pro get_dwd_profile_test, date, intervall=intervall, zrange=zrange, profile=profile
   
   NHEI=1500
   STEPS=10
   t_step=1200
   
   if N_ELEMENTS(profile) ne 0 then begin
      profile=fix(profile)
      if (profile lt 1) && (profile gt 3) then begin
         print, "profile should be in range [1..3]"
         print, " 1: temperature"
         print, " 2: rel. humidity"
         print, " 3: liquid water content"
         return
      endif
   endif else profile=1
   
   if N_ELEMENTS(intervall) eq 0 then  intervall=24
   if N_ELEMENTS(zrange) eq 0 then zrange=[-60.0, 60.0]
   
   if ~(strlen(date) eq 10 || strlen(date) eq 12) then begin 
      print, "Awaiting date-format like ''yyyymmddhh'' or ''yyyymmddhhmmm''"
      return
   endif
   year=fix(strmid(date, 0, 4))
   month=fix(strmid(date, 4, 2))
   day=fix(strmid(date, 6, 2))
   hr=fix(strmid(date, 8, 2))
   if strlen(date) eq 12 then mn=fix(strmid(date, 10, 2)) else mn=0
     
   ix=(JULDAY(month, day, year)-JULDAY(01,01,1970))*24UL*60UL*60UL+hr*3600UL+mn*60UL
   endix=ix+ulong(intervall*3600UL)
   print, 'Start: '+unixtime2bdate(ix, nice=2, /UTC)
   print, 'End  : '+unixtime2bdate(endix, nice=2, /UTC)
   
   hei=findgen(NHEI)*STEPS
   data=fltarr(ulong(intervall*3600UL/t_step),NHEI)*!VALUES.F_NAN
   ;time=ulonarr(ulong(intervall*3600UL/t_step))*0
   time=ulonarr(ulong(intervall*3600UL/t_step))*0
   
   iy=0

   while ix lt endix do begin
      time[iy]=ix

      tmp=get_dwd_profiles(hei, ix, done=done)

      if done gt 0 then begin
         case profile of
            1: data[iy,*]=tmp.temp
            2: if done lt 3 then data[iy,*]=tmp.rHum else data[iy,*]=!VALUES.F_NAN
            3: if done eq 1 then data[iy,*]=tmp.lwc else data[iy,*]=!VALUES.F_NAN
         endcase
      endif else begin
         if profile eq 1 then data[iy,*]=tmp.temp else data[iy,*]=!VALUES.F_NAN
      endelse   
      ix += t_step
      iy += 1
   end  
   print, 'Start: '+unixtime2bdate(time[0], nice=2, /UTC)
   print, 'End  : '+unixtime2bdate(max(time), nice=2, /UTC)
   device, decomposed=0 & ifile=0
      startcolor=30
      loadcolor,startcolor,colorfile='smooth',zcolor=zcolor
      if N_ELEMENTS(path4save) ne 0 then begin
         old_device=!d.NAME
         SET_PLOT, 'Z', /COPY
         device,  SET_RESOLUTION=[1200,800], DECOMPOSED=0, SET_PIXEL_DEPTH=24
      endif else begin
         ;x=(GET_SCREEN_SIZE())[0]/2
         ;y=(GET_SCREEN_SIZE())[1]/2
         window,ifile, xsize=1200, YSIZE=800
      endelse        
        
      sxtf=!x.TICKFORMAT & !x.TICKFORMAT='LABEL_DATE'
      sxtu=!X.TICKUNITS[0] & !X.TICKUNITS[0]='minutes'
      ;dummy = LABEL_DATE(DATE_FORMAT='%h:%i', offset=time_ref)
      dummy = LABEL_DATE(DATE_FORMAT='%H:%I')
      ;dummy = LABEL_DATE(DATE_FORMAT=['%D-%M %H:%I','%Y']) 
      rstart=unixtime2bdate(time[0],nice=2,/utc)
      rend=unixtime2bdate(time[(size(time, /DIMENSIONS))[0]-1],nice=2,/utc)
 
      plottitle='dwd-profiles  ['+rstart+' - '+rend+']'
         
         
      mimage,data, time, [0,uint(NHEI)/100], zrange, zcolor, startcolor, 15, ztitle='Temperature Range',xtitle='Time', heitit='Height AGL [km]', charsize=1.1,thick=1.0
      xyouts,0.49,0.93,plottitle, /noclip, alignment=0.5, /normal, chars=1.1
      !x.TICKFORMAT = sxtf & !X.TICKUNITS[0]=sxtu
      
;      if N_ELEMENTS(path4save) ne 0 then begin
;         img=TVRD(/TRUE)
;         SET_PLOT, old_device
;         file4save=strsplit(fn4ld[ifile],'/', /EXTRACT)
;         file4save=strsplit(file4save[N_ELEMENTS(file4save)-1],'.', /EXTRACT)
;         file4save=TEMPORARY(file4save[0]+file4save[1])
;         print, 'Writing '+file4save+'.png to disc.' 
;         WRITE_PNG, path4save+file4save+'.png', img  
;      endif 
;      window,1, xsize=1060, YSIZE=800
;      plot, unixtime2jul(time), rf, yrange=[0,4], /NOCLIP

end