Pro read1pds_dwell, lun, hdr, ppar,  p_srvinfo, p_data, extract=extract, skip=skip_,RadarConst5=RadarConst5
; read one dwell from pds-files
; Parameter
;   lun         logical unit of the file which has opened before
;   data_size   the size of the 
;   hdr         system parameters with physical units, obtained before by ppar=read_hdr(lun,hdr)
;   ppar        the original ppar structure as read from file
;   p_srvinfo   pointer to struct with dwell header containing information from the log data
;   p_data      pointer to struct with returned data
;   extract     optional array of strings which indicates which profiles should be read and 
;               which should be skipped. it should be a subset of
;                      extrac=['p1','v1','r1','p2','v2','r2','sp1','sp2'].
;               If not given all available profiles are read.
;   skip        skip data from this dwell, this cannot be achieved by setting extract=['']
;               as extract is evaluated only at the first call of read1pds_dwell after reading 
;               the file header.
;   RadarConst5 This keyword parameter contains RadarConst5 id the data is read from a zspc file  
;
;  See the source of gorge_pds_file.pro for information about the structures
;
; Matthias Bauer-Pfundstein 
;
    on_ioerror,ioerr
    if n_elements(skip_) le 0 then skip=0 else skip=skip_

    common initialize,init_extract,init_header_counter
    common read1pds_dwell,extr,spcbuf,cocxbuf,cbuf,r_cocxbuf,ph_cocxbuf,ccoxbuf,ixShiftByNffth,mombuf,intmombuf,expbuf,iqbuf
    common magic4chs,magic4chs,FzkPdsFmt,syspar
    if n_elements(init_extract) eq 0 then begin
        print,'read_hdr has to be called before read1pds_dwell'
        init_extract = 0
        spcbuf = -1
        cbuf = -1
    endif
    if init_extract then begin
        extract_all=['p1','v1','r1','p2','v2','r2','rho','dps','sp1','sp2','cocx','iq1','iq2']
        if n_elements(extract) le 0 then extract=extract_all
        if extract[0] eq 'default' or extract[0] eq '' then extract=extract_all
        extract = strupcase(extract)
        extr = { p1: max(extract eq 'P1' ), $
                 v1: max(extract eq 'V1' ), $
                 r1: max(extract eq 'R1' ), $
                 p2: max(extract eq 'P2' ), $
                 v2: max(extract eq 'V2' ), $
                 r2: max(extract eq 'R2' ), $
                 rhodps: max(extract eq 'rho' ) or max(extract eq 'dps' ), $
                 sp1:max(extract eq 'SP1'), $
                 sp2:max(extract eq 'SP2'), $
                 cocx:max(extract eq 'R_COCX') or max(extract eq 'PH_COCX') or max(extract eq 'CCOX') or max(extract eq 'COCX'), $
                 iq:max(extract eq 'IQ'), $
                 iq1:max(extract eq 'IQ1'), $
                 iq2:max(extract eq 'IQ2')  $
        }
    endif

    lenint=2L
    lenflt=4L
    lenlong=4L
    laprf = hdr.la * hdr.prf 
    nfft=hdr.nfft
    zchan = syspar.zchan
    ;zrg_rpz = hdr.raw_zrg + 2L + syspar.noise3 * (ppar.osc le 1) ; Zahl der RangeGates "Raw Plus Zwei"
    zrg_rpz = hdr.raw_zrg + 2L + syspar.noise3  ; Zahl der RangeGates "Raw Plus Zwei"
    zrg_raw = hdr.raw_zrg
    ; mombufsize = 512L ; k5
    ; mombufsize = 1024L ; 1k
    mombufsize=syspar.mombufsize
    if n_elements(mombuf) ne mombufsize*zchan then begin
        mombuf=fltarr(zchan,mombufsize)
        intmombuf=intarr(zchan,mombufsize*2)
    endif

    items_read=0  ; I think counting items read is obsolete. For every type of data the dwell ends with one designated profile, mostely the spectra
    dwell_done=0
    spc_ex=0
    cocx_ex=0
    ccox_ex=0
    exp_ex=0
    hne_ex=0
    iq_ex=0
    rhodps_ex=0
    cltr_ex=0
    ;print,'entering while read sgnt-blocks of on dwell'
    while dwell_done eq 0 do begin
        sgnt='xxxx' & siz=0L & readu,lun,sgnt,siz
        ; print,sgnt,siz,items_read ; see also print,'osc= ',ppar.osc
        case 1 of
            sgnt eq 'SRVI':begin
                p_srvinfo = ptr_new(read_srvinfo(lun,ppar))
                hdr.len_srvi = siz
            end
            sgnt eq 'HSDV':begin
                if zrg_raw*zchan*lenflt ne siz then stop
                hsdbuf=fltarr(zrg_raw,zchan)
                readu,lun,hsdbuf
                hsd=ptr_new(hsdbuf,/no_copy)   ; It was in the convert spectra section for osc=4
            end
            sgnt eq 'COFA':begin
                if zrg_raw*zchan*lenflt ne siz then stop
                SNRCorFabuf=fltarr(zrg_raw,2)
                ;SNRCorFabuf=fltarr(zrg_raw,zchan)
                readu,lun,SNRCorFabuf
                SNRCorFa=ptr_new(SNRCorFabuf,/no_copy)
                RadarConst5=(*p_srvinfo).DetuneFine
            end
            sgnt eq 'FFTD' or sgnt eq 'ZSPC' or sgnt eq 'ZSPX' or sgnt eq 'ZSPY' or sgnt eq 'SDPD':begin
                ; print,'osc= ',ppar.osc
                zint =  nfft * zchan * zrg_rpz 
                if siz gt 0L and (extr.sp1 or extr.sp2 or extr.iq1 or extr.iq2 or extr.iq) and skip eq 0 then begin
                    case 1 of
                    max(ppar.osc eq [0,2,3,12,16,22,26,30]): begin
                        ; spectra written by the new (since FZK) integer DSP
                        if n_elements(spcbuf) ne zint then begin 
                            if mombufsize eq 512L and ppar.osc eq 0 then begin
                                ; print, 'reading spectra as 64 bit integers and the following dimensions',nfft,zchan,zrg_rpz
                                spcbuf = lonarr(nfft,zchan,zrg_rpz)
                            endif else begin
                                ; print, 'reading spectra as 4 byte floats and the following dimensions',nfft,zchan,zrg_rpz
                                spcbuf = fltarr(nfft,zchan,zrg_rpz)
                            endelse
                        endif
                        readu,lun,spcbuf
                        spc_ex=1
                    end
                    ppar.osc eq 2: begin
                        ; spectra with normal floats and no exponents which where written e.g. by iq2spc.pro
                        if n_elements(spcbuf) ne zint then spcbuf = fltarr(nfft,zchan,zrg_rpz)
                        readu,lun,spcbuf
                        spc_ex=1
                    end
                    ppar.osc eq 4: begin
                        ; concise spectra written by spectraprocessing.pro
                        if n_elements(cbuf) ne long(nfft)*zrg_rpz*zchan then begin
                            cbuf=fltarr(nfft,zrg_rpz,zchan)
                            if sgnt eq 'ZSPX' then begin
                                r_cocxbuf=fltarr(nfft,zrg_rpz)
                                ph_cocxbuf=fltarr(nfft,zrg_rpz)
                            endif
                            if sgnt eq 'ZSPY' then begin
                                ccoxbuf = complexarr(nfft,zrg_rpz) 
                            endif 
                            ixShiftByNffth = (lindgen(nfft)+nfft/2) mod nfft ; this index vector can be used to shift a spectrum by its half length
                        endif
                        if sgnt eq 'ZSPX' then do_cocx=1 else do_cocx=0
                        if sgnt eq 'ZSPY' then do_ccox=1 else do_ccox=0
                        for irg=0,zrg_raw-1 do begin
                            for ichan=0L,zchan-1L do cbuf[*,irg,ichan]=(*hsd)[irg,ichan]
                            if do_cocx then begin
                                r_cocxbuf[*,irg]=0.0
                                ph_cocxbuf[*,irg]=-999.
                            endif
                            if do_ccox then begin
                                ccoxbuf[*,irg] = complex(0.0,0.0) 
                            endif
                            nixabs=0 & readu,lun,nixabs
                            for ixabs=0,nixabs-1 do begin
                                ixa=0 & zix=0 & readu,lun,ixa,zix
                                spcpiece=uintarr(zix,/nozero) & max_=0. 
                                for ichan=0L,zchan-1L do begin
                                    readu,lun,spcpiece,max_,transfer_count=ztrans
                                    cbuf[ixa,irg,ichan] = 1.5260186e-05 * max_ * spcpiece ; in packflt2int.pro the maximum is normalized to 65530. 1/65530=1.5260186e-05
                                endfor
                                if do_cocx then begin
                                    readu,lun,spcpiece,max_
                                    r_cocxbuf[ixa,irg]=1.5260186e-05 * max_ * spcpiece
                                    pmspcpiece=intarr(zix,/nozero)
                                    readu,lun,pmspcpiece,max_
                                    ph_cocxbuf[ixa,irg]=3.0520e-05 * max_ * pmspcpiece ; the angles may be negative. Therefore they where packed by packpmflt2int.pro
                                endif
                                if do_ccox then begin
                                    pmspcpiece_re=intarr(zix,/nozero)
                                    pmspcpiece_im=intarr(zix,/nozero)
                                    max_re=0.0 & max_im=0.0
                                    readu,lun,pmspcpiece_re,max_re,pmspcpiece_im,max_im
                                    ccoxbuf[ixa,irg] = complex(3.0520e-05*max_re*pmspcpiece_re,3.0520e-05*max_im*pmspcpiece_im)
                                endif
                            endfor
                            for ichan=0L,zchan-1L do cbuf[*,irg,ichan]=cbuf[ixShiftByNffth,irg,ichan]

                            ;plot,db(cbuf[*,irg,0L]),yra=[-10,70],yst=1 & oplot,db(cbuf[*,irg,1L])
                            ;wait,0.1
                        endfor
                        if do_cocx and extr.cocx then begin
                            r_cocx = ptr_new( r_cocxbuf[ixShiftByNffth,*] ) 
                            ph_cocx = ptr_new( ph_cocxbuf[ixShiftByNffth,*] )
                            cocx_ex=1
                        endif
                        if do_ccox and extr.cocx then begin
                            ccox = ptr_new( ccoxbuf[ixShiftByNffth,*], /no_copy )
                            ccox_ex=1
                        endif
                        spc_ex=1
                        dwell_done=1 
                    end
                    ppar.osc eq 1: begin
                        ; iqbuf[IoQ,isequence,coOcx,irg]
                        ; IoQ: 0 I, 1 Q
                        ; isequence: 0,..,NFFT-1
                        ; coOcx: 0 co, 1 cx
                        ; irg: RawGate0,..,RawGate1
                        if n_elements(iqbuf) ne zint*2 then iqbuf = intarr(2,nfft,zchan,zrg_rpz)
                        readu,lun,iqbuf
                        iq_ex=1
                    end
                    else: begin
                        print,'unexpected OSC value ',ppar.osc,' Resuming by by trying to read next dwell'
                        goto,ioerr
                    end
                    endcase
                endif else seek,lun,siz
            end
            sgnt eq 'COCX' :begin
                zint =  nfft * zchan * zrg_rpz 
                if siz gt 0L and (extr.cocx) and skip eq 0 then begin
                    case 1 of
                        ppar.osc eq 2:begin   ; spectra from iq2spc old version 
                            if n_elements(cocxbuf) ne zint then cocxbuf = fltarr(nfft,zchan,zrg_rpz)
                            readu,lun,cocxbuf
                            if extr.cocx then begin
                                r_cocx = ptr_new( reform(cocxbuf[*,0,*],nfft,zrg_rpz) ) 
                                ph_cocx = ptr_new( reform(cocxbuf[*,1,*],nfft,zrg_rpz) )
                                cocx_ex=1
                            endif
                        end
                        max(ppar.osc eq [3,16,26]):begin
                            case 1 of 
                                ppar.osc ge 10 and ppar.osc le 19: xspc_normalization = syspar.rx_spc_normalization 
                                ppar.osc ge 20 and ppar.osc le 29: xspc_normalization = syspar.xcrl_spc_normalization
                                else: stop
                            endcase
                            ccoxbuf = fltarr(nfft,zchan,zrg_rpz)
                            readu,lun,ccoxbuf
                            ccoxbuf = reform(complex(ccoxbuf[*,0,*],ccoxbuf[*,1,*]),nfft,zrg_rpz) * xspc_normalization
                            if extr.cocx then ccox = ptr_new( ccoxbuf, /no_copy )
                            ccox_ex=1
                        end
                        else:begin
                        end
                    endcase
                endif else seek,lun,siz
            end
            sgnt eq 'CCOX' :begin
                zint =  nfft * zrg_rpz 
                if siz gt 0L and (extr.cocx) and skip eq 0 then begin
                    ccoxbuf = complexarr(nfft,zrg_rpz)
                    readu,lun,ccoxbuf
                    if extr.cocx then ccox = ptr_new(ccoxbuf,/no_copy)
                    ccox_ex=1
                endif else seek,lun,siz
            end
            sgnt eq 'SNRD': begin
                if siz eq mombufsize*4*zchan and (extr.p1 or extr.p2) and skip eq 0 then begin
                    readu,lun,mombuf
                    if extr.p1 then p1 =  mombuf[0,0:hdr.mom_zrg-1] else p1 = !values.f_nan
                    if extr.p2 then p2 =  mombuf[1,0:hdr.mom_zrg-1] else p2 = !values.f_nan
                    ; if mombuf[0,hdr.mom_zrg] ne (*p_srvinfo).npw1 then goto,ioerr
                    ; if mombuf[0,hdr.mom_zrg+1] ne (*p_srvinfo).cpw1 then goto,ioerr
                    ; if mombuf[1,hdr.mom_zrg] ne (*p_srvinfo).npw2 then goto,ioerr
                    ; if mombuf[1,hdr.mom_zrg+1] ne (*p_srvinfo).cpw2 then goto,ioerr
                endif else begin
                    seek,lun,siz
                    p1 = !values.f_nan
                    p2 = !values.f_nan
                endelse
            end
            sgnt eq 'VELD':begin
                if siz eq mombufsize*4*zchan and (extr.v1 or extr.v2) and skip eq 0 then begin
                    readu,lun,mombuf
                    if extr.v1 then v1 =  laprf * mombuf[0,0:hdr.mom_zrg-1] else v1 = !values.f_nan
                    if extr.v2 then v2 =  laprf * mombuf[1,0:hdr.mom_zrg-1] else v2 = !values.f_nan
                endif else begin
                    seek,lun,siz
                    v1 = !values.f_nan
                    v2 = !values.f_nan
                endelse
            end
            sgnt eq 'RMSD':begin
                if siz eq mombufsize*4*zchan and (extr.r1 or extr.r2) and skip eq 0 then begin
                    readu,lun,mombuf
                    if extr.r1 then r1 = laprf * mombuf[0,0:hdr.mom_zrg-1] else r1 = !values.f_nan
                    if extr.r2 then r2 = laprf * mombuf[1,0:hdr.mom_zrg-1] else r2 = !values.f_nan
                endif else begin
                    seek,lun,siz
                    r1 = !values.f_nan
                    r2 = !values.f_nan
                endelse
            end
            sgnt eq 'XCRD':begin
                if siz eq mombufsize*4*zchan and (extr.rhodps) and skip eq 0 then begin
                    readu,lun,mombuf
                    if extr.rhodps then rho = laprf * mombuf[0,0:hdr.mom_zrg-1] else rho = !values.f_nan
                    if extr.rhodps then dps = laprf * mombuf[1,0:hdr.mom_zrg-1] else dps = !values.f_nan
                    rhodps_ex=1
                endif else begin
                    seek,lun,siz
                    rho = !values.f_nan
                    dps = !values.f_nan
                endelse
            end
            sgnt eq 'EXPD':begin ; gibt es nur bei k5
                if siz eq mombufsize*2*zchan and (extr.sp1 or extr.sp2) and skip eq 0 then begin
                    if n_elements(expbuf) ne mombufsize*zchan then expbuf=intarr(zchan,mombufsize)
                    readu,lun,expbuf
                    exp_ex=1
                endif else seek,lun,siz
                ;if items_read lt 5 then print,'dwel with missing items'
                dwell_done=1
            end
            sgnt eq 'HNED':begin ; gibt es nur beim 1k
               if siz eq mombufsize*4*zchan and skip eq 0 then begin   ; HSDiv1 und HSDiv1 werden immergelesen wenn sie vorhanden sind, es gibt kein extr.HSDiv1/2
                   if ppar.osc eq 1 then begin
                       readu,lun,intmombuf
                       hsdbuf=transpose(float(intmombuf[*,*]))
                   endif else begin
                       readu,lun,mombuf
                       hsdbuf=transpose(mombuf[*,0:hdr.mom_zrg-1])
                   endelse
                   ; eventuell muss statt "0:hdr.mom_zrg-1" das hier eingesetzt werden "ix_exbu =[lindgen(hdr.raw_zrg),hdr.mom_zrg,hdr.mom_zrg+1]"
                   hne_ex=1
               endif else begin
                   seek,lun,siz
                   hsdbuf=!values.f_nan
               endelse
               hsd = ptr_new(hsdbuf,/no_copy)
               ; if items_read ne 5+max(ppar.osc eq [16,26]) then print,'dwel with missing items. Number of items read: ',items_read
               dwell_done=1
            end
            sgnt eq 'CLTR':begin
                if siz ne 0L then begin
                    npatchclutter_=0L & zchan_=0L & rgclutter_=0L & readu,lun,npatchclutter_,zchan_,rgclutter_
                    RemovedSpcs=fltarr(npatchclutter_,zchan_,rgclutter_) & readu,lun,RemovedSpcs
                    cltr_ex=1
                endif
            end
            else: begin
                print,'unrecognized data tag '+sgnt+' with length '+string(siz)+'. skipping over it.'
                ;stop
                if siz gt 0 then begin
                    skipbytes=bytarr(siz) & readu,lun,skipbytes
                endif
            end
        endcase
        ++items_read
    endwhile

    ;print,'exp_ex=',exp_ex,' spc_ex=',spc_ex,' osc=',ppar.osc
    case 1 of
        (spc_ex and ppar.osc eq 2): begin  ; spectra from iq2spc or pds output from other idl programs 
            if extr.sp1 then sp1 = ptr_new( reform(spcbuf[*,0,*],nfft,zrg_rpz) ) else sp1 = ptr_new(!values.f_nan)
            if extr.sp2 then sp2 = ptr_new( reform(spcbuf[*,1,*],nfft,zrg_rpz) ) else sp2 = ptr_new(!values.f_nan)
        end
        (spc_ex and ppar.osc eq 4): begin ; spectra from zspc 
            ; fake the noise and reference spectra so that spectraprocessing can calculate npw and cpw from it.
            cbuf[*,zrg_raw,0L]=(*p_srvinfo).npw1/nfft
            cbuf[*,zrg_raw+1,0L]=(*p_srvinfo).cpw1/nfft
            if zchan eq 2 then begin
                cbuf[*,zrg_raw,1L]=(*p_srvinfo).npw2/nfft
                cbuf[*,zrg_raw+1,1L]=(*p_srvinfo).cpw2/nfft
            endif
            ; hsd=ptr_new(hsdbuf,/no_copy)   ; I dont understand wyh it was here so I put it up to the place where it was read
            if extr.sp1 then sp1 = ptr_new(cbuf[*,*,0L]) else sp1 = ptr_new(!values.f_nan)
            if zchan eq 2 && extr.sp2 then sp2 = ptr_new(cbuf[*,*,1L]) else sp2 = ptr_new(!values.f_nan)
        end
        (spc_ex and (exp_ex or hne_ex)):begin
            if exp_ex then begin
                ix_exbu =[lindgen(hdr.raw_zrg),hdr.mom_zrg,hdr.mom_zrg+1]    ; richtig
                nfftme = replicate(1.,nfft)
                if extr.sp1 then sp1 = ptr_new( syspar.pds_spectra_normalization*reform(float(spcbuf[*,0,*]),nfft,zrg_rpz) * ( nfftme # (2. ^ expbuf[0,ix_exbu])))  else sp1 = ptr_new(!values.f_nan)
                if zchan eq 2 && extr.sp2 then sp2 = ptr_new( syspar.pds_spectra_normalization*reform(float(spcbuf[*,1,*]),nfft,zrg_rpz) * ( nfftme # (2. ^ expbuf[1,ix_exbu])))  else sp2 = ptr_new(!values.f_nan)
            endif else begin ; spectra from uBlaze radarservers or rx_client
                ;if abs(syspar.pds_spectra_normalization -1.0) le 0.1 then rx_client2dsp_scaling=1./(32768.^2) else rx_client2dsp_scaling=1.0 ; (32768./1.852)^2
                case 1 of
                    ppar.osc ge 10 and ppar.osc le 19: normalization = syspar.rx_spc_normalization
                    ppar.osc ge 20 and ppar.osc le 29: normalization = syspar.xcrl_spc_normalization
                    else:normalization = syspar.pds_spectra_normalization
                endcase
                if extr.sp1 then sp1 = ptr_new( normalization*reform(spcbuf[*,0,*],nfft,zrg_rpz) ) else sp1 = ptr_new(!values.f_nan)
                if zchan eq 2 && extr.sp2 and zchan ge 2 then sp2 = ptr_new( normalization*reform(spcbuf[*,1,*],nfft,zrg_rpz) ) else sp2 = ptr_new(!values.f_nan)
            endelse
        end
        else: begin
            sp1 = ptr_new(!values.f_nan)
            sp2 = ptr_new(!values.f_nan)
        end
    endcase
    
    if spc_ex and ppar.osc eq 4 then begin
        if extr.p1 eq 0 and extr.p2 eq 0 and extr.v1 eq 0 and extr.v2 eq 0 and extr.r1 eq 0 and extr.r2 eq 0 then begin
            p1=!values.f_nan
            p2=!values.f_nan
            v1=!values.f_nan
            v2=!values.f_nan
            r1=!values.f_nan
            r2=!values.f_nan
        endif else begin
            print, 'Moment profiles where requested by the extract parameter but they are not present in the concise data'
        endelse
    endif

    if rhodps_ex eq 0 then begin
        rho=!values.f_nan
        dps=!values.f_nan
    endif

    if cocx_ex eq 0 or extr.cocx eq 0 then begin
        r_cocx = ptr_new(!values.f_nan)
        ph_cocx = ptr_new(!values.f_nan)
    endif

    if ccox_ex eq 0 or extr.cocx eq 0 then begin
        ccox = ptr_new(!values.f_nan)
    endif

    if iq_ex then begin
        if  extr.iq1 then iq1=ptr_new( complex(float(iqbuf[*,*,0,*]),0,nfft,zrg_rpz) ) else iq1=ptr_new(!values.f_nan)
        if  extr.iq2 then iq2=ptr_new( complex(float(iqbuf[*,*,1,*]),0,nfft,zrg_rpz) ) else iq2=ptr_new(!values.f_nan)
        if  extr.iq then iq = ptr_new( iqbuf[*,*,*,*] ) else iq=ptr_new(!values.f_nan)
    endif else begin
        iq1 = ptr_new(!values.f_nan)
        iq2 = ptr_new(!values.f_nan)
        iq = ptr_new(!values.f_nan)
    endelse
   
    if n_elements(hsd) eq 0 then hsd=ptr_new(!values.f_nan)
    if n_elements(SNRCorFa) eq 0 then begin
        SNRCorFa=ptr_new(!values.f_nan)
        RadarConst5=!values.f_nan
    endif

    p_data = ptr_new({ $
        p1  : p1  ,   $
        v1  : v1  ,   $
        r1  : r1  ,   $
        p2  : p2  ,   $
        v2  : v2  ,   $
        r2  : r2  ,   $
        sp1 : sp1 ,   $
        sp2 : sp2 ,   $
        r_cocx : r_cocx ,   $
        ph_cocx : ph_cocx ,  $
        ccox : ccox , $
        hsd : hsd ,   $
        SNRCorFa : SNRCorFa ,   $
        iq : iq ,   $
        iq1 : iq1 ,   $
        iq2 : iq2 ,   $
        rho : rho , $
        dps : dps  $
    })
    return
ioerr:
    ;stop
    p_data = ptr_new({ $
        p1  : !values.f_nan  ,   $
        v1  : !values.f_nan  ,   $
        r1  : !values.f_nan  ,   $
        p2  : !values.f_nan  ,   $
        v2  : !values.f_nan  ,   $
        r2  : !values.f_nan  ,   $
        sp1 : ptr_new(!values.f_nan)  ,   $
        sp2 : ptr_new(!values.f_nan)  ,   $
        r_cocx : ptr_new(!values.f_nan)  ,   $
        ph_cocx : ptr_new(!values.f_nan)  ,   $
        ccox : ptr_new(!values.f_nan)  ,   $
        hsd : ptr_new(!values.f_nan) ,   $
        SNRCorFa : ptr_new(!values.f_nan) ,   $
        iq : ptr_new(!values.f_nan)  ,   $
        iq1 : ptr_new(!values.f_nan)  ,   $
        iq2 : ptr_new(!values.f_nan)  ,   $
        rho : !values.f_nan , $
        dps : !values.f_nan  $
    })
end

