pro read1znc_dwell, ncdfid, hdr, ppar,  p_srvinfo, p_data, iti, allsrvinfo, extract=extract
; read one profile of spectra from a znc file
; Parameter
;   ncin        id of the already opened ncdf-file
;   hdr         system parameters with physical units, obtained before by ppar=read_hdr(lun,hdr), probably not needed
;   ppar        the original ppar structure as read from file, probably not needed
;   p_srvinfo   pointer to struct with dwell header containing information from the log data
;   p_data      pointer to struct with returned data
;   extract     optional array of strings which indicates which profiles should be read and 
;               which should be skipped. it should be a subset of
;                      extrac=['p1','v1','r1','p2','v2','r2','sp1','sp2'].
;               If not given all available profiles are read.
;
; Matthias Bauer-Pfundstein 
;
    common magic4chs,magic4chs,FzkPdsFmt,syspar

    nfft=hdr.nfft
    zchan = hdr.pol+1L
    zrg_rpz = hdr.raw_zrg + 2L + syspar.noise3  ; Zahl der RangeGates "Raw Plus Zwei"
    zrg_raw = hdr.raw_zrg
    ;lind = ((lindgen(nfft)+nfft/2) mod nfft) ; the spectra are stored so that the peak is typically in the middle which is more efficient in zspc. Now they have to be resaranged as in pds data.
    lind = lindgen(nfft) ; the spectra are stored so that the peak is typically at fft line 0 or nfft-1 as usually done in case of spectra. Its don different than in zspc where haveing the peak in the miiddle is more efficient

    p_srvinfo = ptr_new(read_srvinfo_nc(iti,allsrvinfo))
    ;hdr.len_srvi = n_tags(p_srvinfo,/bytes)

    if max(extract eq 'SP1') ne 0 then begin
        NCDF_VARGET, ncdfid, 'SPCco',SPCxx,offset=[0L,0L,iti],count=[nfft,zrg_raw,1L]
        ccbuf=fltarr(nfft,zrg_rpz)
        ;ccbuf[0L,0L] = 1/256.*SPCxx[lind,*]
        ccbuf[0L,0L] = SPCxx[lind,*]
        ccbuf[*,zrg_raw]=(*p_srvinfo).npw1/nfft
        ccbuf[*,zrg_raw+1]=(*p_srvinfo).cpw1/nfft
        sp1 = ptr_new(ccbuf,/no_copy)
    endif else begin
        sp1 = ptr_new(!values.f_nan)
    endelse
    if max(extract eq 'SP2') ne 0 then begin
        NCDF_VARGET, ncdfid, 'SPCcx',SPCxx,offset=[0L,0L,iti],count=[nfft,zrg_raw,1L]
        ccbuf=fltarr(nfft,zrg_rpz)
        ;ccbuf[0L,0L] = 1/256.*SPCxx[lind,*]
        ccbuf[0L,0L] = SPCxx[lind,*]
        ccbuf[*,zrg_raw]=(*p_srvinfo).npw2/nfft
        ccbuf[*,zrg_raw+1]=(*p_srvinfo).cpw2/nfft
        sp2 = ptr_new(ccbuf,/no_copy)
    endif else begin
        sp2 = ptr_new(!values.f_nan)
    endelse

    if max(extract eq 'COCX') eq 1 then begin
        NCDF_VARGET, ncdfid, 'SPCcocxRe',SPCcocxRe,offset=[0L,0L,iti],count=[nfft,zrg_raw,1L] 
        NCDF_VARGET, ncdfid, 'SPCcocxIm',SPCcocxIm,offset=[0L,0L,iti],count=[nfft,zrg_raw,1L]
        ;ccox = ptr_new(complex(1/256.*SPCcocxRe[lind,*],1/256.*SPCcocxIm[lind,*]))
        ccox = ptr_new(complex(SPCcocxRe[lind,*],SPCcocxIm[lind,*]))
    endif else begin
        ccox = ptr_new(!values.f_nan)
    endelse

    ; extrac these always if available
    NCDF_VARGET, ncdfid, 'HSDco',hilde_nls_co,offset=[0L,iti],count=[zrg_raw,1L]
    if syspar.zchan ge 2 then begin 
        NCDF_VARGET, ncdfid, 'HSDcx',hilde_nls_cx,offset=[0L,iti],count=[zrg_raw,1L]
        hsd=ptr_new([[hilde_nls_co],[hilde_nls_cx]],/no_copy)
    endif else hsd=ptr_new([[hilde_nls_co]],/no_copy)
    ppar.osc=4      ; prevents recalculation of Hildebrand-Sekhon in Spectraprocessing as for zspc
    if n_elements(hsd) eq 0 then hsd=ptr_new(!values.f_nan)

    NCDF_VARGET, ncdfid, 'SNRCorFaCo',SNRCorFaCo,offset=[0L,iti],count=[zrg_raw,1L]
    if syspar.zchan ge 2 then begin
        NCDF_VARGET, ncdfid, 'SNRCorFaCx',SNRCorFaCx,offset=[0L,iti],count=[zrg_raw,1L]
        SNRCorFa=ptr_new([[SNRCorFaCo],[SNRCorFaCx]],/no_copy)
    endif else SNRCorFa=ptr_new([[SNRCorFaCo],[SNRCorFaCo]],/no_copy)
    if n_elements(SNRCorFa) eq 0 then SNRCorFa=ptr_new(!values.f_nan)

    p_data = ptr_new({ $
        sp1 : sp1 ,   $
        sp2 : sp2 ,   $
        r_cocx : ptr_new(!values.f_nan) ,   $
        ph_cocx : ptr_new(!values.f_nan) ,  $
        ccox : ccox ,  $
        hsd : hsd ,    $
        SNRCorFa : SNRCorFa   $
    })
    return
end
