function un_gorge,hdr,parr_data,itime=itime,irg=irg,ifrq=ifrq,tag=tag
; Extract an array from the pointer array parr_data which is provided by gorge_pds_file.pro.
; This function should do the elaborate pointer gym so the user of gorge_pds_file.pro should
; have no need for careing about how the data is organized in pointers, arrays, and structures.
; The dwell index itime, the range index irg, and the frequency index may be specified as:
;     scalar               for selecting a slice, the coresponding dimension in the output array will degenerate
;     two element vector   for selecting range of elements of the corresponding dimension
;     not specified        for transfering all elements of the corresponding dimension
; Yet un_gorge handles only a subset of the 27 cases which result in these possibilities for setting.
;
; Matthias Bauer-Pfundstein bauer@metek.de
;
    zs_time=n_elements(itime)
    zs_rg=n_elements(irg)
    zs_frq=n_elements(ifrq)
    ztime=n_elements(parr_data)
    if n_elements(tag) eq 1 then begin
        if (size(tag))[1] eq 7 then begin
            ; convert from tag name to tag index
            itag=(where(tag_names(*parr_data[0]) eq strupcase(tag)))[0] 
        endif else itag=long(tag)
    endif else itag=-1l
    if itag lt 0 then begin
        print,'unknown tag name ',strupcase(tag)
        print,'only these tags are in the file: ',tag_names(*parr_data[0])
        return,!values.F_NAN
    endif
    is_profile_or_spc = size((*parr_data[0]).(itag),/type) eq 10
    if is_profile_or_spc then begin
        ; sectra
        zrg=hdr.raw_zrg
        available = n_elements(*(*parr_data[itime[0]]).(itag)) ge hdr.nfft*(hdr.raw_zrg)
        if not available then begin
            print,'No spectra available for tag ' + (tag_names(*parr_data[0]))[itag]
            return,!values.F_NAN
        endif
        case 1 of
        zs_time eq 1 and zs_rg ne 1 and zs_frq ne 1: begin
            ; spectra of one dwell ==> vals[iifrq,iirg]
            if zs_rg eq 0 and zs_frq eq 0 then vals = *(*parr_data[itime]).(itag) else begin
                if zs_rg ne 2 then irg=[0L,hdr.raw_zrg+1]
                if zs_frq ne 2 then ifrq=[0L,hdr.nfft-1]
                vals = (*(*parr_data[itime]).(itag))[ifrq[0]:ifrq[1],irg[0]:irg[1]]
            endelse
        end
        zs_time ne 1 and zs_rg eq 1 and zs_frq ne 1: begin
            ; spectra of one range ==> vals[iifrq,iitime]
            if zs_frq ne 2 then ifrq=[0L,hdr.nfft-1]
            if zs_time ne 2 then itime=[0L,ztime-1]
            vals=fltarr(ifrq[1]-ifrq[0]+1L, itime[1]-itime[0]+1L, /nozero)
            for iitime=itime[0],itime[1] do begin
                if finite( (*(*parr_data[iitime]).(itag))[0] ) then begin
                    vals[0,iitime-itime[0]]=(*(*parr_data[iitime]).(itag))[ifrq[0]:ifrq[1],irg]
                endif else vals[*,iitime-itime[0]] = !values.f_nan
            endfor
        end
        zs_time ne 1 and zs_rg ne 1 and zs_frq ne 1: begin
            ; all spectra ==> vals[iifrq,iirg,iitime]
            if zs_rg ne 2 then irg=[0L,hdr.raw_zrg+1]
            if zs_frq ne 2 then ifrq=[0L,hdr.nfft-1]
            if zs_time ne 2 then itime=[0L,ztime-1]
            vals=fltarr(ifrq[1]-ifrq[0]+1L, irg[1]-irg[0]+1L, itime[1]-itime[0]+1L, /nozero)
            for iitime=itime[0],itime[1] do begin
                if finite( (*(*parr_data[iitime]).(itag))[0] ) then begin 
                    vals[0,0,iitime-itime[0]]=(*(*parr_data[iitime]).(itag))[ifrq[0]:ifrq[1],irg[0]:irg[1]]
                endif else vals[*,0,iitime-itime[0]] = !values.f_nan
            endfor
        end
        else:begin
            print, 'You have chosen a combination of itime, irg, and ifrq which has not been implemeted yet.'
            vals = !values.F_NAN
        end
        endcase
    endif else begin
        ; moment data
        zrg = hdr.mom_zrg
        available = n_elements((*parr_data[itime[0]]).(itag)) ge hdr.mom_zrg
        if not available then begin
            print, 'No moments available for tag ' + (tag_names(*parr_data[0]))[itag]
            return, !values.F_NAN
        endif
        case 1 of
            zs_time eq 1 and zs_rg ne 1: begin
                ; profile of the selected moment ==> vals[iirg]
                if zs_rg eq 0 and zs_frq eq 0 then vals = (*parr_data[itime]).(itag) else begin
                    if zs_rg ne 2 then irg=[0L,hdr.raw_zrg+1]
                    vals = ((*parr_data[itime]).(itag))[irg[0]:irg[1]]
                endelse
            end
            zs_time ne 1 and zs_rg eq 1: begin
                ; time sequence of the selected moment ==> vals[iitime]
                if zs_time ne 2 then itime=[0L,ztime-1]
                vals=fltarr(itime[1]-itime[0]+1L, /nozero)
                for iitime=itime[0],itime[1] do begin
                    if n_elements( ((*parr_data[iitime]).(itag)) ) ge zrg then begin 
                        vals[iitime-itime[0]]=((*parr_data[iitime]).(itag))[irg]
                    endif else vals[iitime-itime[0]] = !values.f_nan
                endfor
            end
            zs_time ne 1 and zs_rg ne 1: begin
                ; time-hight section of the selected moment ==> vals[iirg,iitime]
                if zs_rg ne 2 then irg=[0L,hdr.raw_zrg+1]
                if zs_time ne 2 then itime=[0L,ztime-1]
                vals=fltarr(irg[1]-irg[0]+1L, itime[1]-itime[0]+1L, /nozero)
                for iitime=itime[0],itime[1] do begin
                    if n_elements( ((*parr_data[iitime]).(itag)) ) ge zrg then begin
                        vals[0,iitime-itime[0]]=((*parr_data[iitime]).(itag))[irg[0]:irg[1]]
                    endif else vals[*,iitime-itime[0]] = !values.f_nan
                endfor
            end
            else:begin
                print, 'You have chosen a combination of itime, irg which has not been implemeted yet.'
                vals = !values.F_NAN
            end
        endcase
    endelse
    return,vals
end
