function iqsimu,ppar,targetrange=targetrange,targetpow=targetpow,targetvel=targetvel,targetldr=targetldr,tansfrq=transfrq,pulsewidth=pulsewidth,prf=prf,nfft=nfft,nave=nave,sampfrq=sampfrq,isamprg=isamprg,zpreNappend=zpreNappend,ranges=ranges,pulseshape=pulseshape,IronTxPhase=IronTxPhase,unlimiterfac=unlimiterfac
; generate raw I-Q signals of one pulse cycle. 
; To save memory the I-Q signals are only calculated for a range of samples specified by the parameter isamprg=[isamprg_begin,isamprg_end]
; isamprg corresponds to the height_range = [isamprg_begin,isamprg_end] * c / (2*sampfrq)
;
; Noise is added and the power of the I-Q signals is normalized so that its spectral power is 1.0. To bring it to the equivalent receiver noise level
; the calculated I-Q signals should be multiplied by K_B*T*N_F. The added noise is white. In real I-Q signals typically the highest frequencies 
; are flattened by analog filters. This difference between syntesized and real I-Q signal does no matter as the digital filter cancles these 
; frequencies anyway. The noise power after digital filtering correspons to the bandwidth. 
;
; A list of targets may be specified by the parameters targetrange, targetpow, targetldr
; The target power hast to be specified relative to system noise temperature which was normalized to 1.0 * Bandwidth.
common iqsimucomm,seed,seed1
if n_elements(sampfrq) ne 1 then sampfrq=125e6 ; its the sampling frequency of the digital filter, the frequency at which I-Q pairs are generated is 0.5*sampfrq
c=3e8
co=0L & cx=1L
meterspersamp = c/(sampfrq) ; *2/2
if n_elements(targetrange) eq 0 then targetrange=[1e3]
minrange=min(targetrange) & maxrange=max(targetrange)
ntargets=n_elements(targetrange)
if n_elements(IronTxPhase) ne 1 then IronTxPhase=0
if n_elements(unlimiterfac) ne 1 then unlimiterfac=1.0
if n_elements(targetpow) ne ntargets then targetpow=replicate(1.0,ntargets)
if n_elements(targetvel) ne ntargets then targetvel=replicate(+0.7,ntargets)
if n_elements(targetldr) ne ntargets then targetldr=replicate(1e-15,ntargets)
if size(pulseshape,/type) eq 7 then begin
    pulseshapefile=pulseshape
    openr,lun,pulseshapefile,/get_lun
    n=read_txn(lun,ix,absiq,iqpulse)
    free_lun,lun
    ziqpulse=n_elements(iqpulse)
    iqphase = -1*atan(real_part(iqpulse),imaginary(iqpulse))*31.25/!pi 
    iqdps = cmod(iqphase[1:ziqpulse-1]-iqphase[0:ziqpulse-2],62.5,0)
    ;absiq=abs(iqpulse) ; in the file abs(iq)^2 is saved as absiq which rather the instantaneous power than the magnitude. Here we want ot have the magnitude
    maxiq=max(absiq)
    ixmanpulse = where(absiq ge maxiq*0.3,nix)
    ixstart = ixmanpulse[0]
    ixend = ixmanpulse[nix-1]
    if n_elements(pulsewidth) ne 1 then begin
        pulsewidth=(ixend-ixstart)*16e-9
        print,'Pulse width estimated from '+pulseshapefile+' is ',pulsewidth
    endif
    ixfrom=(ixstart-2)>0 & ixto=(ixend+2)<(n_elements(absiq)-1)
    unlimiter=linear(((iqdps[ixfrom:ixto]-median(iqdps[ixfrom:ixto]))<0.)*unlimiterfac)
    if IronTxPhase then begin
        pulseshape=abs(iqpulse[ixfrom:ixto])*unlimiter
    endif else begin
        pulseshape=iqpulse[ixfrom:ixto]*unlimiter
    endelse
    ;timex=findgen(ixto-ixfrom+1)*16.
    ;plot,timex,pulseshape/max(pulseshape),yrange=[-2,2]
    ;oplot,timex+8,iqdps[ixfrom:ixto]/max(abs(iqdps[ixfrom:ixto])),color=1
    ;stop
endif
if n_elements(pulseshape) gt 0 then begin
        zpulse=n_elements(pulseshape)
        if n_elements(pulsewidth) ne 1 then pulsewidth=2*zpulse/sampfrq
        pulseshape=1./mean(abs(pulseshape))*pulseshape
endif
if n_elements(pulsewidth) ne 1 then begin
    pulsewidth=208e-9
endif
if n_elements(zpulse) ne 1 then zpulse=floor(0.5*pulsewidth*sampfrq+1.5)
if n_elements(pulseshape) eq 0 then pulseshape=replicate(1.0,zpulse)
if n_elements(zpreNappend) ne 1 then zpreNappend=64
if n_elements(isamprg) ne 2 then isamprg=[floor(minrange/meterspersamp),ceil((maxrange+c*pulsewidth/2.)/meterspersamp)]+zpreNappend*[-1,1]
if n_elements(transfrq) ne 1 then transfrq=35.1e9
if n_elements(prf) ne 1 then prf=5000.
if n_elements(nfft) ne 1 then nfft=1
if n_elements(nave) ne 1 then nave=1
lambda=c/transfrq
vel2dph = 4.0*!pi/(prf*lambda)

zsamp=isamprg[1]-isamprg[0]+1 & isamprg0=isamprg[0]
isamp0=isamprg[0]
ranges=(findgen(zsamp)+isamprg[0])*meterspersamp
zchan=1
iq=complex(randomn(seed,zsamp,nfft,nave,zchan),randomn(seed,zsamp,nfft,nave,zchan))/sqrt(2.0)
phasesoftarget=randomn(seed1,ntargets,nave)*2.0*!pi ; adds random phase to each target, even if all targets are placed in the same position
for itarget=0,ntargets-1 do begin
    istart=targetrange[itarget]/meterspersamp-isamprg[0] & istop=istart+zpulse-1
    tpoi=targetpow[itarget] 
    dph = targetvel[itarget]*vel2dph
    for iave=0,nave-1 do begin
        tphi=phasesoftarget[itarget,iave]
        for ifft=0,nfft-1 do begin
            iq[istart:istop,ifft,iave,co] = iq[istart:istop,ifft,iave,co] + tpoi * complex(cos(tphi+ifft*dph),sin(tphi+ifft*dph)) * pulseshape
        endfor
    endfor
endfor
ppar={transfrq:transfrq,pulsewidth:pulsewidth,prf:prf,nfft:nfft,nave:nave,sampfrq:sampfrq,zpreNappend:zpreNappend,ranges:ranges,lambda:lambda,vel2dph:vel2dph,zsamp:zsamp,isamp0:isamp0,zchan:zchan,meterspersamp:meterspersamp,minrange:minrange,maxrange:maxrange}
return,iq
end
