function superraw2spc,iq,ppar,filteredranges=filteredranges,iqfilteredpowerave=iqfilteredpowerave,iqfilteredpower=iqfilteredpower,spc=spc,hsd=hsd,pow=pow,zgatebinwidth=zgatebinwidth,filterwidth=filterwidth,hammingalpha=hammingalpha,RBL_nofilter=RBL_nofilter
; Calculate filtered iq values from raw iq values produced by iqsimu.pro, then calculate the unaveraged spectrum
; A gaussian filter with zgatebinwidth elements is used, or if hammingalpha is specified a hamming generalized hanning filter with cropped zeros on both sides is used. 
; The filter filterwidth is given as standard deviation sigma as on the 10 DMark note, but the maximum of the filter curve is 1.0.
; The output is calculated with maximum possible range oversampling, i.e. the filter is shifted samble by sample for generating the filtered iq range profiles. Like this, for targets located at one range the range weighting function can be found in the range profile.
if n_elements(filterwidth) ne 1 then filterwidth=1.0     ; Default=1.0,such wide window emulates a rectangular window.
if n_elements(hammingalpha) ne 1 then hammingalpha=-1.0
if n_elements(zgatebinwidth) ne 1 then begin
    print,'The filter width zgatebinwidth has to be specified'
endif
if n_elements(RBL_nofilter) ne 1 then RBL_nofilter=5.79557e+11   ; this constant must be set to avgpower/mean_noisepower*bwd  or avg(SNR)*bwd  calculated with no filter: dummy=superraw2spc(iq,ppar,filteredra=fra,iqfilteredpowerave=iqfpa,zgatebinwidth=1,filterwidth=1.0)
ziqfiltered=ppar.zsamp-zgatebinwidth
if zgatebinwidth ge 2 then begin
    if hammingalpha gt 0.0 then begin
        filter=( (hanning(zgatebinwidth+1,alpha=hammingalpha))[1:zgatebinwidth] )^1.13
    endif else begin
        fti = [findgen(zgatebinwidth/2)-zgatebinwidth/2,findgen(zgatebinwidth-zgatebinwidth/2)]*2./ppar.sampfrq
        filter=exp(-0.5*(fti/filterwidth)^2)
    endelse
endif else filter = [1.0]
sumfilter=total(filter)
filter=filter/sqrt(sumfilter)
; estimate the bandwidth of the filter
nfft4filterbwd=256
filter256=replicate(0.0,nfft4filterbwd)
filter256[0]=filter
spcfilter=abs(fft(filter256,-1))^2
df=0.5*ppar.sampfrq / nfft4filterbwd
bwd=total(spcfilter)/max(spcfilter)*df
filter=filter*sqrt(bwd*ppar.pulsewidth)
iqfiltered=complexarr(ziqfiltered,ppar.nfft,ppar.nave,ppar.zchan)
iqfilterepow_ave=fltarr(ziqfiltered,ppar.nfft,ppar.zchan)
filteredranges=fltarr(ziqfiltered)
for ichan=0,ppar.zchan-1 do begin
    for ifiltered=0,ziqfiltered-1 do begin
        for ifft=0,ppar.nfft-1 do begin
            for iave=0,ppar.nave-1 do begin
                iqfiltered[ifiltered,ifft,iave,ichan]=total(filter*iq[ifiltered:ifiltered+zgatebinwidth-1,ifft,iave,ichan])
            endfor
            filteredranges[ifiltered]=ppar.ranges[ifiltered+zgatebinwidth/2]
        endfor
    endfor
endfor

; Spectra
if ppar.nfft gt 1 then begin
    of0=55 
    spc=fltarr(ppar.nfft,ziqfiltered,ppar.zchan)
    hsd=fltarr(ziqfiltered,ppar.zchan)
    pow=fltarr(ziqfiltered,ppar.zchan)
    fftwin=hanning(ppar.nfft) & fftwinloss=sqrt(total(fftwin^2)/ppar.nfft)
    for ichan=0,ppar.zchan-1 do begin
        for ifiltered=0,ziqfiltered-1 do begin
            spcavebuf=replicate(0.0,ppar.nfft)
            for iave=1,ppar.nave-1 do begin
                spccomplex = fft(reform(fftwin*iqfiltered[ifiltered,*,iave,ichan]),-1)*(sqrt(ppar.nfft)/fftwinloss)
                spcx=real_part(spccomplex*conj(spccomplex))
                spcavebuf = spcavebuf + spcx
            endfor
            spcavebuf = spcavebuf / ppar.nave
            spc[*,ifiltered,ichan]=spcavebuf
            hsdx = fhilde(spcavebuf,ofdiv=9.5,ndiv=16,n_spcave=ppar.nave)
            hsd[ifiltered,ichan]=hsdx
            issignal=spcavebuf ge hsdx * (1.0+0.1*of0/sqrt(ppar.nave))
            ixsignal=where(issignal,nixsignal)
            if nixsignal gt 0 then begin
                powx=total(spcavebuf[ixsignal]-hsdx)
            endif else begin
                powx=0.0
            endelse
            ;if ifiltered eq 58 then stop
            pow[ifiltered,ichan]=powx
        endfor
    endfor
endif

; evaluate the signals
print,''
print,'iq signal results with zgatebinwidth '+strgg(zgatebinwidth)+' filterwidth '+strgg(filterwidth) + ' minrange,maxrange '+strgg(ppar.minrange)+' '+strgg(ppar.maxrange) + ' Receiver Bandwidth '+strgg(bwd)

; iqpower = real_part(iq*conj(iq))
zOnlyNoiseh=ppar.ZPRENAPPEND-zgatebinwidth-1
ixnoise_iqf=[findgen(zOnlyNoiseh),ziqfiltered-zOnlyNoiseh+findgen(zOnlyNoiseh)]
ixsignal_iqf = zOnlyNoiseh+findgen(ziqfiltered-2*zOnlyNoiseh)


iqfilteredpower = real_part(iqfiltered*conj(iqfiltered))
iqfilteredpowerave=replicate(0.0,ziqfiltered,ppar.nfft,ppar.zchan)
for iave=1,ppar.nave-1 do begin
    iqfilteredpowerave = iqfilteredpowerave + reform(iqfilteredpower[*,*,iave,*],ziqfiltered,ppar.nfft,ppar.zchan)
endfor
iqfilteredpowerave=iqfilteredpowerave / ppar.nave
mean_noisepower = mean(iqfilteredpowerave[ixnoise_iqf,*,*])

max_powers = fltarr(ppar.nfft) 
for ifft=0,ppar.nfft-1 do begin
    max_powers[ifft]=max(iqfilteredpowerave[ixsignal_iqf,ifft,0],imax)
endfor
imaxpower=imax+zOnlyNoiseh ; Diese Hilfsgroesse wird nur aus der letzten von nfft Durchlauf der obigen Schleife abgeleitet.
max_power=mean(max_powers)-mean_noisepower
print,'Max(SNR):' + strgg(max_power/mean_noisepower) + '  Mean Noise Power:' + strgg(mean_noisepower) 

if ppar.minrange eq ppar.maxrange then begin
   RangeWeightingFunctionWidths=fltarr(ppar.nfft) & for ifft=0,ppar.nfft-1 do RangeWeightingFunctionWidths[ifft]=total(iqfilteredpowerave[ixsignal_iqf,ifft]/max_power)*ppar.meterspersamp & RangeWeightingFunctionWidth=mean(RangeWeightingFunctionWidths)
   print,'Width of the Range Weighting Function:'+strgg(RangeWeightingFunctionWidth)
endif

if ppar.minrange+0.5*zgatebinwidth*ppar.meterspersamp lt ppar.maxrange-0.5*zgatebinwidth*ppar.meterspersamp-5 then begin
    ixfilled=where(ppar.ranges gt ppar.minrange+0.5*zgatebinwidth*ppar.meterspersamp+30. and ppar.ranges lt ppar.maxrange-0.5*zgatebinwidth*ppar.meterspersamp-30.,nixfilted)
    avgpowers=fltarr(ppar.nfft) & for ifft=0,ppar.nfft-1 do avgpowers[ifft]=mean(iqfilteredpowerave[ixfilled,ifft]) & avgpower=mean(avgpowers)
    print,'avg(SNR):' + strgg(avgpower/mean_noisepower) + '  Mean Noise Power:' + strgg(mean_noisepower) + '  estimated RBL:' + strgg(db(avgpower/mean_noisepower*bwd/RBL_nofilter))+' dB'
endif

print,''
return,iqfiltered
end

