function header2ncdf,fn,ppar,hdr,extras=extras,ptypes=ptypes,do_ncdf4zip=do_ncdf4zip,include_special_gates=include_special_gates
    ;  Create the NetCDF file header where the variables and dimensions are defined
    if n_elements(extras) eq 0 then extras=['']
    if n_elements(do_ncdf4zip) eq 0 then do_ncdf4zip=0
    if n_elements(include_special_gates) eq 0 then include_special_gates=0
common magic4chs,magic4chs,FzkPdsFmt,syspar

lopa={$
        convention:'CF-1.0',$
        location:hdr.place,$ ; string(hdr.PLACE) from /icfg/header.ini
        Altitude:'Unknown',$
        Latitude:'Unknown',$
        Longitude:'Unknown',$
        system:syspar.iam,$
        title:'MIRA Cloud Radar Data',$
        institution:'Unknown',$
        source:hdr.name,$ ; put the pds file name created by the radarserver here
        reference:'Ka Band Cloud Radar MIRA, METEK GmbH www.metek.de',$
        Copywright:'Any use of these data is subject to a copyright agreement with the copyright owner',$
        Copywright_Owner:'Unknown'$
    }

    config2stru,lopa,'header2ncdf',[syspar.iam+'.config']

    ; try to get some information like latitude out of the decr field 
    ; which is copied from /icfg/header.ini to the pds files by the radarserver
    tags = tag_names(lopa) 
    desc=hdr.descr
    kpos = strpos(desc,'#')
    if kpos ge 0 then desc=strmid(desc,0,kpos)
    descriptions=strsplit(desc,':',/extract)
    ndes=n_elements(descriptions)
    for ides=0,ndes-1 do begin
        rx=stregex(descriptions[ides],'[       ]*([A-Za-z][A-Za-z_0-9]*)[      ]*=(.*)$',/extr,/sub)
        if n_elements(rx) eq 3 then begin
            key = rx[1]
            val = rx[2]
            tagnumber_of_key = (where(strupcase(key) eq tags))[0]
            if tagnumber_of_key ge 0 then begin
                lopa.(tagnumber_of_key) = val
            endif
        endif
    endfor

    if fix(strmid(!version.RELEASE,0,1)) ge 8 then ncdf4_available=1 else ncdf4_available=0
    if ncdf4_available and do_ncdf4zip then begin
        Id = NCDF_CREATE(fn,/CLOBBER,/NETCDF4_FORMAT)
        gzipq=syspar.gzipq
    endif else begin
        Id = NCDF_CREATE(fn,/CLOBBER)
        gzipq=0
    endelse

    ; Create some global attributes
    NCDF_ATTPUT, Id,  'convention',      lopa.convention, /global, /char
    NCDF_ATTPUT, Id,  'location',        lopa.location, /global, /char                  ; string(hdr.PLACE) from /icfg/header.ini
    NCDF_ATTPUT, Id,  'Altitude',        lopa.Altitude, /global, /char
    NCDF_ATTPUT, Id,  'Latitude',        lopa.Latitude, /global, /char
    NCDF_ATTPUT, Id,  'Longitude',       lopa.Longitude, /global, /char
    NCDF_ATTPUT, Id,  'system',          lopa.system, /global, /char
    NCDF_ATTPUT, Id,  'title',           lopa.title, /global, /char
    NCDF_ATTPUT, Id,  'institution',     lopa.institution, /global, /char
    NCDF_ATTPUT, Id,  'source',          lopa.source, /global, /char     ; put the pds file name created by the radarserver here
    NCDF_ATTPUT, Id,  'reference',       lopa.reference, /global, /char
    NCDF_ATTPUT, Id,  'ppar',            stru2str(ppar), /global, /char
    NCDF_ATTPUT, Id,  'hrd',             stru2str(hdr), /global, /char
    NCDF_ATTPUT, Id,  'Copywright',      lopa.Copywright, /global, /char
    NCDF_ATTPUT, Id,  'Copywright_Owner',lopa.Copywright_Owner, /global, /char
    if syspar.TxRecbandwidthMatchingLoss lt 1.5 then notestxt1='In the reflectiveties 1.7 to 2 dB Receiver Bwd Loss is neglected.' else notestxt1='It is accounted for the Receiver Bwd Loss in Radarconst*, and Z*.'
    notes = notestxt1
    notestxt2='If azi < 370 then time represents the end of the averaging periode of the next data block, If azi > 370 time represents the end of the averaging interval of the current data block.'
    if syspar.no_stamp_shift eq 0 then notes = notes + string(10B) + notestxt2 
    NCDF_ATTPUT, Id,  'notes',notes, /global, /char


    ;  Create dimensions
    if include_special_gates then zrg = hdr.MOM_ZRG+2 else  zrg = hdr.MOM_ZRG
    nfft = hdr.nfft
    zchan = hdr.pol+1L
    ziq = 2L
    Dim1   = NCDF_DIMDEF(Id,'time', 0)   ; 0 gives the variable the status unlimited
    Dim2   = NCDF_DIMDEF(Id,'range',zrg)
    if max(extras eq 'SPC') or max(extras eq 'SPCcocx') then begin
        Dim3 = NCDF_DIMDEF(Id,'doppler',nfft)
    endif
    if max(extras eq 'LastClutterBin') then Dim4   = NCDF_DIMDEF(Id,'channel',zchan)
    if max(extras eq 'IQ') then Dim5   = NCDF_DIMDEF(Id,'re_im',ziq)
    
    ;  Create variables for the most important informations of PPAR
    VarId   = NCDF_VARDEF(Id,'nfft', /Long)
    NCDF_ATTPUT, Id, 'nfft',     'long_name', 'Number of FFT Points' ; hdr.NFFT
    NCDF_ATTPUT, Id, 'nfft',     'units', 'count'

    VarId   = NCDF_VARDEF(Id,'prf', /Long)
    NCDF_ATTPUT, Id, 'prf',     'long_name', 'Pulse Repetition Frequency' ; hdr.prf
    NCDF_ATTPUT, Id, 'prf',     'units', 'Hz'

    VarId   = NCDF_VARDEF(Id,'NyquistVelocity', /FLOAT)
    NCDF_ATTPUT, Id, 'NyquistVelocity', 'long_name', 'Speed of Light  * Pulse Repetition Frequency / (4 * Frequency)' ; hdr.VNY
    NCDF_ATTPUT, Id, 'NyquistVelocity', 'units', 'm s-1' 

    VarId   = NCDF_VARDEF(Id,'nave', /Long)
    NCDF_ATTPUT, Id, 'nave',     'long_name', 'Number of Spectral Avreages' ; hdr.avc ; ave=nave*nfft/prf
    NCDF_ATTPUT, Id, 'nave',     'units', 'count' 

    VarId   = NCDF_VARDEF(Id,'zrg', /Long)
    NCDF_ATTPUT, Id, 'zrg',      'long_name', 'Number of Range Gates' ; hdr.mom_zrg
    NCDF_ATTPUT, Id, 'zrg',      'units', 'count'

    VarId   = NCDF_VARDEF(Id,'rg0', /Long)
    NCDF_ATTPUT, Id, 'rg0',      'long_name', 'Number of Lowest Range Gates' ; ppar.ihp   
    NCDF_ATTPUT, Id, 'rg0',      'units', 'count' ; 

    VarId   = NCDF_VARDEF(Id,'drg', /FLOAT)
    NCDF_ATTPUT, Id, 'drg',      'long_name', 'Range Resolution' ; hdr.DELTA_H
    NCDF_ATTPUT, Id, 'drg',      'unit', 'm'

    VarId   = NCDF_VARDEF(Id,'lambda', /Float)
    NCDF_ATTPUT, Id, 'lambda',   'long_name', 'Wave Length' ; hdr.la
    NCDF_ATTPUT, Id, 'lambda',   'units', 'm'

    VarId   = NCDF_VARDEF(Id,'range', Dim2, /FLOAT)
    NCDF_ATTPUT, Id, 'range',    'long_name', 'Range from Antenna to the Centre of each Range Gate' ; range = ( rg0 + indgen(zrg)) * drg
    NCDF_ATTPUT, Id, 'range',    'units', 'm'
    NCDF_ATTPUT, Id, 'range',    'axis', 'Y'

    if max(extras eq 'SPC') or max(extras eq 'SPCcocx') then begin
        VarId   = NCDF_VARDEF(Id,'doppler', Dim3, /FLOAT)
        NCDF_ATTPUT, Id, 'doppler',    'long_name', 'Doppler Velocity' ; doppler = cmod(dx * findgen(nfft), hdr.VUAR,0)
        NCDF_ATTPUT, Id, 'doppler',    'units', 'm/s'
        NCDF_ATTPUT, Id, 'doppler',    'axis', 'V'
    endif

; Most important information from the SRVI structure
    VarId   = myNCDF_VARDEF(Id,'time', Dim1, /LONG,gzipq)
    NCDF_ATTPUT, Id, 'time',     'long_name', 'Seconds since 01.01.1970 00:00 UTC' ; srvinfo.TIME_T
    NCDF_ATTPUT, Id, 'time',     'units', 'Seconds' 
    NCDF_ATTPUT, Id, 'time',     'axis', 'T'

    VarId   = myNCDF_VARDEF(Id,'microsec', Dim1, /LONG,gzipq)
    NCDF_ATTPUT, Id, 'microsec',     'long_name', 'Micro Seconds' ; srvinfo.microsec
    NCDF_ATTPUT, Id, 'microsec',     'units', 'us'

    VarId   = myNCDF_VARDEF(Id,'tpow', Dim1, /FLOAT,gzipq)
    NCDF_ATTPUT, Id, 'tpow',     'long_name','Average Transmit Power' ;  srvinfo.tpow (* 0.01 in case of some older radars)
    NCDF_ATTPUT, Id, 'tpow',     'units','W' 

    VarId   = myNCDF_VARDEF(Id,'npw1', Dim1, /FLOAT,gzipq)
    NCDF_ATTPUT, Id, 'npw1',     'long_name','Noise Power Co-Channel' ;  srvinfo.npw1
    NCDF_ATTPUT, Id, 'npw1',     'units','Sum over the Spectrum in DSP units'

    VarId   = myNCDF_VARDEF(Id,'npw2', Dim1, /FLOAT,gzipq)
    NCDF_ATTPUT, Id, 'npw2',     'long_name','Noise Power Cross-Channel' ;  srvinfo.npw2
    NCDF_ATTPUT, Id, 'npw2',     'units','Sum over the Spectrum in DSP units'

    VarId   = myNCDF_VARDEF(Id,'cpw1', Dim1, /FLOAT,gzipq)
    NCDF_ATTPUT, Id, 'cpw1',     'long_name','SNR of Calibration Signal Co-Channel' ;  srvinfo.cpw1
    NCDF_ATTPUT, Id, 'cpw1',     'units','Ratio'

    VarId   = myNCDF_VARDEF(Id,'cpw2', Dim1, /FLOAT,gzipq)
    NCDF_ATTPUT, Id, 'cpw2',     'long_name','SNR of Calibration Signal Cross-Channel' ;  srvinfo.cpw2
    NCDF_ATTPUT, Id, 'cpw2',     'units','Ratio'

    VarId   = myNCDF_VARDEF(Id,'grst', Dim1, /LONG,gzipq)
    NCDF_ATTPUT, Id, 'grst',     'long_name','General Radar State' ;  srvinfo.grst
    NCDF_ATTPUT, Id, 'grst',     'units','Enumerate'

    VarId   = myNCDF_VARDEF(Id,'azi', Dim1, /FLOAT,gzipq)
    NCDF_ATTPUT, Id, 'azi',     'long_name','Azimut Angle, if > 370 azi-720 represents the middle of the averaging interval, if azi < 370 then azi represents the position of the end of the next data block.'
    NCDF_ATTPUT, Id, 'azi',     'units','deg'

    VarId   = myNCDF_VARDEF(Id,'elv', Dim1, /FLOAT,gzipq)
    NCDF_ATTPUT, Id, 'elv',     'long_name','Elevation Angle, if > 370 elv-720 represents the middle of the averaging interval, if elv < 370 then elv represents the position of the end of the next data block.'
    NCDF_ATTPUT, Id, 'elv',     'units','deg'

    VarId   = myNCDF_VARDEF(Id,'aziv', Dim1, /FLOAT,gzipq)
    NCDF_ATTPUT, Id, 'aziv',     'long_name','Azimut Angle Velocity'
    NCDF_ATTPUT, Id, 'aziv',     'units','deg/s'

    VarId   = myNCDF_VARDEF(Id,'northangle', Dim1, /FLOAT,gzipq)
    NCDF_ATTPUT, Id, 'northangle',     'long_name','North Angle'
    NCDF_ATTPUT, Id, 'northangle',     'units','deg'

    VarId   = myNCDF_VARDEF(Id,'elvv', Dim1, /FLOAT,gzipq)
    NCDF_ATTPUT, Id, 'elvv',     'long_name','Elevation Angle Velocity'
    NCDF_ATTPUT, Id, 'elvv',     'units','deg/s'

    VarId   = myNCDF_VARDEF(Id,'LO_Frequency', Dim1, /FLOAT,gzipq)
    NCDF_ATTPUT, Id, 'LO_Frequency',     'long_name','Transmit Frequency'
    NCDF_ATTPUT, Id, 'LO_Frequency',     'units','MHz'

    VarId   = myNCDF_VARDEF(Id,'DetuneFine', Dim1, /FLOAT,gzipq)
    NCDF_ATTPUT, Id, 'DetuneFine',     'long_name','Detuning of the Local Oscillator Detected from Txn Footprint'
    NCDF_ATTPUT, Id, 'DetuneFine',     'units','kHz'

    ;  Create vector variables to hold the Output Profiles:
    ; remove ptype 'xxx' which is uses because there is no array with 0 elements in idl
    ix_ValidPtype=where(ptypes ne 'xxx',nptypes)
    if nptypes gt 0 then ptypes=ptypes[ix_ValidPtype]
    for ipty=0,nptypes-1 do begin
        exn=ptypes[ipty]
        VarId   = myNCDF_VARDEF(Id,'SNR'+exn, [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'SNR'+exn,      'long_name', 'Reflectivity SNR'+exn
        NCDF_ATTPUT, Id, 'SNR'+exn,      'units', ' '
        NCDF_ATTPUT, Id, 'SNR'+exn,      'yrange', [db(ppar.of0/(10.*hdr.nfft*sqrt(ppar.avc))),+70.0]
        NCDF_ATTPUT, Id, 'SNR'+exn,      'db', 1
        
        ;if exn eq 'rain' then begin
        ;    VarId   = myNCDF_VARDEF(Id,'LWC'+exn, [Dim2, Dim1], /FLOAT,gzipq)
        ;    NCDF_ATTPUT, Id, 'LWC'+exn,      'long_name', 'LWC '+exn
        ;    NCDF_ATTPUT, Id, 'LWC'+exn,      'units', ' '
        ;    NCDF_ATTPUT, Id, 'LWC'+exn,      'yrange', [95.0,205.0]
        ;    NCDF_ATTPUT, Id, 'LWC'+exn,      'db', 1
        ;    
        ;    VarId   = myNCDF_VARDEF(Id,'RR'+exn, [Dim2, Dim1], /FLOAT,gzipq)
        ;    NCDF_ATTPUT, Id, 'RR'+exn,      'long_name', 'RR '+exn
        ;    NCDF_ATTPUT, Id, 'RR'+exn,      'units', ' '
        ;    NCDF_ATTPUT, Id, 'RR'+exn,      'yrange', [95.0,205.0]
        ;    NCDF_ATTPUT, Id, 'RR'+exn,      'db', 1
        ;endif
        
        if exn eq '1' or exn eq '2' or exn eq '3' then begin
            VarId   = myNCDF_VARDEF(Id,'NN'+exn, [Dim2, Dim1], /FLOAT,gzipq)
            NCDF_ATTPUT, Id, 'NN'+exn,      'long_name', 'Number of Neighbours '+exn
            NCDF_ATTPUT, Id, 'NN'+exn,      'units', ' '
            NCDF_ATTPUT, Id, 'NN'+exn,      'yrange', [0,25]
            NCDF_ATTPUT, Id, 'NN'+exn,      'db', 0
            
            VarId   = myNCDF_VARDEF(Id,'NF'+exn, [Dim2, Dim1], /FLOAT,gzipq)
            NCDF_ATTPUT, Id, 'NF'+exn,      'long_name', 'Number of Friends '+exn
            NCDF_ATTPUT, Id, 'NF'+exn,      'units', ' '
            NCDF_ATTPUT, Id, 'NF'+exn,      'yrange', [0,25]
            NCDF_ATTPUT, Id, 'NF'+exn,      'db', 0
        endif
        
        ;if exn eq 'g' then begin
        ;    VarId   = myNCDF_VARDEF(Id,'SNM'+exn, [Dim2, Dim1], /FLOAT,gzipq)
        ;    NCDF_ATTPUT, Id, 'SNM'+exn,      'long_name', 'Co-Channel Mie-Corrected SNR'
        ;    NCDF_ATTPUT, Id, 'SNM'+exn,      'units', ' '
        ;    NCDF_ATTPUT, Id, 'SNM'+exn,      'yrange', [db(ppar.of0/(10.*hdr.nfft*sqrt(ppar.avc))),+80.0]
        ;    NCDF_ATTPUT, Id, 'SNM'+exn,      'db', 1
        ;endif
        
        VarId   = myNCDF_VARDEF(Id,'VEL'+exn, [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'VEL'+exn,      'long_name', 'Doppler Velocity VEL'+exn
        NCDF_ATTPUT, Id, 'VEL'+exn,      'units', 'm/s'
        NCDF_ATTPUT, Id, 'VEL'+exn,      'yrange', [-hdr.vny,hdr.vny]
        NCDF_ATTPUT, Id, 'VEL'+exn,      'db', 0
        
        VarId   = myNCDF_VARDEF(Id,'RMS'+exn, [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'RMS'+exn,      'long_name', 'Peak Width RMS'+exn
        NCDF_ATTPUT, Id, 'RMS'+exn,      'units', 'm/s'
        NCDF_ATTPUT, Id, 'RMS'+exn,      'yrange', [0,3.]
        NCDF_ATTPUT, Id, 'RMS'+exn,      'db', 0
        
        VarId   = myNCDF_VARDEF(Id,'LDR'+exn, [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'LDR'+exn,      'long_name', 'Linear De-Polarization Ratio LDR'+exn
        NCDF_ATTPUT, Id, 'LDR'+exn,      'units', ' '
        NCDF_ATTPUT, Id, 'LDR'+exn,      'yrange', [-35.,5]
        NCDF_ATTPUT, Id, 'LDR'+exn,      'db', 1

        VarId   = myNCDF_VARDEF(Id,'NPK'+exn, [Dim2, Dim1], /BYTE,gzipq)
        NCDF_ATTPUT, Id, 'NPK'+exn,      'long_name', 'Number of Peaks'+exn
        NCDF_ATTPUT, Id, 'NPK'+exn,      'units', ' '
        NCDF_ATTPUT, Id, 'NPK'+exn,      'yrange', [-1,10]
        NCDF_ATTPUT, Id, 'NPK'+exn,      'db', 0
    endfor
   
    if max(extras eq 'VarPowCo') then begin
        VarId   = myNCDF_VARDEF(Id,'VarPowCo', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'VarPowCo',      'long_name', 'Variance of Power of Single Spectra Co-Channel'
        NCDF_ATTPUT, Id, 'VarPowCo',      'units', 'DSP'
        NCDF_ATTPUT, Id, 'VarPowCo',      'yrange', [-10.,60.]
        NCDF_ATTPUT, Id, 'VarPowCo',      'db', 1
    endif
        
    if max(extras eq 'VarPowCx') then begin
        VarId   = myNCDF_VARDEF(Id,'VarPowCx', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'VarPowCx',      'long_name', 'Variance of Power of Single Spectra Cx-Channel'
        NCDF_ATTPUT, Id, 'VarPowCx',      'units', 'DSP'
        NCDF_ATTPUT, Id, 'VarPowCx',      'yrange', [-10.,60.]
        NCDF_ATTPUT, Id, 'VarPowCx',      'db', 1
    endif
        
    if max(extras eq 'VarVelCo') then begin
        VarId   = myNCDF_VARDEF(Id,'VarVelCo', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'VarVelCo',      'long_name', 'Variance of Velocity of Single Spectra Co-Channel'
        NCDF_ATTPUT, Id, 'VarVelCo',      'units', 'DSP'
        NCDF_ATTPUT, Id, 'VarVelCo',      'yrange', [-10.,60.]
        NCDF_ATTPUT, Id, 'VarVelCo',      'db', 1
    endif
        
    if max(extras eq 'VarVelCx') then begin
        VarId   = myNCDF_VARDEF(Id,'VarVelCx', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'VarVelCx',      'long_name', 'Variance of Velocity of Single Spectra Cx-Channel'
        NCDF_ATTPUT, Id, 'VarVelCx',      'units', 'DSP'
        NCDF_ATTPUT, Id, 'VarVelCx',      'yrange', [-10.,60.]
        NCDF_ATTPUT, Id, 'VarVelCx',      'db', 1
    endif
    if max(extras eq 'VarRmsCo') then begin
        VarId   = myNCDF_VARDEF(Id,'VarRmsCo', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'VarRmsCo',      'long_name', 'Variance of Peak Width of Single Spectra Co-Channel'
        NCDF_ATTPUT, Id, 'VarRmsCo',      'units', 'DSP'
        NCDF_ATTPUT, Id, 'VarRmsCo',      'yrange', [-50.,50.]
        NCDF_ATTPUT, Id, 'VarRmsCo',      'db', 1
    endif
    
    if max(extras eq 'VarRmsCx') then begin
        VarId   = myNCDF_VARDEF(Id,'VarRmsCx', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'VarRmsCx',      'long_name', 'Variance of Peak Width of Single Spectra Cx-Channel'
        NCDF_ATTPUT, Id, 'VarRmsCx',      'units', 'DSP'
        NCDF_ATTPUT, Id, 'VarRmsCx',      'yrange', [-50.,50.]
        NCDF_ATTPUT, Id, 'VarRmsCx',      'db', 1
    endif


    if max(extras eq 'DifPowCo') then begin
        VarId   = myNCDF_VARDEF(Id,'DifPowCo', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'DifPowCo',      'long_name', 'Difference of mean and median of Power of Single Spectra Co-Channel'
        NCDF_ATTPUT, Id, 'DifPowCo',      'units', 'DSP'
        NCDF_ATTPUT, Id, 'DifPowCo',      'yrange', [-50.,50.]
        NCDF_ATTPUT, Id, 'DifPowCo',      'db', 1
    endif

    if max(extras eq 'DifPowCx') then begin
        VarId   = myNCDF_VARDEF(Id,'DifPowCx', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'DifPowCx',      'long_name', 'Difference of mean and median of Power of Single Spectra Cx-Channel'
        NCDF_ATTPUT, Id, 'DifPowCx',      'units', 'DSP'
        NCDF_ATTPUT, Id, 'DifPowCx',      'yrange', [-50.,50.]
        NCDF_ATTPUT, Id, 'DifPowCx',      'db', 1
    endif


    if max(extras eq 'DifVelCo') then begin
        VarId   = myNCDF_VARDEF(Id,'DifVelCo', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'DifVelCo',      'long_name', 'Difference of mean and median of Velocity of Single Spectra Co-Channel'
        NCDF_ATTPUT, Id, 'DifVelCo',      'units', 'DSP'
        NCDF_ATTPUT, Id, 'DifVelCo',      'yrange', [-50.,50.]
        NCDF_ATTPUT, Id, 'DifVelCo',      'db', 1
    endif

    if max(extras eq 'DifVelCx') then begin
        VarId   = myNCDF_VARDEF(Id,'DifVelCx', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'DifVelCx',      'long_name', 'Difference of mean and median of Velocity of Single Spectra Cx-Channel'
        NCDF_ATTPUT, Id, 'DifVelCx',      'units', 'DSP'
        NCDF_ATTPUT, Id, 'DifVelCx',      'yrange', [-50.,50.]
        NCDF_ATTPUT, Id, 'DifVelCx',      'db', 1
    endif
    if max(extras eq 'DifRmsCo') then begin
        VarId   = myNCDF_VARDEF(Id,'DifRmsCo', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'DifRmsCo',      'long_name', 'Difference of mean and median of Peak Width of Single Spectra Co-Channel'
        NCDF_ATTPUT, Id, 'DifRmsCo',      'units', 'DSP'
        NCDF_ATTPUT, Id, 'DifRmsCo',      'yrange', [-50.,50.]
        NCDF_ATTPUT, Id, 'DifRmsCo',      'db', 1
    endif

    if max(extras eq 'DifRmsCx') then begin
        VarId   = myNCDF_VARDEF(Id,'DifRmsCx', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'DifRmsCx',      'long_name', 'Difference of mean and median of Peak Width of Single Spectra Cx-Channel'
        NCDF_ATTPUT, Id, 'DifRmsCx',      'units', 'DSP'
        NCDF_ATTPUT, Id, 'DifRmsCx',      'yrange', [-50.,50.]
        NCDF_ATTPUT, Id, 'DifRmsCx',      'db', 1
    endif

    if max(extras eq 'ShapeCo') then begin
        VarId   = myNCDF_VARDEF(Id,'ShapeCo', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'ShapeCo',      'long_name', 'Standard deviation / mean for single spectra Co-Channel'
        NCDF_ATTPUT, Id, 'ShapeCo',      'units', 'DSP'
        NCDF_ATTPUT, Id, 'ShapeCo',      'yrange', [-50.,50.]
        NCDF_ATTPUT, Id, 'ShapeCo',      'db', 1
    endif
    
    if max(extras eq 'ShapeCx') then begin
        VarId   = myNCDF_VARDEF(Id,'ShapeCx', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'ShapeCx',      'long_name', 'Standard deviation / mean for single spectra Cx-Channel'
        NCDF_ATTPUT, Id, 'ShapeCx',      'units', 'DSP'
        NCDF_ATTPUT, Id, 'ShapeCx',      'yrange', [-50.,50.]
        NCDF_ATTPUT, Id, 'ShapeCx',      'db', 1
    endif
    
    if max(extras eq 'ErrorCo') then begin
        VarId   = myNCDF_VARDEF(Id,'ErrorCo', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'ErrorCo',      'long_name', 'Statistical error Co-Channel'
        NCDF_ATTPUT, Id, 'ErrorCo',      'units', 'DSP'
        NCDF_ATTPUT, Id, 'ErrorCo',      'yrange', [-50.,50.]
        NCDF_ATTPUT, Id, 'ErrorCo',      'db', 1
    endif

    if max(extras eq 'ErrorCx') then begin
        VarId   = myNCDF_VARDEF(Id,'ErrorCx', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'ErrorCx',      'long_name', 'Statistical error Cx-Channel'
        NCDF_ATTPUT, Id, 'ErrorCx',      'units', 'DSP'
        NCDF_ATTPUT, Id, 'ErrorCx',      'yrange', [-50.,50.]
        NCDF_ATTPUT, Id, 'ErrorCx',      'db', 1
    endif

    if max(extras eq 'SNRco') then begin 
        VarId   = myNCDF_VARDEF(Id,'SNRco', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'SNRco',      'long_name', 'Reflectivity SNR Co-Channel'
        NCDF_ATTPUT, Id, 'SNRco',      'units', ' '
        NCDF_ATTPUT, Id, 'SNRco',      'yrange', [-35.,70.]
        NCDF_ATTPUT, Id, 'SNRco',      'db', 1
    endif

    if max(extras eq 'SNRcx') then begin
        VarId   = myNCDF_VARDEF(Id,'SNRcx', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'SNRcx',      'long_name', 'Reflectivity SNR Cx-Channel'
        NCDF_ATTPUT, Id, 'SNRcx',      'units', ' '
        NCDF_ATTPUT, Id, 'SNRcx',      'yrange', [-35.,70.]
        NCDF_ATTPUT, Id, 'SNRcx',      'db', 1
    endif

    if max(extras eq 'SNRcxpdd') then begin
        VarId   = myNCDF_VARDEF(Id,'SNRcxpdd', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'SNRcxpdd',      'long_name', 'Reflectivity SNR Cx-Channel from radar-server'
        NCDF_ATTPUT, Id, 'SNRcxpdd',      'units', ' '
        NCDF_ATTPUT, Id, 'SNRcxpdd',      'yrange', [-35.,70.]
        NCDF_ATTPUT, Id, 'SNRcxpdd',      'db', 1
    endif

    if max(extras eq 'RHO') then begin
        VarId   = myNCDF_VARDEF(Id,'RHO', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'RHO',      'long_name', 'Co-Cross Channel Correlation'
        NCDF_ATTPUT, Id, 'RHO',      'units', ' '
        NCDF_ATTPUT, Id, 'RHO',      'yrange', [0.0,1.0]
        NCDF_ATTPUT, Id, 'RHO',      'db', 0
    endif

    if max(extras eq 'RHOpdd') then begin
        VarId   = myNCDF_VARDEF(Id,'RHOpdd', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'RHOpdd',      'long_name', 'Co-Cross Channel Correlation from xcrl-server'
        NCDF_ATTPUT, Id, 'RHOpdd',      'units', ' '
        NCDF_ATTPUT, Id, 'RHOpdd',      'yrange', [0.0,1.0]
        NCDF_ATTPUT, Id, 'RHOpdd',      'db', 0
    endif

    if max(extras eq 'DPS') then begin
        VarId   = myNCDF_VARDEF(Id,'DPS', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'DPS',      'long_name', 'Differential Phase'
        NCDF_ATTPUT, Id, 'DPS',      'units', 'deg'
        NCDF_ATTPUT, Id, 'DPS',      'yrange', [-180.,180.]
        NCDF_ATTPUT, Id, 'DPS',      'db', 0
    endif

    if max(extras eq 'DPSpdd') then begin
        VarId   = myNCDF_VARDEF(Id,'DPSpdd', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'DPSpdd',      'long_name', 'Differential Phase from xcrl-server'
        NCDF_ATTPUT, Id, 'DPSpdd',      'units', 'deg'
        NCDF_ATTPUT, Id, 'DPSpdd',      'yrange', [-180.,180.]
        NCDF_ATTPUT, Id, 'DPSpdd',      'db', 0
    endif

    if max(extras eq 'RHOwav') then begin
        VarId   = myNCDF_VARDEF(Id,'RHOwav', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'RHOwav',      'long_name', 'Cross Channel Correlation Peak weighted'
        NCDF_ATTPUT, Id, 'RHOwav',      'units', ' '
        NCDF_ATTPUT, Id, 'RHOwav',      'yrange', [0.0,1.0]
        NCDF_ATTPUT, Id, 'RHOwav',      'db', 0
    endif

    if max(extras eq 'DPSwav') then begin ; obsolete
        VarId   = myNCDF_VARDEF(Id,'DPSwav', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'DPSwav',      'long_name', 'Differential Phase peak weighted'
        NCDF_ATTPUT, Id, 'DPSwav',      'units', 'deg'
        NCDF_ATTPUT, Id, 'DPSwav',      'yrange', [-180.,180.]
        NCDF_ATTPUT, Id, 'DPSwav',      'db', 0
    endif

    if max(extras eq 'LDRnormal') then begin
        VarId   = myNCDF_VARDEF(Id,'LDRnormal', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'LDRnormal',      'long_name', 'LDR calculated by weigting both channels withe the SNR of the more powerful channel'
        NCDF_ATTPUT, Id, 'LDRnormal',      'units', ' '
        NCDF_ATTPUT, Id, 'LDRnormal',      'yrange', [-35,5]
        NCDF_ATTPUT, Id, 'LDRnormal',      'db', 1
    endif

    if max(extras eq 'HSDco') then begin
        ; noise profiles estimated by HSdiv
        VarId   = myNCDF_VARDEF(Id,'HSDco', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'HSDco',      'long_name', 'Co-Channel HSdiv Noise Level'
        NCDF_ATTPUT, Id, 'HSDco',      'units', 'DSP'
        NCDF_ATTPUT, Id, 'HSDco',      'yrange', [-20.,40.]
        NCDF_ATTPUT, Id, 'HSDco',      'db', 1
    endif
        
    if max(extras eq 'HSDcx') then begin
        VarId   = myNCDF_VARDEF(Id,'HSDcx', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'HSDcx',      'long_name', 'Cross-Channel HSdiv Noise Level'
        NCDF_ATTPUT, Id, 'HSDcx',      'units', 'DSP'
        NCDF_ATTPUT, Id, 'HSDcx',      'yrange', [-20.,40.]
        NCDF_ATTPUT, Id, 'HSDcx',      'db', 1
    endif
    
    if max(extras eq 'Ze') then begin
        VarId   = myNCDF_VARDEF(Id,'Ze', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'Ze',        'long_name', 'Equivalent Radar Reflectivity Factor Ze of Hydrometeors' + (strlen(syspar.Z_UnitDescription) gt 0? ' ' : '') + syspar.Z_UnitDescription
        NCDF_ATTPUT, Id, 'Ze',        'units', syspar.Z_UnitString
        NCDF_ATTPUT, Id, 'Ze',        'yrange', [-60,30.]
        NCDF_ATTPUT, Id, 'Ze',        'db', 1
    endif

    if max(extras eq 'Zg') then begin
        VarId   = myNCDF_VARDEF(Id,'Zg', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'Zg',        'long_name', 'Equivalent Radar Reflectivity Factor Ze of all Targets' + (strlen(syspar.Z_UnitDescription) gt 0 ? ' ' : '') + syspar.Z_UnitDescription
        NCDF_ATTPUT, Id, 'Zg',        'units', syspar.Z_UnitString
        NCDF_ATTPUT, Id, 'Zg',        'yrange', [-60,30.]
        NCDF_ATTPUT, Id, 'Zg',        'db', 1
    endif
    
    if max(extras eq 'Zgpdd') then begin
        VarId   = myNCDF_VARDEF(Id,'Zgpdd', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'Zgpdd',        'long_name', 'Equivalent Radar Reflectivity Factor Ze-V from radar-server' + (strlen(syspar.Z_UnitDescription) gt 0 ? ' ' : '') + syspar.Z_UnitDescription
        NCDF_ATTPUT, Id, 'Zgpdd',        'units', syspar.Z_UnitString
        NCDF_ATTPUT, Id, 'Zgpdd',        'yrange', [-60,30.]
        NCDF_ATTPUT, Id, 'Zgpdd',        'db', 1
    endif
    
    if max(extras eq 'Zepdd') then begin
        VarId   = myNCDF_VARDEF(Id,'Zepdd', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'Zepdd',        'long_name', 'Equivalent Radar Reflectivity Factor Ze-H from radar-server' + (strlen(syspar.Z_UnitDescription) gt 0 ? ' ':'') + syspar.Z_UnitDescription
        NCDF_ATTPUT, Id, 'Zepdd',        'units', syspar.Z_UnitString
        NCDF_ATTPUT, Id, 'Zepdd',        'yrange', [-60,30.]
        NCDF_ATTPUT, Id, 'Zepdd',        'db', 1
    endif
    
    if max(extras eq 'Zh2l') then begin
        VarId   = myNCDF_VARDEF(Id,'Zh2l', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'Zh2l',        'long_name', 'Equivalent Radar Reflectivity Factor Ze the compsed co channel' + (strlen(syspar.Z_UnitDescription) gt 0 ? ' ':'') + syspar.Z_UnitDescription
        NCDF_ATTPUT, Id, 'Zh2l',        'units', syspar.Z_UnitString
        NCDF_ATTPUT, Id, 'Zh2l',        'yrange', [-60,30.]
        NCDF_ATTPUT, Id, 'Zh2l',        'db', 1
    endif
    
    if max(extras eq 'Z') then begin
        VarId   = myNCDF_VARDEF(Id,'Z', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'Z',        'long_name', 'Radar Reflectivity Factor Z of Hydrometeors (Mie-corrected)' + (strlen(syspar.Z_UnitDescription) gt 0 ? ' ':'') + syspar.Z_UnitDescription
        NCDF_ATTPUT, Id, 'Z',        'units', syspar.Z_UnitString
        NCDF_ATTPUT, Id, 'Z',        'yrange', [-60,30.]
        NCDF_ATTPUT, Id, 'Z',        'db', 1
    endif
     
    if max(extras eq 'RR') then begin
        VarId   = myNCDF_VARDEF(Id,'RR', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'RR',        'long_name', 'Rain Rate'
        NCDF_ATTPUT, Id, 'RR',        'units', 'mm/h ?'
        NCDF_ATTPUT, Id, 'RR',        'yrange', [30,140.]
        NCDF_ATTPUT, Id, 'RR',        'db', 1
    endif

    if max(extras eq 'LWC') then begin
        VarId   = myNCDF_VARDEF(Id,'LWC', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'LWC',        'long_name', 'Liquid Water Content'
        NCDF_ATTPUT, Id, 'LWC',        'units', 'g/m^3 ?'
        NCDF_ATTPUT, Id, 'LWC',        'yrange', [0,180.]
        NCDF_ATTPUT, Id, 'LWC',        'db', 1
    endif

    if max(extras eq 'TEMP') then begin
        VarId   = myNCDF_VARDEF(Id,'TEMP', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'TEMP',        'long_name', 'Temperature'
        NCDF_ATTPUT, Id, 'TEMP',        'units', 'C'
        NCDF_ATTPUT, Id, 'TEMP',        'yrange', [-60,40.]
        NCDF_ATTPUT, Id, 'TEMP',        'db', 0
    endif

    if max(extras eq 'MeltHei') then begin
        VarId   = myNCDF_VARDEF(Id,'MeltHei', [Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'MeltHei',        'long_name', 'Melting Layer Height'
        NCDF_ATTPUT, Id, 'MeltHei',        'units', 'm'
        NCDF_ATTPUT, Id, 'MeltHei',        'yrange', [0,14000]
        NCDF_ATTPUT, Id, 'MeltHei',        'db', 0
    endif

    if max(extras eq 'MeltHeiDet') then begin
        VarId   = myNCDF_VARDEF(Id,'MeltHeiDet', [Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'MeltHeiDet',        'long_name', 'Melting Layer Height detcted from LDR'
        NCDF_ATTPUT, Id, 'MeltHeiDet',        'units', 'm'
        NCDF_ATTPUT, Id, 'MeltHeiDet',        'yrange', [0,14000]
        NCDF_ATTPUT, Id, 'MeltHeiDet',        'db', 0
    endif

    if max(extras eq 'MeltHeiDB') then begin
        VarId   = myNCDF_VARDEF(Id,'MeltHeiDB', [Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'MeltHeiDB',        'long_name', 'Melting Layer Height from Data Base'
        NCDF_ATTPUT, Id, 'MeltHeiDB',        'units', 'm'
        NCDF_ATTPUT, Id, 'MeltHeiDB',        'yrange', [0,14000]
        NCDF_ATTPUT, Id, 'MeltHeiDB',        'db', 0
    endif

    if max(extras eq 'ISDRco') then begin
        VarId   = myNCDF_VARDEF(Id,'ISDRco', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'ISDRco',        'long_name', 'In Spectrum Dynamic Range Co-Channel'
        NCDF_ATTPUT, Id, 'ISDRco',        'units', ' '
        NCDF_ATTPUT, Id, 'ISDRco',        'yrange', [20,65.]
        NCDF_ATTPUT, Id, 'ISDRco',        'db', 1
    endif

    if max(extras eq 'ISDRcx') then begin
        VarId   = myNCDF_VARDEF(Id,'ISDRcx', [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'ISDRcx',        'long_name', 'In Spectrum Dynamic Range Cross-Channel'
        NCDF_ATTPUT, Id, 'ISDRcx',        'units', ' '
        NCDF_ATTPUT, Id, 'ISDRcx',        'yrange', [20,65.]
        NCDF_ATTPUT, Id, 'ISDRcx',        'db', 1
    endif

    if max(extras eq 'IQ') then begin
        VarId   = myNCDF_VARDEF(Id,'IQco', [Dim5,Dim2, Dim1], /short,gzipq)
        NCDF_ATTPUT, Id, 'IQco',        'long_name', 'IQ Signal Co-Channel'
        NCDF_ATTPUT, Id, 'IQco',        'units', 'ADC'
        NCDF_ATTPUT, Id, 'IQco',        'yrange', [-32767,32767]
        NCDF_ATTPUT, Id, 'IQco',        'db', 0
        VarId   = myNCDF_VARDEF(Id,'IQcx', [Dim5,Dim2, Dim1], /short,gzipq)
        NCDF_ATTPUT, Id, 'IQcx',        'long_name', 'IQ Signal Cross-Channel'
        NCDF_ATTPUT, Id, 'IQcx',        'units', 'ADC'
        NCDF_ATTPUT, Id, 'IQcx',        'yrange', [-32767,32767]
        NCDF_ATTPUT, Id, 'IQcx',        'db', 0
    endif

    if max(extras eq 'SPC') then begin
        ;VarId   = myNCDF_VARDEF(Id,'SPCco', [Dim3, Dim2, Dim1], /ULONG,gzipq)
        VarId   = myNCDF_VARDEF(Id,'SPCco', [Dim3, Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, 'SPCco',        'long_name', 'Doppler Spectrum Co-Channel'
        NCDF_ATTPUT, Id, 'SPCco',        'units', 'DSP Units'
        NCDF_ATTPUT, Id, 'SPCco',        'yrange', [0,70]
        NCDF_ATTPUT, Id, 'SPCco',        'db', 1
        if zchan ge 2 then begin
            ;VarId   = myNCDF_VARDEF(Id,'SPCcx', [Dim3, Dim2, Dim1], /ULONG,gzipq)
            VarId   = myNCDF_VARDEF(Id,'SPCcx', [Dim3, Dim2, Dim1], /FLOAT,gzipq)
            NCDF_ATTPUT, Id, 'SPCcx',        'long_name', 'Doppler Spectrum Cross-Channel'
            NCDF_ATTPUT, Id, 'SPCcx',        'units', 'DSP Units'
            NCDF_ATTPUT, Id, 'SPCcx',        'yrange', [0,70]
            NCDF_ATTPUT, Id, 'SPCcx',        'db', 1
        endif
    endif

    if max(extras eq 'SPCcocx') then begin
        varna = 'SPCcocxRe'
        ;VarId   = myNCDF_VARDEF(Id, varna, [Dim3, Dim2, Dim1], /LONG,gzipq)
        VarId   = myNCDF_VARDEF(Id, varna, [Dim3, Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, varna,        'long_name', 'Co-Cross Spectrum Real Part'
        NCDF_ATTPUT, Id, varna,        'units', 'DSP Units'
        NCDF_ATTPUT, Id, varna,        'yrange', 1e5*[-1,1]
        NCDF_ATTPUT, Id, varna,        'db', 0
        varna = 'SPCcocxIm'
        ;VarId   = myNCDF_VARDEF(Id,varna, [Dim3, Dim2, Dim1], /LONG,gzipq)
        VarId   = myNCDF_VARDEF(Id,varna, [Dim3, Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, varna,        'long_name', 'Co-Cross Spectrum Imaginary Part'
        NCDF_ATTPUT, Id, varna,        'units', 'DSP Units'
        NCDF_ATTPUT, Id, varna,        'yrange', 1e5*[-1,1]
        NCDF_ATTPUT, Id, varna,        'db', 0
    endif

    varna='MRMco'
    if max(extras eq varna) then begin
        VarId   = myNCDF_VARDEF(Id,varna, [Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, varna,        'long_name', 'Radiometer Signal Co'
        NCDF_ATTPUT, Id, varna,        'units', ' '
        NCDF_ATTPUT, Id, varna,        'yrange', [1,2]
        NCDF_ATTPUT, Id, varna,        'db', 0
    endif

    varna='MRMcx'
    if max(extras eq varna) then begin
        VarId   = myNCDF_VARDEF(Id,varna, [Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, varna,        'long_name', 'Radiometer Signal Cx'
        NCDF_ATTPUT, Id, varna,        'units', ' '
        NCDF_ATTPUT, Id, varna,        'yrange', [1,2]
        NCDF_ATTPUT, Id, varna,        'db', 0
    endif

    varna='RadarConst'
    if max(extras eq varna) then begin
        VarId   = myNCDF_VARDEF(Id,varna, [Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, varna,        'long_name', 'Radar Constant related to 5 km Height, and 200 ns pulses. Z = SNR * RadarConst * (range/5 km)^2 * SNRCorFaCo. It changes slightly in time due to the measured transmit power' + (strlen(syspar.Z_UnitDescription) gt 0 ? ' ':'') + syspar.Z_UnitDescription
        NCDF_ATTPUT, Id, varna,        'units', syspar.Z_UnitString
        NCDF_ATTPUT, Id, varna,        'yrange', [-35,-20.]
        NCDF_ATTPUT, Id, varna,        'db', 1
    endif

    varna='SNRCorFaCo'
    if max(extras eq varna) then begin
        VarId   = myNCDF_VARDEF(Id,varna, [Dim2,Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, varna,        'long_name', 'Factor to correct Co-channel SNR based on Receiver calibration measurement by Noisecom source'
        NCDF_ATTPUT, Id, varna,        'units', ' '
        NCDF_ATTPUT, Id, varna,        'yrange', [-10,10.]
        NCDF_ATTPUT, Id, varna,        'db', 1
    endif

    varna='SNRCorFaCx'
    if max(extras eq varna) then begin
        VarId   = myNCDF_VARDEF(Id,varna, [Dim2, Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, varna,        'long_name', 'Factor to correct Cx-channel SNR based on Receiver calibration measurement by Noisecom source. LDR = SNRcx/SNRco * SnrCorFaCx/SnrCorFaCo'
        NCDF_ATTPUT, Id, varna,        'units', '-'
        NCDF_ATTPUT, Id, varna,        'yrange', [-10,10.]
        NCDF_ATTPUT, Id, varna,        'db', 1
    endif

    varna='SKWg'
    if max(extras eq varna) then begin
        VarId   = myNCDF_VARDEF(Id,varna, [Dim2,Dim1], /Float,gzipq)
        NCDF_ATTPUT, Id, varna,        'long_name', 'Skewness'
        NCDF_ATTPUT, Id, varna,        'units', ' '
        NCDF_ATTPUT, Id, varna,        'yrange', [-6.0,+6.0]
        NCDF_ATTPUT, Id, varna,        'db', 0
    endif
    
; the following obsolete variables are kept as they may still be used by MAX:
    varna='RadarConstCo'   ; obsolete the difference betrween co and cross channel are in SNRCorFaCo and SNRCorFaCx. Therefore it makes no sense to save Co and Cx versions of this variable 
    if max(extras eq varna) then begin
        VarId   = myNCDF_VARDEF(Id,varna, [Dim2,Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, varna,        'long_name', 'RadarConstant5 * (Range/5 km)^2 * SNRCorFaCo (obsolete, replaced by RadarConst and SNRCorFaCo)' + (strlen(syspar.Z_UnitDescription) gt 0 ? ' ':'') + syspar.Z_UnitDescription
        NCDF_ATTPUT, Id, varna,        'units', syspar.Z_UnitString
        NCDF_ATTPUT, Id, varna,        'yrange', [-35,20.]
        NCDF_ATTPUT, Id, varna,        'db', 1
    endif

    varna='RadarConstCx'   ; obsolete the difference betrween co and cross channel are in SNRCorFaCo and SNRCorFaCx. Therefore it makes no sense to save Co and Cx versions of this variable
    if max(extras eq varna) then begin
        VarId   = myNCDF_VARDEF(Id,varna, [Dim2,Dim1], /FLOAT,gzipq)
        NCDF_ATTPUT, Id, varna,        'long_name', 'RadarConstant5 * (Range/5 km)^2 * SNRCorFaCx (obsolete, replaced by RadarConst and SNRCorFaCx)' + (strlen(syspar.Z_UnitDescription) gt 0 ? ' ':'') + syspar.Z_UnitDescription
        NCDF_ATTPUT, Id, varna,        'units', syspar.Z_UnitString
        NCDF_ATTPUT, Id, varna,        'yrange', [-35,-20.]
        NCDF_ATTPUT, Id, varna,        'db', 1
    endif

    varna='FirstClutterBin'
    if max(extras eq varna) then begin
        VarId   = myNCDF_VARDEF(Id,varna, [Dim4,Dim2,Dim1], /SHORT,gzipq)
        NCDF_ATTPUT, Id, varna,        'long_name', 'First spectral bin contaminated by ground clutter'
        NCDF_ATTPUT, Id, varna,        'units', '-'
        NCDF_ATTPUT, Id, varna,        'yrange', [-1,256]
        NCDF_ATTPUT, Id, varna,        'db', 0
    endif
    
    varna='LastClutterBin'
    if max(extras eq varna) then begin
        VarId   = myNCDF_VARDEF(Id,varna, [Dim4,Dim2,Dim1], /SHORT,gzipq)
        NCDF_ATTPUT, Id, varna,        'long_name', 'Last spectral bin contaminated by ground clutter'
        NCDF_ATTPUT, Id, varna,        'units', '-'
        NCDF_ATTPUT, Id, varna,        'yrange', [-1,256]
        NCDF_ATTPUT, Id, varna,        'db', 0
    endif
    
    ;  Leave definition mode and enter data write mode
    NCDF_CONTROL, Id, /ENDEF

    ;  Write the data
    NCDF_VARPUT, Id, 'nfft', hdr.nfft
    NCDF_VARPUT, Id, 'prf', hdr.prf
    NCDF_VARPUT, Id, 'NyquistVelocity', hdr.VNY
    NCDF_VARPUT, Id, 'nave', hdr.avc
    NCDF_VARPUT, Id, 'zrg', hdr.mom_zrg
    NCDF_VARPUT, Id, 'rg0', ppar.ihp
    NCDF_VARPUT, Id, 'drg', hdr.DELTA_H
    NCDF_VARPUT, Id, 'lambda', hdr.la
    range =  hdr.mom_h_min + lindgen(hdr.mom_zrg) * hdr.DELTA_H
    NCDF_VARPUT, Id, 'range',range 
    if max(extras eq 'SPC') then begin
        dx = hdr.VUAR/float(nfft)
        doppler = cmod(dx * findgen(nfft), hdr.VUAR,0)
        NCDF_VARPUT, Id, 'doppler',doppler
    endif

    ; Writing the data could look like this:
    ; for iti=0L,ztime-1 do begin
    ;     NCDF_VARPUT, Id, 'time', time[iti], count=[1], offset=[iti]
    ;     NCDF_VARPUT, Id, 'VEL', v[*,iti], count=[zrg,1], offset=[0,iti]
    ; endfor
    
    ; Don't forget to:
    ; NCDF_CLOSE, Id
    
    return,Id
end
