function find_extrema,spc,ix_extremum_l,ix_extremum_r,thr,peak_sig_thr
; find the local minima and maxima in spc.
; Returns the array extremum_type which lists the type of the detected extrema (1 maxima, 0 minima).
; Returns a scalar -1 if no extrema where found
; Returns the indices ix_extremum_l and ix_extremum_r, which point to the places where these extrema where found in spc.
; ix_extremum_l points to the left most and ix_extremum_r to the right most element having the extremal value. These may 
; differ when the extrema resides in a flat part of spc.
common find_extrema,ix_ext_l,ix_ext_r,ext_type
    n = n_elements(spc)
    dspc = [spc[1:n-1],spc[0]] - spc ; derivation of spc: dspc[i] = spc[i+1]-spc[i]
    ; find some place which is not in the middle of a flat part of the spectrum
    i = n - 1L
    for i=n-1L,0L,-1L do begin 
        if dspc[i] ne 0.0 then goto,BREAK 
    endfor
    return,-1
BREAK:
    last_slope = dspc[i] gt 0 ; last_slope=1 positive slope, last_slope=0 negative slope 
    ilastchange_l = i ; _l bedeuted, dass sich der index auf den Linken der verglichenen Werte bezieht.
    if n_elements(ix_ext_l) ne n then begin
        ix_ext_l=lonarr(n)
        ix_ext_r=lonarr(n)
        ext_type=intarr(n)
    endif
    iextremum=0
    for i=0,ilastchange_l do begin
        if dspc[i] ne 0.0 then begin
            slope = dspc[i] gt 0.0 ; slope=1 positive slope, slope=0 negative slope
            if slope ne last_slope then begin
                ext_type[iextremum] = last_slope  ; ext_type=1 Maximum, ext_type=0 Minimum
                ix_ext_r[iextremum] = i
                ix_ext_l[iextremum] = (ilastchange_l+1L) mod n
                last_slope = slope
                iextremum++
            endif
            ilastchange_l = i
        endif
    endfor
    
    if peak_sig_thr gt 0.001 then begin
        for ia=0, iextremum-1 do begin
            ib = (ia+1) mod iextremum
            a=spc[ix_ext_l[ia]] & b = spc[ix_ext_l[ib]]
            if a gt thr && b gt thr && abs(2.*(a-b)/(a+b)) lt peak_sig_thr then begin
                ext_type[ia] = -1
                ext_type[ib] = -1
                ia++
            endif
        endfor

        if iextremum lt 2 || (total(ext_type[0:iextremum-1] eq 0) eq 0 || total(ext_type[0:iextremum-1] eq 1) eq 0) then begin ; es ist nicht mal ein min und ein max uebrig
            iextremum=2
            dumy=max(spc,ixmax,SUBSCRIPT_MIN=ixmin)
            ext_type[0:1]=[1,0]
            ix_ext_r[0:1]=[ixmax,ixmin]
            ix_ext_l[0:1]=[ixmax,ixmin]
        endif
        
        ix_good = where(ext_type[0:iextremum-1] ge 0,nix_good)
        if nix_good gt 0 then begin
            ix_extremum_l=ix_ext_l[ix_good]
            ix_extremum_r=ix_ext_r[ix_good]
            return,ext_type[ix_good]
        endif else return,-1
    endif else begin
       ix_extremum_l=ix_ext_l[0:iextremum-1]
       ix_extremum_r=ix_ext_r[0:iextremum-1]
       return,ext_type[0:iextremum-1]
    endelse
end
