function identify_peaks_by_canion,spc,thr,peak_thr_fac=peak_thr_fac,gap4dom=gap4dom,peak_sig_thr=peak_sig_thr,maxis=maxis
; find the dominant peaks in the spectrum spc by investigating the canions between the peaks. 
; peak_thr_fac = typpically 1./sqrt(nave) where nave is the nuber of spectral averages used while calculating spc
    n = n_elements(spc)

    ; find all extrema
    extremum_type = find_extrema(spc,ix_extremum_l,ix_extremum_r,thr,peak_sig_thr)
    ; extremum_type = ffind_extrema(spc,ix_extremum_l,ix_extremum_r,thr,peak_sig_thr) ; ist eigentlich getestet und schneller
    zext = n_elements(extremum_type)

    if extremum_type[0] lt 0 then return,-1

    ; determin which minima are dominant. 
    is_dominant = replicate(0, zext)
    ixmxl = -1L
    for i=0,zext-1 do begin
        if extremum_type[i] eq 0 then begin
            if spc[ix_extremum_l[i]] le thr and (ix_extremum_r[i]-ix_extremum_l[i]+n) mod n ge gap4dom then begin
                is_dominant[i]=1 
                ixmxl = -1L
            endif else begin
                if ixmxl lt 0L then begin
                    ixmxl = ix_extremum_r[(i-1+zext) mod zext]  ; index of max on the left  side of the investgated peak
                    spmi=1e25
                endif
                ixmxr = ix_extremum_l[(i+1) mod zext]  ; index of max on the right side of the investgated peak
                
                ;+; Test the canion by its integral depth
                ;+; integrate the surface between the line connecting the two neighbouring maxima and spc. 
                ;+dmx = ixmxr - ixmxl & if dmx le 0 then dmx += n
                ;+js = lindgen(dmx-1L) + 1L
                ;+canion = 2.0/float(dmx)*total( ((dmx-js)*spc[ixmxl] + js*spc[ixmxr])/float(dmx) - spc[(ixmxl+js) mod n] )
                ;+; compare this surface with an estimate of the stddev of the total power (tot_pow_without_dip) 
                ;+; between these two maxima.
                ;+tot_pow_without_dip = float(dmx)*0.5*(spc[ixmxl]+spc[ixmxr])
                ;+; tot_pow_without_dip = float(dmx) * (spc[ixmxl] > spc[ixmxr])
                ;+; peak_thr_fac should be typically 3/sqrt(avg), with avg=number of spectral averages. 
                ;+; (dmx/2) is the number of averages related to the integration.
                ;+seperate_here = canion gt tot_pow_without_dip * peak_thr_fac / sqrt(dmx/2)
               
                ;*; Test the cannion by its maximum depth
                ;*dmx = ixmxr - ixmxl & if dmx le 0 then dmx += n
                ;*js = lindgen(dmx-1L) + 1L
                ;*spx = spc[(ixmxl+js) mod n]
                ;*; seil = ((dmx-js)*spc[ixmxl] + js*spc[ixmxr]) / float(dmx)
                ;*; canion_depth = max ( seil - spx, ix_maxdepth )
                ;*; Die folgende Zeile berechnet das gleich wie die beiden versändlicheren Zeilen oben. 
                ;*canion_depth = max ( ((dmx-js)*spc[ixmxl] + js*spc[ixmxr]) - spx*float(dmx), ix_maxdepth )/float(dmx)
                ;*seperate_here = canion_depth/spx[ix_maxdepth] gt peak_thr_fac

                ; Test the cannion by its depth at the minimum
                ixmi_ = ix_extremum_r[i]
                if spc[ixmi_] lt spmi then begin
                    spmi = spc[ixmi_]
                    ixmi = ixmi_
                endif
                dmx = ixmxr - ixmxl & if dmx le 0 then dmx += n
                ixmi_rel = (ixmi-ixmxl) & if ixmi_rel le 0 then ixmi_rel += n
                seil = ((dmx-ixmi_rel)*spc[ixmxl] + ixmi_rel*spc[ixmxr]) / dmx
                canion_depth = seil - spc[ixmi]
                seperate_here = canion_depth/spc[ixmi] gt peak_thr_fac
                
                if seperate_here then begin
                    is_dominant[i] = 1
                    ixmxl = -1L
                endif else begin
                    ; When the programm comes to these lines then two peaks will be joined together. 
                    ; In priciple here the max of the joined peak shold be searched. The following line replaces this max search.
                    if spc[ixmxr] ge spc[ixmxl] then ixmxl = -1L
                endelse
            endelse
        endif
    endfor

    ; get the indices which point to the dominant minima in extremum_type, ix_extremum_l, and ix_extremum_r
    ixdominant = where(is_dominant, nixdom) 
    ; note the number of the dominant maxima is equal to the number of dominant minima.
    
    if nixdom eq 0 || total(extremum_type[ixdominant] eq 0) eq 0 then begin
        ; all minima where flaged as non-dominant. 
        ; Now we re-flag the minima as dominant which is farest from the max in spc. 
        dum=max(spc,ixmax)
        maxdist = 0L
        for i=0,zext-1 do begin
            if extremum_type[i] eq 0 then begin
                distx = abs(cmod(ix_extremum_r[i] - ixmax,n)) < abs(cmod(ix_extremum_l[i] - ixmax,n))
                if distx gt maxdist then begin
                    maxdist = distx
                    ireflag = i
                endif
            endif
        endfor
        is_dominant[ireflag]=1
        nixdom = 1
        ixdominant = [ireflag]
    endif

    ; save the ranges of the peaks which are seperated by dominant minima to peak_ranges
    ; peaks that are separated by minima which are flaged as non-dominant are compound by this procedure
    peak_ranges=lonarr(2,nixdom) ; left, and right index of each max will be stored to this array
    maxis = fltarr(nixdom)
    for i=0,nixdom-1 do begin
        peak_ranges[0,i]=[ix_extremum_r[ixdominant[i]],ix_extremum_l[ixdominant[(i+1) mod nixdom]]]
        ; remeber the max spc value enclosed by the actual peak
        ii = ixdominant[i] 
        maxpow=0.0
        repeat begin
            ii = (ii+1L)  mod zext
            maxpow >= spc[ix_extremum_r[ii]]
            ii = (ii+1L) mod zext
        endrep until ii eq ixdominant[(i+1) mod nixdom] 
        maxis[i] = maxpow
    endfor
    return,peak_ranges
end

