pro new_remove_clutter,spc,x,hsd,det_thres,pp,gcmoms,do_pltclutter=do_plt,clutter_level=clutter_level,nixsigmod=nixsigmod,ixsigmod=ixsigmod
    ; new remove clutter from a spectrum which is distrubuted symmetrically around zero velocity (index 0 in spc)
    ; input variables:
    ; spc = Spectrum of one range gate, zero velocity at index 0
    ; hsd = hsd level
    ; det_thres = detection threshold
    ; pp = Structure containing the following Parameters:
    ; Parameters tested with Data from September + October 2005:
    ; pp.max_sholder_dbc=-1.10  ; theoretically the shoulders should be -6 dbc, -1.1 dB works good in practice
    ;                  Non clutter signal may cover the the clutter. The typically one of the shoulders is higer.
    ;                  In this case no clutter removal is done.
    ; pp.fftwin=[0.45,.055,0.014, 0.003,,0.002,,0.001, 0] ; decaying factor for the neighbors typically used for strong clutter
    ; pp.fftwin=[0.31, 0.0042, 0.001, 0, 0, 0, 0, 0] ; decaying factor (fourier transfomed fft window) for normal clutter
    ;
    ; output variables:
    ; spc the filtered spectrum
    ; clutter_level the power of the clutter peak which was removed.
    ; nixsigmod the number of spectral lines which where set to noise level.
    ;
common new_remove_clutter,max_sholder_oc,nfft,nop,ixnon,ixnop,fftwin,np,yra,ix_msr,xpart,dx
common remove_clutter_patchrg,ixpatchrg
    zchan=n_elements(hsd)
    if n_elements(do_plt) ne 1 then do_plt=0
    redo:
    zchan=n_elements(hsd)
    nfftx = n_elements(spc[*,0L,0L])
    if n_elements(nfft) ne 1 || nfftx ne nfft then begin
        max_sholder_oc=linear(pp.max_sholder_dbc)
        nfft = nfftx
        fftwin=pp.fftwin
        nop = max(where( fftwin gt 0.0 ))+1L
        ixnop=1L+lindgen(nop)
        ixnon=nfft-nop+lindgen(nop)
        ixpatchrg=[ixnon,0L,ixnop]
        if do_plt then begin
            print,'Blueish to green is co and red to magenta is cross channel,'
            print,'Solid lines show the original spectra, starts the filtered spectra.'
            print,'set the parameters yra,np'
            print,'use con to go to the next spectrum'
            print,'quit to continue processing silen or stop exit to return to shell'
            np=30
            yra=90
        endif
        ; pp.ngc_zmsr (max search range) diameter of the range in wich a gc maximum is searhed in units of fft lines of the range. ngc_zmsr=1 means the maximum is seareched within 3 fft lines.
        ix_msr = pp.ngc_zmsr eq 0 ? [0L] : [nfft-pp.ngc_zmsr+lindgen(pp.ngc_zmsr),0L,lindgen(pp.ngc_zmsr)+1]
        dx = abs(x[1]-x[0])
        xpart = x[ixpatchrg]
    endif
    if do_plt then ospc=spc
    pgcmax=max(spc[ix_msr,0L,0L],imax) 
    det_thres_co=det_thres[0L]
    if pgcmax gt det_thres_co then begin
        spcpart = spc[ixpatchrg,0,*]
        imax = imax-pp.ngc_zmsr+nop ; max-offset in groesserem Stueck
        iab=lonarr(2)
        WentDown=replicate(0,2)
        for idir = 0,1 do begin ; idir=0 von max nach links, idir=0 von max nach rechts
            ili = imax  
            repeat begin
                ilast = ili
                if idir then ++ili else --ili
                plast = spcpart[ilast,0L,0L]
                pco = spcpart[ili,0L,0L]
                if pco lt plast then WentDown[idir]=1
                pco_good = pco gt det_thres_co
                if pco_good then begin
                    if zchan ge 2 then ldr_ceil = ((spcpart[ili,0L,1L] > det_thres[1L])-hsd[1L])/(pco-hsd[0L]) else ldr_ceil=0.0000001 
                endif
            endrep until pco_good eq 0 || pco gt plast || ldr_ceil lt pp.GcLdrPeakDesc || ili eq 0 || ili eq nop*2
            iab[idir]=ilast
        endfor
        if min(WentDown) eq 1 then begin 
            ncalc=iab[1]-iab[0]+1
            ixcalc=lindgen(ncalc)+iab[0L] & cospccalc=spcpart[ixcalc,0L,0L]-hsd[0L] & xcalc=xpart[ixcalc]
            powco = total(cospccalc)
            if ncalc ge 2 then begin ; ncalc id always ge 1 as here pgcmax gt det_thres_co 
                velo = - total(cospccalc*xcalc)/powco
                vvx = total(cospccalc*xcalc*xcalc)/powco
                rms = sqrt(vvx - velo*velo)
            endif else begin
                velo = - xcalc[0L]
                rms = dx
            endelse
            if zchan ge 2 then begin 
                powcx = total(spcpart[ixcalc,0L,1L]-hsd[1L])
                ldrvalid = powcx gt det_thres[1]
                ldr = ldrvalid ? powcx/powco : det_thres[1L]/powco
            endif else begin
                ldr=!values.f_nan
                ldrvalid=0
            endelse
        endif else begin
            powco=!values.f_nan
            velo=!values.f_nan
            rms=!values.f_nan
            ldrvalid=0
            ldr=!values.f_nan
        endelse
    endif else begin
        powco=!values.f_nan
        velo=!values.f_nan
        rms=!values.f_nan
        ldrvalid=0
        ldr=!values.f_nan
    endelse

    isgc = finite(powco) && rms lt pp.GcMaxRms && abs(velo) lt pp.GcMaxVelo && (ldrvalid eq 0 || ldr gt pp.GcLowestLdr )
     
    if isgc then begin
        ixsigmod=ixpatchrg[ixcalc]
        spc[ixsigmod,0L,0L]=hsd[0L]
        if zchan ge 2 then spc[ixsigmod,0L,1L]=hsd[1L]
        clutter_level=powco
        nixsigmod = ncalc
    endif else begin
        nixsigmod=-1
        ixsigmod=-1L
        clutter_level=0.
    endelse

    gcmoms={powco:powco,velo:velo,rms:rms,ldrvalid:ldrvalid,ldr:ldr,isgc:isgc}

    while do_plt do begin
        ix=[nfft-np+lindgen(np),lindgen(np+1)]
        frq=[-np+lindgen(np),lindgen(np+1)]
        frq_nop=[-nop+lindgen(nop),lindgen(nop+1)]
        plot,frq,db(ospc[ix,0,0]),yra=[-9,yra],psym=10,xst=1,yst=1,ytit='Power Spectrum [dB]',xtit='Number of Frequency Bin'
        oplot,frq,db(ospc[ix,0,0]),color=1,psym=10
        oplot,frq,db(ospc[ix,0,1]),color=2
        if isgc then begin
            oplot,frq,db(spc[ix,0,0]),col=1,psym=2
            oplot,frq,db(spc[ix,0,1]),col=2,psym=2
        endif
        oplot,frq,replicate(db(hsd[0])      ,2*np+1),linest=2,color=6
        oplot,frq,replicate(db(det_thres[0]),2*np+1),linest=1,color=6
        oplot,frq,replicate(db(hsd[1])      ,2*np+1),linest=2,color=3
        oplot,frq,replicate(db(det_thres[1]),2*np+1),linest=1,color=3

        arg = myprompt('Remove Clutter >')
        cmd = arg[0] &  narg=n_elements(arg)
        case 1 of
        cmd eq 'stop': stop
        cmd eq 'quit': begin
            pp.pltclutter=0
            do_plt=0
        end
        cmd eq 'r' or cmd eq 'redo':begin
            nfft=0
            goto, redo
        end
        cmd eq 'x' or cmd eq 'exit' or cmd eq 'con' or cmd eq 'conti': do_plt=0
        else: begin
            setcmd = ''
            for i=0 , narg - 1 do setcmd = setcmd + arg[i] + ' '
            if lmgr(/vm) then print,'VM license can not execute '+setcmd $
            else dummi=execute( setcmd )
        end
        endcase
    endwhile
end
