function RadarConstant, hdr, serverinfo, calida, do_corrbycal, doLowRangeAttenuation, raw=raw, SnrCorfa, RadarConstant5
; Calculate the factors SnrCorfa[*,Co] and SnrCorfa[*,Cx] which should be multiplied with the SNRs to correct for:
;     1) The power in noise gate serverinfo.npowx is smaller than it would be if the LNA would be terminated by a matched load at 300 K
;        As the SNRs had been devided by serverinfo.npowx they are too big and have to be multiplied by a factor > 1.
;     2) vanisching gain in the lowest range gates.
;     3) if do_corrbycal is selected, correct by npow/cpow*cpow0/npow0. 
;        cpow0/npow0 = so called calibration (~ 10 db) measured while noisecom measurement, 
;        cpow /npow  = so called calibration observed at current measuremnet.
;     4) STC-Mode
;     5) Near Field Correction which is calculated by an estimation from ewan depending on the antenna diameter
; and a factor RadarConstant which has to be multiplied to the corrected SNRs to get the Ze values. Also RadarConstant5 = the radar constant at 5 km 
; height is passed to output. The later is the  product: 
;               [ RadarConstant at 5km * SnrCorfa at 5 km and for co    channel, 
;                 RadarConstant at 5km * SnrCorfa at 5 km and for cross channel]
; Note SnrCorfa has nearly no range dependance above 1 km, so SnrCorfa at 5 km = SnrCorfa at last range gate. 
;
; RadarConstant also calculates a factor RadarConstant which has to be multiplied to the corrected SNRs to get the Ze values.

; These factors are range dependend. Therefore this routine provides two vektors (RadarConstant, SnrCorfa[*,Co], and SnrCorfa[*,Cx]) 
; with one element each per selected range gate.
; The range dependance is mainly due to 1/R^2 in the radar equation. There is an additional rage dependance
; because the pin modulators need some time
; to close after the transmitting pulse. This is is measured during the calibration by the Noisecon source. 
; The Data gained from the calibration with the NoiseCom source is provided by the structure calida. 
;
;
; Matthias 
;
common RadarConstant,zrg,SnrCorfa8,C0,refvals,Hq7H0q,PRef7PNoi50,StcHei,doLowRangeAtt
common RaCoFirst,RaCoFirst
common magic4chs,magic4chs,FzkPdsFmt,syspar
    if n_elements(raw) ne 1 then raw=0    ; normally 0 as calculating Ze is normally done with the moment and not the spectra
    if raw eq 0 then zrgh=hdr.mom_zrg else zrgh=hdr.raw_zrg
    if n_elements(zrg) eq 0 || zrg ne zrgh || doLowRangeAttenuation ne doLowRangeAtt then begin
        pp={$
            PTAV0:30.  ,$    ; [W] Reference value of the average transmitter power at the receiver output (The internal thermistor 
                        $    ; is calibrated so that it shows the Power at the receiver output. At the transmitter output the power is usually 1.4 dB greater). 
            tau0:200e-9 ,$   ; [s] Pulse width = 1/Bandwidth
            PRF0:5000. ,$    ; [Hz]   Reference Pulse repetition frequency
            H0:5000.   ,$    ; [m]    Reference Height
            lambda:299792458./syspar.txfrequency ,$; [m]    Wavelength 
            clight:299792458.,$; speed of light m/s
            AGain:linear(49.75),$; Antenna Gain
            ABeamWidth:0.5 * !pi/180.,$
            NfRec0:linear(6.7),$; Reference Value of the total noise figure at the receiver input (including losses in the TR-switch and Couplers). 
                               $; calida.nf[Co/Cx] = value of the receiver noise aktually measured by the noisecom source 
            WGlength:0.65,$     ; single path waveguide length including the inside the feed
            TwoWayWgLoss:-1.0, $;
            AntennaDiameter4NearFieldCorr:1.0,$ ; Antenna Diameter used for the near field correction, if < 0.1 no near field correction is applied
            KBT0 : 4e-21,$ ; [J] 
            kwq : 0.93 $
        }
        config2stru,pp,'radarconstant',[syspar.iam+'.config']
        if pp.TwoWayWgLoss gt 0.0 then begin
            LossWG = linear(pp.TwoWayWgLoss) 
        endif else begin
            LossWG = linear(0.6 * pp.WGlength*2)  ; Losses in the waveguides, both ways (receiving and transmitting, assuming 0.6 dB/m loss). 
        endelse
        zrg=zrgh
        doLowRangeAtt = doLowRangeAttenuation

        C0= (1e18 * 1024.0 * alog(2.0) * pp.lambda^2 * pp.KBT0 * LossWG * syspar.TxRecbandwidthMatchingLoss * pp.NfRec0 * pp.PRF0 * pp.H0^2) $
             /(!pi^3 * pp.clight * pp.kwq * pp.AGain^2 * pp.ABeamWidth^2 * pp.PTAV0 * pp.tau0)

        compressionmode = round(hdr.tx_pulsewidth / hdr.rx_pulsewidth)
        case compressionmode of
            19: compression_loss = syspar.compression_loss19
            else:compression_loss = 1.0
        endcase
        C0 *= compression_loss

        if raw eq 0 then H = findgen(zrg) * hdr.delta_h + hdr.mom_h_min $
                    else H = findgen(zrg) * hdr.delta_h + hdr.raw_h_min
        Hq7H0q = (H/pp.H0)^2
        SnrCorfa8 = fltarr(zrg,2)
        SnrCorfa8[*,0] = calida.nf[0]/(pp.NfRec0*calida.CorrLNAmatch[0])  
        SnrCorfa8[*,1] = calida.nf[1]/(pp.NfRec0*calida.CorrLNAmatch[1])
        if doLowRangeAtt then begin
            zGainVsRange=n_elements( calida.GainVsRange[*,0])
            SnrCorfa8[0:zGainVsRange-1,0] /= calida.GainVsRange[*,0]
            SnrCorfa8[0:zGainVsRange-1,1] /= calida.GainVsRange[*,1]
        endif
        if pp.AntennaDiameter4NearFieldCorr gt 0.1 then begin
            nfc = near_field_correction( 299792458./syspar.txfrequency, pp.AntennaDiameter4NearFieldCorr, H)
            SnrCorfa8[*,0] *= nfc 
            SnrCorfa8[*,1] *= nfc
        endif
        refvals = pp.PTAV0*pp.tau0/pp.PRF0
        PRef7PNoi50 = calida.Calibr0; * calida.CorrLNAmatch  ; Power of internal reference noise at the time of the calibration measurment / receiver noise with matched load @ 300 K
        StcHei = (where(h*2/hdr.c gt 6e-6 ))[0]
        if n_elements(RaCoFirst) eq 0 then RaCoFirst=1
    endif ; end of do only during first call  

    if do_corrbycal then begin
        SnrCorfa = [[SnrCorfa8[*,0] * (serverinfo.npw1 * PRef7PNoi50[0] / serverinfo.cpw1)],$
                    [SnrCorfa8[*,1] * (serverinfo.npw2 * PRef7PNoi50[1] / serverinfo.cpw2)]]
    endif else begin
        ; ### adc noise correction #####
        SnrCorfa = [[SnrCorfa8[*,0] * serverinfo.npw1 / ( serverinfo.npw1 - calida.adcnoise[0] * hdr.nfft )],$
                   [SnrCorfa8[*,1] * serverinfo.npw2 / ( serverinfo.npw2 - calida.adcnoise[1] * hdr.nfft )]]
    endelse

    if hdr.att and syspar.has_stc then begin
        StcCorfaCo=linear(15.0)
        SnrCorfa[0:StcHei-2,0] *= StcCorfaCo 
        StcCorfaCx=linear(15.0)
        SnrCorfa[0:StcHei-2,1] *= StcCorfaCx
    endif
    
    tpowfac=syspar.hybridmode?0.5:1.0
    case syspar.powermetertype of
        0: avgtpow = tpowfac*serverinfo.tpow/hdr.fzk100    ; power is given as average power
        1: avgtpow = serverinfo.tpow * hdr.tx_pulsewidth*hdr.prf ; power is given as peak power in W as in MBX1
        2: avgtpow = serverinfo.tpow * hdr.tx_pulsewidth*hdr.prf * 1000. ; power in given as peak power in kW line in MBC3
        else:begin
            print,'wrong powermetertype in structure syspar: ',syspar.powermetertype
            avgtpow = tpowfac*serverinfo.tpow/hdr.fzk100
        end
    endcase
    ; pulse_width=2.0*hdr.delta_h/hdr.c
    pulse_width = hdr.tx_pulsewidth
    RadarConstant5 = ( C0 * hdr.prf * refvals / (avgtpow * pulse_width) ) 

    if RaCoFirst then begin
        print
        print,'Printing parameters which go into the radar constant. All printed in dB, smaller values mean the system has better sensitivity.'
        print,'lambda^2/lambda(35.5)^2:',db(pp.lambda^2/(299792458./35.5e9)^2)
        print,'Two-Way Waveguide Loss:',db(LossWG)
        print,'ReceiverBwdLoss:',db(syspar.TxRecbandwidthMatchingLoss)
        print,'Antenna Performance 1/(gain^2*beamwidth^2):',-1*db(pp.AGain^2*pp.ABeamWidth^2)
        print,'Noise Figure co/cx in dB:',db(calida.nf[0]),db(calida.nf[1])
        print,'1/Design Noise Figure:',-1*db(pp.NfRec0)
        print,'1 / CorrLNAmatch co/cx:',-1*db(calida.CorrLNAmatch[0]),-1*db(calida.CorrLNAmatch[1])
        print,'PowTxAvg0/PowTxAvg:',db(pp.PTAV0/avgtpow),' PowTxAvg:',avgtpow,' W'
        print,'200 ns/Pulsewidth:',db(pp.tau0/pulse_width)
        print,'PRF/5000 Hz:',db(hdr.prf/pp.PRF0)
        print,'do_corrbycal:',do_corrbycal
        print,'If do_corrbycal=1 correction of next line is applied, if do_corrbycal=0 the ADC correction of next+1 line is applied.'
        print,'CalibrNow/CalibrDuringNoisecomMeas co/cx:',db(serverinfo.npw1 * PRef7PNoi50[0] / serverinfo.cpw1),db(serverinfo.npw2 * PRef7PNoi50[1] / serverinfo.cpw2)
        print,'ADC Noise Corection co/cx:',db(serverinfo.npw1 / ( serverinfo.npw1 - calida.adcnoise[0] * hdr.nfft)),db(serverinfo.npw2 / ( serverinfo.npw2 - calida.adcnoise[1] * hdr.nfft))
        print,''
        print,'Instantaneous Min dBZ@5km = ',db(SnrCorfa[zrg-1,0L]*RadarConstant5),db(SnrCorfa[zrg-1,1L]*RadarConstant5)
        print
        RaCoFirst=0
    endif
    
    RadarConstantHq7H0q  =  RadarConstant5 * Hq7H0q
    ; also eigentlich bei do_corrbycal=0 und wenn adcnoise klein ist
    ; RadarConstantHq7H0q * SnrCorfa = (1e18 * 1024.0 * alog(2.0) * pp.lambda^2 * pp.KBT0 * LossWG * syspar.TxRecbandwidthMatchingLoss * calida.nf * H^2 * hdr.prf ) / (!pi^3 * pp.clight * pp.kwq * pp.AGain^2 * pp.ABeamWidth^2 * avgtpow * pulse_width * calida.CorrLNAmatch )
    ;
    ; RadarConstant5 =  RadarConstant5 * reform(SnrCorfa[zrg-1,*],2)
    ; this was done until adding snrcorfa to the output of mmclx in April 2014
    ; now for calibtation the following should be used 
    ;      Zco = SNRco * RadarConstantHq7H0q * SnrCorfa[*,co]   with co=0
    ;      Zcx = SNRcx * RadarConstantHq7H0q * SnrCorfa[*,cx]   with cx=1
    return, RadarConstantHq7H0q
end

