pro remove_clutter,spc,noise,det_thres,pp,do_plt,clutter_level=clutter_level,nixsigmod=nixsigmod
    ; remove clutter from a spectrum which is distrubuted symmetrically around zero velocity (index 0 in spc)
    ; input variables:
    ; spc = Spectrum of one range gate, zero velocity at index 0
    ; noise = Noise level
    ; det_thres = detection threshold
    ; pp = Structure containing the following Parameters:
    ; Parameters tested with Data from September + October 2005:
    ; pp.max_sholder_dbc=-1.10  ; theoretically the shoulders should be -6 dbc, -1.1 dB works good in practice
    ;                  Non clutter signal may cover the the clutter. The typically one of the shoulders is higer.
    ;                  In this case no clutter removal is done.
    ; pp.fftwin=[0.45,.055,0.014, 0.003,,0.002,,0.001, 0] ; decaying factor for the neighbors typically used for strong clutter
    ; pp.fftwin=[0.31, 0.0042, 0.001, 0, 0, 0, 0, 0] ; decaying factor (fourier transfomed fft window) for normal clutter
    ;
    ; output variables:
    ; spc the filtered spectrum
    ; clutter_level the power of the clutter peak which was removed.
    ; nixsigmod the number of spectral lines to each side which where changed significantly (more than det_thres-noise). 0 means only the center line was changed, -1 means no change at all.
    ;
common remove_clutter,max_sholder_oc,n,nop,ixnon,ixnop,fftwin,np,yra
common remove_clutter_patchrg,ixpatchrg
    if n_elements(do_plt) ne 1 then do_plt=0
    if n_elements(n) ne 1 || n_elements(spc) ne n then begin
        max_sholder_oc=linear(pp.max_sholder_dbc)
        n = n_elements(spc)
        fftwin=pp.fftwin
        nop = max(where( fftwin gt 0.0 ))+1L
        ixnop=1L+lindgen(nop)
        ixnon=n-1-lindgen(nop)
        ixpatchrg=[0L,ixnop,ixnon]
        if do_plt then begin
            print,'white is the original spectrum, red the clutterfiltered spectrum,'
            print,'blue is the spectraprocessing.fftwin which is set in the config file.'
            print,'it can be accessed here as fftwin'
            print,'use w+ <k> to increase fftwin at frequency <k> (only fft win will change in the picture)'
            print,'or  w- <k> to decrease it (only fftwin will change in the picture)'
            print,'set the parameters yra,np'
            print,'use con to go to the next spectrum'
            print,'quit to continue processing silen or stop exit to return to shell'
            np=30
            yra=70
        endif
    endif
    if do_plt then ospc=spc
    pc = spc[0]   ; power of the spectral component at zero Doppler shift
    ;signif_neg = 2*noise - det_thres
    signif_neg = noise
    lshoulder=spc[n-1] & rshoulder=spc[1]
    biggershoulder=lshoulder > rshoulder
    if pc ge det_thres && biggershoulder lt pc && $
            (0.5*(lshoulder+rshoulder)-noise lt (pc-noise)*max_sholder_oc || biggershoulder lt det_thres) then begin
        fftwinPC = fftwin[0:nop-1]*(pc - noise)
        spc[0] = noise 
        spc[ixnop] = (spc[ixnop] - fftwinPC) > signif_neg
        spc[ixnon] = (spc[ixnon] - fftwinPC) > signif_neg
        ixsigmod=where( fftwinPC gt (det_thres-noise),nixsigmod )
        ; if nixsigmod gt 0 then lixsigmod=ixsigmod[nixsigmod-1] else nixsigmod=0
        clutter_level=pc-noise
    endif else begin
        clutter_level=0.0
        nixsigmod=-1
    endelse
    while do_plt do begin
        ix=[n-np+lindgen(np),lindgen(np+1)]
        frq=[-np+lindgen(np),lindgen(np+1)]
        frq_nop=[-nop+lindgen(nop),lindgen(nop+1)]
        plot,frq,db(ospc[ix]),yra=[-9,yra],psym=10,xst=1,yst=1,ytit='Power Spectrum [dB]',xtit='Number of Frequency Bin'
        if ospc[0] ne spc[0] then oplot,frq_nop,db([fftwin[nop-lindgen(nop)-1],1.0,fftwin[lindgen(nop)]]*pc),color=1
        oplot,frq,replicate(db(det_thres),2*np+1),linest=1,color=4
        oplot,frq,replicate(db(noise),2*np+1),linest=2,color=4
        oplot,frq,db(spc[ix]),col=2,psym=2

        ixp=1L+lindgen(n/2)
        ixn=n-1-lindgen(n/2)
        pspc=ospc[ixp]
        nspc=ospc[ixn]

        arg = myprompt('Remove Clutter >')
        cmd = arg[0] &  narg=n_elements(arg)
        case 1 of
        cmd eq 'png': begin
            if narg ge 2 then png_fn = arg[1] else begin
                png_fn =  'clutter' + tag_mom + '.png'
            endelse
            case !d.name of
                'X':write_png, png_fn, TVRD(true=1)
                'Z': begin
                    write_png, png_fn, TVRD() , ct_red, ct_green, ct_blue
                end
                else:
            endcase
        end
        cmd eq 'w+' or cmd eq 'w-' or cmd eq 'w0' or cmd eq 'w1' :begin
            if narg eq 2 then begin
                iiichange=(0>(fix(arg[1])-1))<(nop-1)
                case arg[0] of 
                    'w+':  fftwin[iiichange] *= 1.1
                    'w-':  fftwin[iiichange] *= 0.9
                    'w0':  fftwin[iiichange] = 0
                    'w1':  begin
                        fftwin[iiichange] = 0.25
                        nop = max(where( fftwin gt 0.0 ))+1L
                    end
                    else:
                endcase
                print,format='(99f10.7)',fftwin
            endif else print,'syntax: fftwin <number of bin> <factor>'
        end
        cmd eq 'stop': stop
        cmd eq 'quit': begin
            pp.pltclutter=0
            do_plt=0
        end
        cmd eq 'x' or cmd eq 'exit' or cmd eq 'con' or cmd eq 'conti': do_plt=0
        else: begin
            setcmd = ''
            for i=0 , narg - 1 do setcmd = setcmd + arg[i] + ' '
            if lmgr(/vm) then print,'VM license can not execute '+setcmd $
            else dummi=execute( setcmd )
        end
        endcase
        nop = max(where( fftwin gt 0.0 ))+1L
    endwhile
end
