pro fituw,v,c,s,u,w
; fit the parameters u, and w by least square regression so that the target function
;    v[i] = u c[i] + w s[i] ; i=0,...,ztime-1
; with given values v[i],c[i],s[i] is matched as good as possible.
;
; The least squaer error is
;     LSQERR = total((  v[i] - u c[i] - w s[i]   )^2)
; In the optimum of u and w the partial derivates of dLSQERR/du and dLSQERR/dw are zero.
; This provides the equations which are solved by Cramers rule.
;
; This routine can be replaced by library functions which are more general and therefore less efficient.
; In the task for which the routine was written v are the radial velocities, c=cos(elv[i]) and s=sin(elv[i])
;
; Matthias Bauer-Pfundstein
;
    CC=total(c^2)
    SS=total(s^2)
    CS=total(c*s)
    VC=total(v*c)
    VS=total(v*s)
    D=CC*SS-CS
    u=(VC*SS-VS*CS)/D
    w=(CC*VS-CS*VC)/D
end
