; $Id: linreg.pro,v 1.0 92/01/24 16:05:36 wave Exp $
;
FUNCTION LINREG,X,Y,W,SIG,SIGY
;
;+
; NAME:
;        REGRESS
; PURPOSE:
;        Multiple linear regression fit.
;        Fit the function:
;        Y(i) = A(0)*X(0,i) + A(1)*X(1,i) + ... + 
;                A(Nterms)*X(Nterms,i)
; CATEGORY:
;        G2 - Correlation and regression analysis.
; CALLING SEQUENCE:
;        Coeff = REGRESS(X,Y,W,SIGMA,SIGMAY)
; INPUTS:
;        X = array of independent variable data.  X must 
;                be dimensioned (Nterms, Npoints) where there are Nterms 
;                coefficients to be found (independent variables) and 
;                Npoints of samples.
;        Y = vector of dependent variable points, must 
;                have Npoints elements.
;        W = vector of weights for each equation, must 
;                be a Npoints elements vector.  For no 
;                weighting, set w(i) = 1., for instrumental weighting 
;                w(i) = 1/standard_deviation(Y(i)), for statistical 
;                weighting w(i) = 1./Y(i)
;
; OUTPUTS:
;        Function result = coefficients = vector of 
;                Nterms elements.  Returned as a vector.
;                The corresponding errors are retrurned in the vector
;                sig and sigy which is the standard derivation.
;
; PROCEDURE:
;        Adapted from my brain (especially the estimation of sig, the
;        error of the calculatet koefitients.
;
; MODIFICATION HISTORY:
;        Written, Matthias Bauer 1992.01.24 
;-
;
on_error,2
    SY = SIZE(Y)                ;GET DIMENSIONS OF X AND Y.
    SX = SIZE(X)
    IF (N_ELEMENTS(W) NE SY(1)) OR (SX(0) NE 2) OR (SY(1) NE SX(2)) THEN $
        message, 'Incompatible arrays.'
    NTERM = SX(1)                ;# OF TERMS
    NPTS = SY(1)                 ;# OF OBSERVATIONS
    SW = TOTAL(W)                ;SUM OF WEIGHTS
    XWEIGHT = X * ( REPLICATE ( 1.0 , NTERM ) # W )     
    WMEAN = W / SW

    IHO = X # ( Y * W )
    AA = XWEIGHT # TRANSPOSE(X)
    IF N_ELEMENTS( AA ) NE 1 THEN INVAA = INVERT( AA ) ELSE INVAA = [[1/AA(0)]] 
    KOEF = INVAA # IHO
    SIGMAYQ =  TRANSPOSE( ( Y - TRANSPOSE(KOEF) # X )^2 )  # WMEAN
    sigy = sqrt( SIGMAYQ(0) )
    ;SIG = SQRT(   (  ( INVAA # X )^2  #    W^2   ) * SIGMAYQ(0)   )
    SIG = SQRT(     ( INVAA # X )^2 # W^2    *  SIGMAYQ(0)      )
    RETURN,KOEF
END
