pro mimage, image, xrange, yrange, zrange, zcol, startcol, badcol, nullcol=nullcol, nulltol=nulltol,  $
           INTERP=interp,  charsize=charsize, charthick=charthick,title=title, saturate=saturate, $
           ztitle=ztitle, xtitle=xtitle,ytitle=ytitle,xstyle=xstyle,ystyle=ystyle,heitit=heitit,ticklen=ticklen
; show the two dimensional array image as image with its values mapped to the colours
; The values which lie in zrange are mapped to the color range defined by startcol and zcol.
; A color bar is added which helps to reassign which value is visualized by which color.
;
; Minimum calling sequence:
; colt,fn
;     fn name of a color file e.g. 'snr'
; mimage,imagearr,xra,yra,zra
;     imagearr: two dimensional array with the values that should be plotted by color codes
;     xrange: two element vector with the x-coordinate of the left most and right most elements of the imagearr
;     yrange: two element vector with the y-coordinate of the lowest and highest elements of the imagearr
;     zrange: range for mapping the color table to the values in imagearr
; 
; Matthias Bauer-Pfundstein bauer@metek.de Karen Ruebenacker
;
if n_elements(heitit) ge 1 then ytitle=heitit
default_value,xstyle,1
default_value,ystyle,1
default_value,ticklen,-0.01
common colt,zcolt,fncolt
if n_elements(badcol) ne 1 then badcol=15
if n_elements(startcol) ne 1 then startcol=30
if n_elements(zcol) ne 1 then begin
    if n_elements(zcolt) then zcol=zcolt
endif 
if n_elements(zcol) ne 1 then begin
    print,'number of colors of the colortable has to be provided as 5th argument or alternatively use the subroutine colt,fn for setting the colortable'
    return
endif

default_value,saturate,2
default_value,nulltol,0.1
default_value,ytitle,'Range'

; Dimensions of the image to be wrapped
sz_image = size(image)
if sz_image[0] lt 2 then message, 'Parameter not 2D'
zimage_x = float(sz_image[1])
zimage_y = float(sz_image[2])

;dzra = zrange[1] - zrange[0]
;if  dzra lt 0 then message, 'zrange mut be positive'

; ================
; Draw the image |
; ================

;Position (0=left, 1=right) of the IMage window in NoRMalized (to 1) coordinates
x0im_nrm  = 0.10
x1im_nrm  = 0.87
y0im_nrm  = 0.1
y1im_nrm  = 0.9

; as we don't plot half pixels at the corners the plot ranges hav to be expanded by 1/2 pixle to each side
eyrange = yrange + [-0.5,0.5] * (yrange[1]-yrange[0])/zimage_y
exrange = xrange + [-0.5,0.5] * (xrange[1]-xrange[0])/zimage_x

; define the coordinate system for the image plot
plot,[0,0],/nodata,xrange=exrange,yrange=eyrange, $
          ticklen=ticklen,xstyle=xstyle,ystyle=ystyle, $
          position=[[x0im_nrm,y0im_nrm],[x1im_nrm,y1im_nrm]],$
          ytitle=ytitle, xtitle=xtitle, title=title,$
          charthick=charthick, charsize=charsize, $
          /ynozero

; !x.window are the normalized coordinates of the plot region, !d.x_vsize is the number of pixles of the plot device
; !y.window !d.y_vsize dito for y direction
xim_px = !x.window * !d.x_vsize   ; x range of the image in device coordinates (PiXels) 
xd_im_px = xim_px[1]-xim_px[0]    ; pixel size of the image in x direction

yim_px = !y.window * !d.y_vsize   ; y range of the image in device coordinates (PiXels)
yd_im_px = yim_px[1]-yim_px[0]    ; pixel size of the image in y direction 

scl_image = scale_image(image,zrange,startcol,zcol,badcol,nullcol,nulltol,saturate)
;;image skalieren, dabei die "schlechten" Werte rausschmeissen -> in scl_image speichern.
;ixnan = where( ~ finite(image),nixnan )
;if nixnan gt 0 then image[ixnan] = zrange[0] - 100.0
;definiert = image ge zrange[0] and image le zrange[1]
;a_scaled = ( image-zrange(0) ) * (zcol-1) / ( zrange(1)-zrange(0) ) 
;scl_image = ( a_scaled * definiert + (badcol-startcol) * (definiert ne 1) ) + startcol ; da is was faul
;
;;Falls nullcol gesetzt ist, werden Werte, die genau 0 sind mit nullcol markiert
;if n_elements(nullcol) ne 0 then $
;    scl_image = (image eq 0.) * nullcol + (image ne 0.) * scl_image

;scl_image der aktuellen Windowgroesse entsprechend "aufblasen".
if (!d.flags and 1) ne 0 then begin          ; Postscript
    tv, scl_image, xim_px[0], yim_px[0], $
        xsize=xd_im_px, ysize=yd_im_px
endif else begin     	                     ; screen
    tv, $
        congrid( $                           ; congrid inflates the the scale to a two-dim pixle array
            scl_image,$                           ; array to be inflated
            xd_im_px, yd_im_px, $            ; x- and y-size of the pixle array
            interp=keyword_set(interp) $     ; type of fit. 0 = neares neighbor
        ) , $
        xim_px[0]+1, yim_px[0]+1             ; position where the lower left corner should be placed in device coordinates
endelse

; replot the frame as parts of the time and height scale may be covered by the image
plot,[0,0],/nodata, xrange=exrange,yrange=eyrange, $
          ticklen=ticklen,xstyle=xstyle,ystyle=ystyle, $
          position=[[x0im_nrm,y0im_nrm],[x1im_nrm,y1im_nrm]], $
          charthick=charthick,charsize=charsize, $
          /noerase

; save coordinate system. It will we restored after drawing the scale to provide the
; possibility to oplot something after exiting this procedure.
ssx=!x & ssy=!y & ssp=!p

; ================
; draw the scale |
; ================
scale = findgen(1,zcol) + startcol ; two dimensional array containing the image for the scale. 
                                   ; It will be inflated by poly_2d in case of pixle devices.
sz_scale = size(scale)
zscale_x = float(sz_scale[1])
zscale_y = float(sz_scale[2])


;Position (0=left, 1=right) of the SCale window in NoRMalized (to 1) coordinates
x0sc_nrm = 0.89
x1sc_nrm = 0.91
y0sc_nrm = y0im_nrm
y1sc_nrm = y1im_nrm

; define plot area
plot,[0,0], xrange=[0,1],yrange=[0,1], /nodata, /noerase, xstyle=5, ystyle=5, $
    position=[[x0sc_nrm,y0sc_nrm],[x1sc_nrm,y1sc_nrm]]

; !x.window are the normalized coordinates of the plot region, !d.x_vsize is the number of pixles of the plot device
; !y.window !d.y_vsize dito for y direction
xsc_px = !x.window * !d.x_vsize   ; x range of the scale in device coordinates (PiXels) 
xd_sc_px = xsc_px[1]-xsc_px[0]    ; pixel size of the scale in x direction

ysc_px = !y.window * !d.y_vsize   ; y range of the scale in device coordinates (PiXels)
yd_sc_px = ysc_px[1]-ysc_px[0]    ; pixel size of the scale in y direction 

if (!d.flags and 1) ne 0 then begin     ; Postscript (scalable device)
    tv, scale, xsc_px[0], ysc_px[0],$
        xsize=xd_sc_px, ysize=yd_sc_px
endif else begin                        ; screen
    tv,  $
        congrid( $                             ; poly_2d inflates the the scale to a two-dim pixle array
            scale, $                           ; array to be inflated
            xd_sc_px, yd_sc_px,  $             ; x- and y-size of the pixle array
            keyword_set(interp)  $             ; type of fit. 0 = neares neighbor
        ), $
        xsc_px[0]+1, ysc_px[0]+1               ; position where the lower left corner should be placed in device coordinates
endelse

; replot the frame arround the scale as parts of it may be covered
oplot,[0,1],[0,0],/noclip
oplot,[0,0],[0,1],/noclip
oplot,[0,1],[1,1],/noclip
if zcol le 20 then yticks = zcol else yticks=0
axis, ticklen=0, ystyle=1, yaxis=1, yrange=[zrange(0),zrange(1)], yticks=yticks,$
     ytitle=ztitle, /ynozero,charthick=charthick, charsize=charsize

; restore the image coordinate system for later oplots
!x=ssx & !y=ssy & !p=ssp

return
end
