pro mosaik,image,xgon,ygon,xrange=xrange,yrange=yrange, zrange=zrange, $
         zcol, startcol, badcol,charsize=charsize,charthick=charthick,ticklen=ticklen,ztit=ztit,xtit=xtit,ytit=ytit,xstyle=xstyle,ystyle=ystyle,$
         saturate=saturate,nullcol=nullcol,nulltol=nulltol
;
; Make a color coded image from a vector of values with corresponding n-dragons coordinates 
;    image[igon] igon=0,..,zgon-1
;    xgon[igon,icorner],ygon[igon,icorner] carthesian coordinates of the n-dragons which sall be drawn.
;         icorner=0,zcorner-1  zcorner is the number of corners of each n-dragon. The same number of 
;                              corners must be used for all ndragons.
;    xrange,yrange range in which the mosaic should be drawn
;    startcol the number at which the color table starts (typpicaly 30)
;    zcol number of elements in the color table
;    badcol This color should be used for NaN-values
;    nullcol,nulltol  if abs values in image are smaller than nultol then the color is set to nulcol
;    saturate: if bit 0 is set then values smaller than zrange[0] are set to zrange[0]
;          and if bit 1 is set then values biger than zrange[1] are set to zrange[1]
;    ztit=ztit, xtit=xtit,ytit=ytit
;    charsize=charsize,thick=thick,xstyle=xstyle,ystyle=ystyle    some plotting keywords.
;
; Matthias Bauer-Pfundstein
;
    default_value,xstyle,1
    default_value,ystyle,1
    default_value,saturate,3
    default_value,nulltol,0.1

    ; Dimensions of the image to be rendered
    image_size = size(image)
    if image_size[0] ne 1 then message, 'image should be a one dimensional array'
    zgon = n_elements(image)
    zcorner = n_elements(xgon[0,*])
    if n_elements(xgon[*,0]) ne zgon then message, 'xgon[igon,icorner] should consit of one set of x-coordinates per image mosaic'
    if n_elements(ygon[*,0]) ne zgon then message, 'ygon[igon,icorner] should consit of one set of x-coordinates per image mosaic'
    if n_elements(ygon[0,*]) ne zcorner then message,'xgon and ygon must have the same number of corner coordinated'

    default_value,xrange,[min(xgon,max=maxxgon),maxxgon]
    default_value,yrange,[min(ygon,max=maxygon),maxygon]
    default_value,zrange,[min(image,max=maxz),maxz]
    dzra = zrange[1] - zrange[0]
    if  dzra lt 0 then message, 'zrange must be positive'

    ;Positions of the image window in normalized coordinates
    ;x0im_nrm  = 0.33
    x0im_nrm  = 0.07
    x1im_nrm  = 0.87
    y0im_nrm  = 0.1
    y1im_nrm  = 0.9

    ; define the coordinate system for the image plot
    plot,[0,0],/nodata,xrange=xrange,yrange=yrange, $
            ticklen=ticklen,xstyle=xstyle,ystyle=ystyle, $
            position=[[x0im_nrm,y0im_nrm],[x1im_nrm,y1im_nrm]],$
            ytitle=ytit, xtitle=xtit,charthick=charthick, charsize=charsize
    
    ;image skalieren, dabei die "schlechten" Werte rausschmeissen -> in scl_image speichern.
    ixnan = where( ~ finite(image) ,nixnan )
    nonnan = ( finite(image) ) and abs(image-zrange[0]) lt 7.*abs(zrange[1]-zrange[0])
    if nixnan gt 0 then image[ixnan] = zrange[0] - 10000.0
    definiert = image ge zrange[0] and image le zrange[1]
    scl_image = (( image - zrange[0] ) * (zcol-1) / ( zrange[1]-zrange[0] ) + startcol) * definiert + badcol * (definiert ne 1) 
    ;============================PATRICS MODIFIKATION=======================
    if (saturate and 1) ne 0 then begin
        idx=where(image lt zrange[0] and nonnan, count)
        if count gt 0 then scl_image[idx]=startcol+0
    endif
    if (saturate and 2) ne 0 then begin
        idx=where(image gt zrange[1] and nonnan, count)
        if count gt 0 then scl_image[idx]=startcol+zcol-1
    endif
    ;===========================ENDE PATRICS MODIFIKATION===================
    ;Falls nullcol gesetzt ist, werden Werte, die genau 0 sind mit nullcol markiert
    if n_elements(nullcol) ne 0 then begin
        ixnull=where(abs (image) lt nulltol,nixtol)
        if nixnull then scl_image[ixnull] = nullcol
    endif

    for igon=0L,zgon-1 do begin
        if    min(xgon[igon,*],max=maxxgon) ge xrange[0] && maxxgon le xrange[1] $
          &&  min(ygon[igon,*],max=maxygon) ge yrange[0] && maxygon le yrange[1]  then begin
            polyfill, [reform(xgon[igon,*]),xgon[igon,0]], [reform(ygon[igon,*]),ygon[igon,0]] , color=scl_image[igon]
        endif
    endfor


    ; replot the frame arround the image as parts of it may be covered
    plot,[0,0],/nodata,xrange=xrange,yrange=yrange, $
            ticklen=ticklen,xstyle=xstyle,ystyle=ystyle, $
            position=[[x0im_nrm,y0im_nrm],[x1im_nrm,y1im_nrm]],$
            ytitle=xytit, xtitle=xytit,charthick=charthick, charsize=charsize, /noerase

    
    ; save coordinate system. It will we restored after drawing the scale to provide the
    ; possibility to oplot something after exiting this procedure.
    ssx=!x & ssy=!y & ssp=!p

    ; ===================
    ; draw the colorbar |   Das sollte ich auch mal durch die COLORBAR-Routine von D.Fanning ersetzen
    ; ===================
    scale = findgen(1,zcol) + startcol ; two dimensional array containing the image for the scale. 
                                    ; It will be inflated by poly_2d in case of pixle devices.
    sz_scale = size(scale)
    zscale_x = float(sz_scale[1])
    zscale_y = float(sz_scale[2])


    ;Position (0=left, 1=right) of the SCale window in NoRMalized (to 1) coordinates
    x0sc_nrm = 0.89
    x1sc_nrm = 0.91
    y0sc_nrm = y0im_nrm
    y1sc_nrm = y1im_nrm

    ; define plot area
    plot,[0,0], xrange=[0,1],yrange=[0,1], /nodata, /noerase, xstyle=5, ystyle=5, $
        position=[[x0sc_nrm,y0sc_nrm],[x1sc_nrm,y1sc_nrm]]

    ; !x.window are the normalized coordinates of the plot region, !d.x_vsize is the number of pixles of the plot device
    ; !y.window !d.y_vsize dito for y direction
    xsc_px = !x.window * !d.x_vsize   ; x range of the scale in device coordinates (PiXels) 
    xd_sc_px = xsc_px[1]-xsc_px[0]    ; pixel size of the scale in x direction

    ysc_px = !y.window * !d.y_vsize   ; y range of the scale in device coordinates (PiXels)
    yd_sc_px = ysc_px[1]-ysc_px[0]    ; pixel size of the scale in y direction 

    if (!d.flags and 1) ne 0 then begin     ; Postscript (scalable device)
        tv, scale, xsc_px[0], ysc_px[0],$
            xsize=xd_sc_px, ysize=yd_sc_px
    endif else begin                        ; screen
        tv,  $
            congrid( $                             ; poly_2d inflates the the scale to a two-dim pixle array
                scale, $                           ; array to be inflated
                xd_sc_px, yd_sc_px,  $             ; x- and y-size of the pixle array
                keyword_set(interp)  $             ; type of fit. 0 = neares neighbor
            ), $
            xsc_px[0]+1, ysc_px[0]+1               ; position where the lower left corner should be placed in device coordinates
    endelse

    ; replot the frame arround the scale as parts of it may be covered
    oplot,[0,1],[0,0],/noclip
    oplot,[0,0],[0,1],/noclip
    oplot,[0,1],[1,1],/noclip
    if zcol le 20 then yticks = zcol else yticks=0
    axis, ticklen=0, ystyle=1, yaxis=1, yrange=[zrange[0],zrange[1]], yticks=yticks,$
        ytitle=ztit, /ynozero,charthick=charthick, charsize=charsize

    ; restore the image coordinate system for later oplots
    !x=ssx & !y=ssy & !p=ssp

end

