function peak11gc,x,a
; Model function of ground clutter + atmospheric peak for lmfit 
; It returns: 
;       y = C * fftwin[round(abs(x))] + A * exp(-1 * (x-x_0)^2/(2*sigma^2) )
; x = integer x-Values in fft bins e.g. [-3,-2,-1,0,1,2,3,4]
; a[0] = C
; a[1] = A
; a[2] = x_0
; a[3] = 2*sigma^2
;
    common fftwin,fftwin
    if n_elements(fftwin) eq 0 then begin
        fftwin=replicate(0.0,512)
        fftwin[0] = [1.,0.257,0.01]
    endif
    return,[a[0] * fftwin[round(abs(x))] + a[1]/sqrt(!pi*a[3]) * exp(-1 * (x-a[2])^2 / a[3] ),$
               fftwin[round(abs(x))],$
               1./sqrt(!pi*a[3]) * exp(-1 * (x-a[2])^2 / a[3]),$
               2.*(x-a[2])*a[1]/(a[3]*sqrt(!pi*a[3])) * exp(-1 * (x-a[2])^2 / a[3] ),$
               ((x-a[2])^2 - 0.5*a[3]) * a[1]/(a[3]^2*sqrt(!pi*a[3])) * exp(-1 * (x-a[2])^2 / a[3] ) $
           ]
end
