pro plotsioux,u,v,x,y,missing=missing,maxmag=maxmag,oplo=oplo,box=box,sym=sym,color=color,thick=thick,framethick=framethick,charsize=charsize,arrowsize=arrowsize,framecolori=framecolor,rand=rand
;
;+
; NAME:
;	plotsioux
; PURPOSE:
;	Produce a two dimensional velocity field plot.
;		Draw a directed arrow at each point showing
;		the direction and magnitude of the field.
;		
; CALLING SEQUENCE:
;	plotsioux,U,V,X,Y, Missing=max_length , maxmag=15, oplo=0 , box=[1,50] 
; INPUTS:
;	U = The X component of the two dimensional field.  Both
;		U and V must be two dimensional and have the same
;		dimensions.
;	V = The Y component of the two dimensional field.  The
;		vector at point (i,j) has the magnitude of
;		(U[i,j]^2 + V[i,j]^2)^0.5 and a direction of
;		ATAN2(V[i,j],U[i,j]).
; OPTIONAL INPUT PARAMETERS:
;	X = Optional abcissae values, Size of X must = 1st dimension
;		of U and V.  Must be a vector.
;	Y = Optional ordinate values.  Same rules as X.
; Keyword Input Parameters:
;	Missing = missing data value.  (U,V) Vectors with a LENGTH longer
;		than missing are ignored.
;       maxmag = gibt vor welche Pfeill"ange bei length=1 ein Feld
;               ausf"ullt. (damit man bei aufeinanderfolgenden Bildern
;               eine gemeinsame Sklierung vorgeben kann)
;       oplp   != 0  bedeutet es soll ein Oplot durchgef"uhrt werden
;       box    = [xbox,ybox] die Pfeile sollen in einer box liegen.
;                wenn nicht angegeben, dann minimaler Abstand von x bzw y
; SIDE EFFECTS:
;	Plotting on the selected device is performed.  System
;	variables concerning Plotting are changed.
; MODIFICATION HISTORY:
;       1993 Matthias Bauer Windpfeile constanter L"ange U^2+V^2 sollen
;       auf einem Kreis liegen
;-
;
	;on_error,2                      ;Return to caller if an error occurs
	s = size(u)
	t = size(v)
 	if s[0] ne 2 or total(abs(s[0:2]-t[0:2])) ne 0 then begin 
            message, 'U and V parameters must be 2D and same size.'
        endif

	if n_elements(oplo) ne 1 then oplo = 0
        if n_elements(rand) ne 1 then rand = 1.
	if n_elements(x) ne s[1] then begin
            if oplo then begin
                xrs=!x.CRANGE & dxrs=(xrs[1]-xrs[0])/float(s[1])
                x = orgel(s[1],xrs+[+0.5,-0.5]*dxrs*rand)
            endif else x = findgen(s[1]) 
	endif
	if n_elements(y) ne s[2] then begin
            if oplo then begin
                yrs=!y.CRANGE & dyrs=( yrs[1]-yrs[0] )/float(s[2])
                y = orgel(s[2],yrs+[+0.5,-0.5]*dyrs*rand)
            endif else y = findgen(s[2]) 
	endif
	nx = n_elements(x)
	ny = n_elements(y)

	if n_elements(sym) ne 1 then sym = 0
        if n_elements(color) ne 1 then begin
            if !d.name eq 'X' or !d.name eq 'WIN' then color=255 else color=0
        endif
	if n_elements(thick) ne 1 then thick=1.2
	if n_elements(framethick) ne 1 then framethick=1.2
	if n_elements(charsize) ne 1 then charsize=1.0
	if n_elements(charthick) ne 1 then charthick=1.0
	if n_elements(arrowsize) ne 1 then arrowsize=1.0
        if n_elements(framecolor) ne 1 then begin
            if !d.name eq 'X' or !d.name eq 'WIN' then framecolor=255 else framecolor=0
        endif
	if n_elements(box) ne 2 then begin  
            box=fltarr(2)
	    box[0] = min(x[1:nx-1]-x[0:nx-2])
	    box[1] = min(y[1:ny-1]-y[0:ny-2])
	endif

	if oplo eq 0 then begin
	    x0 = min(x,max=x1)
	    y0 = min(y,max=y1)
	    plot,[x0,x1],[y1,y0],/nodata,/xst,/yst,charsize=charsize,$
		charthick=framethick,color=framecolor
	    plots,[!x.crange[0],!x.crange[0],!x.crange[1],!x.crange[1],!x.crange[0]],[!y.crange[0],!y.crange[1],!y.crange[1],!y.crange[0],!y.crange[0]],thick=thick,color=framecolor
	endif

	mag = sqrt(double(u)^2+double(v)^2)
	if n_elements(missing) eq 1 then begin
		good = where(mag lt missing) 
	endif else begin
		good = lindgen(s[1],s[2])
	endelse

        if good[0,0] ge 0 then begin  
	    if n_elements(maxmag) ne 1 then maxmag = max(mag[good])

	    ugood = u[good]
	    vgood = v[good]
	    xgood = x[good mod s[1]]
	    ygood = y[good / s[1]]

	    plotpfeile,ugood,vgood,xgood,ygood,maxmag,box,sym=sym,color=color,thick=thick,arrowsize=arrowsize
        endif 
end
