pro polar2image,image,radsl,radsr,fisl,fisr,xrange=xrange,yrange=yrange,scalefacx=scalefacx,scalefacy=scalefacy, zrange=zrange, $
         zcol, startcol, badcol,charsize=charsize,charthick=charthick,ticklen=ticklen,ztit=ztit,xtit=xtit,ytit=ytit,tit=tit,xstyle=xstyle,ystyle=ystyle,$
         saturate=saturate,nullcol=nullcol,nulltol=nulltol,dfimax=dfimax,ringr=ringr,ringname=ringname,prepend2heitit=prepend2heitit,oplo=oplo
;
; Plot RHI or PPI scans.
;    image[rg,ti] rg=0..zrg-1 number of range gates, ti=0..zti-1 number of data chunks in the selected time range
;    radsl[rg],radsr[rg] lower and upper bound of the range covered by each range gate
;    fisl[ti],fisr[ti]   left and right bound of the angle (elevation or arumuth) which where covered dureing the measuring time of each chunk of data
;    xrange,yrange range in which the polar diagrams should be drawn
;    startcol the number at which the color table starts (typpicaly 30)
;    zcol number of elements in the color table
;    badcol This color should be used for NaN-values
;    nullcol,nulltol  if abs values in image are smaller than nulltol then the color is set to nulcol
;    saturate: if bit 0 is set then values smaller than zrange[0] are set to zrange[0]
;          and if bit 1 is set then values biger than zrange[1] are set to zrange[1]
;    ztit=ztit,xtit=xtit,ytit=ytit,rtit=rtit
;    charsize=charsize,thick=thick,xstyle=xstyle,ystyle=ystyle    some plotting keywords.
;
; A method for gridding data related to sperical coordinates which is not used below:
;ix=where((elv-ym) le 90 and  abs(azic) le 90 )
;xx=sin(!pi/180.*azic[ix])*cos(!pi/180.*(elv[ix]-ym))
;yy=sin(!pi/180.*(elv[ix]-ym)) & pco=pcoa[ix]
;TRIANGULATE, xx, yy, trp
;pcog=griddata(xx,yy,pco,dimen=[400,200],delta=[1,1.04]/200.,start=[-1,-0.04]-[1,1.04]/400.,method="NearestNeighbor",TRIANGLES=trp)
;mimage,pcog,[-1,1],[-0.04,1],[-20,60],zcolor, startcolor,badcol,ztit='dB',heitit='Elevation [deg]',charsi=1.5
;WRITE_PNG, fn+'.Co_vol.png', TVRD(true=1)
;
; Matthias Bauer-Pfundstein
;
common polarimagexyp,ssx,ssy,ssp,zzora
    zzoramax=10
    if n_elements(zzora) ne 1 then begin
        ssx=ptrarr(zzoramax)
        ssy=ptrarr(zzoramax)
        ssp=ptrarr(zzoramax)
        zzora=0
    endif
    default_value,xstyle,1
    default_value,ystyle,1
    default_value,saturate,2
    default_value,nulltol,0.1
    default_value,dfimax,15.
    default_value,tit,''
    nrings=n_elements(ringr)
    if nrings gt 0 then dumi=where(ringr gt 0.0,nrings)
    default_value,ticklen, 0.007
    maxrad=max(radsl)
    default_value,xrange,[-maxrad,maxrad]; *scalefacx
    xra=xrange*scalefacx
    default_value,zrange,[min(image,max=maxz),maxz]
    zzora = n_elements(yrange)/2 ; number of zoom ranges
    if zzora gt zzoramax then begin
        print,"yrange may not have more than 2*",zzoramax,' Dimensions'
        return
    endif
    yzora=reform(yrange,2,zzora)
    dummi = where(yzora[1,*]-yzora[0,*] gt 0.0,zzora)
    if zzora eq 0 then begin
        yzora[*,0]=[-maxrad,maxrad]
        zzora = 1
    endif
    default_value,prepend2heitit,replicate('',zzora)
    default_value,oplo,0

    ; Dimensions of the data to be rendered
    image_size = size(image)
    if image_size[0] lt 2 then message, 'image not 2D'
    ztime = long(image_size[1])
    zrg = long(image_size[2])
    if n_elements(radsl) ne n_elements(radsr) or n_elements(radsr) ne zrg then begin
        print,"radal, radsr, and image[*,0] must have the same dimensions"
        return
    endif
    if n_elements(fisl) ne n_elements(fisr) or n_elements(fisr) ne ztime then begin
        print,"fisl, fisr, and image[0,*] must have the same dimensions"
        return
    endif

    ymgbot=[0.2,4.2] & ymgtop=[0.2,3.2] ; the first element is for the zooms which are nnoott in the bot or tob respectively, the second element is for zoom which aarree in the bot/tob
    xmg=[10,13]
    if oplo eq 0 then erase
    for izora=0,zzora-1 do begin
        bot = izora eq 0
        tob = izora eq zzora-1 
        yra = yzora[*,izora]*scalefacy

        if oplo eq 0 then begin
            ; define the coordinate system for the image plot
            !p.multi=[zzora-izora-1,1,zzora]
            if bot then begin 
                xtit_=xtit 
                xtickfmt_=''
            endif else begin
                xtit_=''
                xtickfmt_='(A1)'
            endelse
            ytit_=prepend2heitit[izora]+' '+ytit
            ticklenx=ticklen/abs(!y.window[1]-!y.window[0]) & tickleny=ticklen/abs(!x.window[1]-!x.window[0])
            ; define the coordinate system
            plot,[0,0],/nodata,xrange=xra,yrange=yra, $
                    xstyle=xstyle,ystyle=ystyle, $
                    ymargin=[ymgbot[bot],ymgtop[tob]],xmargin=[xmg],$
                    ytitle=ytit_, xtitle=xtit_,xtickformat=xtickfmt_, charthick=charthick, charsize=charsize,$
                    /noerase,xticklen=ticklenx,yticklen=tickleny
        endif else begin
            !x=*(ssx[izora]) & !y=*(ssy[izora]) & !p=*(ssp[izora])
        endelse

        ; Draw the sectors 
        scl_image = scale_image(image,zrange,startcol,zcol,badcol,nullcol,nulltol,saturate)
        for iti=0L,ztime-1 do begin
            zwiebel_fi  = [  fisr[iti],  fisl[iti],  fisl[iti],  fisr[iti]]
            dfi = abs(fisr[iti] - cmod(fisl[iti],360.,fisr[iti]))
            if dfi le dfimax then begin
                for irg=0L,zrg-1 do begin
                    zwiebel_rad = [ radsr[irg], radsr[irg], radsl[irg], radsl[irg]]
                    zwiebelx = zwiebel_rad * sin(zwiebel_fi*!PI/180.) * scalefacx
                    zwiebely = zwiebel_rad * cos(zwiebel_fi*!PI/180.) * scalefacy
                    ix=where(zwiebelx lt xra[0],nix)
                    if nix gt 0 then zwiebelx[ix]=xra[0] 
                    if nix lt 4 then begin
                        ix=where(zwiebelx gt xra[1],nix)
                        if nix gt 0 then zwiebelx[ix]=xra[1]
                        if nix lt 4 then begin
                            ix=where(zwiebely lt yra[0],nix)
                            if nix gt 0 then zwiebely[ix]=yra[0]
                            if nix lt 4 then begin
                                ix=where(zwiebely gt yra[1],nix)
                                if nix gt 0 then zwiebely[ix]=yra[1]
                                if nix lt 4 then begin
                                    if oplo eq 0 || scl_image[iti,irg] ne badcol then polyfill, zwiebelx , zwiebely , color=scl_image[iti,irg]
                                endif
                            endif
                        endif
                    endif
                endfor
            endif
        endfor

        if nrings gt 0 then begin
            neck=72 & th=2*!pi*findgen(neck+1)/float(neck)
            ringcol=255
            for iring=0,nrings-1 do begin
                oplot,ringr[iring]*sin(th)*scalefacx,ringr[iring]*cos(th)*scalefacy,color=ringcol
                xyouts,0.0,ringr[iring]*scalefacy+(yra[1]-yra[0])*0.02,ringname[iring],chars=charsize,alignment=0.5,color=ringcol
            endfor
        endif
        if oplo eq 0 then begin

            ; replot the frame arround the image as parts of it may be covered
            plot,[0,0],/nodata,xrange=xra,yrange=yra, $
                    xstyle=xstyle,ystyle=ystyle, $
                    ymargin=[ymgbot[bot],ymgtop[tob]],xmargin=[xmg],$
                    ytitle=ytit_, xtitle=xtit_,xtickformat=xtickfmt_, charthick=charthick, charsize=charsize,$
                    /noerase,xticklen=ticklenx,yticklen=tickleny
            
            if bot then begin
                botplotcorner=[!x.window[1],!y.window[0]]
                zanotationspace=!x.window[0]
            endif
            if tob then begin
                topplotcorner=[!x.window[1],!y.window[1]]
            endif

            ; save coordinate system. It will we restored after drawing the scale to provide the
            ; possibility to oplot something after exiting this procedure.
            ssx[izora]=ptr_new(!x) & ssy[izora]=ptr_new(!y) & ssp[izora]=ptr_new(!p)
        endif
    endfor ; izora

    if oplo eq 0 then begin
        ; Add the title
        if n_elements(tit) gt 0 then xyouts,0.49,topplotcorner[1]+0.018,tit, /noclip,alignment=0.5,/normal,chars=charsize

        ; ===================
        ; draw the colorbar |
        ; ===================
        scale = findgen(1,zcol) + startcol ; two dimensional array containing the image for the scale. It will be inflated by poly_2d in case of pixle devices.
        sz_scale = size(scale) & zscale_x = float(sz_scale[1]) & zscale_y = float(sz_scale[2])

        ;Position (0=left, 1=right) of the SCale window in NoRMalized (to 1) coordinates
        x0sc_nrm = botplotcorner[0]+0.009
        x1sc_nrm = 1.0-zanotationspace*0.97
        y0sc_nrm = botplotcorner[1]+zanotationspace
        y1sc_nrm = topplotcorner[1]-zanotationspace

        ; define plot area
        plot,[0,0], xrange=[0,1],yrange=[0,1], /nodata, /noerase, xstyle=5, ystyle=5, $
            position=[[x0sc_nrm,y0sc_nrm],[x1sc_nrm,y1sc_nrm]]

        ; !x.window are the normalized coordinates of the plot region, !d.x_vsize is the number of pixles of the plot device
        ; !y.window !d.y_vsize dito for y direction
        xsc_px = !x.window * !d.x_vsize   ; x range of the scale in device coordinates (PiXels) 
        xd_sc_px = xsc_px[1]-xsc_px[0]    ; pixel size of the scale in x direction

        ysc_px = !y.window * !d.y_vsize   ; y range of the scale in device coordinates (PiXels)
        yd_sc_px = ysc_px[1]-ysc_px[0]    ; pixel size of the scale in y direction 

        if (!d.flags and 1) ne 0 then begin     ; Postscript (scalable device)
            tv, scale, xsc_px[0], ysc_px[0],$
                xsize=xd_sc_px, ysize=yd_sc_px
        endif else begin                        ; screen
            tv,  $
                congrid( $                             ; poly_2d inflates the the scale to a two-dim pixle array
                    scale, $                           ; array to be inflated
                    xd_sc_px, yd_sc_px,  $             ; x- and y-size of the pixle array
                    keyword_set(interp)  $             ; type of fit. 0 = neares neighbor
                ), $
                xsc_px[0]+1, ysc_px[0]+1               ; position where the lower left corner should be placed in device coordinates
        endelse

        ; replot the frame arround the scale as parts of it may be covered
        oplot,[0,1],[0,0],/noclip
        oplot,[0,0],[0,1],/noclip
        oplot,[0,1],[1,1],/noclip
        if zcol le 20 then begin
            axis, ticklen=ticklen, ystyle=1, yaxis=1, yrange=[zrange[0],zrange[1]], yticks=zcol,$
                ytitle=ztit, /ynozero,charthick=charthick, charsize=charsize
        endif else begin
            axis, ticklen=0, ystyle=1, yaxis=1, yrange=[zrange[0],zrange[1]],$
                ytitle=ztit, /ynozero,charthick=charthick, charsize=charsize
        endelse

        ; restore the image coordinate system for later oplots. If multible zoom ranges are defined then it restores to the las plot.
    endif
    !x=*(ssx[zzora-1]) & !y=*(ssy[zzora-1]) & !p=*(ssp[zzora-1])
end
