pro profseq22image, image, x_links,x_rechts,y_bot,y_top,xrange=xrange, yrange=yrange, $
           zrange, zcol, startcol, badcol, nullcol=nullcol,nulltol=nulltol, saturate=saturate, $
           tollerable_tgap=tollerable_tgap,tollerable_zgap=tollerable_zgap ,$
           INTERP=interp,  charsize=charsize, charthick=charthick,title=title, $
           ztitle=ztitle, xtitle=xtitle,heitit=heitit,xstyle=xstyle,ystyle=ystyle,ticklen=ticklen,prepend2heitit=prepend2heitit
; plot the two dimensional array image as image with its values mapped to the colours
; The values which lie in zrange are mapped to the color range defined by startcol and zcol.
; A color bar is added which helps to reassign which value is visualized by which color.
;
; In oposite to mimage the x-axis values (usually the time) are not assumed to be linearly growing.
; Each profile image[*,itime] is plottet at the time range from x_links[itime] to x_rechts[itime].
; The difference between these two values is usually the average time. 
; It may be different from profile to profile.
; 
; Each profile may be plottet at a different height range from y_bot to y_top. This is useful 
; if the profiles measured with different elevations should be plottet with correctly in height.
; 
;    zcol number of elements in the color table
;    badcol This color should be used for NaN-values
;    nullcol,nulltol  if abs values in image are smaller than nultol then the color is set to nulcol
;    saturate: if bit 0 is set then values smaller than zrange[0] are set to zrange[0]
;          and if bit 1 is set then values biger than zrange[1] are set to zrange[1]
;
; by Karen Ruebenacker and Matthias Bauer-Pfundstein bauer@metek.de
;
common imagexyp,ssx,ssy,ssp,zzora
    zzoramax=10
    if n_elements(zzora) ne 1 then begin
        ssx=ptrarr(zzoramax)
        ssy=ptrarr(zzoramax)
        ssp=ptrarr(zzoramax)
        zzora=0
    endif
    default_value,xstyle,1
    default_value,ystyle,1
    default_value,saturate,2
    default_value,nulltol,0.1
    default_value,ticklen, 0.006
    zzora = n_elements(yrange)/2 ; number of zoom ranges
    if zzora gt zzoramax then begin
        print,"yrange may not have more than 2*",zzoramax,' Dimensions'
        return
    endif
    yzora=reform(yrange,2,zzora)
    dummi = where(yzora[0,*] ge 0.0,nyrangegt0)
    zzora=nyrangegt0

    default_value,heitit,'Range'
    default_value,prepend2heitit,replicate('',zzora)

    ; Dimensions of the image to be rendered
    image_size = size(image)
    if image_size[0] lt 2 then begin 
        message, 'image not 2D'
        return
    endif
    ztime = float(image_size[1])
    zrg = float(image_size[2])
    if n_elements(x_rechts) ne n_elements(x_links) or n_elements(x_rechts) ne ztime then begin
        print,"x_links, x_rechts, and image[0,*] must have the same dimensions"
        return
    endif

    case n_elements(y_bot) of 
        ztime:y_bo=y_bot
        1:y_bo=replicate(y_bot,ztime)
        else: begin
            print,"y_bot should have the same number of elements as the x dimention of the data plotted."
        end
    endcase

    case n_elements(y_top) of 
        ztime:y_to=y_top
        1:y_to=replicate(y_top,ztime)
        else: begin
            print,"y_top should have the same number of elements as the x dimension of the data plotted."
        end
    endcase

    default_value,xrange,[ min([x_links,x_rechts], max=maxx), maxx ]
    dxrange=xrange[1]-xrange[0]

    if n_elements(tollerable_tgap) ne 1 then tollerable_tgap=2.6*min(x_rechts-x_links)
    if n_elements(tollerable_zgap) ne 1 then tollerable_zgap=1.6*min(y_to-y_bo)/float(zrg)
    ave = x_rechts-x_links
    if n_elements(tollerable_avecha) ne 1 then tollerable_avecha=min(ave)*0.1

    ; find time ranges with continuous data
    ixp0=lindgen(ztime-1) & ixp1=ixp0+1L
    ix_discont=where(abs(x_rechts[ixp0]-x_links[ixp1]) gt tollerable_tgap $
                or abs(y_bo[ixp0]-y_bo[ixp1]) gt tollerable_zgap  $
                or abs(y_to[ixp0]-y_to[ixp1]) gt tollerable_zgap  $
                or abs(ave[ixp0]-ave[ixp1]) gt tollerable_avecha $
            ,nix_discont)

    npieces=nix_discont+1
    if nix_discont gt 0L then begin
        ixpiece0=[0L,ix_discont+1]
        ixpiece1=[ix_discont,ztime-1]
    endif else begin
        ixpiece0=[0L]
        ixpiece1=[ztime-1]
    endelse

    ; ================
    ; Draw the image |
    ; ================

    ymgbot=[0.18,4.2] & ymgtop=[0.18,3.2] ; the first element is for the zooms which are nnoott in the bot or tob respectively, the second element is for zoom which aarree in the bot/tob
    xmg=[10,13]
    erase
    for izora=0,zzora-1 do begin
        bot = izora eq 0
        tob = izora eq zzora-1 
        
        if n_elements(yzora[*,izora]) ne 2 || yzora[1,izora] le yzora[0,izora] then begin
            yrange_=[min([y_bo-0.5*(y_to-y_bo)/zrg,y_to+0.5*(y_to-y_bo)/zrg],max=maxx),maxx] 
            ; The term 0.5*(..-..)/zrg provides that the pixels at the top and bottom are not squezed to 1/2 of 
            ; the size of the other pixels.
        endif else yrange_=yzora[*,izora]
        dyrange = yrange_[1] - yrange_[0]

        ; define the coordinate system for the image plot
        !p.multi=[zzora-izora-1,1,zzora]
        if bot then begin 
            xtitle_=xtitle 
        endif else begin
            xtitle_=''
            dummy = LABEL_DATE(DATE_FORMAT=' ')
        endelse
        heitit_=prepend2heitit[izora]+' '+heitit
        plot,[0,0],/nodata,xrange=xrange,yrange=yrange_, $
                xstyle=xstyle,ystyle=ystyle, $
                ymargin=[ymgbot[bot],ymgtop[tob]],xmargin=[xmg],$
                ytitle=heitit_, xtitle=xtitle_, charthick=charthick, charsize=charsize, $
                /noerase,xticklen=ticklen,yticklen=ticklen

        scl_image = scale_image(image,zrange,startcol,zcol,badcol,nullcol,nulltol,saturate)

        ; !x.window are the normalized coordinates of the plot region, !d.x_vsize is the number of pixles of the plot device
        ; !y.window !d.y_vsize dito for y direction
        img_xpos_pix = !x.window * !d.x_vsize   ; x range of the image in device coordinates (PiXels) 
        dimg_xpos_pix = img_xpos_pix[1]-img_xpos_pix[0]    ; pixel size of the image in x direction
        img_ypos_pix = !y.window * !d.y_vsize   ; y range of the image in device coordinates (PiXels)
        dimg_ypos_pix = img_ypos_pix[1]-img_ypos_pix[0]    ; pixel size of the image in y direction 

        for ipiece=0,npieces-1 do begin
            ixpi0 = ixpiece0[ipiece] ; pointing to first index of the contiguous piece
            ixpi1 = ixpiece1[ipiece] ; pointing to last index of the contiguous piece
            ztp = ixpi1-ixpi0+1 ; number of profiles belonging to this contigous piece 
            x_links_pix = (x_links[ixpi0]-xrange[0]) / dxrange * dimg_xpos_pix + img_xpos_pix[0] ; left bound of piece in dev coord
            x_rechts_pix = (x_rechts[ixpi1]-xrange[0]) / dxrange * dimg_xpos_pix + img_xpos_pix[0] ; right bound of piece in dev coord
            x_rml_pix = x_rechts_pix - x_links_pix + 1 ; x diameter in dev coordinates
            y_bo_ = total(y_bo[ixpi0:ixpi1])/float(ztp)  ; the image of the profiles of this piece should be plotted
            y_to_ = total(y_to[ixpi0:ixpi1])/float(ztp)  ; between y_bo_ and y_to_, which may be a subset of yrange
            hei=y_bo_+(y_to_-y_bo_)*findgen(zrg)/(zrg-1)
            ixinyra = where( hei gt yrange_[0] and hei lt yrange_[1],nixinyra) 
            if nixinyra gt 0 then begin
                if hei[ixinyra[         0]] lt hei[ixinyra[nixinyra-1]] then begin
                    y_bo_ = hei[ixinyra[         0]]
                    y_to_ = hei[ixinyra[nixinyra-1]]
                endif else begin
                    y_to_ = hei[ixinyra[         0]]
                    y_bo_ = hei[ixinyra[nixinyra-1]]
                    ixinyra=reverse(ixinyra)
                endelse
                y_bo_pix = (y_bo_-yrange_[0])/dyrange * dimg_ypos_pix + img_ypos_pix[0]
                y_to_pix = (y_to_-yrange_[0])/dyrange * dimg_ypos_pix + img_ypos_pix[0]
                y_tmb_pix = y_to_pix - y_bo_pix + 1

                ;scl_image der aktuellen Windowgroesse entsprechend "aufblasen".
                if (!d.flags and 1) ne 0 then begin          ; Postscript
                    tv, scl_image[ixpi0:ixpi1,ixinyra], x_links_pix, y_bo_pix, $
                        xsize=x_rml_pix, ysize=y_tmb_pix
                endif else begin     	                     ; screen
                    tv, $
                        congrid( $                           ; congrid inflates the the scale to a two-dim pixle array
                            scl_image[ixpi0:ixpi1,ixinyra],$       ; array to be inflated
                            x_rml_pix, y_tmb_pix, $          ; x- and y-size of the pixle array
                            interp=keyword_set(interp) $     ; type of fit. 0 = neares neighbor
                        ) , $
                        x_links_pix, y_bo_pix ; position where the lower left corner should be placed in device coordinates
                endelse
            endif
        endfor

        ; replot the frame as parts of the time and height scale may be covered by the image
        !p.multi=[zzora-izora-1,1,zzora]
        ticklenx=ticklen/abs(!y.window[1]-!y.window[0]) & tickleny=ticklen/abs(!x.window[1]-!x.window[0])
        plot,[0,0],/nodata,xrange=xrange,yrange=yrange_, $
                xstyle=xstyle,ystyle=ystyle, $
                ymargin=[ymgbot[bot],ymgtop[tob]],xmargin=[xmg],$
                ytitle=heitit_, xtitle=xtitle_, charthick=charthick, charsize=charsize, $
                /noerase,xticklen=ticklenx,yticklen=tickleny

        if bot then begin
            botplotcorner=[!x.window[1],!y.window[0]]
            zanotationspace=!x.window[0]
        endif
        if tob then begin
            topplotcorner=[!x.window[1],!y.window[1]]
        endif

        ; save coordinate system. It will we restored after drawing the scale to provide the
        ; possibility to oplot something after exiting this procedure.
        ssx[izora]=ptr_new(!x) & ssy[izora]=ptr_new(!y) & ssp[izora]=ptr_new(!p)
    endfor

    ; Add the title
    if n_elements(title) gt 0 then xyouts,0.49,topplotcorner[1]+0.018,title, /noclip,alignment=0.5,/normal,chars=charsize

    ; ===================
    ; draw the colorbar |  Das sollte ich auch mal durch die COLORBAR-Routine von D.Fanning ersetzen
    ; ===================
    scale = findgen(1,zcol) + startcol ; two dimensional array containing the image for the scale. 
                                    ; It will be inflated by poly_2d in case of pixle devices.

    ;Position (0=left, 1=right) of the SCale window in NoRMalized (to 1) coordinates
    x0sc_nrm = botplotcorner[0]+0.009
    x1sc_nrm = 1.0-zanotationspace*0.97
    y0sc_nrm = botplotcorner[1]+zanotationspace
    y1sc_nrm = topplotcorner[1]-zanotationspace

    ; define plot area
    plot,[0,0], xrange=[0,1],yrange=[0,1], /nodata, /noerase, xstyle=5, ystyle=5, $
        position=[[x0sc_nrm,y0sc_nrm],[x1sc_nrm,y1sc_nrm]]

    ; !x.window are the normalized coordinates of the plot region, !d.x_vsize is the number of pixles of the plot device
    ; !y.window !d.y_vsize dito for y direction
    xsc_px = !x.window * !d.x_vsize   ; x range of the scale in device coordinates (PiXels) 
    xd_sc_px = xsc_px[1]-xsc_px[0]    ; pixel size of the scale in x direction

    ysc_px = !y.window * !d.y_vsize   ; y range of the scale in device coordinates (PiXels)
    yd_sc_px = ysc_px[1]-ysc_px[0]    ; pixel size of the scale in y direction 

    if (!d.flags and 1) ne 0 then begin     ; Postscript (scalable device)
        tv, scale, xsc_px[0], ysc_px[0],$
            xsize=xd_sc_px, ysize=yd_sc_px
    endif else begin                        ; screen
        tv,  $
            congrid( $                             ; poly_2d inflates the the scale to a two-dim pixle array
                scale, $                           ; array to be inflated
                xd_sc_px, yd_sc_px,  $             ; x- and y-size of the pixle array
                keyword_set(interp)  $             ; type of fit. 0 = neares neighbor
            ), $
            xsc_px[0]+1, ysc_px[0]+1               ; position where the lower left corner should be placed in device coordinates
    endelse

    ; replot the frame arround the scale as parts of it may be covered
    oplot,[0,1],[0,0],/noclip
    oplot,[0,0],[0,1],/noclip
    oplot,[0,1],[1,1],/noclip
    if zcol le 20 then yticks = zcol else yticks=0
    axis, ticklen=0, ystyle=1, yaxis=1, yrange=[zrange[0],zrange[1]], yticks=yticks,$
        ytitle=ztitle, /ynozero,charthick=charthick, charsize=charsize

    ; restore the image coordinate system for later oplots
    !x=*(ssx[zzora-1]) & !y=*(ssy[zzora-1]) & !p=*(ssp[zzora-1])
end
