pro profseq2image, image, x_links,x_rechts,y_bot,y_top,xrange=xrange, yrange=yrange, $
           zrange, zcol, startcol, badcol, nullcol=nullcol,nulltol=nulltol, saturate=saturate, $
           tollerable_tgap=tollerable_tgap,tollerable_zgap=tollerable_zgap ,$
           INTERP=interp,  charsize=charsize, thick=thick,title=title, $
           ztitle=ztitle, xtitle=xtitle,heitit=heitit,xstyle=xstyle,ystyle=ystyle
; plot the two dimensional array image as image with its values mapped to the colours
; The values which lie in zrange are mapped to the color range defined by startcol and zcol.
; A color bar is added which helps to reassign which value is visualized by which color.
;
; In oposite to mimage the x-axis values (usually the time) are not assumed to be linearly growing.
; Each profile image[*,itime] is plottet at the time range from x_links[itime] to x_rechts[itime].
; The difference between these two values is usually the average time. 
; It may be different from profile to profile.
; 
; Each profile may be plottet at a different height range from y_bot to y_top. This is useful 
; if the profiles measured with different elevations should be plottet with correctly in height.
; 
;    zcol number of elements in the color table
;    badcol This color should be used for NaN-values
;    nullcol,nulltol  if abs values in image are smaller than nultol then the color is set to nulcol
;    saturate: if bit 0 is set then values smaller than zrange[0] are set to zrange[0]
;          and if bit 1 is set then values biger than zrange[1] are set to zrange[1]
;
; by Karen Ruebenacker and Matthias Bauer-Pfundstein bauer@metek.de
;
    default_value,xstyle,1
    default_value,ystyle,1
    default_value,saturate,2
    default_value,nulltol,0.1
    ticklen=-0.01

    default_value,heitit,'Range'

    ; Dimensions of the image to be rendered
    image_size = size(image)
    if image_size[0] lt 2 then begin 
        message, 'image not 2D'
        return
    endif
    ztime = float(image_size[1])
    zrg = float(image_size[2])
    if n_elements(x_rechts) ne n_elements(x_links) or n_elements(x_rechts) ne ztime then begin
        print,"x_links, x_rechts, and image[0,*] must have the same dimensions"
        return
    endif

    case n_elements(y_bot) of 
        ztime:y_bo=y_bot
        1:y_bo=replicate(y_bot,ztime)
        else: begin
            print,"y_bot should have the same number of elements as the x dimention of the data plotted."
        end
    endcase

    case n_elements(y_top) of 
        ztime:y_to=y_top
        1:y_to=replicate(y_top,ztime)
        else: begin
            print,"y_top should have the same number of elements as the x dimension of the data plotted."
        end
    endcase

    default_value,xrange,[ min([x_links,x_rechts], max=maxx), maxx ]
    dxrange=xrange[1]-xrange[0]

    if n_elements(yrange) ne 2 || yrange[1] le yrange[0] then begin
        yrange_=[min([y_bo-0.5*(y_to-y_bo)/zrg,y_to+0.5*(y_to-y_bo)/zrg],max=maxx),maxx] 
        ; The term 0.5*(..-..)/zrg provides that the pixels at the top and bottom are not squezed to 1/2 of 
        ; the size of the other pixels.
    endif else yrange_=yrange
    dyrange = yrange_[1] - yrange_[0]

    if n_elements(tollerable_tgap) ne 1 then tollerable_tgap=2.6*min(x_rechts-x_links)
    if n_elements(tollerable_zgap) ne 1 then tollerable_zgap=1.6*min(y_to-y_bo)/float(zrg)
    ave = x_rechts-x_links
    if n_elements(tollerable_avecha) ne 1 then tollerable_avecha=min(ave)*0.1

    ; ================
    ; Draw the image |
    ; ================

    ;Positions of the image window in normalized coordinates
    x0im_nrm  = 0.07
    x1im_nrm  = 0.87
    y0im_nrm  = 0.1
    y1im_nrm  = 0.9

    ; define the coordinate system for the image plot
    plot,[0,0],/nodata,xrange=xrange,yrange=yrange_, $
            ticklen=ticklen,xstyle=xstyle,ystyle=ystyle, $
            position=[[x0im_nrm,y0im_nrm],[x1im_nrm,y1im_nrm]],$
            ytitle=heitit, xtitle=xtitle, title=title,$
            charthick=thick, charsize=charsize, $
            /ynozero

    scl_image = scale_image(image,zrange,startcol,zcol,badcol,nullcol,nulltol,saturate)

    ; !x.window are the normalized coordinates of the plot region, !d.x_vsize is the number of pixles of the plot device
    ; !y.window !d.y_vsize dito for y direction
    img_xpos_pix = !x.window * !d.x_vsize   ; x range of the image in device coordinates (PiXels) 
    dimg_xpos_pix = img_xpos_pix[1]-img_xpos_pix[0]    ; pixel size of the image in x direction
    img_ypos_pix = !y.window * !d.y_vsize   ; y range of the image in device coordinates (PiXels)
    dimg_ypos_pix = img_ypos_pix[1]-img_ypos_pix[0]    ; pixel size of the image in y direction 

    ; find time ranges with continuous data
    ixp0=lindgen(ztime-1) & ixp1=ixp0+1L
    ix_discont=where(abs(x_rechts[ixp0]-x_links[ixp1]) gt tollerable_tgap $
                or abs(y_bo[ixp0]-y_bo[ixp1]) gt tollerable_zgap  $
                or abs(y_to[ixp0]-y_to[ixp1]) gt tollerable_zgap  $
                or abs(ave[ixp0]-ave[ixp1]) gt tollerable_avecha $
            ,nix_discont)

    npieces=nix_discont+1
    if nix_discont gt 0L then begin
        ixpiece0=[0L,ix_discont+1]
        ixpiece1=[ix_discont,ztime-1]
    endif else begin
        ixpiece0=[0L]
        ixpiece1=[ztime-1]
    endelse

    for ipiece=0,npieces-1 do begin
        ixpi0 = ixpiece0[ipiece] ; pointing to first index of the contiguous piece
        ixpi1 = ixpiece1[ipiece] ; pointing to last index of the contiguous piece
        ztp = ixpi1-ixpi0+1 ; number of profiles belonging to this contigous piece 
        x_links_pix = (x_links[ixpi0]-xrange[0]) / dxrange * dimg_xpos_pix + img_xpos_pix[0] ; left bound of piece in dev coord
        x_rechts_pix = (x_rechts[ixpi1]-xrange[0]) / dxrange * dimg_xpos_pix + img_xpos_pix[0] ; right bound of piece in dev coord
        x_rml_pix = x_rechts_pix - x_links_pix + 1 ; x diameter in dev coordinates
        y_bo_ = total(y_bo[ixpi0:ixpi1])/float(ztp)  ; the image of the profiles of this piece should be plotted
        y_to_ = total(y_to[ixpi0:ixpi1])/float(ztp)  ; between y_bo_ and y_to_, which may be a subset of yrange
        hei=y_bo_+(y_to_-y_bo_)*findgen(zrg)/(zrg-1)
        ixinyra = where( hei gt yrange_[0] and hei lt yrange_[1],nixinyra) 
        if nixinyra gt 0 then begin
            if hei[ixinyra[         0]] lt hei[ixinyra[nixinyra-1]] then begin
                y_bo_ = hei[ixinyra[         0]]
                y_to_ = hei[ixinyra[nixinyra-1]]
            endif else begin
                y_to_ = hei[ixinyra[         0]]
                y_bo_ = hei[ixinyra[nixinyra-1]]
                ixinyra=reverse(ixinyra)
            endelse
            y_bo_pix = (y_bo_-yrange_[0])/dyrange * dimg_ypos_pix + img_ypos_pix[0]
            y_to_pix = (y_to_-yrange_[0])/dyrange * dimg_ypos_pix + img_ypos_pix[0]
            y_tmb_pix = y_to_pix - y_bo_pix + 1

            ;scl_image der aktuellen Windowgroesse entsprechend "aufblasen".
            if (!d.flags and 1) ne 0 then begin          ; Postscript
                tv, scl_image[ixpi0:ixpi1,ixinyra], x_links_pix, y_bo_pix, $
                    xsize=x_rml_pix, ysize=y_tmb_pix
            endif else begin     	                     ; screen
                tv, $
                    congrid( $                           ; congrid inflates the the scale to a two-dim pixle array
                        scl_image[ixpi0:ixpi1,ixinyra],$       ; array to be inflated
                        x_rml_pix, y_tmb_pix, $          ; x- and y-size of the pixle array
                        interp=keyword_set(interp) $     ; type of fit. 0 = neares neighbor
                    ) , $
                    x_links_pix, y_bo_pix ; position where the lower left corner should be placed in device coordinates
            endelse
        endif
    endfor

    ; replot the frame as parts of the time and height scale may be covered by the image
    plot,[0,0],/nodata, xrange=xrange,yrange=yrange_, $
            ticklen=ticklen,xstyle=xstyle,ystyle=ystyle, $
            position=[[x0im_nrm,y0im_nrm],[x1im_nrm,y1im_nrm]], $
            charthick=thick,charsize=charsize, $
            /noerase

    ; save coordinate system. It will we restored after drawing the scale to provide the
    ; possibility to oplot something after exiting this procedure.
    ssx=!x & ssy=!y & ssp=!p

    ; ===================
    ; draw the colorbar |  Das sollte ich auch mal durch die COLORBAR-Routine von D.Fanning ersetzen
    ; ===================
    scale = findgen(1,zcol) + startcol ; two dimensional array containing the image for the scale. 
                                    ; It will be inflated by poly_2d in case of pixle devices.
    sz_scale = size(scale)
    zscale_x = float(sz_scale[1])
    zscale_y = float(sz_scale[2])


    ;Position (0=left, 1=right) of the SCale window in NoRMalized (to 1) coordinates
    x0sc_nrm = 0.89
    x1sc_nrm = 0.91
    y0sc_nrm = y0im_nrm
    y1sc_nrm = y1im_nrm

    ; define plot area
    plot,[0,0], xrange=[0,1],yrange=[0,1], /nodata, /noerase, xstyle=5, ystyle=5, $
        position=[[x0sc_nrm,y0sc_nrm],[x1sc_nrm,y1sc_nrm]]

    ; !x.window are the normalized coordinates of the plot region, !d.x_vsize is the number of pixles of the plot device
    ; !y.window !d.y_vsize dito for y direction
    xsc_px = !x.window * !d.x_vsize   ; x range of the scale in device coordinates (PiXels) 
    xd_sc_px = xsc_px[1]-xsc_px[0]    ; pixel size of the scale in x direction

    ysc_px = !y.window * !d.y_vsize   ; y range of the scale in device coordinates (PiXels)
    yd_sc_px = ysc_px[1]-ysc_px[0]    ; pixel size of the scale in y direction 

    if (!d.flags and 1) ne 0 then begin     ; Postscript (scalable device)
        tv, scale, xsc_px[0], ysc_px[0],$
            xsize=xd_sc_px, ysize=yd_sc_px
    endif else begin                        ; screen
        tv,  $
            congrid( $                             ; poly_2d inflates the the scale to a two-dim pixle array
                scale, $                           ; array to be inflated
                xd_sc_px, yd_sc_px,  $             ; x- and y-size of the pixle array
                keyword_set(interp)  $             ; type of fit. 0 = neares neighbor
            ), $
            xsc_px[0]+1, ysc_px[0]+1               ; position where the lower left corner should be placed in device coordinates
    endelse

    ; replot the frame arround the scale as parts of it may be covered
    oplot,[0,1],[0,0],/noclip
    oplot,[0,0],[0,1],/noclip
    oplot,[0,1],[1,1],/noclip
    if zcol le 20 then yticks = zcol else yticks=0
    axis, ticklen=0, ystyle=1, yaxis=1, yrange=[zrange[0],zrange[1]], yticks=yticks,$
        ytitle=ztitle, /ynozero,charthick=thick, charsize=charsize

    ; restore the image coordinate system for later oplots
    !x=ssx & !y=ssy & !p=ssp

    return
end
