#!/usr/bin/perl
# Core of mk_detect4system
# 
# The first part performes df and warns if the space on the harddrive is below 15 %
#
# The main part of this scrip checks if all the files given as keyword parameters are being written during the
# last 400 s. 
#
# Matthias Bauer-Pfundstein bauer@metek.de
#
use strict;
use Filesys::Df;
my $ref_home = df("/home");
my $ref_raid = df("/mnt/raid");

my $imd=1;
my $now_is = time; 
my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime($now_is);
my $dati = sprintf "%2.2d.%2.2d.%2.2d %2.2d:%2.2d",$mday,$mon+1,$year%100,$hour,$min;

printf "[LW01] $dati   root 000000 000000 %6d  %% <%1d> root file system including home \n",$ref_home->{per},$ref_home->{per} > 85;
printf "[LW02] $dati   data 000000 000000 %6d  %% <%1d> scsi raid array for data and log\n",$ref_raid->{per},$ref_raid->{per} > 85;

while ( @ARGV && $ARGV[0] =~ /(\w+)=(.*)/ && shift ){
    my $ftype = $1 ; 
    my $fname = $2 ;
    my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks)= stat($fname);
    my $age_s = ($now_is-$mtime);
    my $err;
    if ( $ftype eq 'accu' ){  # obsolete 
        $err = $size  ? 7 : 0;
    } else { 
        $err = ($age_s < 400) ? 0 : 8;
    }
    printf "[MD%2.2d] $dati %6s %5ds %5dh %6d kB <%1d> $fname\n",$imd,$ftype,$age_s % 3600,$age_s/3600,$size/1024,$err;
    $imd++;
}

