function calc_moms_mie,spc,xx,noise4snr,mietab,nue,temp,dmpstru,SNRco
; calculate the three spectral moments from the selected spectral points spc which are at the given positions xx
    nspc = n_elements(spc)
    dmpst = dmpstru

    ; first three moments:
    if spc[0] lt 0.0 then spc[0]=0.
    if spc[nspc-1] lt 0.0 then spc[nspc-1]=0.0
    pwx = total(spc)
    SNRco = pwx/noise4snr
    dmpst.SNR = fix(1000.*alog10(SNRco))
    vlx = total(spc*xx)/pwx
    dmpst.VEL = - vlx
    vvx = total(spc*xx*xx)/pwx
    if nspc gt 1 then begin
        dmpst.RMS = sqrt(vvx - vlx*vlx)
    endif else begin 
        dmpst.RMS = 0.0
    endelse
    
    ; calculate diameter from the terminal fall velocity under the assumption 
    ; 1.) the target is a water dropplet, 2.) the vertical wind velocity is small 
    D = ( -0.0016 * alog( 0.9369 - 0.097087 * ((xx/nue) < 9.25) )) > 0.00016
    mie = mietab.mie7ray[long((D-mietab.DIA0)/mietab.DDIA)<(mietab.NDIA-1),(0L>long((temp-mietab.temp0)/mietab.dtemp))<(mietab.NTEMP-1)]
    SNMi = spc / mie  ; mie-corrected Spectrum
    dmpst.SNM = fix(1000.*alog10(total(SNMi)/noise4snr))
    if vlx gt 0.6 then begin ; do not calculate LWC and RR is the Fallvelocities are too small
        LWCi = SNMi / ((1000.*D)^3)
        LWCx = total(LWCi)/noise4snr
        RRi = LWCi*xx 
        RRx = total(RRi)/noise4snr
        ; if abs(1000.*alog10(LWCx)) gt 10000. or abs(1000.*alog10(RRx)) gt 10000. then stop
        dmpst.LWC = fix(1000.*alog10(LWCx))
        dmpst.RR = fix(1000.*alog10(RRx))
        ; if vlx gt 4.0 and max(xx) gt 8.7 and min(xx) lt 1.8 and dB(SNRco) gt 56 then stop
        ; if vlx gt 1.4 and temp gt -5 and dmpst.LWC/100. gt 120 then stop 
        ;w-error if vlx gt 0.7 and temp gt -5 and temp lt 15 and dmpst.LWC/100. gt 90 then begin
        ;w-error     D = ( -0.0016 * alog( 0.9369 - 0.097087 * (( (xx-.1) /nue) < 9.2) )) > 0.00017
        ;w-error     mie=mietab.mie7ray[long((D-mietab.DIA0)/mietab.DDIA)<(mietab.NDIA-1), $
        ;w-error         (0L>long((temp-mietab.temp0)/mietab.dtemp))<(mietab.NTEMP-1)]
        ;w-error     LWCi = spc / (D^3*mie)
        ;w-error     print,vlx,-dmpst.LWC/100.+db(total(LWCi)/noise4snr),dmpst.LWC/100.,temp
        ;w-error endif
    endif
    
    return,dmpst
end
