pro cocrodp,arguments
; 
; Reads IQ-data from pds files (saved with osc=1) and writes spectra to the output file. 
;
; Calling Sequence: 
;
; unix> idl iq2spc infile=data/20040602_1000.pds outfile=20040602.spc.pdf
;
; IDL > iq2spc, 'infile=data/20040602_1000.pds outfile=xxx_off.pdf'
; 
; infile      File to read the pds data from. Multible files with contain successive 
;             (in time) data may be given as array of strings.
;
; outfile     File for writing the output data.
;
; See the structure pph below for a deskription of the other keyword parameters.
;
; Matthias Bauer-Pfundstein bauer@metek.de
;
common startup,startup_done
common magic4chs,magic4chs,FzkPdsFmt,syspar
if n_elements(startup_done) ne 1 then begin
@startup
endif

; Arguments can be provided by two methods:
; - If iq2spc is called by an interactive IDL session the first (and only) 
; string parameter arguments which contains a space separated list is split to
; the strarr argv.
; - Aditionally strings contained in the environment variables ARG1, ARG2, ..., 
; ARG9, ARG10, ... are prepended to argv. The script idl packs its calling arguments
; these environment variables. This is done as IDL can not extract the shell 
; variables like $1, $2,...
argv = env2argv(arguments)

kp = { $
    infile:'', $
    gziped:-1 , $
    iam:'nobody', $   ; radar name, may be specified to evaluate data from another MIRA36
    config:'', $
    outfile:'' $
}
argv2stru,argv,kp
kp.iam = whoami(kp.iam)

; Default values for the Ploting parameters. These Parameters can be changed by the command "set"
pp={ $
phase:0.0,$ ; parameter for improving DP
dummy:0$
}

config2stru,pp,'cocrodp',[kp.iam+'.config',kp.config]
argv2stru,argv,pp


get_lun,outlun & openw,outlun,kp.outfile
rgstop=30

zchan=2L ; number of channels
zrg = -1L
nfft=0L  ; nfft of the spectra in momory. If hdr.nfft differs then a new buffer for noisspectra is created.
zspectra = 0L
skipi=0L

if kp.infile eq 'data' then begin
    zfiles=1
    infiles=[kp.infile]
endif else begin
    infiles = findfile(kp.infile)
    infiles_oldsyntax = strsplit(kp.infile,':',/extract)
    if n_elements(infiles_oldsyntax) gt n_elements(infiles) then infiles=infiles_oldsyntax
    if infiles[0] eq '' then begin
        print,'infile not found: ' + kp.infile
        goto,ende
    endif else zfiles = n_elements(infiles)
endelse
for ifiles=0,zfiles-1 do begin
    ; open the pds file and read the fileheader
    if infiles[ifiles] eq 'data' then begin
        spawn,"tail_s2m 2 /data/\*.pds",unit=lun
        online = 1
    endif else begin
        if kp.gziped ge 0 then gziped=kp.gziped else begin
            if strpos(infiles[ifiles],'.zspc') eq strlen(infiles[ifiles])-5 or $ 
               strpos(infiles[ifiles],'.zspca') eq strlen(infiles[ifiles])-6 or $
               strpos(infiles[ifiles],'.gz')  eq strlen(infiles[ifiles])-3       then gziped=1 else gziped=0
        endelse
        openr,lun,infiles[ifiles],/get_lun,compress=gziped
        online = 0
    endelse
    ; read obsolete header which is in the first 1024 bytes of each file
    ppar = read_hdr(lun,hdr)
    ; write_hdr,outlun, ppar,1

    while not eof(lun) do begin
        ; read one dwell of iq-data
        while read_mainframe_header(lun, hdr) eq 0 do begin
            ; read the header which appears in the data each time any parameter was changed and at the begining
            ppar = read_ppar_frame( lun, hdr)
        endwhile
        if hdr.nfft ne nfft then begin
            nfft = hdr.nfft
        endif
        if zspectra ne nfft*(hdr.raw_zrg+2)*zchan then begin
            ; create some arrays
            zrg = hdr.raw_zrg+2 ; noise and calibration height are treated as the other ranges.
            zspectra = nfft*zrg*zchan
            fft_window = hanning(nfft)
            spectra = fltarr(nfft,zchan,zrg)
        endif
        if ptr_valid(p_srvinfo) then ptr_free,p_srvinfo
        if ptr_valid(p_data) then heap_free,p_data    ; replaces ptr_free,(*p_data).sp1 & ptr_free,(*p_data).sp2 & ptr_free,p_data
        ; read one IQ sequence here (nfft pulse cycles):
        read1pds_dwell, lun, hdr, ppar, p_srvinfo, p_data, extract=['IQ1','IQ2']
        if n_elements( (*(*p_data).iq1)[*,0] ) ne nfft then begin
            if eof(lun) then goto,ende
            print,'corrupted data'
            stop
        endif
        if n_elements(iq_tagnr) ne 2 then begin
            iq_tagnr = [(where(tag_names(*p_data) eq 'IQ1'))[0],(where(tag_names(*p_data) eq 'IQ2'))[0]]
        endif

        ; do the FFT for each spectrum
        ; help,(*(*p_data).(iq_tagnr[ichan]))
        ; <PtrHeapVar4>   COMPLEX   = Array[256, 400]
        iq_co = (*(*p_data).(iq_tagnr[0]))[*,*] 
        iq_cx = (*(*p_data).(iq_tagnr[1]))[*,*] 
        phco=180./!pi*atan(iq_co,/phase)
        phcx=180./!pi*atan(iq_cx,/phase)
        dp0 = median(cmod(phcx-phco,360))
        dp1 = median(cmod(phcx-phco,360,dp0))
        printf,outlun,(*p_srvinfo).azipos,(*p_srvinfo).elvpos,dp1
    endwhile

    ende:
    close,lun
endfor
close,outlun
if ptr_valid(p_srvinfo) then heap_free,p_srvinfo
if ptr_valid(p_data) then heap_free,p_data
exit
end

