pro dmp2nc,arguments
; 
; Reads dmp data and writes the global moments in NetCDF format
;
;
; Matthias Bauer-Pfundstein
;
common startup,startup_done
common magic4chs,magic4chs,FzkPdsFmt
if n_elements(startup_done) ne 1 then begin
@startup
endif


argv = env2argv(arguments)

kp = { $
    infile:'', $
    iam:'nobody', $   ; radar name, may be specified to evaluate data from another MIRA36
    tail:0,$
    outfile:'' $
}
argv2stru,argv,kp
kp.iam = whoami(kp.iam)

; Default values for the Ploting parameters. These Parameters can be changed by the command "set"
pp={ $
of0LDR:45,$         ; For calculating SNRminLDR, which is needed to determine the minimal LDR that could be detected. of0LDR may be a bit smaller than of0.
drg:5$      ; Number of ranges which are taken in to account for the clustering
}
argv2stru,argv,pp


; Konstanten:
co=0
cx=1
npeak_max=16b ; maximum number of peaks that may show up in the eformat. This parameter must be the same as in spcs2moms.
zrg = -1L


dmpstru = {SNR:-20000,        SNM:-20000,        LWC:-20000, RR:-20000,               LDR:-20000,        VEL:!values.f_nan, RMS: !values.f_nan, ixa:-20000, ixb:20000}

; Almost the same as dmpstru but here SNR,LWC,RR,LDR are saved as floats and lineaized. 
momstru = {SNR:!values.f_nan, SNM:!values.f_nan, LWC:!values.f_nan, RR:!values.f_nan, LDR:!values.f_nan, VEL:!values.f_nan, RMS: !values.f_nan, ixa:-20000, ixb:20000}


infiles = findfile(kp.infile)
infiles_oldsyntax = strsplit(kp.infile,':',/extract)
if n_elements(infiles_oldsyntax) gt n_elements(infiles) then infiles=infiles_oldsyntax
if infiles[0] eq '' then begin
    print,'infile not found: ' + kp.infile
    goto,ende   
endif else zfiles = n_elements(infiles)
for ifiles=0,zfiles-1 do begin
    ; open the pds file and read the fileheader
    openr,lun,infiles[ifiles],/get_lun

    ppar = read_hdr(lun,hdr)
    
    if ifiles eq 0 then begin
        ptypes=['g']
        ncdfid = header2ncdf(kp.outfile,ppar,hdr,ptypes=ptypes,extras=['HSDco','HSDcx'])
        otime=0
    endif

    while not eof(lun) do begin
    resume:
        ; read one dwell of data:
        ; first read its srvi header
        if hdr.sn eq 'fzk' then begin
            while (rmh=read_mainframe_header(lun, hdr)) eq 0 do begin
                ; we got a ppar header
                if rmh eq -1L then begin
                    print,'read_mainframe_header hit eof'
                    goto,ende
                endif
                ppar = read_ppar_frame( lun, hdr)
            endwhile
        endif else begin
            ; vermutlich kann die Fallunterscheidung weg
            if ~ find_sgnt(lun,magic4chs) then begin
                print,'cannot find '+magic4chs+' frame header'
                goto,ende
            endif
            siz=0L & readu,lun,siz
        endelse
        if ptr_valid(p_srvinfo) then ptr_free,p_srvinfo
        if ptr_valid(p_npeaks) then ptr_free,p_npeaks
        if ptr_valid(p_dmp_data) then ptr_free,p_dmp_data
        ; read the dmp moments here:
        read1dmp_dwell_mie, lun, hdr, dmpstru, p_srvinfo, p_hsd, p_npeaks, p_dmp_data 
        if ~ ptr_valid(p_npeaks) || n_elements(*p_npeaks) ne hdr.mom_zrg then begin
            if eof(lun) then begin
                print,'read1dmp_dwell_mie could not read valid data.'
                goto,ende
            endif
            print,'corrupted data'
            goto,resume
        endif
        
        outdoortemp = float((*p_srvinfo).ps_err) - 100.
        ; print,'outdoortemp:',outdoortemp
        if outdoortemp lt -60. or outdoortemp gt 60. then begin
            outdoortemp=!values.f_nan
        endif

        if zrg ne hdr.mom_zrg then begin
            if zrg ne -1L then begin
                print,'zrg changed. That will not work. Aborting'
                goto,ende
            endif
            ; create some arrays
            zrg = hdr.mom_zrg
            zrgFlue = lindgen(zrg)
            hei = zrgFlue * hdr.delta_h + hdr.mom_h_min 
            zchan = hdr.pol +1L
            data = replicate(momstru,zrg,npeak_max)   ; the moments of all peaks are saved here
            npk = replicate(-1,zrg)                   ; npk[*,itme] is the number of peakes in data at each range gate.
            hsd_noise = fltarr(zrg,zchan)
            nfft=hdr.nfft & nfftme=nfft-1 
            outGlobal = replicate(momstru,zrg)
            SNRg = fltarr(zrg)
            if pp.of0LDR gt 0.0 then of0LDR=pp.of0LDR else of0LDR = ppar.of0
            SNRminLDR = of0LDR / (10*nfft*sqrt(ppar.avc))
        endif

        ; reorganize the data read by read1dmp_dwell_mie to a rectangular arrays:
        dmp2arr_mie, p_npeaks, p_dmp_data, data, npk
        hsd_noise[0,0] = reform(*p_hsd,zrg,zchan)
        srvinfos = *p_srvinfo 

        ; Calculate global 0-moments SNRg
        for irg=0, zrg-1 do begin 
            if npk[irg] gt 0 then begin
                SNRg[irg] = total(data[irg,0:npk[irg]-1].SNR)
            endif else begin
                SNRg[irg] = 0.0
            endelse
        endfor
        
        ; Evaluate the global moments
        for irg=0, zrg-1 do begin 
            ; filter some peaks which are very definitely no meteorological signal?
            if npk[irg] gt 0 then begin
                LDRdetectable=intarr(npk[irg])
                for ipeak=0,npk[irg]-1 do begin
                    ixb9ixa = (data[irg,ipeak].ixb-data[irg,ipeak].ixa+nfft) mod nfft
                    LDRmin = sqrt(ixb9ixa) * SNRminLDR * hsd_noise[irg,cx] / ( data[irg,ipeak].SNR * srvinfos.NPW2 )
                    LDRdetectable[ipeak] =  data[irg,ipeak].LDR gt LDRmin 
                endfor
                outGlobal[irg] = total_dmp_mie(data[irg,0:npk[irg]-1], LDRdetectable )
            endif else outGlobal[irg] = momstru
        endfor
            
        
        ; write result in cdf format. 
        srvinfo2ncdf,ncdfid,otime,srvinfos,hdr.fzk100
            
        ; Global moments, only three path echos are omitted
        NCDF_VARPUT, ncdfid, 'SNRg', outGlobal[0:zrg-1].SNR, count=[zrg,1], offset=[0,otime]
        NCDF_VARPUT, ncdfid, 'VELg', outGlobal[0:zrg-1].VEL, count=[zrg,1], offset=[0,otime]
        NCDF_VARPUT, ncdfid, 'RMSg', outGlobal[0:zrg-1].RMS, count=[zrg,1], offset=[0,otime]
        NCDF_VARPUT, ncdfid, 'LDRg', outGlobal[0:zrg-1].LDR, count=[zrg,1], offset=[0,otime]
        NCDF_VARPUT, ncdfid, 'HSDco', hsd_noise[*,co], count=[zrg,1], offset=[0,otime]
        NCDF_VARPUT, ncdfid, 'HSDcx', hsd_noise[*,cx], count=[zrg,1], offset=[0,otime]
        otime++
    endwhile ; eof

    free_lun,lun
endfor ; ifile

ende:
NCDF_CLOSE, ncdfid 
if ptr_valid(p_srvinfo) then heap_free,p_srvinfo
if ptr_valid(p_data) then heap_free,p_data
exit
end
