pro read_ceilo_nc_chm15k, filename, utime, bases

; procedure reads Jenoptik CHM15k Ceilometer netCDF file, which contains cloud bases and other information, and
; returns arrays utime (time series) and bases (cloud base height) as called-by-reference parameters

; Syntax:
;
; result = read_ceilo_nc_chm15k(filename, utime, bases)
;
; Arguments:
;
; filename  : full path to the file, which contains the ceilometer data
;
; Example:
;
; result = read_ceilo_nc_chm15k('/home/alex/work/dwd/ceilo/CHM060012_2010-04-14.nc', utime, bases)
;
; Fewald 08.05.2013

ncdfid = NCDF_OPEN(filename,/NOWRITE)
NCDF_VARGET, ncdfid, 'layer', layer 
NCDF_VARGET, ncdfid, 'time', utimex
ncdfidv = NCDF_VARID(ncdfid, 'cbh' )
NCDF_VARGET, ncdfid, ncdfidv, bases
NCDF_CLOSE, ncdfid

utime = utimex - (JULDAY(01,01,1970)-JULDAY(01,01,1904))*24UL*60UL*60UL


end

