Pro read1pds_dwell, lun, hdr, ppar,  p_srvinfo, p_data, extract=extract, skip=skip_
; read one dwell from pds-files
; Parameter
;   lun         logical unit of the file which has opened before
;   data_size   the size of the 
;   hdr         system parameters with physical units, obtained before by ppar=read_hdr(lun,hdr)
;   ppar        the original ppar structure as read from file
;   p_srvinfo   pointer to struct with dwell header containing information from the log data
;   p_data      pointer to struct with returned data
;   extract     optional array of strings which indicates which profiles should be read and 
;               which should be skipped. it should be a subset of
;                      extrac=['p1','v1','r1','p2','v2','r2','sp1','sp2'].
;               If not given all available profiles are read.
;   skip        skip data from this dwell, this cannot be achieved by setting extract=['']
;               as extract is evaluated only at the first call of read1pds_dwell after reading 
;               the file header.
;
;  See the source of gorge_pds_file.pro for information about the structures
;
; Matthias Bauer-Pfundstein 
;
    on_ioerror,ioerr
    if n_elements(skip_) le 0 then skip=0 else skip=skip_

    common initialize,init_extract,init_header_counter
    common read1pds_dwell,extr,spcbuf,cbuf,lind,mombuf,expbuf,iqbuf
    common magic4chs,magic4chs,FzkPdsFmt,syspar
    if n_elements(init_extract) eq 0 then begin
        print,'read_hdr has to be called before read1pds_dwell'
        init_extract = 0
        spcbuf = -1
        cbuf = -1
    endif
    if init_extract then begin
        extract_all=['p1','v1','r1','p2','v2','r2','sp1','sp2','iq1','iq2']
        if n_elements(extract) le 0 then extract=extract_all
        if extract[0] eq 'default' or extract[0] eq '' then extract=extract_all
        extract = strupcase(extract)
        extr = { p1: max(extract eq 'P1' ), $
                 v1: max(extract eq 'V1' ), $
                 r1: max(extract eq 'R1' ), $
                 p2: max(extract eq 'P2' ), $
                 v2: max(extract eq 'V2' ), $
                 r2: max(extract eq 'R2' ), $
                 sp1:max(extract eq 'SP1'), $
                 sp2:max(extract eq 'SP2'), $
                 iq1:max(extract eq 'IQ1'), $
                 iq2:max(extract eq 'IQ2')  $
        }
    endif

    lenint=2L
    lenflt=4L
    lenlong=4L
    laprf = hdr.la * hdr.prf 
    nfft=hdr.nfft
    zchan = hdr.pol+1L
    zrg_rpz = hdr.raw_zrg + 2L + syspar.noise3   ; Zahl der RangeGates "Raw Plus Zwei"
    zrg_raw = hdr.raw_zrg
    ; mombufsize = 512L ; k5
    ; mombufsize = 1024L ; 1k
    mombufsize=syspar.mombufsize
    if n_elements(mombuf) ne mombufsize*2 then mombuf=fltarr(2,mombufsize)

    items_read=0
    dwell_done=0
    spc_ex=0
    exp_ex=0
    iq_ex=0
    while dwell_done eq 0 do begin
        sgnt='xxxx' & siz=0L & readu,lun,sgnt,siz
        ; help,sgnt,siz
        case 1 of
            sgnt eq 'SRVI':begin
                p_srvinfo = ptr_new(read_srvinfo(lun))
                hdr.len_srvi = siz
            end
            sgnt eq 'HSDV':begin
                if zrg_raw*zchan*lenflt ne siz then stop
                hsdbuf=fltarr(zrg_raw,zchan)
                readu,lun,hsdbuf
            end
            sgnt eq 'FFTD' or sgnt eq 'ZSPC':begin
                zint =  nfft * 2L * zrg_rpz 
                if siz gt 0L and (extr.sp1 or extr.sp2 or extr.iq1 or extr.iq2) and skip eq 0 then begin
                    case ppar.osc of
                    0: begin
                        ; spectra written by the new (since FZK) integer DSP
                        if n_elements(spcbuf) ne zint then begin 
                            if mombufsize eq 512L then begin
                                spcbuf = lonarr(nfft,2,zrg_rpz)
                            endif else begin
                                spcbuf = fltarr(nfft,2,zrg_rpz)
                            endelse
                        endif
                        readu,lun,spcbuf
                        spc_ex=1
                    end
                    2: begin
                        ; spectra with normal floats and no exponents which where written e.g. by iq2spc.pro
                        if n_elements(spcbuf) ne zint then spcbuf = fltarr(nfft,2,zrg_rpz)
                        readu,lun,spcbuf
                        spc_ex=1
                        ; dwell_done=1
                    end
                    4: begin
                        ; concise spectra written by spectraprocessing.pro
                        if n_elements(cbuf) ne long(nfft)*zrg_rpz*zchan then begin
                            cbuf=fltarr(nfft,zrg_rpz,zchan)
                            lind = (lindgen(nfft)+nfft/2) mod nfft
                        endif
                        for irg=0,zrg_raw-1 do begin
                            for ichan=0L,zchan-1L do cbuf[*,irg,ichan]=hsdbuf[irg,ichan]
                            nixabs=0 & readu,lun,nixabs
                            for ixabs=0,nixabs-1 do begin
                                ixa=0 & zix=0 & readu,lun,ixa,zix
                                spcpiece=uintarr(zix) & max=0. 
                                  for ichan=0L,zchan-1L do begin
                                      readu,lun,spcpiece,max 
                                      cbuf[ixa,irg,ichan] = 1.5260186e-05 * max * spcpiece
                                  endfor
                            endfor
                            for ichan=0L,zchan-1L do cbuf[*,irg,ichan]=cbuf[lind,irg,ichan]
                            ;plot,db(cbuf[*,irg,0L]),yra=[-10,70],yst=1 & oplot,db(cbuf[*,irg,1L])
                            ;wait,0.1
                        endfor
                        spc_ex=1
                        dwell_done=1 
                    end
                    1: begin
                        ; iqbuf[IoQ,isequence,coOcx,irg]
                        ; IoQ: 0 I, 1 Q
                        ; isequence: 0,..,NFFT-1
                        ; coOcx: 0 co, 1 cx
                        ; irg: RawGate0,..,RawGate1
                        if n_elements(iqbuf) ne zint*2L then iqbuf = intarr(2,nfft,2,zrg_rpz)
                        readu,lun,iqbuf
                        iq_ex=1
                    end
                    else: goto,ioerr
                    endcase
                endif else seek,lun,siz
            end
            sgnt eq 'SNRD': begin
                if siz eq mombufsize*8 and (extr.p1 or extr.p2) and skip eq 0 then begin
                    readu,lun,mombuf
                    if extr.p1 then p1 =  mombuf[0,0:hdr.mom_zrg-1] else p1 = !values.f_nan
                    if extr.p2 then p2 =  mombuf[1,0:hdr.mom_zrg-1] else p2 = !values.f_nan
                    ; if mombuf[0,hdr.mom_zrg] ne (*p_srvinfo).npw1 then goto,ioerr
                    ; if mombuf[0,hdr.mom_zrg+1] ne (*p_srvinfo).cpw1 then goto,ioerr
                    ; if mombuf[1,hdr.mom_zrg] ne (*p_srvinfo).npw2 then goto,ioerr
                    ; if mombuf[1,hdr.mom_zrg+1] ne (*p_srvinfo).cpw2 then goto,ioerr
                endif else begin
                    seek,lun,siz
                    p1 = !values.f_nan
                    p2 = !values.f_nan
                endelse
            end
            sgnt eq 'VELD':begin
                if siz eq mombufsize*8 and (extr.v1 or extr.v2) and skip eq 0 then begin
                    readu,lun,mombuf
                    if extr.v1 then v1 =  laprf * mombuf[0,0:hdr.mom_zrg-1] else v1 = !values.f_nan
                    if extr.v2 then v2 =  laprf * mombuf[1,0:hdr.mom_zrg-1] else v2 = !values.f_nan
                endif else begin
                    seek,lun,siz
                    v1 = !values.f_nan
                    v2 = !values.f_nan
                endelse
            end
            sgnt eq 'RMSD':begin
                if siz eq mombufsize*8 and (extr.r1 or extr.r2) and skip eq 0 then begin
                    readu,lun,mombuf
                    if extr.r1 then r1 = laprf * mombuf[0,0:hdr.mom_zrg-1] else r1 = !values.f_nan
                    if extr.r2 then r2 = laprf * mombuf[1,0:hdr.mom_zrg-1] else r2 = !values.f_nan
                endif else begin
                    seek,lun,siz
                    r1 = !values.f_nan
                    r2 = !values.f_nan
                endelse
            end
            sgnt eq 'EXPD':begin ; gibt es nur bei k5
                if siz eq mombufsize*4 and (extr.sp1 or extr.sp2) and skip eq 0 then begin
                    if n_elements(expbuf) ne mombufsize*2 then expbuf=intarr(2,mombufsize)
                    readu,lun,expbuf
                    exp_ex=1
                endif else seek,lun,siz
                if items_read ne 5 then print,'dwel with missing items'
                dwell_done=1
            end
            sgnt eq 'HNED':begin ; gibt es nur beim 1k
               if siz eq mombufsize*4 and (extr.sp1 or extr.sp2) and skip eq 0 then begin
                   if n_elements(expbuf) ne mombufsize*2 then expbuf=intarr(2,mombufsize)
                   readu,lun,expbuf
                   hne_ex=1
               endif else seek,lun,siz
               if items_read ne 5 then print,'dwel with missing items'
               dwell_done=1
            end
            else: goto,ioerr
        endcase
        ++items_read
    endwhile

    ;print,'exp_ex=',exp_ex,' spc_ex=',spc_ex,' osc=',ppar.osc
    case 1 of
        (spc_ex and ppar.osc eq 2): begin
            if extr.sp1 then sp1 = ptr_new( reform(spcbuf[*,0,*],nfft,zrg_rpz) ) else sp1 = ptr_new(!values.f_nan)
            if extr.sp2 then sp2 = ptr_new( reform(spcbuf[*,1,*],nfft,zrg_rpz) ) else sp2 = ptr_new(!values.f_nan)
        end
        (spc_ex and ppar.osc eq 4): begin
            ; fake the noise and reference spectra so that spectraprocessing can calculate npw and cpw from it.
            cbuf[*,zrg_raw,0L]=(*p_srvinfo).npw1/nfft
            cbuf[*,zrg_raw+1,0L]=(*p_srvinfo).cpw1/nfft
            cbuf[*,zrg_raw,1L]=(*p_srvinfo).npw2/nfft
            cbuf[*,zrg_raw+1,1L]=(*p_srvinfo).cpw2/nfft
            hsd=ptr_new(hsdbuf,/no_copy)
            if extr.sp1 then sp1 = ptr_new(cbuf[*,*,0L]) else sp1 = ptr_new(!values.f_nan)
            if extr.sp2 then sp2 = ptr_new(cbuf[*,*,1L]) else sp2 = ptr_new(!values.f_nan)
        end
        (spc_ex and (exp_ex or mombufsize ne 512L)):begin
            if mombufsize eq 512L then begin
                ix_exbu =[lindgen(hdr.raw_zrg),hdr.mom_zrg,hdr.mom_zrg+1]    ; richtig
                nfftme = replicate(1.,nfft)
                if extr.sp1 then sp1 = ptr_new( syspar.pds_spectra_normalization*reform(float(spcbuf[*,0,*]),nfft,zrg_rpz) * ( nfftme # (2. ^ expbuf[0,ix_exbu])))  else sp1 = ptr_new(!values.f_nan)
                if extr.sp2 then sp2 = ptr_new( syspar.pds_spectra_normalization*reform(float(spcbuf[*,1,*]),nfft,zrg_rpz) * ( nfftme # (2. ^ expbuf[1,ix_exbu])))  else sp2 = ptr_new(!values.f_nan)
            endif else begin
                if extr.sp1 then sp1 = ptr_new( syspar.pds_spectra_normalization*reform(spcbuf[*,0,*],nfft,zrg_rpz) ) else sp1 = ptr_new(!values.f_nan)
                if extr.sp2 then sp2 = ptr_new( syspar.pds_spectra_normalization*reform(spcbuf[*,1,*],nfft,zrg_rpz) ) else sp2 = ptr_new(!values.f_nan)
            endelse
        end
        else: begin
            sp1 = ptr_new(!values.f_nan)
            sp2 = ptr_new(!values.f_nan)
        end
    endcase
    
    if spc_ex and ppar.osc eq 4 then begin
        if extr.p1 eq 0 and extr.p2 eq 0 and extr.v1 eq 0 and extr.v2 eq 0 and extr.r1 eq 0 and extr.r2 eq 0 then begin
            p1=!values.f_nan
            p2=!values.f_nan
            v1=!values.f_nan
            v2=!values.f_nan
            r1=!values.f_nan
            r2=!values.f_nan
        endif else begin
            print, 'Moment profiles where requested by the extract parameter but they are not present in the concise data'
        endelse
    endif

    if iq_ex then begin
        if  extr.iq1 then iq1=ptr_new( complex(float(iqbuf[*,*,0,*]),0,nfft,zrg_rpz) ) else iq1=ptr_new(!values.f_nan)
        if  extr.iq2 then iq2=ptr_new( complex(float(iqbuf[*,*,1,*]),0,nfft,zrg_rpz) ) else iq2=ptr_new(!values.f_nan)
    endif else begin
        iq1 = ptr_new(!values.f_nan)
        iq2 = ptr_new(!values.f_nan)
    endelse
   
    if n_elements(hsd) eq 0 then hsd=ptr_new(!values.f_nan)

    p_data = ptr_new({ $
        p1  : p1  ,   $
        v1  : v1  ,   $
        r1  : r1  ,   $
        p2  : p2  ,   $
        v2  : v2  ,   $
        r2  : r2  ,   $
        sp1 : sp1 ,   $
        sp2 : sp2 ,   $
        hsd : hsd ,   $
        iq1 : iq1 ,   $
        iq2 : iq2     $
    })
    return
ioerr:
    p_data = ptr_new({ $
        p1  : !values.f_nan  ,   $
        v1  : !values.f_nan  ,   $
        r1  : !values.f_nan  ,   $
        p2  : !values.f_nan  ,   $
        v2  : !values.f_nan  ,   $
        r2  : !values.f_nan  ,   $
        sp1 : ptr_new(!values.f_nan)  ,   $
        sp2 : ptr_new(!values.f_nan)  ,   $
        iq1 : ptr_new(!values.f_nan)  ,   $
        iq2 : ptr_new(!values.f_nan)      $
    })
end

