Function hdr2phys,ppar,_description,len_pparx
; calculation the tags of the physically meaningful structure hdr
common hdr2phys,name,place,oper,descr,len_ppar
common magic4chs,magic4chs,FzkPdsFmt,syspar
    c = 2.998e8
    xmt = syspar.txfrequency 

    if ppar.pol gt 2 then begin
        pol=1l
        tx_pulsewidth=ppar.pdr*1e-9
        pulse_width_i=ppar.pol
        ppar.pdr = pulse_width_i    ; vielleich besser weg lassen
    endif else begin
        pol=ppar.pol
        if ppar.pdr le 9 then begin
            pulse_width_i = (syspar.RecPulseWidths)[ppar.pdr] 
            ; pulse_width = ([100,200,300,400,500,600,700,800,900,1000])[ppar.pdr] ; devices before and including MPI
            ; pulse_width = ([96,192,288,400,512,576,706,832,912,1040])[ppar.pdr] ; between halo and DLR
            ; pulse_width = ([112,208,306,400,512,624,706,832,912,1040])[ppar.pdr] ; 1khg devices
            ppar.pdr = pulse_width_i
        endif else begin
            pulse_width_i = ppar.pdr
        endelse
        tx_pulsewidth=pulse_width_i*1e-9
    endelse
    pulse_width=pulse_width_i*1e-9
    delta_h = 0.5 * c * pulse_width
    mom_h_min   = ppar.ihp * delta_h
    mom_zrg = ppar.chg-2                     ; -2 is applied as chg includes the two noise 
                                             ; estimates which are appended after the higest range gate
    ; mom_height = [findgen(mom_zrg) * delta_h + mom_h_min,15000.,15000.]

    raw_h_min = (ppar.raw_gate1 )*delta_h
    raw_zrg = ppar.raw_gate2 - ppar.raw_gate1+1 - syspar.noise3
    ; raw_height = [findgen(raw_zrg) * delta_h + raw_h_min,15000.,15000.]

    if ppar.prf lt 10 then prf = 2500. * (ppar.prf + 1) else prf = float(ppar.prf)
    if ppar.sft ge 64 then nfft=long(ppar.sft) else nfft= 128L * 2L ^ ppar.sft
    ave = float(ppar.avc) * float(nfft) / prf

    if n_elements(_description) gt 0 then begin
        name=_description.name
        place=_description.place
        oper=strmid(_description.oper,0,63)
        descr=strmid(_description.descr,0,255)
    endif
    if N_elements(len_pparx) then len_ppar=len_pparx
    if N_elements(len_ppar) eq 0 then  len_ppar=-1
    len_srvi = -1L

    hdr={ $
        name:name, $                    ; original name of the pds file
        oper:oper, $                    ; operator, should begin with 'fzk' 
        descr:descr,$                   ; may contain informations like Altitude given in /icfg/header.ini
        place:place, $                  ; e.g. string from the gps receiver 
        sn:'fzk',$                      ; device
        len_ppar:len_ppar, $            ; number of bytes of the ppar structure
        len_srvi:len_srvi, $            ; number of bytes of the srvi structure
        prf: prf, $                     ; Pulse repetition frequency
        pulse_width: pulse_width, $     ; puls diameter as choosen in the magnetron mode
        nfft: nfft, $                   ; number of FFT points
        avc: ppar.avc, $                ; number of spectral averages
        ave: ave, $                     ; averaging time
        c: c, $                         ; speed of light
        xmt: xmt, $                     ; transmitting frequency. This is a constant and not from the BITE data!
        fzk100: syspar.fzk100 , $       ; serverinfo.tpow/fzk100 = average tx power in Watts. 
        delta_h: delta_h , $            ; range spacing
        mom_h_min: mom_h_min, $         ; height of the lowes range gate of the moment data
        mom_zrg: mom_zrg, $             ; number of range gates of the moment data
        raw_h_min: raw_h_min, $         ; height of the lowes range gate of the raw (or spectrum) data
        raw_zrg: raw_zrg, $             ; number of range gates of the raw data
        do_raw: ppar.raw, $             ; flag -- oscillosgramm or spectra data available
        vuar:  c * prf / (2.0*xmt), $   ; diameter of the range of velocities which can be allocated without ambiguity 
                                    $   ; (Velocity Un-Ambigiuty Range)
        vny: c * prf / (4.0*xmt), $     ; nyquist velocity = vuar/2
        la: c / xmt, $                  ; wave length
        pol: pol,  $               ; cross polarized data on/off
        att: ppar.att $                 ; 1 if attn or STC is switched on
    }
    return,hdr
end

function read_ppar,lun
; read the PPAR chunk
    ppar    = { $
            prf          : 0l, $
            pdr          : 0l, $
            sft          : 0l, $
            avc          : 0l, $
            ihp          : 0l, $
            chg          : 0l, $
            pol          : 0l, $
            att          : 0l, $
            tx           : 0l, $
            adcgain0     : 0., $
            adcgain1     : 0., $
            wnd          : 0l, $
            pos          : 0l, $
            add          : 0l, $
            len          : 0l, $
            cal          : 0l, $
            nos          : 0l, $
            of0          : 0l, $
            of1          : 0l, $
            swt          : 0l, $

            sum          : 0l, $
            osc          : 0l, $
            tst          : 0l, $
            cor          : 0l, $
            ofs          : 0l, $
            HSBn         : 0l, $

            HSBa         : 0l, $

            calibrpower_m : 0., $
            calibrsnr_m   : 0., $
            calibrpower_s : 0., $
            calibrsnr_s   : 0., $

            raw_gate1     : 0l, $
            raw_gate2     : 0l, $
            raw           : 0l, $
            prc           : 0l $
    }
    readu,lun,ppar
    return,ppar
end

function read_hdr, lun, hdr
; read the header which appears only once at the begining of each pds-file
common fhdr1024,_description,fill1024
common initialize,init_extract,init_header_counter
init_extract=1 ; forces recalculation of init_extract in read1pds_dwell
init_header_counter=1 ; be silent if header_indes dose not match
    str32   = STRING(REPLICATE(32B,32))
    str64   = STRING(REPLICATE(32B,64))
    str128  = STRING(REPLICATE(32B,128))
    str256  = STRING(REPLICATE(32B,256))

    _description    = {           $ 
            name         : str32, $
            time         : str32, $
            oper         : str64, $
            place        : str128,$
            descr        : str256 $
    }

    readu, lun, _description
    ppar = read_ppar(lun)
    len_ppar = N_Tags(ppar, /Length)
    fill1024  = replicate(32b,512-len_ppar)
    readu, lun, fill1024
    hdr = hdr2phys(ppar,_description,len_ppar)

    return, ppar
end

