pro guesss,arguments
; 
; tool for testing differen methods for moment estimation for mira36 cloudradar spectra.
;
; 
; 
; unix> idl guesss infile=data/20040602_1000.pds of0=40 
;
; IDL > guesss, 'infile=data/20040602_1000.pds hilde_method=1'
; 
; infile      File to read the pds data from. Multible files with contain successive 
;             (in time) data may be given as array of strings.
;
; nrange      set to some reasonable value (e.g nrange=-10:10) to switch on plotting 
;             time sequence of the noise and cal values, a feature which actually should
;             included to some other program. This feature needs to be activated at program
;             startup (not at the command line).
;
; See the set command for a description of the other parameters. 
;
; After launching the pds file is openend and the firsp profile of spectra is read. 
; Then a cmd line interface is launched.
; 
; The following commands can be used:
;-----------------------------------------------------------------------------------------------------------------
;     Guesss > set rg 137     # sets the range gate at which moment estimation should be performed
;     Guesss > set chan 1     # sets the channel (0 co, 1 cross) for moment estimation
;     Guesss > guess          # do moment estimation
;     Guesss > r              # repeat last action (guess) cmd
;
;     Guesss > + [increment]  # read the next dwell from the pds file
;                               # noise estimation works only correctly after reaching dwell # 16.
;
;-----------------------------------------------------------------------------------------------------------------
;    Guesss > set <parameter> <valu1> [<value2> ...]
;
; Change the default value of a plotting parameter. If <value1> is omited the
; current value is plotted. All the plotting parameters which 
; are available can be shown by
;
;    Guesss > set 
;
; Their curren Value can be queried by
;
;    Guesss > set rg          # prints the range gate on which moment estimation is performed.
;
;    Guesss > postscript <filename>
;
; Directs the output of the following plot commands to the given postscript filename.
; for flushing and closing the postscript file the command
;
;    Guesss > close
;
; must be given. After that the plot out put is directed to the screen again.
;----------------------------------------------------------------------------------------------------------------
;    Guesss > png <filename>
;
; exports the screen to png-format
;-----------------------------------------------------------------------------------------------------------------
;    Guesss > info
;
; gives some information from the file header
;-----------------------------------------------------------------------------------------------------------------
;    Guesss > x
;
; exits guesss
;----------------------------------------------------------------------------------------------------------------
; If no command is recognized then the command is executed by the IDL interpreter.
;
; Matthias Bauer-Pfundstein bauer@metek.de
;
common startup,startup_done
if n_elements(startup_done) ne 1 then begin
@startup
endif

; Arguments can be provided by two methods:
; - If guesss is called by an interactive IDL session the first (and only) 
; string parameter arguments which contains a space separated list is split to
; the strarr argv.
; - Aditionally strings contained in the environment variables ARG1, ARG2, ..., 
; ARG9, ARG10, ... are prepended to argv. The script idl packs its calling arguments
; these environment variables. This is done as IDL can not extract the shell 
; variables like $1, $2,...
argv = env2argv(arguments)

kp = { $
    infile:'', $
    max_dwells:6000 $
}
argv2stru,argv,kp


; Default values for the Ploting parameters. These Parameters can be changed by the command "set"
pp={ $
rg:50,$          ; see comments below!
chan:0,$     
of0:-1,$     
hilde_method:3,$ 
ofdiv:9.,$
ndiv:16,$
rgclutter:0,$
max_sholder_dbc:-0.20,$
fftwin:[0.75, 0.3, 0.16, 0.065, 0.023, 0.008, 0.0025],$
gsm_n:5, $
gsm_alpha:8.6, $ 
gap4dom:4L, $
peak_thr:9.0 ,$ 
peak_sig_thr:0.05,$
peak_thr_wider:8., $
peak_thr_higher:2., $
kkk:1L, $
peak_method:1, $
badcol :15, $
srange:[-35,75.], $
nrange:[-10,-20.], $
vrange:[!values.f_nan,!values.f_nan], $
ldr:1,$
charsize:1.1, $
thick:1.0, $
b4tit:'',$
ti:'!3',$
colt:'smooth' ,$
nnave:16 $
}
argv2stru,argv,pp

; online descripion for the above parameters
pph={ $
rg:      'gate for calculating moments', $
chan:      'channel (co or  cross) for calculating moments', $
of0:      'ofs0, parameter for detection threshold', $
hilde_method: 'enumerate: 1 hildebrand-sekhon, 2 hildebrand-fischer-henemut, 3 HSdiv', $
ofdiv: 'Used by HSdiv, similar to ofs0 for determination of detection threshold. divisions from minimum to this thresold are regarded as noise',$
ndiv: 'Used by HSdiv, number of parts to which the spectrum is divided', $
rgclutter:'maximum height in which clutter should be removed', $
max_sholder_dbc:'theroretically the sholders of the clutter in the spectra should be -6 dbc. Non clutter signal may cover the sholders. Then no clutter removal is performed',$
fftwin:'decaying factor for the neighbor spectral lines in clutter removal', $
gsm_n:'number of points of convol. Kernel for smoothing before peak search', $
gsm_alpha:'1/width of gausian kernel for smoothing, see gsm_n', $
peak_thr: 'Threshold for peak separation, canion method. Canions sepatate peaks if the relative depth of the canion inbtween is greater than peak_thr.', $ 
peak_sig_thr: 'Aux variable for identify peak', $ 
gap4dom: 'Peaks are separated in any case if more than gap4dom values are below detection thresold between them, canion method', $
peak_thr_wider: '1st Threshold for peak separation, 2nd-moment method', $
peak_thr_higher: '2nd Threshold for peak separation, 2nd-moment method', $
kkk: 'peak identification and moment estimation is repeated kkk times. Useful for determination of execution time',$
peak_method: 'Method for identyfying peaks. 1 canion method, 2 2ndmoment method', $
badcol : 'Grey, color for plotting values which are not plausible.', $
srange:  'Zrange for spectra', $
nrange:  'Zrange for noise spectra', $
vrange:  'velocity range for plotting spectra', $
ldr:  'show cross channel and ldr as dotted line', $
charsize: 'See graphic keywords in IDL documentation', $
thick:   'See graphic keywords in IDL documentation', $
b4tit:  'Can be used to pass a Hershly-style font directive used only for title lines', $
ti:     'Can be uses to pass a Hershly-style font directive used for all annotations', $
colt:   'Color table, the predefined colortables reside in the directory idl/colors' $
}

startcolor=30
ocolt='none'
td=1000.

!p.multi=[0,1,1]
if !d.name eq 'X' then begin
    while !d.window ge 0 do wdelete
    screen_size=get_screen_size()
    scale=screen_size/[1280.,1024]
    maxcolor=512
    if pp.nrange[1] gt pp.nrange[0] then begin
        window,/free,retain=2,colors=maxcolor,title='guesss',xpos=500*scale[0],ypos=500*scale[1], $
            xsize=680*scale[0],ysize=400*scale[1]
            noise_win=!d.window
        tnbuf=4000
        tnsp_co=fltarr(tnbuf)
        tcsp_co=fltarr(tnbuf)
        tnsp_cx=fltarr(tnbuf)
        tcsp_cx=fltarr(tnbuf)
        dnsp_co=fltarr(tnbuf)
        dcsp_co=fltarr(tnbuf)
        dnsp_cx=fltarr(tnbuf)
        dcsp_cx=fltarr(tnbuf)
        itnbuf=0
    endif
    window,/free,retain=2,colors=maxcolor,title='guesss',xpos=600*scale[0],ypos=600*scale[1], $
        xsize=680*scale[0],ysize=400*scale[1]
        spc_win=!d.window
    wset,spc_win
endif

; open the pds file and read the fileheader
get_lun,lun & openr,lun,kp.infile
file_pos = lonarr(kp.max_dwells)
ipos=0L
ppar = read_hdr(lun,hdr)
fzk=0
if fzk then begin
    ppar = read_ppar_frame( lun, hdr)
endif

zchan=2 ; number of channels
nfft=0  ; nfft of the noise spectra in momory. If hdr.nfft differs then a new buffer for noisspectra is created.
read1dwell=1
while 1 do begin
    if read1dwell then begin
        ; remeber the positions in file where the dwells begin. Needed for the '-' command!
        point_lun, -lun, posx & file_pos[ipos] = posx 
        if ++ipos ge kp.max_dwells then begin
            shiftpos=kp.max_dwells/2
            file_pos[0]=file_pos[shiftpos:kp.max_dwells-1L]
            ipos -= shiftpos
        endif
        ; read parameter headers if necessary
        if fzk then begin
            while read_mainframe_header(lun, hdr) eq 0 do begin
                ppar = read_ppar_frame( lun, hdr)
            endwhile
        endif
        ; gnerate some arrays that depend on the parameters in pparm
        if hdr.nfft ne nfft then begin
            nfft=hdr.nfft
            noise_spectras = fltarr(nfft, zchan, pp.nnave)
            noise_values = fltarr(zchan, pp.nnave)
            inave=0
            nnave=0
            ix2ord = (lindgen(nfft) + nfft/2 ) mod nfft
            dx = hdr.VUAR/nfft
            x = cmod(dx * findgen(nfft), hdr.VUAR,0)
            xord=x[ix2ord]
        endif
        if n_elements(p_srvinfo) then heap_free,p_srvinfo
        if n_elements(p_data) then heap_free,p_data
        ; read the p_srvinfo structure and the spectra
        read1pds_dwell, lun, hdr, ppar, p_srvinfo, p_data
        ; transfer the noise spectra to a buffer containing the last 16 noise spectra
        noise_spectras[0,0,inave] = (*(*p_data).sp1)[*,hdr.raw_zrg]
        noise_spectras[0,1,inave] = (*(*p_data).sp2)[*,hdr.raw_zrg]
        ; transfer the noise values to a buffer containing the last 16 noise spectra
        noise_values[0,inave] = total(noise_spectras[*,0,inave])/float(nfft)
        noise_values[1,inave] = total(noise_spectras[*,1,inave])/float(nfft)
        inave = ++inave mod pp.nnave
        if nnave lt pp.nnave then ++nnave
        ; e.g. the spectra are stored in
        ; (*(*p_data).sp1)[*,rg4sp]           co channel
        ; (*(*p_data).sp2)[*,rg4sp]        cross channel
        ; As one piece of code should be able to handle ...sp1 or ...sp2 depending on pp.chan we need to
        ; address the structure tags of *(*p_data) by indexes
        ; *(*p_data).(sp_tagnr[0]) is a substitute for *(*p_data).sp1
        ;    and
        ; *(*p_data).(sp_tagnr[1]) is a substitute for *(*p_data).sp2
        if n_elements(sp_tagnr) ne 2 then begin
            sp_tagnr=[(where(tag_names(*p_data) eq 'SP1'))[0],(where(tag_names(*p_data) eq 'SP2'))[0]]
            p_tagnr=[(where(tag_names(*p_data) eq 'P1'))[0],(where(tag_names(*p_data) eq 'P2'))[0]]
            v_tagnr=[(where(tag_names(*p_data) eq 'V1'))[0],(where(tag_names(*p_data) eq 'V2'))[0]]
            r_tagnr=[(where(tag_names(*p_data) eq 'R1'))[0],(where(tag_names(*p_data) eq 'R2'))[0]]
            noi_tagnr=[(where(tag_names(*p_srvinfo) eq 'NPW1'))[0],(where(tag_names(*p_srvinfo) eq 'NPW2'))[0]]
            cal_tagnr=[(where(tag_names(*p_srvinfo) eq 'CPW1'))[0],(where(tag_names(*p_srvinfo) eq 'CPW2'))[0]]
        endif
        read1dwell=0
    endif
    
    ; color table
    if ocolt ne pp.colt then begin
        loadcolor,startcolor,colorfile=pp.colt,zcolor=zcolor 
        ocolt=pp.colt
    endif

    pp.rg = (pp.rg > ppar.ihp) < (ppar.ihp+ppar.chg-3) 
    rgx = ( pp.rg < (ppar.RAW_GATE2+2)) > (ppar.RAW_GATE1)
    if pp.rg ne rgx then begin
        print,'range has been set to the range of gates in which saved spectra are available',ppar.RAW_GATE1,ppar.RAW_GATE2
        pp.rg = rgx
    endif
    rg4sp = pp.rg - ppar.RAW_GATE1  ; richtig
    if pp.rg lt ppar.ihp  or  pp.rg gt (ppar.ihp+ppar.chg-1) then begin
        print,'This program requires that moments are saved for the ranges in which spectra are saved.'
        stop
    endif
    ; extract the data fromt the range gate that should be investigated
    rg4mom = pp.rg - ppar.IHP 
    spc = (*(*p_data).(sp_tagnr[pp.chan]))[*,rg4sp]
    if finite(((*p_data).(p_tagnr[pp.chan]))[rg4mom]) then begin
        dsp_snr = ((*p_data).(p_tagnr[pp.chan]))[rg4mom]
        dsp_vel = -((*p_data).(v_tagnr[pp.chan]))[rg4mom]
        dsp_rms = ((*p_data).(r_tagnr[pp.chan]))[rg4mom]
        dsp_NoiseLevel = (*p_srvinfo).(noi_tagnr[pp.chan])/float(nfft) ; the noise an calibration power saved in the srvinfo structure are P_noise and not NoiseLevel
        dsp_pow = dsp_snr * ( dsp_NoiseLevel * float(nfft) ) ; integral first moment in DSP units and not relative to P_noise
        mom_ex=1
    endif else begin
        mom_ex=0
        dsp_pow=-1
        dsp_vel=-1
    endelse
    my_noise = total(noise_values[pp.chan,0:nnave-1])/float(nnave)
    if pp.of0 gt 0 then of0 = pp.of0 else of0 = ppar.of0
    det_thres = my_noise * ( 1 + of0/10. / sqrt(ppar.avc))
    det_thres_dyn = det_thres > max(spc)/linear(ppar.SWT)


    hs_noise = hilde(spc,method=pp.hilde_method, ofdiv=pp.ofdiv, ndiv=pp.ndiv, n_spcave=ppar.avc)
    hs_det_thres = hs_noise * ( 1 + of0/10. / sqrt(ppar.avc))
    ix_hs2 = where(spc gt hs_det_thres, nixhs)
    if nixhs gt 0 then begin
        above_9noise = spc[ix_hs2] - hs_noise
        hs_pow=total(above_9noise)
        hs_vel=total(above_9noise*x[ix_hs2])/hs_pow
    endif else begin
        hs_pow=-1.
        hs_vel=9999.
    endelse
    
    P_cal = (*p_srvinfo).(cal_tagnr[pp.chan])

    ; remove clutter
    if pp.rg le pp.rgclutter then begin
        remove_clutter,spc,hs_noise,hs_det_thres,pp
    endif
    

    ; momentestimation as in the DSP "global moments"
    ix_my=where(spc gt det_thres_dyn, nixmy)
    if nixmy gt 0 then begin
        above_9noise = spc[ix_my] - my_noise
        my_pow=total(above_9noise) 
        my_vel=total(above_9noise * x[ix_my]) / my_pow
    endif else begin
        my_pow=-1.
        my_vel=9999.
    endelse

    ; multipeak moment estimation
    spcord=spc[ix2ord] 
    for kk=0,pp.kkk-1 do begin
        spc_thr = spcord  > hs_det_thres
        if pp.gsm_n gt 0 then spc_thr = gsmooth(spc_thr,n=pp.gsm_n,alpha=pp.gsm_alpha)
        case pp.peak_method of
            1: begin
                peak_ifrqs = identify_peaks_by_canion(spc_thr,hs_det_thres,peak_thr_fac=pp.peak_thr/sqrt(ppar.avc),gap4dom=pp.gap4dom,peak_sig_thr=pp.peak_sig_thr)
                npeaks=n_elements(peak_ifrqs[0,*]) 
                if peak_ifrqs[0,0] ne -1 then begin
                    mpow = fltarr(npeaks)
                    for i=0,npeaks-1 do begin
                        ix4spc = ab2ix4spc(peak_ifrqs[0,i],peak_ifrqs[1,i],nfft)
                        mpow[i] = total(spcord[ix4spc] - hs_noise)
                    endfor
                    ix_sortpeaks = sort(-mpow)
                endif else npeaks=0
            end
            2:begin
                peak_ifrqs = identify_peaks_by_2ndmom(spc_thr,hs_det_thres,xord,moms,peak_thr_wider=pp.peak_thr_wider,peak_thr_higher=pp.peak_thr_higher, peak_sig_thr=pp.peak_sig_thr)
                npeaks=n_elements(peak_ifrqs[0,*])
                if peak_ifrqs[0,0] eq -1 then npeaks=0 else begin
                    mpow = reform(moms[0,*],npeaks)
                    ix_sortpeaks = lindgen(npeaks)
                endelse
            end
            else: 
        endcase
    endfor

    if finite(pp.vrange[0]) then xra=pp.vrange else xra=[-0.5,.5]*hdr.vuar
    tit = ' dsp, my, hs; pow:' + string(/pri,db(dsp_pow),db(my_pow),db(hs_pow),format='(f7.1,f7.1,f7.1)') + $
                       ' vel:' + string(/pri,dsp_vel    ,    my_vel,    hs_vel,format='(f7.3,f7.3,f7.3)')
    dati = unixtime2bdate((*p_srvinfo).TIME_T, nice=2,utc=1)
    plot,xord,db(spcord), $
        xtit='Velocity [m/s]',ytit='Spectral power, rel [dB]',tit='RG='+string(pp.rg)+tit+' '+dati, $
        yrange=pp.srange,xst=1,yst=1,psym=10, xrange=xra
    if pp.ldr && pp.chan eq 0 then begin
        spcx = (*(*p_data).(sp_tagnr[1]))[*,rg4sp]
        hs_noisex = hilde(spcx,method=pp.hilde_method, ofdiv=pp.ofdiv, ndiv=pp.ndiv, n_spcave=ppar.avc)
        hs_det_thresx = hs_noisex * ( 1 + of0/10. / sqrt(ppar.avc))
        spcxord=spcx[ix2ord]
        ix_ldr = where(spcxord gt hs_det_thresx and spcord gt hs_det_thres,nix_ldr)
        oplot,xord,db(spcxord),psym=3
        if nix_ldr gt 0 then $
            oplot,xord[ix_ldr],db(spcxord[ix_ldr]-hs_noisex)-db(spcord[ix_ldr]-hs_noise),col=5,psym=10
    endif

    ; if ix_hs[0] ge 0 then oplot,x[ix_hs],db(spc[ix_hs]),psym=2,color=2
    oplot,[-15,15],[1,1]*db(my_noise),color=1
    oplot,[-15,15],[1,1]*db(det_thres_dyn),color=1
    oplot,[-15,15],[1,1]*db(hs_noise),linest=2,color=2
    oplot,[-15,15],[1,1]*db(hs_det_thres),linest=2,color=2
    oplot,dsp_vel*[1,1],pp.srange,color=1
    oplot,hs_vel*[1,1],pp.srange,color=2,linest=2
    for i=0,npeaks-1 do begin
        ix4spc = ab2ix4spc(peak_ifrqs[0,ix_sortpeaks[i]],peak_ifrqs[1,ix_sortpeaks[i]],nfft)
        oplot,xord[ix4spc],db(spcord[ix4spc]),color=1+i mod 12,psym=10
    endfor

    if pp.nrange[1] gt pp.nrange[0] then begin
        wset,noise_win
        nspcord_co = ((*(*p_data).sp1)[*,hdr.raw_zrg])[ix2ord]
        cspcord_co = ((*(*p_data).sp1)[*,hdr.raw_zrg+1])[ix2ord]
        nspcord_cx = ((*(*p_data).sp2)[*,hdr.raw_zrg])[ix2ord]
        cspcord_cx = ((*(*p_data).sp2)[*,hdr.raw_zrg+1])[ix2ord]
        plot,xord,db( nspcord_co ),yrange=pp.nrange,xst=1,yst=1
        oplot,xord,db( nspcord_co ),color=1
        oplot,xord,db( cspcord_co ),color=2
        oplot,xord,db( nspcord_cx ),color=4
        oplot,xord,db( cspcord_cx ),color=5
        wset,spc_win
        tnsp_co[itnbuf]=total(nspcord_co)/float(nfft)
        tcsp_co[itnbuf]=total(cspcord_co)/float(nfft)
        tnsp_cx[itnbuf]=total(nspcord_cx)/float(nfft)
        tcsp_cx[itnbuf]=total(cspcord_cx)/float(nfft)
        dnsp_co[itnbuf]=(*p_srvinfo).npw1/float(nfft)
        dcsp_co[itnbuf]=(*p_srvinfo).cpw1/float(nfft)
        dnsp_cx[itnbuf]=(*p_srvinfo).npw2/float(nfft)
        dcsp_cx[itnbuf]=(*p_srvinfo).cpw2/float(nfft)
        itnbuf=itnbuf+1
    endif

    
    
    arg = myprompt('Guesss >')
artarg:
    cmd = arg[0] &  narg=n_elements(arg)
    case 1 of
        ; cmd eq 'redraw' or cmd eq 'r': begin
        ;     arg = last_draw_arg
        ;     goto,artarg
        ; end
        ; cmd eq 'guess' or cmd eq 'guesss' or cmd eq 'g':begin
        ;     ; do it
        ; end
        cmd eq '+': begin 
            if narg eq 2 then stride = long(arg[1]) else stride = 1L 
            if stride gt 1 then begin
                for i=0,stride-2 do begin
                    if fzk then begin
                        while read_mainframe_header(lun, hdr) eq 0 do begin
                            ppar = read_ppar_frame( lun, hdr)
                        endwhile
                    endif
                    read1pds_dwell, lun, hdr, ppar, p_srvinfo, p_data
                endfor
            endif
            read1dwell=1
        end
        cmd eq '-': begin 
            if narg eq 2 then stride = long(arg[1]) else stride = 1L 
            point_lun,lun,file_pos[(ipos-stride-1) > 0L]
            ipos = (ipos - stride - 1L) > 0L
            read1dwell=1
        end
        cmd eq 'u': begin 
            if narg eq 2 then stride = long(arg[1]) else stride = 1L 
            pp.rg = pp.rg + stride 
        end
        cmd eq 'd': begin 
            if narg eq 2 then stride = long(arg[1]) else stride = 1L 
            pp.rg = pp.rg - stride 
        end
        cmd eq 'postscript' or cmd eq 'ps': begin
            if narg eq 2 then psfile = arg[1] else psfile='guesss.ps'
            set_plot,'PS'
            device,/color,bits_per_pixel=8,filename=psfile ; ,yoffset= 27,xoffset=0.7
        end
        cmd eq 'close':begin
            device,/close
            set_plot,'X'
        end
        cmd eq 'png': begin
            if narg ge 2 then png_fn = arg[1] else begin
                png_fn = infile + '_' + tag_mom + '.png'
            endelse
            screen_dump = TVRD(true=1)
            WRITE_PNG, png_fn, screen_dump
        end
        cmd eq 'set': begin
            if narg ge 2 then set_tag_name = arg[1] else set_tag_name='HELP'
            if strupcase(set_tag_name) eq 'HELP' then begin
                pp_tags = tag_names(pp)
                pph_tags = tag_names(pph)
                for i=0,n_elements(pp_tags)-1 do begin
                    helptext_tag_number = (where(pph_tags eq pp_tags[i]))[0]
                    helptext = helptext_tag_number ge 0 ? pph.(helptext_tag_number) : ''
                    print_table,pp_tags[i]+':',helptext,tabw=20,linew=79
                endfor
            endif else begin
                set_tag_number = (where(strupcase(set_tag_name) eq tag_names(pp)))[0]
                if set_tag_number ge 0 then begin
                    if narg eq 2 then print,set_tag_name+' :',pp.(set_tag_number) else begin
                        for i=0,(narg-3) < (n_elements(pp.(set_tag_number))-1) do begin
                            if strupcase(arg[i+2]) ne 'SKIP' then pp.(set_tag_number)[i] = arg[i+2]
                        endfor
                    endelse
                endif else print,'unknown variable name'
            endelse
        end
        cmd eq 'info': begin
            help,hdr,/struct
            help,ppar,/struct
        end
        cmd eq 'stop': if lmgr(/vm) then goto,ende else stop
        cmd eq 'x': goto,ende
        else: begin
            setcmd = ''
            for i=0 , narg - 1 do setcmd = setcmd + arg[i] + ' '
            if lmgr(/vm) then print,'VM license can not execute '+setcmd $
            else dummi=execute( setcmd )
        end
    endcase
endwhile

ende:
close,lun
if !d.name eq 'X' then begin
    while !d.window ge 0 do wdelete
endif
if n_elements(p_srvinfo) then heap_free,p_srvinfo
if n_elements(p_data) then heap_free,p_data
end
