pro read_and_write_ppar_iq2s,lun, outlun, cocxlun, hdr, nave
        ppar = read_ppar_frame( lun, hdr)
        out_ppar = ppar
        out_ppar.osc = 2L ; set osc=2 in the output data to indicate that the file contains spectra in float format with no extra exponents.
        out_ppar.prc = 0L ; set prc=0 as the moment data which may be in the input data is striped
        out_ppar.avc = nave
        write_ppar_frame,outlun, out_ppar
        if cocxlun ne 0 then write_ppar_frame,cocxlun, out_ppar
end

pro iq2spc,arguments
; 
; Reads IQ-data from pds files (saved with osc=1) and writes spectra to the output file. 
;
; Calling Sequence: 
;
; unix> idl iq2spc infile=data/20040602_1000.pds outfile=20040602.spc.pdf
;
; IDL > iq2spc, 'infile=data/20040602_1000.pds outfile=xxx_off.pdf'
; 
; infile      File to read the pds data from. Multible files with contain successive 
;             (in time) data may be given as array of strings.
;
; outfile     File for writing the output data.
;
; See the structure pph below for a deskription of the other keyword parameters.
;
; Matthias Bauer-Pfundstein bauer@metek.de
;
common startup,startup_done
common magic4chs,magic4chs,FzkPdsFmt,syspar
if n_elements(startup_done) ne 1 then begin
@startup
endif

; Arguments can be provided by two methods:
; - If iq2spc is called by an interactive IDL session the first (and only) 
; string parameter arguments which contains a space separated list is split to
; the strarr argv.
; - Aditionally strings contained in the environment variables ARG1, ARG2, ..., 
; ARG9, ARG10, ... are prepended to argv. The script idl packs its calling arguments
; these environment variables. This is done as IDL can not extract the shell 
; variables like $1, $2,...
argv = env2argv(arguments)

kp = { $
    infile:'', $
    iam:'nobody', $   ; radar name, may be specified to evaluate data from another MIRA36
    outcocx:'', $ 
    outfile:'' $
}
argv2stru,argv,kp
kp.iam = whoami(kp.iam)

; Default values for the Ploting parameters. These Parameters can be changed by the command "set"
pp={ $
nave:200L, $ ; number of spectra averaged in-coherently
epsilon:-!pi/180.*1.2,$ ; parameter for improving DP
phase:0.0,$ ; parameter for improving DP
dummy:0$
}
argv2stru,argv,pp

cpx_epsilon = pp.epsilon * complex(cos(pp.phase*!pi/180.),sin(pp.phase*!pi/180.))

co=0
cx=1

; open the pds file and read the fileheader
if kp.infile eq '' then lun=0 else begin
    openr,lun,kp.infile,/get_lun
endelse
openw,outlun,kp.outfile,/get_lun
if kp.outcocx ne '' then openw,cocxlun,kp.outcocx,/get_lun

; read obsolete header which is in the first 1024 bytes of each file
ppar = read_hdr(lun,hdr)
write_hdr,outlun, ppar,1
write_hdr,cocxlun, ppar,1

zchan=2L ; number of channels
zrg = -1L
nfft=0L  ; nfft of the spectra in momory. If hdr.nfft differs then a new buffer for noisspectra is created.
zspectra = 0L
while not eof(lun) do begin
    ; read one dwell of iq-data
    while read_mainframe_header(lun, hdr) eq 0 do begin
        ; read the header which appears in the data each time any parameter was changed and at the begining
        read_and_write_ppar_iq2s,lun, outlun, cocxlun, hdr, pp.nave
    endwhile
    if hdr.nfft ne nfft then begin
        nfft = hdr.nfft
        iave=0L
    endif
    if zspectra ne nfft*(hdr.raw_zrg+2)*zchan then begin
        ; create some arrays
        zrg = hdr.raw_zrg+2 ; noise and calibration height are treated as the other ranges.
        zspectra = nfft*zrg*zchan
        fft_window = hanning(nfft)
        AvPowSpcs = fltarr(nfft,zchan,zrg)
        if kp.outcocx ne '' then begin
            AvCplxCoCx = complexarr(nfft,zrg)
            AvAbsCoCx = fltarr(nfft,zrg)
            CoCxOut = fltarr(nfft,zchan,zrg)
        endif
    endif
    if iave eq 0L then begin
        AvPowSpcs[*,*,*] = 0.0
        if kp.outcocx ne '' then begin
            AvCplxCoCx[*,*] = Complex(0.0,0.0)
            AvAbsCoCx[*,*] = 0.0
        endif
    endif
    if ptr_valid(p_srvinfo) then ptr_free,p_srvinfo
    if ptr_valid(p_data) then heap_free,p_data    ; replaces ptr_free,(*p_data).sp1 & ptr_free,(*p_data).sp2 & ptr_free,p_data
    ; read one IQ sequence here (nfft pulse cycles):
    read1pds_dwell, lun, hdr, ppar, p_srvinfo, p_data, extract=['IQ1','IQ2']
    if n_elements( (*(*p_data).iq1)[*,0] ) ne nfft then begin
        if eof(lun) then goto,ende
        print,'corrupted data'
        stop
    endif
    if n_elements(iq_tagnr) ne 2 then begin
        iq_tagnr = [(where(tag_names(*p_data) eq 'IQ1'))[0],(where(tag_names(*p_data) eq 'IQ2'))[0]]
    endif
    
    ; do the FFT for each spectrum
    ; help,(*(*p_data).(iq_tagnr[ichan]))
    ; <PtrHeapVar4>   COMPLEX   = Array[256, 400]
    for irg = 0, zrg-1 do begin
        iq_co = (*(*p_data).(iq_tagnr[0]))[*,irg]
        iq_cx = (*(*p_data).(iq_tagnr[1]))[*,irg] + cpx_epsilon * iq_co
        CplxSpc_co = fft(iq_co * fft_window ,-1)  
        CplxSpc_cx = fft(iq_cx * fft_window ,-1)
        PowSpc_co = real_part(CplxSpc_co)^2 + imaginary(CplxSpc_co)^2
        PowSpc_cx = real_part(CplxSpc_cx)^2 + imaginary(CplxSpc_cx)^2
        AvPowSpcs[0,co,irg] = AvPowSpcs[*,co,irg] + PowSpc_co
        AvPowSpcs[0,cx,irg] = AvPowSpcs[*,cx,irg] + PowSpc_cx
        if kp.outcocx ne '' then begin
            CplxCoCx = CplxSpc_co * conj(CplxSpc_cx)
            AvCplxCoCx[0,irg] = AvCplxCoCx[*,irg] + CplxCoCx
            AvAbsCoCx[0,irg] = AvAbsCoCx[*,irg] + abs(CplxCoCx)
        endif
        ; if irg eq 25 and ichan eq 0 then stop
    endfor  ; irg

    if ++iave ge pp.nave then begin
        ; write the AvPowSpcs to file
        len_mainframe = 8L + hdr.len_srvi + 8L + zspectra*4L + 3L * (8L) + 8L 
        edn = 1./float(iave)
        writeu,outlun,magic4chs,len_mainframe
        writeu,outlun,'SRVI',hdr.len_srvi,*p_srvinfo
        writeu,outlun,'FFTD', zspectra*4L, AvPowSpcs * edn
        writeu,outlun,'SNRD', 0L
        writeu,outlun,'VELD', 0L
        writeu,outlun,'RMSD', 0L
        writeu,outlun,'EXPD', 0L
        if kp.outcocx ne '' then begin
            for irg=0,zrg-1 do begin
                Normalization = 1. / sqrt( AvPowSpcs[*,co,irg] * AvPowSpcs[*,co,irg] )
                CoCxOut[0,co,irg] = abs(AvCplxCoCx[*,irg]) * Normalization
                CoCxOut[0,cx,irg] = atan(AvCplxCoCx[*,irg],/phase)*(180/!pi) 
            endfor
            writeu,cocxlun,magic4chs,len_mainframe
            writeu,cocxlun,'SRVI',hdr.len_srvi,*p_srvinfo
            writeu,cocxlun,'FFTD', zspectra*4L, CoCxOut
            writeu,cocxlun,'SNRD', 0L
            writeu,cocxlun,'VELD', 0L
            writeu,cocxlun,'RMSD', 0L
            writeu,cocxlun,'EXPD', 0L
        endif
        iave=0L
    endif
endwhile

ende:
close,lun
close,outlun
if ptr_valid(p_srvinfo) then heap_free,p_srvinfo
if ptr_valid(p_data) then heap_free,p_data
exit
end

