function ldr2melt,f,n=kn
common ldr2melt,n,g
default_value,kn,9L
if n_elements(n) ne 1 then begin
    n=-1L 
endif
if n ne kn then begin
   n = kn 
   g = hanning(n) - (n*0.5-1.)/float(n)
endif
sf = convol(f,g,/edge_wrap,/center)
return,sf
end

pro mmclx,arguments
;
; Calling Sequence: 
;
; unix> idl mmclx infile=data/20040602_1000.dmp outfile=20040602.9.pds nmulti=9
;
; IDL > mmclx, 'infile=data/20040602_1000.dmp outfile=xxx_off.pds'
; 
; infile      File to read the pds data from. The data must be in dmp format (dmpformat=1 in spcs2moms.pro). 
;             Multible files with contain successive 
;             (in time) data may be given as array of strings.
;
; outfile     File for writing the output data.
;
; Arguments are passes as keyword parameters in the form <keyword>=<value1>[:<value2>...]
; - If mmclx is called by an interactive IDL session all keyword parameters are 
;   passed in the first string argument as a space separated list. Each keword-value pair must
;   not contain blanks. 
; - if mmclx is called from the csh-script idl each argument ($1, $2, ...) is idl 
;   processed as one keword-value pair. Again, each keword-value pair may not contain blanks.
;
; See the comments in the definition of the structure pp below for a description of the other 
; keyword parameters.
;
; Matthias Bauer-Pfundstein
;
common startup,startup_done
common magic4chs,magic4chs,FzkPdsFmt,syspar
if n_elements(startup_done) ne 1 then begin
@startup
endif


argv = env2argv(arguments)

kp = { $
    infile:'', $
    iam:'nobody', $   ; radar name, may be specified to evaluate data from another MIRA36
    tail:0,    $
    config:'', $
    ttoutfile:'', $
    outfile:'' $
}
argv2stru,argv,kp
kp.iam = whoami(kp.iam)


; Default values for the Ploting parameters. These Parameters can be changed by the command "set"
pp={ $
dwellsperfile:18000L,$ ; Number of dwells that are saved per file. Files should not get biger than 1GByte.
FirstfileWo00:1,$  ;if set then the first file will not have the suffix .00
of0LDR:45.,$         ; For calculating SNRminLDR, which is needed to determine the minimal LDR that could be detected. of0LDR may be a bit smaller than of0.
plankHy:linear(-18.),$ ; Peaks with smaller LDR are flagged as Hy
RainLdr:linear(-20.),$ ; Only Peaks with smaller LDR are flagged as Rain. 
LowesPlanktop:-10000.,$ ; Below this range in m plankton is always expected
plankVEL:-7.7,     $ ; Peaks with fall velocities below plankVEL are never flagged as plankton 
AboveClutterRg:7, $ ; estimate the whether condition  above MeltHei but at least above AboveClutterRG
nRg4RainDet:25,   $ ; number of ranges used for rain detection
ZminRain:linear(-23.), $ ; If global dBZ values above Melting layer are grater than this threshold then an enhanced LDR may occur => plankparRain 
SNRcovered:linear(-16.), $ ; If there is almost no signal above the melting layer then plankton may occur slightly above the melting layer
SNRsaturation:linear(53.), $ ; If SNRco is above this level LDR may be overestimated. Then a large LDR is no reliable plankton indicator 
MeltHei:-90000.,$         ; height of the melting layer.
outdoortemp2metlhei:152., $ ; If outdoortemp is saved in the ps_err tag of the serverinfo structrure the MeltRgExtern is calculatete from it.
agl2asl:0.0, $ ;  The height information in the radiosoundings and in the model data are given above sea level ASL, the height in the radar data is given in AGL. agl2asl=ASLheight of the radar is used to convert between both.
TempDifMetar7Mira36:0.0, $ ; Typical difference in the temperature between the location of the radar and the metar airport (negative if airport is lower)
prefere_modle:0, $          ; If set the external temperature will be taken from the modle data, even if the radio sonde data is available.
MeltSearchbzt: 3, $       ; Set the center for searching the melting layer this number of range gates below the zero C height.
MeltLdrMin:linear(-16.),$ ; Minimal LDR for melting layer detection
MeltGradVelMin:0.4,$      ; Minimal Velocity gradient for melting layer detection
RMSrain:0.01,$       ; Peaks with lager width can savely be identified as rain. Does not work as expected!
VELrain:-1.1,$      ; Peaks with smaller velocity can safely be identified as rain. Does not work as expected!
VELrainImpl:-4.7,$  ; Peaks with smaller velocity can safely be identified as rain.
                    ; Instead Peaks with (VEL < VELrain and RMS < RMSrain)  or VEL < VELrainImpl are flaged as rain.               
; hy_absvel_max:3.5,$ ; obsolete above melting layer peaks are marked as Hy if their velocity is in the range hy_veloff+/-hy_absvel_max
; hy_veloff:-0.1,$    ; obsolete (up-drafts typically are stronger than down-drafts, this compensates typpical fall velocities.) 
noscanner:0, $        ; If set ELVPOS will be set to 90
minHyNon3hyclx:11,$      ; Peaks are passed to plank-out (SNRplank,...) if they are flagged as Plankton || if the number of plankton peaks in 
minHyNon3hyclxBT:21,$   ; the 3-D neighborhood multiplied with NplankZuNhyFac is greater than nHyNon (= number of Hy and unidentified peaks
NplankZuNhyFac:2.6,$    ; in the neighborhood) || if the nHyNon is less than minHyNon3hyclx. All other peaks are passed to hy-output (SNR,...).
aboveStrongPlank:25, $  ;  In the boundery layer minHyNon3hyclxBT is used in stead of minHyNon3hyclx. For this purpose boundery layer are the  
StrongPlankSnr:linear(3.),$; ranges where strong plankton is detected and pp.aboveStrongPlank ranges above. Strong Plankton is 
StrongPlankPercentage:1.0,$; 
nHySig:9, $            ; Peaks are also flaged as hy if more than nHySig of the peaks in the neighborhood are hy due to low LDRs.
MinRain3clust:3,$    ; The hy-Peaks are divided to rain and smaller dropelts. I there are more than MinRain3clust peaks marked as rain in 
$                    ; the 3d neighborhood then the peak is passed to the rain-output, else wise it is passed to the cl-output.
rms2d:4.5, $         ; Peaks which cannot be identified safely from their charakteristics (LDR, VEL, range, RMS) are assigned to a
$                    ; scatterer type by the cluster algorithm. For each such 'Delinquent' peak in each of his neighbor gates/dwells the 
dVELmax:0.7, $       ; closest Peak is searched. It is considered to be in the cluster when abs(VELdeli-VELngb)-(RMSdeli+RMSngb)*rms2d < dVELmax   
naveNoiseSnr:60, $     ; The quantmed from naveNoiseSnr values of pnoi is taken for the division in the re-snr calculation. 
$                      ; This prevents distortion of the reflectivity due to radiometric noise which is increased only during very strong rain.
snr_radiometer:linear(54.),$; Above this return SNR significant radiometric noise is added to the noise in noise gate.
do_radiomer:1,$          ; If 1, calculate the sliding average of noise, which might be useful for detection of radiometric noise
nav14: 60, $            ; Number of range gates beginning from the top which is used for radiometer calculation. Schould be the same as naveNoiseSnr
do_corrbycal:0,$         ; Correct the calibration based on internal calibration noise
do_hscorr:linear(30.0),$ ; if SNR > do_hscorr and do_hscorr > 0 then add the power between the raised noise floor and the receiver noise to the power of the peaks.
do_saturationcorr:1,$    ; if set, reconstruct co chan power from the assuption that LDR in rain is independant of height. 
PolDecoupl:linear(-28.),$ ; In case there are no unsatturated ranges below the melting layer where the polarization decoupling can be detected it is set to PolDecoupl
doLowRangeAttenuation:1,$ ; do low range attenuation correction
nip:0,$     ; Number of Cl-Peaks with spowes fall velocity which are passed to output.
dtime:5, $  ; Number of dwells which are taken in to account for the clustering
drg:5$      ; Number of ranges which are taken in to account for the clustering
}

config2stru,pp,'mmclx',[kp.iam+'.config',kp.config]

argv2stru,argv,pp

;case kp.iam of
;    'DWD': begin
;    end
;    'NASA':begin
;    end
;    'FZK':begin
;    end
;    'MPI':begin
;    end
;    'HALO':begin
;    end
;    'NUIG':begin
;    end
;    'CNRP':begin
;    end
;    'JUEL':begin
;    end
;    'DWD1':begin
;    end
;    else:begin
;       print,'I dont know who Iam: '+kp.iam 
;       exit
;    end
;endcase
if pp.PolDecoupl lt 0 then pp.PolDecoupl=linear(-29.)

; ############ get the information from receiver calibration by calibrated noisecom source #######################
get_noisecom,calida



plankparNorm={$
MeltSearchRadius: 18, $     ; number of rangegates above and below pp.MeltRg where melting level is searched.
MeltDiaBelow:9,$
MeltDiaAbove:8,$
plankBelow:linear(-19),$
plankIn:linear(-4), $
plankAbove:linear(-6)$
}
config2stru,plankparNorm,'plankparNorm',['plankpar']

plankparRain={$
MeltSearchRadius: 31, $ 
MeltDiaBelow:26, $
MeltDiaAbove:12,$
plankBelow:linear(-17),$
plankIn:linear(-4), $
plankAbove:linear(-6)$
}
config2stru,plankparRain,'plankparRain',['plankpar']

plankparSun={$
MeltSearchRadius: 0, $ 
MeltDiaBelow:3,$
MeltDiaAbove:12,$
plankBelow:linear(-20),$
plankIn:linear(-19), $
plankAbove:linear(-10)$
}
config2stru,plankparSun,'plankparSun',['plankpar']

plankparCold={$
MeltSearchRadius: 17, $ 
MeltDiaBelow:23,$
MeltDiaAbove:12,$
plankBelow:linear(-14),$
plankIn:linear(-5), $
plankAbove:linear(-6)$
}
config2stru,plankparCold,'plankparCold',['plankpar']

plankpar=plankparRain ; prevent crash due to undefined variables in the beginning

; Konstanten:
co=0
cx=1
nip=pp.nip ; number of Cl-Peaks which are passed to output indiviually
npeak_max=16b ; maximum number of peaks that may show up in the eformat. This parameter must be the same as in spcs2moms.
zrg = -1L

BitPlankton = 1b
BitHy = 2b
BitLDRdetectable = 8b
BitThree=16b
BitsPlkHy = BitPlankton or BitHy
BitRain=4b
dth=pp.dtime/2
drh=pp.drg/2
 
time_temp = 0UL ; time for which teperatures where retrived from the data base last. 

; if a tag in dmpstru is changed the following files must be syncronized:
; total_dmp_mie.pro
; dmp2arr_mie.pro
; spcs2dmp_mie.pro
; calc_moms_mie.pro
; mmclx_mie.pro
; ncdf/header2ncdf.pro
dmpstru = {SNR:-20000,        SNM:-20000,        LWC:-20000, RR:-20000,               LDR:-20000,        VEL:!values.f_nan, RMS: !values.f_nan, ixa:-20000, ixb:20000}

; Almost the same as dmpstru but here SNR,LWC,RR,LDR are saved as floats and lineaized. 
momstru = {SNR:!values.f_nan, SNM:!values.f_nan, LWC:!values.f_nan, RR:!values.f_nan, LDR:!values.f_nan, VEL:!values.f_nan, RMS: !values.f_nan, ixa:0, ixb:20000}
; in momstru.ixa speichert total_dmp_mie die Zahl der Peaks

if kp.ttoutfile ne '' then begin
    openw,ttout,kp.ttoutfile,/get_lun
endif else ttout=0

infiles = findfile(kp.infile)
infiles_oldsyntax = strsplit(kp.infile,':',/extract)
if n_elements(infiles_oldsyntax) gt n_elements(infiles) then infiles=infiles_oldsyntax
if infiles[0] eq '' then begin
    print,'infile not found: ' + kp.infile
    goto,ende   
endif else zfiles = n_elements(infiles)
for ifiles=0,zfiles-1 do begin
    ; open the pds file and read the fileheader
    if kp.tail eq 1 and ifiles eq zfiles-1 then begin
        spawn,"tail_mmclx "+infiles[ifiles],unit=lun
    endif else begin
        openr,lun,infiles[ifiles],/get_lun
    endelse

    ppar = read_hdr(lun,hdr)

    oisRainy=-1 & oisSunny=-1 & oisCold=-1
    ; itme is a index pointing to the current dwell. For clustering all data is saved to arrays which can hold pp.dtime dwells in a ring.
    while not eof(lun) do begin
    resume:
        ; read one dwell of data:
        ; first read its srvi header
        if hdr.sn eq 'fzk' then begin
            while (rmh=read_mainframe_header(lun, hdr)) eq 0 do begin
                ; we got a ppar header
                if rmh eq -1L then begin
                    print,'read_mainframe_header hit eof'
                    goto,ende
                endif
                ppar = read_ppar_frame( lun, hdr)
            endwhile
        endif else begin
            ; vermutlich kann die Fallunterscheidung weg
            if ~ find_sgnt(lun,magic4chs) then begin
                print,'cannot find '+magic4chs+' frame header'
                goto,ende
            endif
            siz=0L & readu,lun,siz
        endelse
        if ptr_valid(p_srvinfo) then ptr_free,p_srvinfo
        if ptr_valid(p_npeaks) then ptr_free,p_npeaks
        if ptr_valid(p_dmp_data) then ptr_free,p_dmp_data
        ; ##########  read the dmp moments here ##################################
        read1dmp_dwell_mie, lun, hdr, dmpstru, p_srvinfo, p_hsd, p_npeaks, p_dmp_data 
        if ~ ptr_valid(p_npeaks) || n_elements(*p_npeaks) ne hdr.mom_zrg then begin
            if eof(lun) then begin
                print,'read1dmp_dwell_mie could not read valid data.'
                goto,ende
            endif
            print,'corrupted data'
            goto,resume
        endif

        if syspar.fzkscanner then (*p_srvinfo).ELVPOS = (*p_srvinfo).ELVPOS + 90.
        if pp.noscanner then (*p_srvinfo).ELVPOS = 90.0

        ; ################   initialize arrays     #########################################
        if zrg ne hdr.mom_zrg then begin
            if zrg ne -1L then begin
                print,'zrg changed. That will not work. Aborting'
                goto,ende
            endif
            ; create some arrays
            zrg = hdr.mom_zrg
            zrgFlue = lindgen(zrg)
            hei = zrgFlue * hdr.delta_h + hdr.mom_h_min 
            SNRg = fltarr(zrg)
            zchan = hdr.pol +1L
            data = replicate(momstru,zrg,npeak_max,pp.dtime)   ; the moments of all peaks are saved here
            flag = replicate(0b,zrg,npeak_max,pp.dtime)        ; the results of indiviual classification of each peak is saved here
            npk = replicate(-1,zrg,pp.dtime)                   ; npk[*,itme] is the number of peakes in data at each range gate.
            hsd_noise = fltarr(zrg,zchan,pp.dtime)
            radarconstants5 = fltarr(zchan,pp.dtime)
            nHys=intarr(zrg,npeak_max)
            ttypes=intarr(zrg,npeak_max)
            nPlanktons=intarr(zrg,npeak_max)
            nRains=intarr(zrg,npeak_max)
            nNons=intarr(zrg,npeak_max)
            nFriends=intarr(zrg,npeak_max)
            ixHy=lonarr(npeak_max)
            ixRain=lonarr(npeak_max)
            ixCl=lonarr(npeak_max)
            ixPlankton=lonarr(npeak_max)
            nfft=hdr.nfft & nfftme=nfft-1 
            outGlobal = replicate(momstru,zrg,pp.dtime)
            isRainys = intarr(pp.dtime)
            itime=0L   ; the loaded data is saved to iti=itime mod pp.dtime
            otime=0L   ; the output data is clustered at oti=otime mod pp.dtime
            intus=0
            if pp.of0LDR gt 0.0 then of0LDR=pp.of0LDR else of0LDR = ppar.of0
            SNRminLDR = of0LDR / (10*nfft*sqrt(ppar.avc))
            SNRmin = ppar.of0 / (10*nfft*sqrt(ppar.avc))
            noise_snr_avgbuf = fltarr(pp.naveNoiseSnr, zchan) 
            noise_snr_avgbuf_valid = 0
            inoise=0L
            if pp.MeltHei gt -80000. then begin
                MeltRgExtern = fix((pp.MeltHei-hdr.mom_h_min)/hdr.delta_h) > 0
                print,'setting MeltRgExtern to '+string(MeltRgExtern)+' due to keyword parameter MeltHei'
            endif
            nMeltRgDet = 9
            iMeltRgDet = 0
            MeltRgDets = intarr(nMeltRgDet)
            MeltRgDetTimes = lonarr(nMeltRgDet)
            MeltRgDetGdAt=0L
            histo = load_history( 'mmclx', {noise_snr:replicate(0.0,zchan),MeltRg:-1L}, (*p_srvinfo).TIME_T, utime_shift=utime_shift)
            if histo.noise_snr[0] gt 0.0 then begin
                if utime_shift lt 3600L*12L then begin 
                    for ich=0,zchan-1 do noise_snr_avgbuf[*,ich] = histo.noise_snr[ich]
                    ;noise_snr_avgbuf_valid=1
                    noise_snr_avgbuf_valid=0 ; Damit noise_14_avgbuf und noise_snr_avgbuf gleich behandelt werden setze ich hier trotz aller Mühe noise_snr_avgbuf_valid=0
                endif
                if utime_shift lt 1800L and histo.MeltRg ne -1 then begin
                    print, 'Setting Melting range according to history file to', histo.MeltRg
                    MeltRgDetGd = histo.MeltRg   ; this gate number will be used for plankton detection. It may be changed by melting layer detection
                    MeltRgDetGdAt = (*p_srvinfo).TIME_T - utime_shift
                endif
            endif 
            srvinfos = replicate(*p_srvinfo,pp.dtime)
            sinelvs = fltarr(pp.dtime)
        endif

        ; open mmclx file
        otime7file = otime mod pp.dwellsperfile
        if otime7file eq 0L and (intus mod pp.dtime) eq 0 then begin
            FileNameNumber=otime/pp.dwellsperfile
            if FileNameNumber ge 1L and (kp.tail eq 0 or ifiles lt zfiles-1) then ncdf_close,ncdfid
            ptypes=['','g','plank','rain','cl']
            if nip gt 0 then ptypes=[ptypes,string(indgen(nip)+1,format='(i1.1)')]
            extras=['Ze','Zg','Z','RR','LWC','MeltHei','MeltHeiDB','MeltHeiDet','TEMP','HSDco','HSDcx','RadarConstCo','RadarConstCx']
            if pp.do_hscorr gt 0 then extras = [ extras, 'ISDRco','ISDRcx' ]
            if pp.do_radiomer gt 0 then extras = [ extras, 'MRMco','MRMcx' ]
            if pp.FirstfileWo00 and FileNameNumber eq 0 then begin
                ncdfFileName=kp.outfile
            endif else begin
                ncdfFileName=kp.outfile+'.'+string(FileNameNumber,format='(i2.2)')
            endelse
            ncdfid = header2ncdf(ncdfFileName,ppar,hdr,ptypes=ptypes,extras=extras)
            if kp.tail eq 1 and ifiles eq zfiles-1 then NCDF_CLOSE, ncdfid
        endif

        ; ################ reorganize the data read by read1dmp_dwell_mie to a rectangular arrays in the ring buffers data, npk, and hsd_noise #############
        iti = itime mod pp.dtime ; mit iti wird auf die Ringbuffer zugegriffen, in denen die Daten zum clustenrn abgelegt werden.
        dmp2arr_mie, p_npeaks, p_dmp_data, data, npk, itime=iti
        hsd_noise[0,0,iti] = reform(*p_hsd,zrg,zchan)
        srvinfos[iti] = *p_srvinfo
        if srvinfos[iti].ELVPOS gt 1 and srvinfos[iti].ELVPOS lt 179 then sinelv=sin(srvinfos[iti].ELVPOS*!pi/180.) else sinelv=1.
        sinelvs[iti]=sinelv
        npow = [(*p_srvinfo).npw1,(*p_srvinfo).npw2]   ; Vorsicht das gehoert immer zu den Daten vom Zeitschritt iti


        ; ################   get melting hight from external data     #########################################
        outdoortemp = float((*p_srvinfo).ps_err) - 100.
        ; print,'outdoortemp from dmp data:',outdoortemp
        if outdoortemp lt -60. or outdoortemp gt 60. then begin
            outdoortemp=!values.f_nan
        endif else begin
            outdoortemp=outdoortemp+pp.TempDifMetar7Mira36
        endelse

        if abs( (*p_srvinfo).TIME_T - time_temp ) gt 900 then begin
            time_temp = (*p_srvinfo).TIME_T
            temp = temperature(hei*sinelvs[iti],time_temp,MeltHei=pp.MeltHei,done=temp_valid,outdoortemp2metlhei=pp.outdoortemp2metlhei,prefere_modle=pp.prefere_modle,agl2asl=pp.agl2asl)
            ; print,'outdoortemp from spawn:',temp[0]
            if (temp_valid eq 1 or temp_valid eq 2) and n_elements(temp) eq zrg then begin
                ixBelowZero = where(temp le 0.0,nBelowZero)
                if nBelowZero ge 1 then MeltRgExtern=ixBelowZero[0] else MeltRgExtern=zrg-1
                MeltRgExtern = MeltRgExtern - pp.MeltSearchbzt
                ; print,'setting MeltRgExtern to '+string(MeltRgExtern)+' due to Temp profile from the database'
            endif else begin
                if finite(outdoortemp) then begin
                    ; eigentlich wäre es sinnvol statt der nächsten Zeile temperature( ... MeltHei=outdoortemp/pp.outdoortemp2metlhei) zu setzen! 
                    MeltRgExtern = round((outdoortemp * pp.outdoortemp2metlhei - hei[0]) / hdr.delta_h ) - pp.MeltSearchbzt 
                    ; print,'Setting MeltRgExtern acording to outdoortemp from the serverinfo structure:',MeltRgExtern
                endif else begin
                    ; print,'Leaving MeltRgExtern as set by the keyword parameter MeltHei as outdoortemp and DB access are not available.'
                endelse
            endelse
            if n_elements(MeltRgExtern) eq 0 then begin
                print,'All methods for getting an initial value for melting height (MeltRgExtern) failed'
                exit
            endif
        endif
        ; set MeltRgFinal acording to Data Base or Keyword parameter (MeltRgExtern) or acording to a detected Melting Layer Range (MeltRgDetGd)
        if (*p_srvinfo).TIME_T - MeltRgDetGdAt lt 7200 then MeltRgFinal=MeltRgDetGd else MeltRgFinal=MeltRgExtern
        
        ; ### Das erhoehte Rauschen, das bei starkem Signal auftritt, ist ein Teil der reflektieten Leistung,         ########
        ; ### die aber nicht Dopplerisiert im Spektrum erscheint. Die Leistung um die sich das Rauschen erhoeht muss  ########
        ; ### eigentlich den groessten Peaks, entsprechen deren Gewichte, zugeordnet werden.                          ########
        ; Das bring nur einwenig am Kompressionspunkt. Dort wäre eine steilere Korrektur sinnvoll. So wie es is bringt es ~ 4 dB dynamik.
        if pp.do_hscorr gt 0 then begin
            sgco = fltarr(zrg)
            sgcx = fltarr(zrg)
            for irg = 0, zrg-1 do begin
                if npk[irg,iti] gt 0 then begin
                    sgco[irg] = total(data[irg,0:npk[irg,iti]-1,iti].SNR)
                    ; set NaNs in data.LDR to -40 dB for avoiding further problems
                    ixnan = where(~ finite(data[irg,0:npk[irg,iti]-1,iti].LDR),nixnan)
                    if nixnan gt 0 then data[irg,ixnan,iti].LDR = 0.0001
                    snrcx_ =        data[irg,0:npk[irg,iti]-1,iti].SNR * data[irg,0:npk[irg,iti]-1,iti].LDR
                    sgcx[irg] = total(snrcx_)
                endif else begin
                    sgco[irg] = 0.
                    sgcx[irg] = 0.
                endelse
                if sgco[irg] ge pp.do_hscorr then begin 
                    data[irg,0:npk[irg,iti]-1,iti].SNR *= 1.0 + ( hsd_noise[irg,co,iti]/npow[Co] - 1.0 ) / sgco[irg]
                endif
                if sgcx[irg] ge pp.do_hscorr then begin
                    snrcx_[0:npk[irg,iti]-1]           *= 1.0 + ( hsd_noise[irg,cx,iti]/npow[Cx] - 1.0 ) / sgcx[irg]
                    data[irg,0:npk[irg,iti]-1,iti].LDR = reform( snrcx_[0:npk[irg,iti]-1] / data[irg,0:npk[irg,iti]-1,iti].SNR,1,npk[irg,iti])
                endif
            endfor
            ; if max(sgco) gt linear(61) then stop
            ISDRco = sgco * npow[Co] / hsd_noise[*,co,iti]   ; echter Signal to Noise Ratio fuer ISDR-Ausgabe
            ISDRcx = sgcx * npow[Cx] / hsd_noise[*,cx,iti]   
        endif

        ; ############## calculate sliding average of noise which might be useful for estimating radiometric noise
        ; Bei den Radaren vor Halo hat das radiometrische Rauschen den Rauschlevel in der Rauschmesshöhe vergrößert
        ; da diese in spcs2dmp fuer die Berechnung von SNR verwendet wurde, musste hier korrigiert werden.
        ; Bei den neueren Radaren sind die Pin-Modualtoren aus, während im Noise-Gate gemessen wird, und dadurch wird
        ; serverinfo.npwx nicht mehr von radiometerstrahlung beeinflusse. Statt dessen is eine ander Korrektur nötig
        ; die LNA waehend der rauschmessung nicht abgeschlossen (matched) betrieben wird. in Diesem Zustand rauscht er
        ; zu wenig. Siehe calida.CorrLNAmatch in radarconstant.pro
        if pp.do_radiomer then begin
            noise_14co = total(hsd_noise[zrg-pp.nav14:zrg-1,Co,iti])/float(pp.nav14)
            noise_14cx = total(hsd_noise[zrg-pp.nav14:zrg-1,Cx,iti])/float(pp.nav14)
            noise_snr_avgbuf[inoise,0] = npow[Co] & noise_snr_avgbuf[inoise,1] = npow[Cx]
            inoise = (inoise+1) mod pp.naveNoiseSnr
            if ~ noise_snr_avgbuf_valid then begin    ; in case no noise from history was available at startup of mmclx
                noise_snr_avgbuf[*,Co] = npow[Co]
                noise_snr_avgbuf[*,Cx] = npow[Cx]
            endif
            qm1=quantmed(noise_snr_avgbuf[*,0],0.50)  &  qm2=quantmed(noise_snr_avgbuf[*,1],0.50)
            radiometer=[noise_14co/qm1,noise_14cx/qm2]
        endif else begin
            qm1=-1.0 & qm2=-1.0
        endelse

        ; Calculate global 0-moments SNRg which are used for some rough estimations of the weather condition (isRany, isSunny).
        ; SNRg is also needed for strong rain detection (radiometric noise level).
        for irg=0, zrg-1 do begin 
            if npk[irg,iti] gt 0 then begin
                SNRg[irg] = total(data[irg,0:npk[irg,iti]-1,iti].SNR)   ; sieht aehnlich aus wie oben, aber data wurde oben korrigiert!
            endif else begin
                SNRg[irg] = 0.0
            endelse
        endfor

        ; ########## calculate the radarconstant * 1/R^2 and SnrCorfa which should be used for correcting the SNR profiles calculated by the moment estimation 
        RadarConstCo = RadarConstant( hdr, *p_srvinfo, calida, pp.do_corrbycal, pp.doLowRangeAttenuation, SnrCorfa, RadarConst5)
        ;SnrCorfa[*,*]=1.0   ;matthias: diese Zeile wieder loeschen sobald wir eine noisecom-Messung haben
        
        radarconstants5[*,iti]=RadarConst5  ; wegspeiern der Radarconstanten*SnrCorfa die in 5 km gueltig ist für die Ausgabe

        ; apply this to SNRg
        SNRg *= SnrCorfa[*,Co]
        Zeg = SNRg * RadarConstCo

        ; Estimate the weather condition. If there is strong signal above the melting layer a broad bright band can be expected               
        ; which may have an exteded region whith large LDR values. ...
        ; The LDR threshold plank is set depending on general weather condition (isRainy) and depending on the height
        ; The general weather condition is estimated from the ranges above the melting layer MeltRgX which makes it more robust against plankton.
        ; Like this unfortunately Clouds below the melting layer are missed
        MeltRgX = ((MeltRgFinal > pp.AboveClutterRg)/sinelvs[iti]) < (zrg-1L)
        ZegAbove =  Zeg[MeltRgX:(MeltRgX+pp.nRg4RainDet/sinelvs[iti]) < (zrg-1L)]
        nheavyz =  total(Zeg[pp.AboveClutterRg:80] ge linear(18.))
        isRainy = (nheavyz gt 10) || (quantmed( ZegAbove , 0.85) ge pp.ZminRain) ; isRainy rather means that there is a fair probability for a melting layer with increases LDR 
        isRainys[iti] = isRainy
        isSunny = quantmed(SNRg[(MeltRgX+1)<(zrg-2L):*],0.92) lt pp.SNRcovered ; rather means that there are no high clouds
        ; print,unixtime2bdate(srvinfos[iti].TIME_T,nice=0,utc=1),db(quantmed( Zeg[MeltRgX:(MeltRgX+pp.nRg4RainDet) < (zrg-1L)], 0.85)),db(quantmed(SNRg[(MeltRgX+9)<(zrg-2L):*],0.9))
        if finite(outdoortemp) then isCold = outdoortemp lt 7.0 else isCold = MeltRgExtern lt 20
        if isRainy ne oisRainy or isSunny ne oisSunny or isCold ne oisCold then begin
            print,unixtime2bdate(srvinfos[iti].TIME_T,nice=0,utc=1),' isRainy isSunny isCold: ', isRainy, isSunny, isCold
            oisRainy=isRainy & oisSunny=isSunny & oisCold=isCold
        endif
        case 1 of 
            isCold: plankpar=plankparCold
            isRainy: plankpar=plankparRain
            isSunny: plankpar=plankparSun
            else:    plankpar=plankparNorm
        endcase
        ;belowMeltRg = ((MeltRgFinal/sinelvs[iti] - plankpar.MeltDiaBelow/sinelvs[iti]) > 0L ) < (zrg-2)
        ;aboveMeltRg = (MeltRgFinal/sinelvs[iti] + plankpar.MeltDiaAbove/sinelvs[iti]) < (zrg-1)
        LowesPlanktopRg = long(pp.LowesPlanktop/sinelvs[iti])
        belowMeltRg = ((MeltRgFinal/sinelvs[iti] - plankpar.MeltDiaBelow/sinelvs[iti]) > LowesPlanktopRg ) < (zrg-2)   
        aboveMeltRg = ((MeltRgFinal/sinelvs[iti] + plankpar.MeltDiaAbove/sinelvs[iti]) > LowesPlanktopRg ) < (zrg-1)
        planks =  plankpar.plankBelow * (zrgFlue lt belowMeltRg) $
                + plankpar.plankIn    * (zrgFlue ge belowMeltRg and zrgFlue le aboveMeltRg) $
                + plankpar.plankAbove * (zrgFlue gt aboveMeltRg)
       

        ; ################# per-peak (individual) classification ############################################################
        ; ****************  Set some Flags: *********************************************************************************
        ; IsPlankton (bit 0)        =  peak that definitely seems to be plankton
        ;                           =  ( LDR > pp.plank  &&  LDR > LDRmin   <=>   LDR > (pp.plank > LDRmin) ) && irg < MeltRgFinal
        ;
        ; IsHy (bit 1)              =  LDR indicates that peak is no plankton ("Hydro meteor", this information is never 100 % relible) 
        ;                           =  LDR < pp.plankton && LDRmin < pp.plankton   <=> (LDR > LDRmin) < pp.plankton
        ;
        ; isRain                    =  IsHy && RMS > pp.RMSrain && VEL < pp.VELrain
        ;
        ; SNRcx = SNRco * LDR    
        ;
        ; Definition: SNR = zeroMom/srvi.NPWci     and       S2N = zeroMom/HSDiv               (ci=co or cx)
        ; The first is used for calculating cabibrated dBZ, the later is germane (relevant) for deciding if a peak is significant.
        ; Note S2NciMin = SNRminLDR     where SNRminLDR = pp.of0LDR/(10*nfft*sqrt(ppar.avc))   !!!
        ; S2Nci = SNRci / hsd_noise7n14[igr,ci,iti]
        ; From these relations the minimal detectable LDR follows:
        ; The minimal detectable LDR is most likely limited by the minimal detectable S2N in the cross channel.
        ; LDR = SNRcx / SNRco = S2Ncx * hsd_noise7n14[igr,cx,iti] / SNRco 
        ; LDRmin = SNRminLDR * hsd_noise7n14[igr,cx,iti] / SNRco
        ; ====================================================================================================================
        ; LDRmin = (SNRminLDR/SNRco) * hsd_noise7n14[igr,cx,iti] where SNRminLDR = pp.of0LDR/(10*nfft*sqrt(ppar.avc)) 
        ; ====================================================================================================================
        
        flag[*,*,iti] = 0b
        for irg=0, zrg-1 do begin 
            for ipeak=0, npk[irg,iti]-1 do begin
                data[irg,ipeak,iti].SNR *= SnrCorfa[irg,Co]
                data[irg,ipeak,iti].SNM *= SnrCorfa[irg,Co]
                data[irg,ipeak,iti].LWC *= SnrCorfa[irg,Co]
                data[irg,ipeak,iti].RR  *= SnrCorfa[irg,Co]
                data[irg,ipeak,iti].LDR *= SnrCorfa[irg,Cx]/SnrCorfa[irg,Co]
                SNRco = data[irg,ipeak,iti].SNR
                LDR = data[irg,ipeak,iti].LDR
                ; the minimal power in cross channel which is significant depends on the witdth of the range in wich SNRcx was estimated
                ixb9ixa = (data[irg,ipeak,iti].ixb-data[irg,ipeak,iti].ixa+nfft) mod nfft
                LDRmin = sqrt(ixb9ixa) * SNRminLDR * hsd_noise[irg,cx,iti] / (SNRco * npow[Cx])
                ; print,'SNRco=',db(SNRco),' LDR=',LDR,' LDRmin=',LDRmin,' ipeak=',ipeak 
                LDRdetectable = LDR gt LDRmin
                if finite(LDR) ne 1 then begin
                    LDR = LDRmin 
                    ; data[irg,ipeak,iti].LDR = LDR   ; probably not neccessary   test!!!!
                endif
                ; IsHy      = LDR lt pp.plankHy && LDRdetectable
                IsHy       = LDRdetectable ? LDR lt pp.plankHy : LDRmin le pp.plankHy
                ; if IsHy and LDR gt pp.plankHy then stop
                
                vertically = (sinelvs[iti] gt 0.994 ) 
                notvertically = vertically ne 1
                fallVEL = data[irg,ipeak,iti].VEL * vertically    ; fall velocity is set to zero if the elevation deviates more than 2 deg from vertically
                IsPlankton = LDR ge planks[irg] && LDRdetectable && fallVEL ge pp.plankVEL && SNRg[irg] lt (0.9*pp.SNRsaturation)
                if irg lt aboveMeltRg then begin
                    IsRain = (IsPlankton ne 1) $
                              && ( (data[irg,ipeak,iti].RMS ge pp.RMSrain && ((data[irg,ipeak,iti].VEL lt pp.VELrain) || notvertically ) ) $
                                          || (vertically && data[irg,ipeak,iti].VEL lt pp.VELrainImpl) $
                                        ;  || Ze ge pp.Ze_rain    ; aus mmclx910 kopieren.
                                 )
                endif else IsRain=0
                flag[irg,ipeak,iti] = BitPlankton*IsPlankton or BitHy * IsHy or IsRain * BitRain $
                    or LDRdetectable * BitLDRdetectable 
                ;if IsPlankton eq 0 then stop
            endfor
        endfor
        
        ; Evaluate the global moments (above only the global SNRg was calculated which was needed in advance for estimating the weathercondition)
        for irg=0, zrg-1 do begin 
            ; filter some peaks which are very definitely no meteorological signal?
            ; nothing is filtered at the moment as ThreePathEchoes are not filtered any more.  
            if npk[irg,iti] gt 0 then begin
                outGlobal[irg,iti] = total_dmp_mie(data[irg,0:npk[irg,iti]-1,iti], (flag[irg,0:npk[irg,iti]-1,iti] and BitLDRdetectable) ne 0)
            endif else begin
                outGlobal[irg,iti] = momstru
            endelse
        endfor

        intus = (intus+1) < pp.dtime  ; so viel Zeitschritte hat der cluster buffer intus
        
        ; ############################# count how many neighbouring peaks have been given the different individual (per-peak) classifications  ################
        while intus eq pp.dtime && ( otime le ( itime - dth ) || (eof(lun) && (otime le itime) ) ) do begin
            ; Note, Except at the beginning and at the end of the data this while loop is executed exactly once.
            oti = otime mod pp.dtime
            
            ;-; Calculate the variabiliti of LDR
            ;-outV = replicate(momstru,zrg)
            ;-SNR5 = outGlobal[0:zrg-1,*].SNR - smooth(outGlobal[0:zrg-1,*].SNR,[pp.drg,pp.dtime-1],/edge_trunc,/nan)
            ;-LDR5 = outGlobal[0:zrg-1,*].LDR - smooth(outGlobal[0:zrg-1,*].LDR, [pp.drg,pp.dtime-1],/edge_trunc,/nan)
            ;-otis = [otime+(pp.dtime-1),otime,(otime+1)] mod pp.dtime
            ;-for irg=0L,zrg-1 do begin
            ;-    rgc = irg > drh < (zrg-1L-drh) & rg0=rgc-drh & rg1=rgc+drh
            ;-    outV[irg].SNR = sqrt(avg((SNR5[rg0:rg1,otis])^2)) / outGlobal[irg,oti].SNR
            ;-    outV[irg].LDR = sqrt(avg((LDR5[rg0:rg1,otis])^2)) 
            ;-endfor

            ; Count the number of neighbors of different types.
            ; For each delinquent peak (irg,ipeak,oti) for which we are trying to assign a target type (plankton, hy, rain, cl) first in
            ; each neighbor heigth-time-point the ipeak matching best by velocity is searched. These peaks will be denoted as three
            ; dimensional neighbors as they are neighbors in time, height and velocity. In pp.dtime and pp.drg determine how many 
            ; neighbors in in time and height are accounted for. In the velocity dimension only the closes neighbor is accounted for.
            ; This choice has been made for feasibility.
            rgtop=zrg-1 
            rgbot=0L
            for irg=rgbot,rgtop do begin
                rgc = irg > (rgbot+drh) < (rgtop-drh) & rg0=rgc-drh & rg1=rgc+drh
                for ipeak=0,npk[irg,oti]-1 do begin
                    if ~ (flag[irg,ipeak,oti] and BitPlankton) then begin   ; save CPU time, if a peak is marked as plankton by its own moments then it will be output as plankton regardless of its neighbors
                        VELx=data[irg,ipeak,oti].VEL & RMSx=data[irg,ipeak,oti].RMS 
                        nHy=0 ; Counter for the number of 'similar' hy peaks in the neighborhood. 
                        nRain=0 ; Counter for the number of 'similar' Rain peaks in the neighborhood. 
                        nPlankton=0 ; Counter for the number of 'similar' plankton peaks in the neighborhood. 
                        nNon=0 ; Counter for the number of 'similar' peaks in the neighborhood that are neither hy nore plankton 
                        nFriend=0 ; Counter for the number Friends. Friend Peaks are both:
                                ;                 closest to the delinquent on the VEL-axis within the peaks at the test pos ngbrg,ngbti
                                ;         and     the delinquent is the closest to the Friend within the peaks of the delinquent position.
                        for ngbrg=rg0,rg1 do begin
                            for ngbti=0,pp.dtime-1 do begin
                                if (npk_ngb=npk[ngbrg,ngbti]) gt 0 $
                                        && ~ (ngbrg eq irg && ngbti eq oti $ ; wichtig? Zeitintensiv?
                                        ) then begin
                                    dVEL = min(abs(data[ngbrg,0:npk_ngb-1,ngbti].VEL-VELx),ixdVEL)
                                    if ((dVEL-(data[ngbrg,ixdVEL,ngbti].RMS+RMSx)*pp.rms2d)>0) le pp.dVELmax then begin
                                        flagNgb = flag[ngbrg,ixdVEL,ngbti]
                                        nHy += (flagNgb and BitHy) ne 0
                                        nRain += (flagNgb and BitRain) ne 0
                                        nPlankton += (flagNgb and BitPlankton) ne 0
                                        nNon += ~ ( flagNgb and BitsPlkHy )
                                        ;++; very expensive search for reproducable multi peaks ()
                                        ;++dummy=min(abs(data[ngbrg,ixdVEL,ngbti].VEL-data[irg,0:npk[irg,oti]-1,oti].VEL),ixRival)
                                        ;++if ixRival eq ipeak then nFriend++ 
                                    endif
                                endif
                            endfor
                        endfor
                        nHys[irg,ipeak] = nHy
                        nPlanktons[irg,ipeak] = nPlankton
                        nRains[irg,ipeak] = nRain
                        nNons[irg,ipeak] = nNon
                        nFriends[irg,ipeak] = nFriend
                    endif else begin
                        nHys[irg,ipeak] = 0
                        nPlanktons[irg,ipeak] = 0
                        nNons[irg,ipeak] = 0
                        nFriends[irg,ipeak] = 0
                    endelse
                endfor
            endfor

            ; Evaluate average Power and LDR of the peaks flagged as Plankton and use it to change the cluster threshold minHyNon3hyclx
            minHyNon3hyclx=replicate(pp.minHyNon3hyclx, zrg)
            nPlanks=fltarr(zrg) & SnrPlanks=fltarr(zrg)  ; these arrays are onla for visualization during parameter adjustment.
            ;--LdrPlanks=fltarr(zrg) 
            for irg=0,zrg-1 do begin
                nPlank=0 & SnrPlank=0.0 
                ;-- LdrPlank=0.0 
                for ngbti=0,pp.dtime-1 do begin
                    plankocured=0
                    for ipeak=0,npk[irg,ngbti]-1 do begin
                        if flag[irg,ipeak,ngbti] and BitPlankton ne 0 then begin
                            SnrPlank += data[irg,ipeak,ngbti].SNR
                            plankocured=1
                            ;-- LdrPlank += data[irg,ipeak,ngbti].LDR
                        endif
                    endfor
                    if plankocured then nPlank++ 
                endfor
                nPlanks[irg]=nPlank
                if nPlank gt 0 then begin
                    SnrPlanks[irg] = SnrPlank/nPlank 
                    ;-- LdrPlanks[irg] = LdrPlank/nPlank
                    if SnrPlanks[irg] gt pp.StrongPlankSnr and nPlank ge pp.dtime*pp.StrongPlankPercentage then begin
                        minHyNon3hyclx[irg:(irg+pp.aboveStrongPlank/sinelvs[iti])<(zrg-1)] = pp.minHyNon3hyclxBT
                    endif
                    ;-- detect average LDR of plankton and deduce plank below from it.
                endif
            endfor
            

            ; Now assign the peaks of each range gate at the current point in time (oti) to these target types: 
            ;     1. Plankton 
            ;     2. Hy    
            ;             2.1 Rain
            ;             2.2 Cl
            ; Stars are not passed to the output
            outHy = replicate(momstru,zrg)
            outPlankton = replicate(momstru,zrg)
            outRain = replicate(momstru,zrg)
            outCl = replicate(momstru,zrg)
            if nip gt 0 then outClip = replicate(momstru,zrg,nip)
            for irg=0,zrg-1 do begin
                nixHy = 0 
                nixPlankton=0
                nixRain=0
                nixCl=0
                nixMelt=0
                for ipeak=0,npk[irg,oti]-1 do begin
                    flagx = flag[irg,ipeak,oti]
                    nHyNonx = nHys[irg,ipeak] + nNons[irg,ipeak]
                    probablyHy = ((flagx and BitHy) ne 0) && nHys[irg,ipeak] ge pp.nHySig
                    if (probablyHy eq 0) $
                            && ( $
                                ( (flagx and BitPlankton) ne 0 )  $
                                || ( $
                                        nHyNonx le (minHyNon3hyclx[irg] > (pp.NplankZuNhyFac*nPlanktons[irg,ipeak])) $
                                    ) $
                            ) $
                            then begin
                        ixPlankton[nixPlankton++]=ipeak
                        ttype = 0
                    endif else begin
                        ;if data[irg,ipeak,oti].LDR gt linear(-21) then stop
                        ixHy[nixHy++] = ipeak
                        if (flagx and BitRain) ne 0 || nRains[irg,ipeak] ge pp.MinRain3clust then begin
                            ixRain[nixRain++] = ipeak 
                            ttype = 3
                        endif else begin
                            ixCl[nixCl++] = ipeak
                            ttype = 1
                        endelse
                    endelse
                    ttypes[irg,ipeak] = ttype
                endfor ; ipeak

                ;  #################   integrate the three moments and LDR of the peaks of each type #########################
                if nixRain gt 0 then begin
                    iix=ixRain[0:nixRain-1]
                    outRain[irg] = total_dmp_mie(data[irg,iix,oti], (flag[irg,iix,oti] and BitLDRdetectable) ne 0)
                endif
                if nixCl gt 0 then begin
                    iix=ixCl[0:nixCl-1]
                    outCl[irg] = total_dmp_mie(data[irg,iix,oti], (flag[irg,iix,oti] and BitLDRdetectable) ne 0)
                endif
                if nixHy gt 0 then begin
                    iix=ixHy[0:nixHy-1]
                    outHy[irg] = total_dmp_mie(data[irg,iix,oti], (flag[irg,iix,oti] and BitLDRdetectable) ne 0)
                    ; search for a Peak which can be used as indicator for the vertical wind speed (wInd).
                    ; experimentally pass nip peaks which are the top candidates
                    if nip gt 0 then begin
                        iisx = sort(-data[irg,iix,oti].VEL)
                        if n_elements(iisx) gt nip then begin
                            iisx=iisx[0:nip-1L]
                            nixC=nip-1L
                        endif else nixC=nixHy-1L
                        outClip[irg,0:nixC] = data[irg,iisx,oti] 
                        ix9Ldr = where((flag[irg,iisx,oti] and BitLDRdetectable) eq 0,nix9Ldr)
                        if nix9Ldr gt 0 then outClip[irg,ix9Ldr].LDR = !values.f_nan 
                        outClip[irg,0:nixC].LWC = nFriends[irg,iisx] 
                        outClip[irg,0:nixC].RR = nHys[irg,iisx] + nNons[irg,iisx]
                    endif
                endif
                if nixPlankton gt 0 then begin
                    iix=ixPlankton[0:nixPlankton-1]
                    outPlankton[irg] = total_dmp_mie(  data[irg,iix,oti], (flag[irg,iix,oti] and BitLDRdetectable) ne 0  )
                endif
            endfor ; irg

            if ttout ne 0 then begin
                writeu,ttout,'TTOUT'
                writeu,ttout,zrg
                writeu,ttout,srvinfos[oti].TIME_T
                for irg=0,zrg-1 do begin
                    writeu,ttout,npk[irg,oti]
                    for ipeak=0,npk[irg,oti]-1 do begin
                        writeu,ttout,data[irg,ipeak,oti].ixa
                        writeu,ttout,data[irg,ipeak,oti].ixb
                        writeu,ttout,ttypes[irg,ipeak]
                    endfor
                endfor
            endif

            ; ###################### satturation correction  #####################################################
            if pp.do_saturationcorr and hdr.att eq 0 then begin
                ; If the receiver is saturated below the melting layer then reconstruct the co-channel SNR by the assumption 
                ; that LDR is constant.
                minRangeFulSens=400.
                LowestRg9att=round((minRangeFulSens-hdr.MOM_H_MIN)/hdr.DELTA_H) > 0L
                BelowMelt=(MeltRgFinal - long(300./hdr.DELTA_H)) > 0L
                if LowestRg9att lt BelowMelt then ixSatByRain = where ( outHy[LowestRg9att:BelowMelt].SNR gt pp.SNRsaturation, nixSatByRain ) else nixSatByRain=0
                if nixSatByRain gt 2 then begin
                    ; find the smallest LDR below the melting lager which where the co-channel is not saturated:
                    LDRrain = pp.PolDecoupl < min((outHy[LowestRg9att:BelowMelt].LDR),ixminldr) & ixminldr += LowestRg9att-1
                    ; correct the combined Hy profile
                    SatCorfa = outHy[0:ixminldr].LDR/LdrRain
                    outHy[0:ixminldr].SNR *= SatCorfa
                    outHy[0:ixminldr].SNM *= SatCorfa
                    outHy[0:ixminldr].LDR = LdrRain

                    ; correct the Rain profile
                    outRain[0:ixminldr].SNR *= SatCorfa
                    outRain[0:ixminldr].SNM *= SatCorfa
                    outRain[0:ixminldr].RR  *= SatCorfa
                    outRain[0:ixminldr].LWC *= SatCorfa
                    outRain[0:ixminldr].LDR = LdrRain
                    ; !!! the other profiles (e.g. Cl, and Plank Global) are neglegted yet
                endif
            endif ; pp.do_saturationcorr

            outZg =         RadarConstCo * outGlobal[0:zrg-1,oti].SNR
            outZe =         RadarConstCo * outHy[*].SNR
            outZ  =         RadarConstCo * outHy[*].SNM
            outRR = 0.524 * RadarConstCo * outRain[*].RR
            outLWC = 0.524 * RadarConstCo * outRain[*].LWC ; der Faktor rohwasser fehlt!!!
            
            ; *****************************************************
            ; Find the melting layer from the hydro meteor data
            ; *****************************************************
            hshift=1
            ixvel = lindgen(zrg-hshift) & ixldr=ixvel+hshift 
            LdrShifedOrg = outHy[ixldr].LDR
            LdrShifed = 1.1 * ldr2melt(LdrShifedOrg,n=round(11./sqrt(sinelvs[oti])))
            LdrMinEnv = fltarr(zrg-hshift) 
            for i=0,zrg-hshift-1 do begin 
                LdrMinEnv[i] = min(LdrShifedOrg[0>(i-18):(zrg-hshift-1)<(i+18)])
            endfor
            VelShifted = outHy[ixvel].VEL
            GradVel = gsmooth(mderiv(VelShifted),n=5)
            ixMeltRg = where( $
                        GradVel * LdrShifed gt pp.MeltGradVelMin*pp.MeltLdrMin*1.8 $  
                    and GradVel             gt pp.MeltGradVelMin $
                    and           LdrShifed gt                   pp.MeltLdrMin $
                    and LdrMinEnv lt linear(-21.) $
                    and abs(gsmooth(VelShifted,n=3)+2.2) le 2.8 $
                    and outHy[ixldr].SNR gt SNRmin*pp.plankHy $   ; die Leistung is gross genug, so dass man zwischen Hy und Plank unters
                    and zrgFlue lt (MeltRgExtern+plankpar.MeltSearchRadius)/sinelvs[oti] $
                    and zrgFlue gt (MeltRgExtern-plankpar.MeltSearchRadius)/sinelvs[oti] $
            , nix)
    ;       if strmid(unixtime2bdate(srvinfos[oti].TIME_T,nice=0,utc=1),6,4) ge '1114' then stop
            if nix ge 2 and isRainys[oti] then begin
                MeltRgDetK = ( fix(konsensus(ixMeltRg,8,in_clust=nMeltK)) )*sinelvs[oti]
            endif else nMeltK=0
            if nMeltK ge 2 then begin
                MeltRgDet = MeltRgDetK
                MeltRgDets[iMeltRgDet] = MeltRgDet
                MeltRgDetTimes[iMeltRgDet] = srvinfos[oti].TIME_T
                iMeltRgDet = ++iMeltRgDet mod nMeltRgDet
                ixMeltRgDetCluster = where(abs(srvinfos[oti].TIME_T - MeltRgDetTimes) lt 1200 and abs(MeltRgDets-MeltRgDet) lt 4, nixMeltRgDetCluster)
                if nixMeltRgDetCluster ge 6 then begin
                    ; For the next plankton detection the detected meling layer height will be used if the following line is active:
                    MeltRgDetGd = MeltRgDet
                    MeltRgDetGdAt = srvinfos[oti].TIME_T
                endif
                print,unixtime2bdate(srvinfos[oti].TIME_T,nice=0,utc=1),' MltRgSrchCnt:' , fix(MeltRgExtern), ' det.:' , fix(MeltRgDet),' nk=',byte(nMeltK),' ncl=',byte(nixMeltRgDetCluster)
            endif else MeltRgDet=-1.
            
            if kp.tail eq 1 and ifiles eq zfiles-1 then ncdfid = NCDF_OPEN( ncdfFileName, /WRITE )
            ; write result in cdf format. 
            otime7file = otime mod pp.dwellsperfile
            srvinfo2ncdf,ncdfid,otime7file,srvinfos[oti],hdr.fzk100
            
            ; Hydro meteors peaks
            NCDF_VARPUT, ncdfid, 'SNR', outHy[0:zrg-1].SNR, count=[zrg,1], offset=[0,otime7file]
            NCDF_VARPUT, ncdfid, 'VEL', outHy[0:zrg-1].VEL, count=[zrg,1], offset=[0,otime7file]
            NCDF_VARPUT, ncdfid, 'RMS', outHy[0:zrg-1].RMS, count=[zrg,1], offset=[0,otime7file]
            NCDF_VARPUT, ncdfid, 'LDR', outHy[0:zrg-1].LDR, count=[zrg,1], offset=[0,otime7file]
            NCDF_VARPUT, ncdfid, 'NPK', outHy[0:zrg-1].ixa, count=[zrg,1], offset=[0,otime7file]
            
            ; Global moments
            NCDF_VARPUT, ncdfid, 'SNRg', outGlobal[0:zrg-1,oti].SNR, count=[zrg,1], offset=[0,otime7file]
            ; NCDF_VARPUT, ncdfid, 'SNMg', outGlobal[0:zrg-1,oti].SNM, count=[zrg,1], offset=[0,otime7file]
            NCDF_VARPUT, ncdfid, 'VELg', outGlobal[0:zrg-1,oti].VEL, count=[zrg,1], offset=[0,otime7file]
            NCDF_VARPUT, ncdfid, 'RMSg', outGlobal[0:zrg-1,oti].RMS, count=[zrg,1], offset=[0,otime7file]
            NCDF_VARPUT, ncdfid, 'LDRg', outGlobal[0:zrg-1,oti].LDR, count=[zrg,1], offset=[0,otime7file]
            NCDF_VARPUT, ncdfid, 'NPKg', outGlobal[0:zrg-1,oti].ixa, count=[zrg,1], offset=[0,otime7file]
            
            ; rain peaks
            NCDF_VARPUT, ncdfid, 'SNRrain', outRain[0:zrg-1].SNR, count=[zrg,1], offset=[0,otime7file]
            NCDF_VARPUT, ncdfid, 'VELrain', outRain[0:zrg-1].VEL, count=[zrg,1], offset=[0,otime7file]
            NCDF_VARPUT, ncdfid, 'RMSrain', outRain[0:zrg-1].RMS, count=[zrg,1], offset=[0,otime7file]
            NCDF_VARPUT, ncdfid, 'LDRrain', outRain[0:zrg-1].LDR, count=[zrg,1], offset=[0,otime7file]
            
            ; Hydro meteor peaks that are no rain peaks
            NCDF_VARPUT, ncdfid, 'SNRcl', outCl[0:zrg-1].SNR, count=[zrg,1], offset=[0,otime7file]
            NCDF_VARPUT, ncdfid, 'VELcl', outCl[0:zrg-1].VEL, count=[zrg,1], offset=[0,otime7file]
            NCDF_VARPUT, ncdfid, 'RMScl', outCl[0:zrg-1].RMS, count=[zrg,1], offset=[0,otime7file]
            NCDF_VARPUT, ncdfid, 'LDRcl', outCl[0:zrg-1].LDR, count=[zrg,1], offset=[0,otime7file]
            
            ; Candidates for wInd peaks.
            for ip=0,nip-1 do begin
                sip=string(ip+1,format='(i1.1)')
                NCDF_VARPUT, ncdfid, 'SNR'+sip, outClip[0:zrg-1,ip].SNR, count=[zrg,1], offset=[0,otime7file]
                NCDF_VARPUT, ncdfid, 'VEL'+sip, outClip[0:zrg-1,ip].VEL, count=[zrg,1], offset=[0,otime7file]
                NCDF_VARPUT, ncdfid, 'RMS'+sip, outClip[0:zrg-1,ip].RMS, count=[zrg,1], offset=[0,otime7file]
                NCDF_VARPUT, ncdfid, 'LDR'+sip, outClip[0:zrg-1,ip].LDR, count=[zrg,1], offset=[0,otime7file]
                ;++ NCDF_VARPUT, ncdfid, 'NF'+sip, outClip[0:zrg-1,ip].LWC, count=[zrg,1], offset=[0,otime7file]
                NCDF_VARPUT, ncdfid, 'NN'+sip,  outClip[0:zrg-1,ip].RR, count=[zrg,1], offset=[0,otime7file]
            endfor
            
            NCDF_VARPUT, ncdfid, 'SNRplank', outPlankton[0:zrg-1].SNR, count=[zrg,1], offset=[0,otime7file]
            NCDF_VARPUT, ncdfid, 'VELplank', outPlankton[0:zrg-1].VEL, count=[zrg,1], offset=[0,otime7file]
            NCDF_VARPUT, ncdfid, 'RMSplank', outPlankton[0:zrg-1].RMS, count=[zrg,1], offset=[0,otime7file]
            NCDF_VARPUT, ncdfid, 'LDRplank', outPlankton[0:zrg-1].LDR, count=[zrg,1], offset=[0,otime7file]
            NCDF_VARPUT, ncdfid, 'NPKplank', outPlankton[0:zrg-1].ixa, count=[zrg,1], offset=[0,otime7file]

            ; extras
            NCDF_VARPUT, ncdfid, 'Zg', outZg, count=[zrg,1], offset=[0,otime7file]
            NCDF_VARPUT, ncdfid, 'Ze', outZe, count=[zrg,1], offset=[0,otime7file] 
            NCDF_VARPUT, ncdfid, 'Z', outZ, count=[zrg,1], offset=[0,otime7file] 
            NCDF_VARPUT, ncdfid, 'RR', outRR, count=[zrg,1], offset=[0,otime7file] 
            NCDF_VARPUT, ncdfid, 'LWC', outLWC, count=[zrg,1], offset=[0,otime7file] 
            NCDF_VARPUT, ncdfid, 'TEMP', temp, count=[zrg,1], offset=[0,otime7file] 
            NCDF_VARPUT, ncdfid, 'MeltHei', MeltRgFinal*hdr.delta_h+hdr.mom_h_min, count=[1], offset=[otime7file]
            NCDF_VARPUT, ncdfid, 'MeltHeiDet', MeltRgDet ge 0 ? MeltRgDet*hdr.delta_h+hdr.mom_h_min : -1, count=[1], offset=[otime7file]
            NCDF_VARPUT, ncdfid, 'MeltHeiDB', MeltRgExtern*hdr.delta_h+hdr.mom_h_min, count=[1], offset=[otime7file]
            if pp.do_hscorr gt 0 then begin
                NCDF_VARPUT, ncdfid, 'ISDRco', ISDRco, count=[zrg,1], offset=[0,otime7file]
                NCDF_VARPUT, ncdfid, 'ISDRcx', ISDRcx, count=[zrg,1], offset=[0,otime7file]
            endif
            if pp.do_radiomer then begin
                NCDF_VARPUT, ncdfid, 'MRMco', radiometer[Co], count=[1], offset=[otime7file]
                NCDF_VARPUT, ncdfid, 'MRMcx', radiometer[Cx], count=[1], offset=[otime7file]
            endif
            NCDF_VARPUT, ncdfid, 'HSDco', hsd_noise[*,co,oti], count=[zrg,1], offset=[0,otime7file]
            NCDF_VARPUT, ncdfid, 'HSDcx', hsd_noise[*,cx,oti], count=[zrg,1], offset=[0,otime7file]

            NCDF_VARPUT, ncdfid, 'RadarConstCo', radarconstants5[co,oti], count=[1], offset=[otime7file]
            NCDF_VARPUT, ncdfid, 'RadarConstCx', radarconstants5[cx,oti], count=[1], offset=[otime7file]

            if kp.tail eq 1 and ifiles eq zfiles-1 then NCDF_CLOSE, ncdfid
            
            otime++ 
        endwhile
        itime++
    endwhile

    free_lun,lun
endfor ; ifile

ende:
if n_elements(histo) gt 0 then begin
    histo.noise_snr = [qm1,qm2]
    if srvinfos[oti].TIME_T-MeltRgDetGdAt lt 3600 then histo.MeltRg = MeltRgDetGd else histo.MeltRg=-1
    update_history, srvinfos[oti].TIME_T, histo
endif
if kp.tail ne 1 then NCDF_CLOSE, ncdfid 
if ptr_valid(p_srvinfo) then heap_free,p_srvinfo
if ptr_valid(p_data) then heap_free,p_data
if ttout ne 0 then free_lun,ttout
exit
end
