function header2ncdf,fn,ppar,hdr,extras=extras,ptypes=ptypes
    if n_elements(extras) eq 0 then extras=['']
    ; if n_elements(ptypes) eq 0 then ptypes=['']
    ;  Create the NetCDF file

    lopa={$
        convention:'CF-1.0',$
        location:hdr.place,$ ; string(hdr.PLACE) from /icfg/header.ini
        Altitude:'Unknown',$
        Latitude:'Unknown',$
        Longitude:'Unknown',$
        system:hdr.name,$
        title:'MIRA-36 Cloud Radar Data',$
        institution:'Unknown',$
        source:hdr.name,$ ; put the pds file name created by the radarserver here
        reference:'Ka Band Cloud Radar MIRA-36, METEK GmbH www.metek.de',$
        Copywright:'Any use of these data is subject to a copyright agreement with the copyright owner',$
        Copywright_Owner:'Unknown'$
    }
common magic4chs,magic4chs,FzkPdsFmt,syspar
    config2stru,lopa,'header2ncdf',[syspar.iam+'.config']

    ; try to get some information like latitude out of the decr field 
    ; which is copied from /icfg/header.ini to the pds files by the radarserver
    tags = tag_names(lopa) 
    desc=hdr.descr
    kpos = strpos(desc,'#')
    if kpos ge 0 then desc=strmid(desc,0,kpos)
    descriptions=strsplit(desc,':',/extract)
    ndes=n_elements(descriptions)
    for ides=0,ndes-1 do begin
        rx=stregex(descriptions[ides],'[       ]*([A-Za-z][A-Za-z_0-9]*)[      ]*=(.*)$',/extr,/sub)
        if n_elements(rx) eq 3 then begin
            key = rx[1]
            val = rx[2]
            tagnumber_of_key = (where(strupcase(key) eq tags))[0]
            if tagnumber_of_key ge 0 then begin
                lopa.(tagnumber_of_key) = val
            endif
        endif
    endfor


    Id = NCDF_CREATE(fn,/CLOBBER)

    ; Create some global attributes
    NCDF_ATTPUT, Id,  'convention',      lopa.convention, /global, /char
    NCDF_ATTPUT, Id,  'location',        lopa.location, /global, /char                  ; string(hdr.PLACE) from /icfg/header.ini
    NCDF_ATTPUT, Id,  'Altitude',        lopa.Altitude, /global, /char
    NCDF_ATTPUT, Id,  'Latitude',        lopa.Latitude, /global, /char
    NCDF_ATTPUT, Id,  'Longitude',       lopa.Longitude, /global, /char
    NCDF_ATTPUT, Id,  'system',          lopa.system, /global, /char
    NCDF_ATTPUT, Id,  'title',           lopa.title, /global, /char
    NCDF_ATTPUT, Id,  'institution',     lopa.institution, /global, /char
    NCDF_ATTPUT, Id,  'source',          lopa.source, /global, /char     ; put the pds file name created by the radarserver here
    NCDF_ATTPUT, Id,  'reference',       lopa.reference, /global, /char
    NCDF_ATTPUT, Id,  'ppar',            stru2str(ppar), /global, /char
    NCDF_ATTPUT, Id,  'hrd',             stru2str(hdr), /global, /char
    NCDF_ATTPUT, Id,  'Copywright',      lopa.Copywright, /global, /char
    NCDF_ATTPUT, Id,  'Copywright_Owner',lopa.Copywright_Owner, /global, /char


    ;  Create dimensions
    zrg = hdr.MOM_ZRG
    nfft = hdr.nfft
    Dim1   = NCDF_DIMDEF(Id,'time', 0)   ; 0 gives the variable the status unlimited
    Dim2   = NCDF_DIMDEF(Id,'range',zrg)
    if max(extras eq 'SPC') then Dim3 = NCDF_DIMDEF(Id,'doppler',nfft)

    ;  Create variables for the most important informations of PPAR
    VarId   = NCDF_VARDEF(Id,'nfft', /Long)
    NCDF_ATTPUT, Id, 'nfft',     'long_name', 'Number of FFT Points' ; hdr.NFFT
    NCDF_ATTPUT, Id, 'nfft',     'units', 'count'

    VarId   = NCDF_VARDEF(Id,'prf', /Long)
    NCDF_ATTPUT, Id, 'prf',     'long_name', 'Pulse Repetition Frequency' ; hdr.prf
    NCDF_ATTPUT, Id, 'prf',     'units', 'Hz'

    VarId   = NCDF_VARDEF(Id,'NyquistVelocity', /FLOAT)
    NCDF_ATTPUT, Id, 'NyquistVelocity', 'long_name', 'Speed of Light  * Pulse Repetition Frequency / (4 * Frequency)' ; hdr.VNY
    NCDF_ATTPUT, Id, 'NyquistVelocity', 'units', 'm s-1' 

    VarId   = NCDF_VARDEF(Id,'nave', /Long)
    NCDF_ATTPUT, Id, 'nave',     'long_name', 'Number of Spectral Avreages' ; hdr.avc ; ave=nave*nfft/prf
    NCDF_ATTPUT, Id, 'nave',     'units', 'count' 

    VarId   = NCDF_VARDEF(Id,'zrg', /Long)
    NCDF_ATTPUT, Id, 'zrg',      'long_name', 'Number of Range Gates' ; hdr.mom_zrg
    NCDF_ATTPUT, Id, 'zrg',      'units', 'count'

    VarId   = NCDF_VARDEF(Id,'rg0', /Long)
    NCDF_ATTPUT, Id, 'rg0',      'long_name', 'Number of Lowest Range Gates' ; ppar.ihp   
    NCDF_ATTPUT, Id, 'rg0',      'units', 'count' ; 

    VarId   = NCDF_VARDEF(Id,'drg', /FLOAT)
    NCDF_ATTPUT, Id, 'drg',      'long_name', 'Range Resolution' ; hdr.DELTA_H
    NCDF_ATTPUT, Id, 'drg',      'unit', 'm'

    VarId   = NCDF_VARDEF(Id,'lambda', /Long)
    NCDF_ATTPUT, Id, 'lambda',   'long_name', 'Wave Length' ; hdr.la
    NCDF_ATTPUT, Id, 'lambda',   'units', 'm'

    VarId   = NCDF_VARDEF(Id,'range', Dim2, /FLOAT)
    NCDF_ATTPUT, Id, 'range',    'long_name', 'Range from Antenna to the Centre of each Range Gate' ; range = ( rg0 + indgen(zrg)) * drg
    NCDF_ATTPUT, Id, 'range',    'units', 'm'
    NCDF_ATTPUT, Id, 'range',    'axis', 'Y'

    if max(extras eq 'SPC') then begin
        VarId   = NCDF_VARDEF(Id,'doppler', Dim3, /FLOAT)
        NCDF_ATTPUT, Id, 'doppler',    'long_name', 'Doppler Velocity' ; doppler = cmod(dx * findgen(nfft), hdr.VUAR,0)
        NCDF_ATTPUT, Id, 'doppler',    'units', 'm/s'
        NCDF_ATTPUT, Id, 'doppler',    'axis', 'V'
    endif

; Most important information from the SRVI structure
    VarId   = NCDF_VARDEF(Id,'time', Dim1, /LONG)
    NCDF_ATTPUT, Id, 'time',     'long_name', 'Seconds since 01.01.1970 00:00 UTC' ; srvi.TIME_T
    NCDF_ATTPUT, Id, 'time',     'units', 'Seconds' 
    NCDF_ATTPUT, Id, 'time',     'axis', 'T'

    VarId   = NCDF_VARDEF(Id,'microsec', Dim1, /LONG)
    NCDF_ATTPUT, Id, 'microsec',     'long_name', 'Micro Seconds' ; srvi.TIME_T
    NCDF_ATTPUT, Id, 'microsec',     'units', 'us' 

    VarId   = NCDF_VARDEF(Id,'tpow', Dim1, /FLOAT)
    NCDF_ATTPUT, Id, 'tpow',     'long_name','Average Transmit Power' ;  srvi.tpow * 0.01
    NCDF_ATTPUT, Id, 'tpow',     'units','W' 

    VarId   = NCDF_VARDEF(Id,'npw1', Dim1, /FLOAT)
    NCDF_ATTPUT, Id, 'npw1',     'long_name','Noise Power Co-Channel' ;  srvi.npw1
    NCDF_ATTPUT, Id, 'npw1',     'units','Sum over the Spectrum in DSP units'

    VarId   = NCDF_VARDEF(Id,'npw2', Dim1, /FLOAT)
    NCDF_ATTPUT, Id, 'npw2',     'long_name','Noise Power Cross-Channel' ;  srvi.npw2
    NCDF_ATTPUT, Id, 'npw2',     'units','Sum over the Spectrum in DSP units'

    VarId   = NCDF_VARDEF(Id,'cpw1', Dim1, /FLOAT)
    NCDF_ATTPUT, Id, 'cpw1',     'long_name','SNR of Calibration Signal Co-Channel' ;  srvi.cpw1
    NCDF_ATTPUT, Id, 'cpw1',     'units','Ratio'

    VarId   = NCDF_VARDEF(Id,'cpw2', Dim1, /FLOAT)
    NCDF_ATTPUT, Id, 'cpw2',     'long_name','SNR of Calibration Signal Cross-Channel' ;  srvi.cpw2
    NCDF_ATTPUT, Id, 'cpw2',     'units','Ratio'

    VarId   = NCDF_VARDEF(Id,'azi', Dim1, /FLOAT)
    NCDF_ATTPUT, Id, 'azi',     'long_name','Azimut Angle' ;  srvi.cpw2
    NCDF_ATTPUT, Id, 'azi',     'units','deg'

    VarId   = NCDF_VARDEF(Id,'north', Dim1, /FLOAT)
    NCDF_ATTPUT, Id, 'north',     'long_name','North Angle' ;  srvi.cpw2
    NCDF_ATTPUT, Id, 'north',     'units','deg'

    VarId   = NCDF_VARDEF(Id,'elv', Dim1, /FLOAT)
    NCDF_ATTPUT, Id, 'elv',     'long_name','Elevation Angle' ;  srvi.cpw2
    NCDF_ATTPUT, Id, 'elv',     'units','deg'

    VarId   = NCDF_VARDEF(Id,'aziv', Dim1, /FLOAT)
    NCDF_ATTPUT, Id, 'aziv',     'long_name','Azimut Angle Velocity' ;  srvi.cpw2
    NCDF_ATTPUT, Id, 'aziv',     'units','deg/s'

    VarId   = NCDF_VARDEF(Id,'elvv', Dim1, /FLOAT)
    NCDF_ATTPUT, Id, 'elvv',     'long_name','Elevation Angle Velocity' ;  srvi.cpw2
    NCDF_ATTPUT, Id, 'elvv',     'units','deg/s'

    ;  Create vector variables to hold the Output Profiles:
    for ipty=0,n_elements(ptypes)-1 do begin
        exn=ptypes[ipty]
        VarId   = NCDF_VARDEF(Id,'SNR'+exn, [Dim2, Dim1], /FLOAT)
        NCDF_ATTPUT, Id, 'SNR'+exn,      'long_name', 'Reflectivity SNR'+exn
        NCDF_ATTPUT, Id, 'SNR'+exn,      'units', ' '
        NCDF_ATTPUT, Id, 'SNR'+exn,      'yrange', [db(ppar.of0/(10.*hdr.nfft*sqrt(ppar.avc))),+70.0]
        NCDF_ATTPUT, Id, 'SNR'+exn,      'db', 1
        
        ;if exn eq 'rain' then begin
        ;    VarId   = NCDF_VARDEF(Id,'LWC'+exn, [Dim2, Dim1], /FLOAT)
        ;    NCDF_ATTPUT, Id, 'LWC'+exn,      'long_name', 'LWC '+exn
        ;    NCDF_ATTPUT, Id, 'LWC'+exn,      'units', ' '
        ;    NCDF_ATTPUT, Id, 'LWC'+exn,      'yrange', [95.0,205.0]
        ;    NCDF_ATTPUT, Id, 'LWC'+exn,      'db', 1
        ;    
        ;    VarId   = NCDF_VARDEF(Id,'RR'+exn, [Dim2, Dim1], /FLOAT)
        ;    NCDF_ATTPUT, Id, 'RR'+exn,      'long_name', 'RR '+exn
        ;    NCDF_ATTPUT, Id, 'RR'+exn,      'units', ' '
        ;    NCDF_ATTPUT, Id, 'RR'+exn,      'yrange', [95.0,205.0]
        ;    NCDF_ATTPUT, Id, 'RR'+exn,      'db', 1
        ;endif
        
        if exn eq '1' or exn eq '2' or exn eq '3' then begin
            VarId   = NCDF_VARDEF(Id,'NN'+exn, [Dim2, Dim1], /FLOAT)
            NCDF_ATTPUT, Id, 'NN'+exn,      'long_name', 'Number of Neighbours '+exn
            NCDF_ATTPUT, Id, 'NN'+exn,      'units', ' '
            NCDF_ATTPUT, Id, 'NN'+exn,      'yrange', [0,25]
            NCDF_ATTPUT, Id, 'NN'+exn,      'db', 0
            
            VarId   = NCDF_VARDEF(Id,'NF'+exn, [Dim2, Dim1], /FLOAT)
            NCDF_ATTPUT, Id, 'NF'+exn,      'long_name', 'Number of Friends '+exn
            NCDF_ATTPUT, Id, 'NF'+exn,      'units', ' '
            NCDF_ATTPUT, Id, 'NF'+exn,      'yrange', [0,25]
            NCDF_ATTPUT, Id, 'NF'+exn,      'db', 0
        endif
        
        ;if exn eq 'g' then begin
        ;    VarId   = NCDF_VARDEF(Id,'SNM'+exn, [Dim2, Dim1], /FLOAT)
        ;    NCDF_ATTPUT, Id, 'SNM'+exn,      'long_name', 'Co-Channel Mie-Corrected SNR'
        ;    NCDF_ATTPUT, Id, 'SNM'+exn,      'units', ' '
        ;    NCDF_ATTPUT, Id, 'SNM'+exn,      'yrange', [db(ppar.of0/(10.*hdr.nfft*sqrt(ppar.avc))),+80.0]
        ;    NCDF_ATTPUT, Id, 'SNM'+exn,      'db', 1
        ;endif
        
        VarId   = NCDF_VARDEF(Id,'VEL'+exn, [Dim2, Dim1], /FLOAT)
        NCDF_ATTPUT, Id, 'VEL'+exn,      'long_name', 'Doppler Velocity VEL'+exn
        NCDF_ATTPUT, Id, 'VEL'+exn,      'units', 'm/s'
        NCDF_ATTPUT, Id, 'VEL'+exn,      'yrange', [-hdr.vny,hdr.vny]
        NCDF_ATTPUT, Id, 'VEL'+exn,      'db', 0
        
        VarId   = NCDF_VARDEF(Id,'RMS'+exn, [Dim2, Dim1], /FLOAT)
        NCDF_ATTPUT, Id, 'RMS'+exn,      'long_name', 'Peak Width RMS'+exn
        NCDF_ATTPUT, Id, 'RMS'+exn,      'units', 'm/s'
        NCDF_ATTPUT, Id, 'RMS'+exn,      'yrange', [0,3.]
        NCDF_ATTPUT, Id, 'RMS'+exn,      'db', 0
        
        VarId   = NCDF_VARDEF(Id,'LDR'+exn, [Dim2, Dim1], /FLOAT)
        NCDF_ATTPUT, Id, 'LDR'+exn,      'long_name', 'Linear De-Polarization Ratio LDR'+exn
        NCDF_ATTPUT, Id, 'LDR'+exn,      'units', ' '
        NCDF_ATTPUT, Id, 'LDR'+exn,      'yrange', [-35.,5]
        NCDF_ATTPUT, Id, 'LDR'+exn,      'db', 1

        VarId   = NCDF_VARDEF(Id,'NPK'+exn, [Dim2, Dim1], /BYTE)
        NCDF_ATTPUT, Id, 'NPK'+exn,      'long_name', 'Number of Peaks'+exn
        NCDF_ATTPUT, Id, 'NPK'+exn,      'units', ' '
        NCDF_ATTPUT, Id, 'NPK'+exn,      'yrange', [-1,10]
        NCDF_ATTPUT, Id, 'NPK'+exn,      'db', 0
    endfor
   
    if max(extras eq 'HSDco') then begin
        ; noise profiles estimated by HSdiv
        VarId   = NCDF_VARDEF(Id,'HSDco', [Dim2, Dim1], /FLOAT)
        NCDF_ATTPUT, Id, 'HSDco',      'long_name', 'Co-Channel HSdiv Noise Power'
        NCDF_ATTPUT, Id, 'HSDco',      'units', 'DSP'
        NCDF_ATTPUT, Id, 'HSDco',      'yrange', [-10.,60.]
        NCDF_ATTPUT, Id, 'HSDco',      'db', 1
    endif
        
    if max(extras eq 'HSDcx') then begin
        VarId   = NCDF_VARDEF(Id,'HSDcx', [Dim2, Dim1], /FLOAT)
        NCDF_ATTPUT, Id, 'HSDcx',      'long_name', 'Cross-Channel HSdiv Noise Power'
        NCDF_ATTPUT, Id, 'HSDcx',      'units', 'DSP'
        NCDF_ATTPUT, Id, 'HSDcx',      'yrange', [-10.,60.]
        NCDF_ATTPUT, Id, 'HSDcx',      'db', 1
    endif
    
    if max(extras eq 'Ze') then begin
        VarId   = NCDF_VARDEF(Id,'Ze', [Dim2, Dim1], /FLOAT)
        NCDF_ATTPUT, Id, 'Ze',        'long_name', 'Equivalent Radar Reflectivity Factor Ze of Hydrometeors'
        NCDF_ATTPUT, Id, 'Ze',        'units', 'mm^6/m^3'
        NCDF_ATTPUT, Id, 'Ze',        'yrange', [-60,30.]
        NCDF_ATTPUT, Id, 'Ze',        'db', 1
    endif
    if max(extras eq 'Zg') then begin
        VarId   = NCDF_VARDEF(Id,'Zg', [Dim2, Dim1], /FLOAT)
        NCDF_ATTPUT, Id, 'Zg',        'long_name', 'Equivalent Radar Reflectivity Factor Ze of all Targets'
        NCDF_ATTPUT, Id, 'Zg',        'units', 'mm^6/m^3'
        NCDF_ATTPUT, Id, 'Zg',        'yrange', [-60,30.]
        NCDF_ATTPUT, Id, 'Zg',        'db', 1
    endif
    if max(extras eq 'Z') then begin
        VarId   = NCDF_VARDEF(Id,'Z', [Dim2, Dim1], /FLOAT)
        NCDF_ATTPUT, Id, 'Z',        'long_name', 'Radar Reflectivity Factor Z of Hydrometeors (Mie-corrected)'
        NCDF_ATTPUT, Id, 'Z',        'units', 'mm^6/m^3'
        NCDF_ATTPUT, Id, 'Z',        'yrange', [-60,30.]
        NCDF_ATTPUT, Id, 'Z',        'db', 1
    endif
    if max(extras eq 'RR') then begin
        VarId   = NCDF_VARDEF(Id,'RR', [Dim2, Dim1], /FLOAT)
        NCDF_ATTPUT, Id, 'RR',        'long_name', 'Rain Rate'
        NCDF_ATTPUT, Id, 'RR',        'units', 'mm/h ?'
        NCDF_ATTPUT, Id, 'RR',        'yrange', [30,140.]
        NCDF_ATTPUT, Id, 'RR',        'db', 1
    endif
    if max(extras eq 'LWC') then begin
        VarId   = NCDF_VARDEF(Id,'LWC', [Dim2, Dim1], /FLOAT)
        NCDF_ATTPUT, Id, 'LWC',        'long_name', 'Liquid Water Content'
        NCDF_ATTPUT, Id, 'LWC',        'units', 'g/m^3 ?'
        NCDF_ATTPUT, Id, 'LWC',        'yrange', [0,180.]
        NCDF_ATTPUT, Id, 'LWC',        'db', 1
    endif
    if max(extras eq 'TEMP') then begin
        VarId   = NCDF_VARDEF(Id,'TEMP', [Dim2, Dim1], /FLOAT)
        NCDF_ATTPUT, Id, 'TEMP',        'long_name', 'Temperature'
        NCDF_ATTPUT, Id, 'TEMP',        'units', 'C'
        NCDF_ATTPUT, Id, 'TEMP',        'yrange', [-60,40.]
        NCDF_ATTPUT, Id, 'TEMP',        'db', 0
    endif
    if max(extras eq 'MeltHei') then begin
        VarId   = NCDF_VARDEF(Id,'MeltHei', [Dim1], /FLOAT)
        NCDF_ATTPUT, Id, 'MeltHei',        'long_name', 'Melting Layer Height'
        NCDF_ATTPUT, Id, 'MeltHei',        'units', 'm'
        NCDF_ATTPUT, Id, 'MeltHei',        'yrange', [0,14000]
        NCDF_ATTPUT, Id, 'MeltHei',        'db', 0
    endif
    if max(extras eq 'MeltHeiDet') then begin
        VarId   = NCDF_VARDEF(Id,'MeltHeiDet', [Dim1], /FLOAT)
        NCDF_ATTPUT, Id, 'MeltHeiDet',        'long_name', 'Melting Layer Height detcted from LDR'
        NCDF_ATTPUT, Id, 'MeltHeiDet',        'units', 'm'
        NCDF_ATTPUT, Id, 'MeltHeiDet',        'yrange', [0,14000]
        NCDF_ATTPUT, Id, 'MeltHeiDet',        'db', 0
    endif
    if max(extras eq 'MeltHeiDB') then begin
        VarId   = NCDF_VARDEF(Id,'MeltHeiDB', [Dim1], /FLOAT)
        NCDF_ATTPUT, Id, 'MeltHeiDB',        'long_name', 'Melting Layer Height from Data Base'
        NCDF_ATTPUT, Id, 'MeltHeiDB',        'units', 'm'
        NCDF_ATTPUT, Id, 'MeltHeiDB',        'yrange', [0,14000]
        NCDF_ATTPUT, Id, 'MeltHeiDB',        'db', 0
    endif
    if max(extras eq 'ISDRco') then begin
        VarId   = NCDF_VARDEF(Id,'ISDRco', [Dim2, Dim1], /FLOAT)
        NCDF_ATTPUT, Id, 'ISDRco',        'long_name', 'In Spectrum Dynamic Range Co-Channel'
        NCDF_ATTPUT, Id, 'ISDRco',        'units', ' '
        NCDF_ATTPUT, Id, 'ISDRco',        'yrange', [00,40.]
        NCDF_ATTPUT, Id, 'ISDRco',        'db', 1
    endif
    if max(extras eq 'ISDRcx') then begin
        VarId   = NCDF_VARDEF(Id,'ISDRcx', [Dim2, Dim1], /FLOAT)
        NCDF_ATTPUT, Id, 'ISDRcx',        'long_name', 'In Spectrum Dynamic Range Cross-Channel'
        NCDF_ATTPUT, Id, 'ISDRcx',        'units', ' '
        NCDF_ATTPUT, Id, 'ISDRcx',        'yrange', [00,40.]
        NCDF_ATTPUT, Id, 'ISDRcx',        'db', 1
    endif

    if max(extras eq 'SPC') then begin
        VarId   = NCDF_VARDEF(Id,'SPCco', [Dim3, Dim2, Dim1], /FLOAT)
        NCDF_ATTPUT, Id, 'SPCco',        'long_name', 'Doppler Velocity Co-Spectrum'
        NCDF_ATTPUT, Id, 'SPCco',        'units', 'm/s'
        NCDF_ATTPUT, Id, 'SPCco',        'yrange', [0,70]
        NCDF_ATTPUT, Id, 'SPCco',        'db', 1
        VarId   = NCDF_VARDEF(Id,'SPCcx', [Dim3, Dim2, Dim1], /FLOAT)
        NCDF_ATTPUT, Id, 'SPCcx',        'long_name', 'Doppler Velocity Cross-Spectrum'
        NCDF_ATTPUT, Id, 'SPCcx',        'units', 'm/s'
        NCDF_ATTPUT, Id, 'SPCcx',        'yrange', [0,70]
        NCDF_ATTPUT, Id, 'SPCcx',        'db', 1
    endif

    varna='MRMco'
    if max(extras eq varna) then begin
        VarId   = NCDF_VARDEF(Id,varna, [Dim1], /FLOAT)
        NCDF_ATTPUT, Id, varna,        'long_name', 'Radiometer Signal Co'
        NCDF_ATTPUT, Id, varna,        'units', ' '
        NCDF_ATTPUT, Id, varna,        'yrange', [1,2]
        NCDF_ATTPUT, Id, varna,        'db', 0
    endif
    varna='MRMcx'
    if max(extras eq varna) then begin
        VarId   = NCDF_VARDEF(Id,varna, [Dim1], /FLOAT)
        NCDF_ATTPUT, Id, varna,        'long_name', 'Radiometer Signal Cx'
        NCDF_ATTPUT, Id, varna,        'units', ' '
        NCDF_ATTPUT, Id, varna,        'yrange', [1,2]
        NCDF_ATTPUT, Id, varna,        'db', 0
    endif
    varna='RadarConstCo'
    if max(extras eq varna) then begin
        VarId   = NCDF_VARDEF(Id,varna, [Dim1], /FLOAT)
        NCDF_ATTPUT, Id, varna,        'long_name', 'Radiometer Signal Co'
        NCDF_ATTPUT, Id, varna,        'units', 'mm^6/m^3'
        NCDF_ATTPUT, Id, varna,        'yrange', [-35,20.]
        NCDF_ATTPUT, Id, varna,        'db', 1
    endif
    varna='RadarConstCx'
    if max(extras eq varna) then begin
        VarId   = NCDF_VARDEF(Id,varna, [Dim1], /FLOAT)
        NCDF_ATTPUT, Id, varna,        'long_name', 'Radar Constant at 5 km Height'
        NCDF_ATTPUT, Id, varna,        'units', 'mm^6/m^3'
        NCDF_ATTPUT, Id, varna,        'yrange', [-35,-20.]
        NCDF_ATTPUT, Id, varna,        'db', 1
    endif
    
    ;  Leave definition mode and enter data write mode
    NCDF_CONTROL, Id, /ENDEF

    ;  Write the data
    NCDF_VARPUT, Id, 'nfft', hdr.nfft
    NCDF_VARPUT, Id, 'prf', hdr.prf
    NCDF_VARPUT, Id, 'NyquistVelocity', hdr.VNY
    NCDF_VARPUT, Id, 'nave', hdr.avc
    NCDF_VARPUT, Id, 'zrg', hdr.mom_zrg
    NCDF_VARPUT, Id, 'rg0', ppar.ihp
    NCDF_VARPUT, Id, 'drg', hdr.DELTA_H
    NCDF_VARPUT, Id, 'lambda', hdr.la
    range = ( ppar.ihp + lindgen(hdr.mom_zrg) ) * hdr.DELTA_H
    NCDF_VARPUT, Id, 'range',range 
    if max(extras eq 'SPC') then begin
        dx = hdr.VUAR/float(nfft)
        doppler = cmod(dx * findgen(nfft), hdr.VUAR,0)
        NCDF_VARPUT, Id, 'doppler',doppler
    endif

    ; Writing the data could look like this:
    ; for iti=0L,ztime-1 do begin
    ;     NCDF_VARPUT, Id, 'time', time[iti], count=[1], offset=[iti]
    ;     NCDF_VARPUT, Id, 'VEL', v[*,iti], count=[zrg,1], offset=[0,iti]
    ; endfor
    
    ; Don't forget to:
    ; NCDF_CLOSE, Id
    
    return,Id
end
