function average_profiles,sel_region,buffer,time,HSD,heis,itr
    noisecom_ex=0
    trmouse = sel_region.x 
    seconds = time
    itr = [first(where(seconds ge trmouse[0])),last(where(seconds le trmouse[1]))]
    if itr[1] le itr[0] or itr[0] lt 0 or itr[1] ge n_elements(seconds)  then begin
        print,"selected range invalid"
        noisecom_ex=0
    endif else begin
        noisecom_ex=1
        prof = (*(buffer.ptr_var))[*,itr[0]:itr[1]]
        heis = *(buffer.ptr_range) 
        nhei=n_elements(heis)
        nutime=n_elements(prof[0,*])
        HSD=fltarr(nhei) & ednutime=1.0/float(nutime) & for ihei=0,nhei-1 do HSD[ihei]=total(prof[ihei,*])*ednutime 
    endelse
    return,noisecom_ex
end

pro viewncdf,arguments
; Tool for visualizing profiles of the MIRA-36 cloud radar saved in ncdf format. 
;
; viewncdf can be called from the IDL prompt or from a unix shell:
; 
; unix> idl viewncdf
;
; IDL > viewncdf
; 
; The Data which should be visualized can be loaded to two buffers by the following commands:
; 
; View NCDF > load <buffer number> <filename> <variable name>    # loads the data to buffer 1
;
; Different visualizations can be created by the following commands:
;
;-----------------------------------------------------------------------------------------------------------------
;     View NCDF > thc <buffer number>
;
; Plot a Time Range Cross section of the profiles idicated by the first argument.
; The default value of the z-range of each type of profiles is stored in the
; parameter <f>range where <f> is the first letter of the profiles name. For 
; example the default value of the z-range for the ldr plots cam be changed by
;
;    View NCDF > set lrange -30 -1 ; trc ldr
;
;-----------------------------------------------------------------------------------------------------------------
;    View NCDF > set <parameter> <valu1> [<value2> ...]
;
; Change the default value of a plotting parameter. If <value1> is omited the
; current value is plotted. All the plotting parameters which 
; are available can be shown by
;
;    View NCDF > set help
;-----------------------------------------------------------------------------------------------------------------
;    View NCDF > tr <begin_hour> <end_hour>
;
; limits the time range of the trc plots. <begin_hour> <end_hour> are given as decimal numbers:
; 17.5 means 17:30. The hours ar countet from 00:00 of the first day contained in the data
; which is loaded.
;-----------------------------------------------------------------------------------------------------------------
;    View NCDF > postscript <filename>
;
; Directs the output of the following plot commands to the given postscript filename.
; for flushing and closing the postscript file the command
;
;    View NCDF > close
;
; must be given. After that the plot out put is directed to the screen again.
;----------------------------------------------------------------------------------------------------------------
;    View NCDF > png <filename>
;
; exports the screen to png-format
;-----------------------------------------------------------------------------------------------------------------
;    View NCDF > info
;
; gives some information from the file header
;-----------------------------------------------------------------------------------------------------------------
;    View NCDF > x
;
; exits viewncdf
;----------------------------------------------------------------------------------------------------------------
; If no command is recognized then the command is executed by the IDL interpreter.
;
; Matthias Bauer-Pfundstein bauer@metek.de
;
common startup,startup_done
if n_elements(startup_done) ne 1 then begin
@startup
endif

startup_device=!d.name

; Arguments can be provided by two methods:
; - If viewncdf is called by an interactive IDL session the first (and only) 
; string parameter arguments which contains a space separated list is split to
; the strarr argv.
; - Aditionally strings contained in the environment variables ARG1, ARG2, ..., 
; ARG9, ARG10, ... are prepended to argv. The script idl packs its calling arguments
; these environment variables. This is done as IDL can not extract the shell 
; variables like $1, $2,...
argv = env2argv(arguments)
kp = { $
    infile1:'',$
    config:'', $
    iam:'nobody',$
    max_dwells:70000L $
}
argv2stru,argv,kp
kp.iam=whoami(kp.iam)

; Default values for the Ploting parameters. These Parameters can be changed by the command "set"
pp={ $
hr:[130.,13000.],$          ; see comments below!
heira:[1,-1.],$   
td:1000.,$
tdunit:"km",$
valuetitle:'',$
rhixra:[1.,-1.],$
ScanExpectN:160,$
ScanSmoothVel:7,$ 
ScanVelThres:0.18,$
posg:0.4,$
northangle:-999,$
tr:[9d9,-9d9],$
wxsize:-1,$
wysize:-1,$
badcol :15, $
saturate:3b,$
min_txn_pow: 8. ,$
BeginTime:0UL,$
rec_err_good_bits:'0000'X,$
rec_err_bad_bits:'0000'X, $
xtu:'Minutes',$
tolerable_gap:11d0, $
takeat_gap:'right', $
maxtdsyn:-1.,$
scatter_rg_shift:0L,$
scattertol:-1.0,$
InterpoltOffset:0L,$
InterpolrOffset:0.0,$
InterpolrFactor:1.0,$
rangeoffset:0.0,$
timeoffset:0L,$
vel_chs:1,$
nbin:100,$
pADC:linear(-12.06),$
mrm_hei:4.0,$
mrm_fac:2,$
mrm_off:-0.35,$
pNoiseCom:linear(22.45-0.2),$
zrsym:3,$
charsize:1.1, $
thctit:'',$
tit:'',$
thick:1.0, $
MeltThick:0.5, $
CircleThick:0.4, $
SymColor:0, $
b4tit:'',$
ti:'!3',$
colt:'smooth' $
}
argv2stru,argv,pp
config2stru,pp,'viewncdf',[kp.iam+'.config',kp.config]

pph={ $
hr:      "Range of range gates which will be used for plotting and other things", $
heira:   "Shown height range of the image plots given in km", $
td:      "range scaling factor, 1000 for km",$
tdunit:  "range unit, km for for td=1000",$
valuetitle:'Override the annotation of the plotted values',$
rhixra:  "x range im m of the rhi plots. The yrange is defined by heira",$
ScanVelThres:"Parameter for autodetection of ppi or rhi scans. A scan is a contigous region in which the smoothed angle velocity is less in magnutude than ScanVelThres",$
ScanSmoothVel:"Parameter for autodetection of ppi or rhi scans. Width of box smoothing applied before scan detection, see ScanVelThres.",$
ScanExpectN:"In a scan win less than ScanExpectN is detected (see ScanVelThres) it is dicarded.",$
posg:    "weighting factor between 0 and 1 for finetuning the assignement between azi/elv and data chunk",$
northangle:"angle to which the antenna at is pointing after moveabs x 0, moveabs y 0",$
tr:      "Time range for plotting", $
badcol : "Grey, color for plotting values which are not plausible.", $
saturate : "If values are outside bset <buffer> yrange they are mapped to the range boundaries. bit 0 for lower, bit 2 for higher range.", $
min_txn_pow: "Only profiles are plotted by trc in which srvinfo.tpow (transmitted avg power * 100) is grater than min_txn_pow", $
BeginTime: "Load only data with time stamps later or equal to BeginTime.",$
rec_err_good_bits: "Only profiles are plotted by trc in which rec_err_good_bits in tsrvinfo.re_err are set", $
rec_err_bad_bits: "Only profiles are plotted by trc in which rec_err_bad_bits in tsrvinfo.re_err are not set", $
xtu:     "Parameter to control the number of ticks on the time axis. 'minutes'|'time'|'hours'..", $
tolerable_gap: "A gap in the time sequence will be detected when difference between the time stamps of two dwells is greater than tolerable_gap*median dwelltime", $
takeat_gap: "Describes how itr should be changed when a gap is detected", $
maxtdsyn: "max allowed time shift for data shown in the scatter plots",$
scatter_rg_shift: 'range shift for scatter plots (Behelfs-Pfusch) given in nuber of range gates',$
scattertol: 'when calculating difference and variance of the scatter plot only data with smaller abs difference is accounted for',$
InterpoltOffset: 'time shift given in s, which is added to the target ranges begore doing interpolt',$
InterpolrOffset: 'range shift given in m, which is added to the target ranges begore doing interpolr',$
InterpolrFactor: 'range factor, which can be used for scaling the target ranges begore doing interpolr',$
rangeoffset: 'an offset which is added to the range axis loaded from the ncdf file',$
timeoffset: 'an offset which is added to the range axis loaded from the ncdf file',$
nbin: 'Number of bins for histograms',$
pNoiseCom:'Power (linear) of noise source above thermal power.',$
pADC:'Noise Power (HSD, linear) if IfAttn is set to -25 dB',$
mrm_hei:'Parameter for MRM command: height where radiometer curve should be plotted',$
mrm_fac:'Parameter for MRM command: The db values of the radiometer curve are multiplied by this factor.',$
mrm_off:'Parameter for MRM command: This parameter is added to the db values of the radiometer curve.',$
zrsym: 'Symbols for isolated points in zr-plots',$
charsize: "See graphic keywords in IDL documentation", $
thick:   "See graphic keywords in IDL documentation", $
MeltThick:  "Make the Bullets indicating the melting layer heigt thicker.", $
CircleThick:  "Make the Bullets indicating the Ceilo Bases thicker.", $
SymColor:  "Color for the Bullets indicating the melting layer heigt", $
b4tit:  "Can be used to pass a Hershly-style font directive used only for title lines", $
ti:     "Can be uses to pass a Hershly-style font directive used for all annotations", $
colt:   "Color table, the predefined colortables reside in the directory idl/colors" $
}

buffer_proto={ $
ptr_var:ptr_new(),$
ptr_prof:ptr_new(),$
ptr_time:ptr_new(),$
ptr_microsec:ptr_new(),$
ptr_ave:ptr_new(),$
ptr_hours:ptr_new(),$
itr:[-1L,-2L],$
ptr_range:ptr_new(),$
ptr_hei:ptr_new(),$
ptr_tpow:ptr_new(),$
ptr_cpw1:ptr_new(),$
ptr_cpw2:ptr_new(),$
ptr_npw1:ptr_new(),$
ptr_npw2:ptr_new(),$
ptr_azi:ptr_new(),$
ptr_elv:ptr_new(),$
ihr:[-1,-2L], $
drg:-1.,$
nfft:-1,$
location:'unknown',$
filename:'unknown',$
name:'unknown',$
long_name:'unknown',$
units:'',$
valuetitle:'',$
yrange:[-15,75.],$
db:0L $
}
buffer_help={ $
name:'Name of the profile used in the axis annotation, use thc after chanhing it.',$
filename:'File name of the profile used in the axis annotation of the scatter plots.',$
long_name:'Name of the profile used in the title, use thc after chanhing it.',$
units:'Unit used in the axis annotation, use thc after chanhing it.',$
valuetitle:'Description of values for the axis anotation',$
yrange:'plotting range for the values, use thc after chanhing it.',$
db:'1 if the data shold be plottet in dB.' $
}
nbuffer = 9
buffer=replicate(buffer_proto,nbuffer)

startcolor=30
ocolt='none'

!p.multi = [0,1,1]
if !d.name eq "X" then begin
    if !d.window ge 0 then wdelete
    screen_size=get_screen_size()
    scale=screen_size/[1280.,1024]
    maxcolor=512
    if pp.wxsize gt 0 then xsize=pp.wxsize else xsize=1180*scale[0]
    if pp.wysize gt 0 then ysize=pp.wysize else ysize=800*scale[1]
    window,/free,retain=2,colors=maxcolor,title="viewncdf",xpos=90*scale[0],ypos=200*scale[1], $
    xsize=xsize,ysize=ysize
endif
if !d.name eq "Z" then begin
    Device, Set_Resolution=[1200,800], Z_Buffer=0
endif

ipcs1=0L
ipcs2=0L
last_draw_arg='trc'
while 1 do begin
    if ocolt ne pp.colt then begin
        loadcolor,startcolor,colorfile=pp.colt,zcolor=zcolor 
        ocolt=pp.colt
        TVLCT, ct_red, ct_green, ct_blue, /Get
    endif
    
    arg = myprompt('View NCDF >')
artarg:
    cmd = arg[0] &  narg=n_elements(arg)
    case 1 of
        cmd eq 'redraw' or cmd eq 'r': begin
            arg = last_draw_arg
            goto,artarg
        end
        cmd eq 'recent': begin
            pp.BeginTime = ulong(SYSTIME( /SECONDS )) - ulong(arg[1])
        end
        cmd eq 'load' or cmd eq 'ld': begin
            bufnr = fix(arg[1])-1
            if bufnr ge 0 && bufnr lt nbuffer-1 then begin 
                varname = arg[2]
                if ~ ( narg lt 4 || arg[3] eq '"' ) then begin
                    fn4ld = findfile(arg[3])
                    infiles_oldsyntax = strsplit(arg[3],':',/extract)
                    if n_elements(infiles_oldsyntax) gt n_elements(fn4ld) then fn4ld=infiles_oldsyntax
                    fn4title = arg[3]
                endif else begin
                    if n_elements(fn4ld) eq 0 then fn4ld=['']
                endelse
                if fn4ld[0] eq '' then begin 
                    nfiles=0 & print,'file not found'
                endif else nfiles=n_elements(fn4ld)
                firstrun=1
                for ifile = 0, nfiles-1 do begin
                    catch,error
                    IF error NE 0 THEN BEGIN
                        print,'An error with index ',error,' occurred'
                        print,!error_state.msg
                        catch,/canc
                        if ifile ge nfiles then print,'ifile ge nfiles',ifile,nfiles $
                        else print,'error while reading from file: ',fn4ld[ifile]
                        goto,NEXT_FILE
                    endif
                    gzipped = strpos(fn4ld[ifile],'.gz') eq strlen(fn4ld[ifile])-3
                    if gzipped then begin
                        spawn,'echo $$',pid
                        fn4ldx='/tmp/viewncdf_unzipped.'+pid[0]+'.nc'
                        spawn,'cp '+fn4ld[ifile]+'  '+ fn4ldx+'.gz'
                        spawn,'gunzip '+fn4ldx+'.gz'
                    endif else fn4ldx=fn4ld[ifile]
                    ncdfid = NCDF_OPEN(fn4ldx,/NOWRITE)
                    ; Test if varname is a thc variable
                    ncdfnvars=(ncdf_inquire(ncdfid)).NVARS
                    ncdfvarnames=strarr(ncdfnvars)
                    for i=0,ncdfnvars-1 do begin 
                        ncdfvarnames[i] = (ncdf_varinq(ncdfid,i)).name
                    endfor
                    ncdfixv = where(ncdfvarnames eq varname,ncdfnixv)
                    if ncdfnixv eq 1 then begin
                        ncdfidv = NCDF_VARID(ncdfid, varname ) 
                        NCDF_VARGET, ncdfid, 'range', rangex & zrg=n_elements(rangex)
                        rangex = rangex + pp.rangeoffset
                        NCDF_VARGET, ncdfid, 'time', timex
                        timex = timex + pp.timeoffset
                        ixld=where(timex ge pp.BeginTime, ztime)
                        if ztime gt 0 then begin
                            timex=timex[ixld]
                            TimeOffset=ixld[0]
                            ncdfidv = NCDF_VARID(ncdfid, varname ) 
                            NCDF_ATTGET, ncdfid, 'location', location,/global
                            NCDF_VARGET, ncdfid, 'tpow', tpowx, count=[ztime], offset=[TimeOffset]
                            if max(tpowx) gt 0.05 and max(tpowx) lt 0.75 then tpowx=tpowx*100.  ; bad bug fix
                            NCDF_VARGET, ncdfid, 'cpw1', cpw1x, count=[ztime], offset=[TimeOffset]
                            NCDF_VARGET, ncdfid, 'cpw2', cpw2x, count=[ztime], offset=[TimeOffset]
                            NCDF_VARGET, ncdfid, 'npw1', npw1x, count=[ztime], offset=[TimeOffset]
                            NCDF_VARGET, ncdfid, 'npw2', npw2x, count=[ztime], offset=[TimeOffset]
                            ncdfixv = where(ncdfvarnames eq 'azi',ncdfnixv)
                            if ncdfnixv eq 1 then begin
                                NCDF_VARGET, ncdfid, 'azi', azix, count=[ztime], offset=[TimeOffset]
                                NCDF_VARGET, ncdfid, 'elv', elvx, count=[ztime], offset=[TimeOffset]
                                NCDF_VARGET, ncdfid, 'microsec', microsecx, count=[ztime], offset=[TimeOffset]
                            endif else begin
                                azix=replicate(-999.,ztime)
                                elvx=replicate(90,ztime)
                                microsecx=replicate(0L,ztime)
                            endelse
                            NCDF_VARGET, ncdfid, 'prf',prfx
                            NCDF_VARGET, ncdfid, 'nfft',nfftx
                            NCDF_VARGET, ncdfid, 'nave',navex
                            avex=replicate(float(navex)*nfftx/prfx,n_elements(timex))
                            NCDF_VARGET, ncdfid, ncdfidv, varx, count=[zrg,ztime], offset=[0L,TimeOffset]
                            NCDF_ATTGET, ncdfid, ncdfidv,'yrange', yrangex
                            NCDF_ATTGET, ncdfid, ncdfidv,'long_name', long_namex
                            NCDF_ATTGET, ncdfid, ncdfidv,'units', unitsx
                            NCDF_ATTGET, ncdfid, ncdfidv,'db', dbx
                            NCDF_VARGET, ncdfid, 'drg', drgx
                            if n_elements(first_day_0000) ne 1 then first_day_0000 = long((timex[0]+70L)/(3600d0*24d0))*24d0
                            pp.tr[0] <= min(timex)/36d2 - first_day_0000 
                            pp.tr[1] >= max(timex)/36d2 - first_day_0000 
                            if firstrun eq 1 then begin
                                firstrun=0
                                buffer[bufnr] = buffer_proto
                                buffer[bufnr].ptr_var = ptr_new(varx)
                                buffer[bufnr].ptr_time = ptr_new(timex)
                                buffer[bufnr].ptr_range = ptr_new(rangex)
                                buffer[bufnr].ptr_tpow = ptr_new(tpowx)
                                buffer[bufnr].ptr_cpw1 = ptr_new(cpw1x)
                                buffer[bufnr].ptr_cpw2 = ptr_new(cpw2x)
                                buffer[bufnr].ptr_npw1 = ptr_new(npw1x)
                                buffer[bufnr].ptr_npw2 = ptr_new(npw2x)
                                buffer[bufnr].ptr_microsec = ptr_new(microsecx)
                                buffer[bufnr].ptr_ave = ptr_new(avex)
                                buffer[bufnr].ptr_azi = ptr_new(azix)
                                buffer[bufnr].ptr_elv = ptr_new(elvx)
                                buffer[bufnr].filename = fn4title
                                buffer[bufnr].name = varname
                                buffer[bufnr].location = string(location)
                                buffer[bufnr].nfft = nfftx
                                buffer[bufnr].long_name = string(long_namex)
                                buffer[bufnr].units = string(unitsx)
                                buffer[bufnr].yrange = yrangex
                                buffer[bufnr].db = dbx 
                                buffer[bufnr].drg = drgx 
                            endif else begin
                                ztime_old = n_elements(*buffer[bufnr].ptr_time)
                                ztime_new = n_elements(timex)
                                zrgx=n_elements(*buffer[bufnr].ptr_range)
                                if zrgx ne zrg then begin
                                    print,'two files with different number of range gates hav been appended',zrg,zrgx
                                endif
                                zrg = zrgx < zrg
                                varx_old = (*buffer[bufnr].ptr_var)[0:zrg-1,*] 
                                *buffer[bufnr].ptr_var = fltarr(zrg, ztime_old+ztime_new)
                                (*buffer[bufnr].ptr_var)[0:zrg-1,0:ztime_old-1] = varx_old
                                (*buffer[bufnr].ptr_var)[0:zrg-1,ztime_old:ztime_old+ztime_new-1L] = varx[0:zrg-1,*]
                                *buffer[bufnr].ptr_time = [*buffer[bufnr].ptr_time,timex]
                                *buffer[bufnr].ptr_microsec = [*buffer[bufnr].ptr_microsec,microsecx]
                                *buffer[bufnr].ptr_ave = [*buffer[bufnr].ptr_ave,avex]
                                *buffer[bufnr].ptr_tpow = [*buffer[bufnr].ptr_tpow,tpowx]
                                *buffer[bufnr].ptr_cpw1 = [*buffer[bufnr].ptr_cpw1,cpw1x]
                                *buffer[bufnr].ptr_cpw2 = [*buffer[bufnr].ptr_cpw2,cpw2x]
                                *buffer[bufnr].ptr_npw1 = [*buffer[bufnr].ptr_npw1,npw1x]
                                *buffer[bufnr].ptr_npw2 = [*buffer[bufnr].ptr_npw2,npw2x]
                                *buffer[bufnr].ptr_azi = [*buffer[bufnr].ptr_azi,azix]
                                *buffer[bufnr].ptr_elv = [*buffer[bufnr].ptr_elv,elvx]
                            endelse
                        endif
                    endif else begin
                        ; print a list thc of variables which are in the ncdf-file 
                        for i=0,ncdfnvars-1 do begin
                            if (ncdf_varinq(ncdfid,i)).NDIMS eq 2 && (ncdf_varinq(ncdfid,i)).NATTS ge 4 then begin
                                NCDF_ATTGET, ncdfid, i,'long_name', long_namex
                                print,form='(a12,": ",a)',(ncdf_varinq(ncdfid,i)).name,string(long_namex)
                            endif
                        endfor
                    endelse
NEXT_FILE:
                    catch,/canc
                    NCDF_CLOSE, ncdfid
                    if gzipped then spawn,'rm '+fn4ldx
                endfor
                if ncdfnixv eq 1 then begin
                    ixnan=where(finite(*buffer[bufnr].ptr_elv) eq 0,nixnan)
                    if nixnan gt 0 then (*buffer[bufnr].ptr_elv)[ixnan]=90.
                    ixnan=where(finite(*buffer[bufnr].ptr_azi) eq 0,nixnan)
                    if nixnan gt 0 then (*buffer[bufnr].ptr_azi)[ixnan]=0.
                endif
            endif else print,'usage: load <BufferNumber> <ProfileName>  # <BufferNumber>= 1, ..,'+string(nbuffer)
            catch,/canc
        end
        cmd eq 'melt' or cmd eq 'meltDB' or cmd eq 'meltDet' or cmd eq 'MRM': begin
            if ~ ( narg lt 2 || arg[1] eq '"' ) then fn4ld = findfile(arg[1]) else begin
                if n_elements(fn4ld) eq 0 then fn4ld=['']
            endelse
            if fn4ld[0] eq '' then begin 
                nfiles=0 
                if narg eq 2 then print,'file not found:'+arg[1] else print,'From which file should MeltRg be taken'
            endif else nfiles=n_elements(fn4ld)
            for ifile = 0, nfiles-1 do begin
                gzipped = strpos(fn4ld[ifile],'.gz') eq strlen(fn4ld[ifile])-3
                if gzipped then begin
                    spawn,'echo $$',pid
                    fn4ldx='/tmp/viewncdf_unzipped.'+pid[0]+'.nc'
                    spawn,'cp '+fn4ld[ifile]+'  '+ fn4ldx+'.gz'
                    spawn,'gunzip '+fn4ldx+'.gz'
                endif else fn4ldx=fn4ld[ifile]
                ncdfid = NCDF_OPEN(fn4ldx,/NOWRITE)
                if cmd eq 'melt'   then NCDF_VARGET, ncdfid, 'MeltHei', MeltHeis
                if cmd eq 'meltDB' then NCDF_VARGET, ncdfid, 'MeltHeiDB', MeltHeisDB
                if cmd eq 'meltDet' then NCDF_VARGET, ncdfid, 'MeltHeiDet', MeltHeisDB
                if cmd eq 'MRM' then begin & NCDF_VARGET, ncdfid, 'MRMco', MRMco & NCDF_VARGET, ncdfid, 'MRMcx', MRMcx & endif
                NCDF_VARGET, ncdfid, 'time', timex
                NCDF_CLOSE, ncdfid
                if gzipped then spawn,'rm '+fn4ldx
                usercirc,pp.MeltThick
                if cmd eq 'melt'   then oplot,unixtime2jul(timex),MeltHeis/pp.td,color=pp.SymColor
                if cmd eq 'meltDB' then oplot,unixtime2jul(timex),MeltHeisDB/pp.td,color=pp.SymColor
                if cmd eq 'meltDet' then oplot,unixtime2jul(timex),MeltHeisDB/pp.td,color=pp.SymColor,psym=8,thick=pp.thick
                if cmd eq 'MRM' then begin
                    oplot,unixtime2jul(timex),pp.mrm_hei+(db(0.5*(MRMco+MRMcx))+pp.mrm_off)*pp.mrm_fac,color=pp.SymColor,thick=pp.thick
                    oplot,unixtime2jul(timex),0*timex+pp.mrm_hei,color=pp.SymColor,thick=pp.thick
                endif
            endfor
        end
        cmd eq 'bases': begin    ; read ceilometer bases from the jenoptik-dat files and oplot them to the thc plots
            if findfile(arg[1]) ne '' then begin
                read_ceilo_dat_chm15k,arg[1],timex,bases
                nbasest = n_elements(bases[0,*])
                nbasesr = n_elements(bases[*,0])
                nbases = nbasest * nbasesr
                xxx = dblarr(nbases)
                yyy = fltarr(nbases)
                nbases=0L
                for iti=0,nbasest-1 do begin
                    hour = timex[iti] ; /36d2 - first_day_0000
                    for irg=0,nbasesr-1 do begin
                        if finite(bases[irg,iti]) then begin
                            xxx[nbases] = hour
                            yyy[nbases] = bases[irg,iti]
                            nbases++
                        endif
                    endfor
                endfor
                if nbases gt 0 then begin
                    usercirc,pp.CircleThick
                    oplot,unixtime2jul(xxx[0:nbases-1]),yyy[0:nbases-1]/pp.td,psym=8,color=pp.SymColor
                endif
            endif else print,'ceilometer data file: ' + arg[1] + ' not found'
        end
        cmd eq 'cbh': begin    ; read ceilometer bases from the jenoptik-nc files and oplot them to the thc plots
            if (file_test(arg[1],/read)) then begin
                read_ceilo_nc_chm15k,arg[1],timex,bases
                usercirc,pp.CircleThick
                oplot,unixtime2jul(timex),bases[0,*]/pp.td,psym=8,color=pp.SymColor
            endif else print,'ceilometer netCDF file: ' + arg[1] + ' not found'
        end
        cmd eq 'ceilo' or cmd eq 'ceilonc': begin
            ;bufnr = fix(arg[1])-1
            if findfile(arg[2]) ne '' then begin
                frm = cmd eq 'ceilonc'
                if bufnr ge 0 && bufnr lt nbuffer-1 then begin
                    read_ceilometer_bsc,arg[2],rangex,timex,varx,frm=frm
                    ; for i=0,(ncdf_inquire(ncdfid)).NVARS-1 do print, (ncdf_varinq(ncdfid,i)).name
                    long_namex = 'Ceilometer Power'
                    unitsx = ' '
                    yrangex = [-5,30.]
                    dbx = 1
                    drgx = 10
                    buffer[bufnr] = buffer_proto
                    buffer[bufnr].ptr_var = ptr_new(varx)
                    buffer[bufnr].ptr_time = ptr_new(timex)
                    if n_elements(first_day_0000) ne 1 then first_day_0000 = long((timex[0]+70L)/(3600d0*24d0))*24d0
                    pp.tr[0] <= min(timex)/36d2 - first_day_0000 
                    pp.tr[1] >= max(timex)/36d2 - first_day_0000 
                    buffer[bufnr].ptr_range = ptr_new(rangex)
                    buffer[bufnr].ptr_tpow = ptr_new(replicate(30.,n_elements(timex)))
                    buffer[bufnr].filename = arg[2]
                    buffer[bufnr].name = 'PWR'
                    buffer[bufnr].long_name = string(long_namex)
                    buffer[bufnr].units = string(unitsx)
                    buffer[bufnr].yrange = yrangex
                    buffer[bufnr].db = dbx 
                    buffer[bufnr].drg = drgx 
                endif else print,'usage: load <BufferNumber> <ProfileName>  # <BufferNumber>= 1, ..,'+string(nbuffer)
            endif else print,'Ceilometer data file '+arg[2]+ ' not found'
        end
        cmd eq 'interpolt': begin
            bufnr1 = fix(arg[1])-1
            bufnr2 = fix(arg[2])-1
            if bufnr1 ge 0 && bufnr1 lt nbuffer && bufnr2 ge 0 && bufnr2 lt nbuffer then begin
                time_org = *buffer[bufnr1].ptr_time + pp.InterpoltOffset
                time_new = *buffer[bufnr2].ptr_time
                t0=time_org[0]
                ztime_org = n_elements(time_org)
                ztime_new = n_elements(time_new)
                zrg_org = n_elements(*buffer[bufnr1].ptr_range)
                zrg_new = n_elements(*buffer[bufnr2].ptr_range)
                ;ptr_free,ptr_var_new
                ptr_var_new=ptr_new(fltarr(zrg_org,ztime_new))
                for irg=0,zrg_org-1 do begin
                    (*ptr_var_new)[irg,*] = interpol(float((*buffer[bufnr1].ptr_var)[irg,*]),time_org-t0,time_new-t0,/quadratic)
                endfor
                ptr_free,buffer[bufnr1].ptr_var
                buffer[bufnr1].ptr_var = ptr_var_new
                ptr_free,buffer[bufnr1].ptr_time
                buffer[bufnr1].ptr_time = ptr_new(time_new)
                ptr_free,buffer[bufnr1].ptr_microsec
                buffer[bufnr1].ptr_microsec = ptr_new(*buffer[bufnr2].ptr_microsec)
                ptr_free,buffer[bufnr1].ptr_ave
                buffer[bufnr1].ptr_ave = ptr_new(*buffer[bufnr2].ptr_ave)
                ptr_elv_new = ptr_new( interpol(*buffer[bufnr1].ptr_elv,time_org-t0,time_new-t0,/quadratic))
                ptr_free,buffer[bufnr1].ptr_elv
                buffer[bufnr1].ptr_elv = ptr_elv_new 
                ptr_tpow_new = ptr_new( interpol(*buffer[bufnr1].ptr_tpow,time_org-t0,time_new-t0,/quadratic))
                ptr_free,buffer[bufnr1].ptr_tpow
                buffer[bufnr1].ptr_tpow = ptr_tpow_new 
            endif else begin
               print,'usage: thc <BufferNumber>'
               print,'The buffer <BufferNumber> must be filled by load-cmd before.'
            endelse
        end
        cmd eq 'interpolr': begin
            ; Veraendert (interpoliert) die Messdaten in bufnr1 so, dass sie zu bufnr2 passen.
            ; Bevor man scatterplots macht, oder calc auf die Messdaten unterschiedlicher Geraete anwendet
            ; sollt man interpolr und interpolt verwenden. 
            bufnr1 = fix(arg[1])-1
            bufnr2 = fix(arg[2])-1
            if bufnr1 ge 0 && bufnr1 lt nbuffer && bufnr2 ge 0 && bufnr2 lt nbuffer then begin
                ztime_org = n_elements( *buffer[bufnr1].ptr_time )
                ranges_org = *buffer[bufnr1].ptr_range * pp.InterpolrFactor + pp.InterpolrOffset
                zrg_org = n_elements(ranges_org)
                ranges_new = *buffer[bufnr2].ptr_range 
                zrg_new = n_elements(ranges_new)
                ; ptr_free,ptr_var_new
                ptr_var_new=ptr_new(fltarr(zrg_new,ztime_org))
                for iti=0,ztime_org-1 do begin
                    (*ptr_var_new)[0,iti] = interpol(float((*buffer[bufnr1].ptr_var)[*,iti]),ranges_org,ranges_new,/quadratic)
                endfor
                ptr_free,buffer[bufnr1].ptr_var
                buffer[bufnr1].ptr_var = ptr_var_new
                ptr_free,buffer[bufnr1].ptr_range
                buffer[bufnr1].ptr_range = ptr_new(ranges_new)
                buffer[bufnr1].drg = buffer[bufnr2].drg
            endif else begin
               print,'usage: interpolr <BufferNumber1> <BufferNumber2>'
               print,'The buffer <BufferNumber1> and <BufferNumber2> must be filled by load-cmd before.'
               print,'The ranges of buffer <BufferNumber1> are interpolated to match <BufferNumber2>.'
            endelse
        end
        cmd eq 'fillt': begin
            if narg gt 1 && fix(arg[1])-1 ge 0 && fix(arg[1])-1 lt nbuffer && n_elements(*buffer[fix(arg[1])-1].ptr_prof) gt 1 then begin
                bufnr1 = fix(arg[1])-1
                time_org = *buffer[bufnr1].ptr_time 
                ztime_org = n_elements(time_org)
                zrg_org = n_elements(*buffer[bufnr1].ptr_range)
                t0=time_org[0]
                tvon=time_org[0]
                tbis=time_org[ztime_org-1]
                tdelta = long(median(abs(time_org[1:ztime_org-1]-time_org[0:ztime_org-2]))+0.4)
                ztime_new = (tbis-tvon) / tdelta + 1
                time_new = tvon + lindgen(ztime_new)*tdelta
                zrg_new = zrg_org
                ;ptr_free,ptr_var_new
                ptr_var_new=ptr_new(fltarr(zrg_org,ztime_new))
                create=1
                for irg=0,zrg_org-1 do begin
                    ;(*ptr_var_new)[irg,*] = interpol(float((*buffer[bufnr1].ptr_var)[irg,*]),time_org-t0,time_new-t0,/quadratic)
                    (*ptr_var_new)[irg,*] = fueller(float((*buffer[bufnr1].ptr_var)[irg,*]),time_org-t0,time_new-t0,missing=-99999.,create=create)
                    create=0
                endfor
                ptr_free,buffer[bufnr1].ptr_var
                buffer[bufnr1].ptr_var = ptr_var_new
                ptr_free,buffer[bufnr1].ptr_time
                buffer[bufnr1].ptr_time = ptr_new(time_new)
                ptr_tpow_new = ptr_new( interpol(*buffer[bufnr1].ptr_tpow,time_org-t0,time_new-t0,/quadratic))
                ptr_free,buffer[bufnr1].ptr_tpow
                buffer[bufnr1].ptr_tpow = ptr_tpow_new 
            endif else begin
               print,'usage: fillt <BufferNumber> tvon tbis tdelta'
               print,'The buffer <BufferNumber> must be filled by load-cmd before.'
            endelse
        end
        cmd eq 'fillr' or cmd eq 'fillh' : begin
            ; fillr <bufnr1> <hvon> <hbis> <hdelta> 
            ; Gridded die Messdaten auf ein neues Hoehenraster 
            ; Wenn <hbis> und /oder <hdelta> weg gelassen werden, werden sie entsprechend den Originaldaten gesetzt
            bufnr1 = fix(arg[1])-1
            hvon = float(arg[2])
            if bufnr1 ge 0 && bufnr1 lt nbuffer && n_elements(*buffer[bufnr1].ptr_prof) gt 1 then begin
                ranges_org = *buffer[bufnr1].ptr_range
                zrg_org = n_elements(ranges_org)
                if narg gt 3 then hbis = float(arg[3]) else hbis=ranges_org[zrg_org-1]
                if narg gt 4 then hdelta = float(arg[4]) else hdelta=min(abs(ranges_org[1:zrg_org-1]-ranges_org[0:zrg_org-2]))
                ztime_org = n_elements( *buffer[bufnr1].ptr_time )
                zrg_new = (hbis-hvon) / hdelta + 1
                ranges_new = hvon + findgen(zrg_new)*hdelta
                ; ptr_free,ptr_var_new
                ptr_var_new=ptr_new(fltarr(zrg_new,ztime_org))
                create=1
                for iti=0,ztime_org-1 do begin
                    ;(*ptr_var_new)[0,iti] = interpol(float((*buffer[bufnr1].ptr_var)[*,iti]),ranges_org,ranges_new,/quadratic)
                    (*ptr_var_new)[0,iti] = fueller(float((*buffer[bufnr1].ptr_var)[*,iti]),ranges_org,ranges_new,missing=-99999.,create=create)
                    create=0
                endfor
                ptr_free,buffer[bufnr1].ptr_var
                buffer[bufnr1].ptr_var = ptr_var_new
                ptr_free,buffer[bufnr1].ptr_range
                buffer[bufnr1].ptr_range = ptr_new(ranges_new)
                buffer[bufnr1].drg = hdelta
            endif else begin
               print,'usage: fillr <BufferNumber> <heivon> <heibis> <deltahei>'
               print,'The buffer <BufferNumber> must be filled by load-cmd before.'
               print,'The buffer <BufferNumber> s interpolated in range according to <heivon> <heibis> <deltahei>.'
            endelse
        end
        cmd eq 'thc' or cmd eq 'trc' or cmd eq 'tdc': begin
            ; time range cross section
            last_draw_arg = arg
            if narg ge 2 then bufnr = fix(arg[1])-1
            if bufnr ge 0 && bufnr lt nbuffer && buffer[bufnr].name ne 'unknown' then begin
                if n_elements(*(buffer[bufnr].ptr_range)) gt 1 then begin
                    hei = *(buffer[bufnr].ptr_range)
                    ihr = [first(where(hei ge pp.hr[0])),last(where(hei le pp.hr[1]))]
                    buffer[bufnr].ihr = ihr
                    ptr_free,buffer[bufnr].ptr_hei & buffer[bufnr].ptr_hei = ptr_new(hei[ihr[0]:ihr[1]])
                    
                    hours = *(buffer[bufnr].ptr_time)/3600d0 - first_day_0000
                    itr = [first(where(hours ge pp.tr[0])),last(where(hours le pp.tr[1]))]
                    if itr[1] le itr[0] or itr[0] lt 0 or itr[1] ge n_elements(hours)  then begin
                         itr[1]=n_elements(hours)-1 & itr[0]=0
                         pp.tr=hours[itr]
                         print, "setting tr to ", pp.tr
                    endif
                    buffer[bufnr].itr = itr
                    ptr_free,buffer[bufnr].ptr_hours & buffer[bufnr].ptr_hours = ptr_new(hours[itr[0]:itr[1]])
                   
                    if pp.valuetitle ne '' then valuetitle=pp.valuetitle else begin
                        valuetitle = buffer[bufnr].name+' ' + (buffer[bufnr].db ? 'dB' : '') + buffer[bufnr].units
                    endelse
                    buffer[bufnr].valuetitle = valuetitle
                    
                    prof = (*(buffer[bufnr].ptr_var))[ihr[0]:ihr[1],itr[0]:itr[1]]
                    if buffer[bufnr].db then prof = db(prof)

                    good_times = (*(buffer[bufnr].ptr_tpow))[itr[0]:itr[1]] ge pp.min_txn_pow 
                    zrg = n_elements(prof[*,0])
                    ptr_free,buffer[bufnr].ptr_prof 
                    ptr_prof = ptr_new( transpose(   prof +  replicate(1.,zrg) # ( (good_times ne 1) * 999999.)  ) )
                    buffer[bufnr].ptr_prof = ptr_prof
                    if buffer[bufnr].yrange[0] gt buffer[bufnr].yrange[1] then begin
                        ; do auto range
                        arbuf = *ptr_prof
                        buffer[bufnr].yrange = autorange(arbuf,99.99,8.0)
                    endif
                    sxtf=!x.TICKFORMAT & !x.TICKFORMAT='LABEL_DATE'
                    sxtu=!X.TICKUNITS[0] & !X.TICKUNITS[0]=pp.xtu
                    dummy = LABEL_DATE(DATE_FORMAT='%h:%i')
                    dara_title =     + unixtime2bdate((*(buffer[bufnr].ptr_time))[itr[0]], nice=2,utc=1) $ 
                        +  ' - '     + unixtime2bdate((*(buffer[bufnr].ptr_time))[itr[1]], nice=2,utc=1)
                    ixtr=lindgen(itr[1]-itr[0]+1)+itr[0]
                    utime_itr=(*(buffer[bufnr].ptr_time))[ixtr]
                    microsec_itr=double((*(buffer[bufnr].ptr_microsec))[ixtr]) * 1d-6
                    jultime_right = unixtime2jul(utime_itr, microsec_itr)
                    jultime_left  = unixtime2jul(utime_itr, microsec_itr - double((*(buffer[bufnr].ptr_ave))[ixtr]))
                    elv_itr = pp.posg * (*(buffer[bufnr].ptr_elv))[(ixtr-1L)>0] + (1.-pp.posg) * (*(buffer[bufnr].ptr_elv))[(ixtr-2L)>0] 
                    ;elv_itr= 0.5 * ( (*(buffer[bufnr].ptr_elv))[(ixtr-0L)>0] + (*(buffer[bufnr].ptr_elv))[(ixtr-1L)>0] )
                    case cmd of
                        'thc':begin
                            rangescalingfactor_itr = sin(!pi/180.* elv_itr)
                            heitit = 'Height AGL ' + pp.tdunit
                        end
                        'tdc':begin
                            rangescalingfactor_itr = cos(!pi/180.* elv_itr)
                            heitit = 'Horizontal Distance ' + pp.tdunit
                        end
                        'trc':begin
                            rangescalingfactor_itr = 1.
                            heitit = 'Range ' + pp.tdunit
                        end
                        else:begin
                            rangescalingfactor_itr = 1.
                            heitit = 'Range ' + pp.tdunit
                        end
                    endcase
                    y_bot = hei[ihr[0]]/pp.td * rangescalingfactor_itr
                    y_top = hei[ihr[1]]/pp.td * rangescalingfactor_itr
                    profseq2image, *ptr_prof, jultime_left, jultime_right, y_bot, y_top, yrange=pp.heira,buffer[bufnr].yrange, $
                        zcolor, startcolor,pp.badcol,charsize=pp.charsize,thick=pp.thick, $
                        ztitle=pp.ti+valuetitle,xtitle=pp.ti+'Time UTC', $
                        heitit=pp.ti+heitit,saturate=pp.saturate
                    tit_filename = stregex(buffer[bufnr].filename,'[0-9_]+\.mmclx',/extract)
                    if pp.thctit eq '' then begin
                        loca = buffer[bufnr].location eq 'Meteorologisches Observatorium Lindenberg' ? 'MOL' : buffer[bufnr].location
                        thctit = pp.ti+pp.b4tit + tit_filename + ' '+ buffer[bufnr].long_name +"   "+dara_title+ ' ' + loca
                    endif else thctit=pp.thctit
                    xyouts,0.49,0.93,thctit, $
                        /noclip,alignment=0.5,/normal,chars=pp.charsize
                    !x.TICKFORMAT = sxtf & !X.TICKUNITS[0]=sxtu
                endif else print,'No data for plotting in buffer number' + arg[2]
            endif else begin
               print,'usage: thc <BufferNumber>'
               print,'The buffer <BufferNumber> must be filled by load-cmd before.'
            endelse
        end
        cmd eq 'ppi': begin
            last_draw_arg = arg
            if narg ge 2 then bufnr = fix(arg[1])-1
            Daumenkino=0
            if narg ge 3 then begin
                nthppi=long(arg[2]) 
            endif else begin
                nthppi=-1
                if (!d.name eq 'PS') then begin
                    filenumber=0
                    Daumenkino=1
                endif
            endelse
            if bufnr ge 0 && bufnr lt nbuffer && buffer[bufnr].name ne 'unknown' then begin
                hours = *(buffer[bufnr].ptr_time)/3600d0 - first_day_0000
                itr = [first(where(hours ge pp.tr[0])),last(where(hours le pp.tr[1]))]
                if itr[1] le itr[0] or itr[0] lt 0 or itr[1] ge n_elements(hours)  then begin
                     itr[1]=n_elements(hours)-1 & itr[0]=0
                     pp.tr=hours[itr]
                     print, "setting tr to ", pp.tr
                endif
                buffer[bufnr].itr = itr
                ptr_free,buffer[bufnr].ptr_hours & buffer[bufnr].ptr_hours = ptr_new(hours[itr[0]:itr[1]])
                    
                ztime_itr = itr[1]-itr[0]+1L
                ixtr=lindgen(ztime_itr)+itr[0]
                utime_itr=(*(buffer[bufnr].ptr_time))[ixtr]
                microsec_itr=double((*(buffer[bufnr].ptr_microsec))[ixtr]) * 1d-6
                time_itr = double(utime_itr) + microsec_itr 
                dtime_itr = time_itr[1:ztime_itr-1] - time_itr[0:ztime_itr-2] 
                ; jultime_right = unixtime2jul(utime_itr, microsec_itr)
                ; jultime_left  = unixtime2jul(utime_itr, microsec_itr - double((*(buffer[bufnr].ptr_ave))[ixtr]))
                g=pp.posg & emg=1.-g
                elv_itr = g*(*(buffer[bufnr].ptr_elv))[(ixtr-1L)>0] + emg*(*(buffer[bufnr].ptr_elv))[(ixtr-2L)>0]  ; Correction due to Romans delay
                if pp.northangle ge 0 then northangle=pp.northangle else northangle=0.0 ; !!! hier sollte north_angle aus den mmclx-daten eingelesen werden.
                azi_itr = ave2ang( (*(buffer[bufnr].ptr_azi))[(ixtr-1L)>0] , (*(buffer[bufnr].ptr_azi))[(ixtr-2L)>0] , g) + northangle ; Correction due to Romans delay
                azi_itr = (azi_itr+360.) mod 360
                ; range of angle in which measuenents of each profile where made:
                ; elv_intermed = 0.5 * (elv_itr[0:ztime_itr-2L]+elv_itr[1:ztime_itr-1L])
                ; elv_left=[elv_intermed,elv_itr[ztime_itr-1L]]
                ; elv_right=[elv_itr[0],elv_intermed]
                azi_intermed=ave2ang(azi_itr[0:ztime_itr-2L],azi_itr[1:ztime_itr-1L])
                azi_left=[azi_intermed,azi_itr[ztime_itr-1L]]
                azi_right=[azi_itr[0],azi_intermed]
                dazi_itr = cmod(azi_itr[1:ztime_itr-1] - azi_itr[0:ztime_itr-2],360.)
                azivel_itr = dazi_itr / dtime_itr
                aziaccel_itr = azivel_itr[1:ztime_itr-2]-azivel_itr[0:ztime_itr-3]
                aziavel_itr = (azivel_itr[1:ztime_itr-2]+azivel_itr[0:ztime_itr-3])*0.5
                nppi = get_ixaixb(round(abs(aziaccel_itr) lt 0.25 or abs(aziavel_itr) gt 0.25),ixppia,ixppib,nixtrue)
                iixx=where((ixppib-ixppia+1) gt 8,niixx)
                if niixx gt 0 then begin
                    ixppia=ixppia[iixx]
                    ixppib=ixppib[iixx]
                    nppi=niixx
                endif else nppi=niixx

                if nthppi ge 0 then begin
                    i0ppi = nthppi < (nppi-1) & nppi=i0ppi+1
                endif else i0ppi=0l
                
                drh=buffer[bufnr].drg/2
                zrg = n_elements((*buffer[bufnr].ptr_prof)[0,*])
                for ippi=i0ppi, nppi-1 do begin
                    if Daumenkino eq 1 then begin
                        device,/close
                        device,filename=psfile + string(filenumber,format='(i3.3)')
                        filenumber++
                    endif
                    nixppi = ixppib[ippi]-ixppia[ippi]+5
                    ixppi = ((ixppia[ippi]-1) > 0 + lindgen(nixppi) ) < (ztime_itr-1)
                    radices = *buffer[bufnr].ptr_hei * cos(!pi/180.*median(elv_itr[ixppi]))
                    maxradi=radices[zrg-1]
                    td7=1./pp.td
                    polarimage,(*buffer[bufnr].ptr_prof)[ixppi,*],(radices-drh)*td7,(radices+drh)*td7,azi_left[ixppi],azi_right[ixppi],$
                        xrange=[-1.,1.]*(maxradi+drh)*td7,yrange=[-1.,1.]*(maxradi+drh)*td7, zrange=buffer[bufnr].yrange, $
                        zcolor, startcolor,pp.badcol,charsize=pp.charsize,thick=pp.thick, $
                        ztit=buffer[bufnr].valuetitle , xytit='horizontal range '+pp.tdunit
                    tit_filename = stregex(buffer[bufnr].filename,'[0-9_]+\.mmclx',/extract)
                    ixmitte=(ixppia[ippi]+ixppib[ippi])/2
                    dati_title =  unixtime2bdate((*(buffer[bufnr].ptr_time))[ixppi[nixppi-1]], nice=2,utc=1)
                    tit='Azimuth Scan, Elv=' + strgg(elv_itr[ixmitte],'(i5)') + '  '
                    if pp.tit eq '' then begin
                        tit = pp.ti + pp.b4tit + tit_filename + ' '+ buffer[bufnr].long_name +"  "+dati_title+ ' ' + buffer[bufnr].location
                    endif else tit=pp.tit
                    xyouts,0.49,0.93,tit, /noclip,alignment=0.5,/normal,chars=pp.charsize
                    if ippi lt nppi-1 then wait,0.2
                endfor
            endif
        end
        cmd eq 'rhi': begin
            last_draw_arg = arg
            FnRhi = 'rhi_'+buffer[bufnr].name
            nthrhi=-1
            if narg ge 2 then bufnr = fix(arg[1])-1
            if narg ge 3 then begin
                if stregex(arg[2],'[+-.0-9]+') eq 0 then nthrhi=long(arg[2]) else FnRhi=arg[2]
            endif
            if narg ge 4 then FnRhi=arg[3]
            if bufnr ge 0 && bufnr lt nbuffer && buffer[bufnr].name ne 'unknown' then begin
                hours = *(buffer[bufnr].ptr_time)/3600d0 - first_day_0000
                itr = [first(where(hours ge pp.tr[0])),last(where(hours le pp.tr[1]))]
                if itr[1] le itr[0] or itr[0] lt 0 or itr[1] ge n_elements(hours)  then begin
                     itr[1]=n_elements(hours)-1 & itr[0]=0
                     pp.tr=hours[itr]
                     print, "setting tr to ", pp.tr
                endif
                buffer[bufnr].itr = itr
                ptr_free,buffer[bufnr].ptr_hours & buffer[bufnr].ptr_hours = ptr_new(hours[itr[0]:itr[1]])
                    
                ztime_itr = itr[1]-itr[0]+1L
                ixtr=lindgen(ztime_itr)+itr[0]
                utime_itr=(*(buffer[bufnr].ptr_time))[ixtr]
                microsec_itr=double((*(buffer[bufnr].ptr_microsec))[ixtr]) * 1d-6
                time_itr = double(utime_itr) + microsec_itr 
                dtime_itr = time_itr[1:ztime_itr-1] - time_itr[0:ztime_itr-2] 
                jultime_right = unixtime2jul(utime_itr, microsec_itr)
                jultime_left  = unixtime2jul(utime_itr, microsec_itr - double((*(buffer[bufnr].ptr_ave))[ixtr]))
                g=pp.posg & emg=1.-g
                elv_itr = g*(*(buffer[bufnr].ptr_elv))[(ixtr-1L)>0] + emg*(*(buffer[bufnr].ptr_elv))[(ixtr-2L)>0]  ; Correction due to Romans delay
                if pp.northangle ge 0 then northangle=pp.northangle else northangle=0.0 ; !!! hier sollte north_angle aus den mmclx-daten eingelesen werden.
                azi_itr = ave2ang( (*(buffer[bufnr].ptr_azi))[(ixtr-1L)>0] , (*(buffer[bufnr].ptr_azi))[(ixtr-2L)>0] , g) + northangle ; Correction due to Romans delay
                azi_itr = (azi_itr+360.) mod 360
                ;range of angle in which measuenents of each profile where made:
                elv_intermed = 0.5 * (elv_itr[0:ztime_itr-2L]+elv_itr[1:ztime_itr-1L])
                elv_left=[elv_intermed,elv_itr[ztime_itr-1L]]
                elv_right=[elv_itr[0],elv_intermed]
                ;azi_intermed=ave2ang(azi_itr[0:ztime_itr-2L],azi_itr[1:ztime_itr-1L])
                ;azi_left=[azi_intermed,azi_itr[ztime_itr-1L]]
                ;azi_right=[azi_itr[0],azi_intermed]
                delv_itr = cmod(elv_itr[1:ztime_itr-1] - elv_itr[0:ztime_itr-2],360.)
                elvvel_itr = delv_itr / dtime_itr
                elvaccel_itr = elvvel_itr[1:ztime_itr-2]-elvvel_itr[0:ztime_itr-3]
                elvavel_itr = (elvvel_itr[1:ztime_itr-2]+elvvel_itr[0:ztime_itr-3])*0.5
                abs_scan_vel=abs(smooth(elvavel_itr,pp.ScanSmoothVel))
                nrhi = get_ixaixb(abs_scan_vel gt pp.ScanVelThres and abs_scan_vel lt 1.6,ixrhia,ixrhib,nixtrue)
                iixx=where((ixrhib-ixrhia+1) ge pp.ScanExpectN,niixx)
                print,'Number of Chunks in the scans detected from Elv scan velocity: ',ixrhib-ixrhia+1
                print,'Number of detected Scans, Number of accepted scans: ',nrhi,niixx
                if niixx gt 0 then begin
                    ixrhia=ixrhia[iixx]
                    ixrhib=ixrhib[iixx]
                endif
                nrhi=niixx

                if nthrhi ge 0 then begin
                    i0rhi = nthrhi < (nrhi-1) & i1rhi=i0rhi+1
                endif else begin
                    i0rhi=0l & i1rhi=nrhi
                endelse
                
                drh=buffer[bufnr].drg/2
                zrg = n_elements((*buffer[bufnr].ptr_prof)[0,*])
                nmovie=i1rhi-i0rhi+1
                for irhi=i0rhi, i1rhi-1 do begin
                    nixrhi = ixrhib[irhi]-ixrhia[irhi]
                    ixrhi = ((ixrhia[irhi]-0) > 0 + lindgen(nixrhi) ) < (ztime_itr-1)
                    radices = *buffer[bufnr].ptr_hei
                    maxradi=radices[zrg-1]
                    if pp.heira[1] gt pp.heira[0] then heira=pp.heira else begin
                        heira=[0,pp.hr[1]+drh]
                    endelse
                    td7=1./pp.td
                    if pp.rhixra[1] gt pp.rhixra[0] then rhixrange=pp.rhixra else rhixrange=[-1.,1.]*(maxradi+drh)
                    polarimage,(*buffer[bufnr].ptr_prof)[ixrhi,*],(radices-drh)*td7,(radices+drh)*td7,90.-elv_left[ixrhi],90.-elv_right[ixrhi],$
                        xrange=rhixrange*td7,$
                        yrange=heira*td7, $
                        zrange=buffer[bufnr].yrange, $
                        zcolor, startcolor,pp.badcol,charsize=pp.charsize,thick=pp.thick, $
                        ztit=buffer[bufnr].valuetitle , xytit='range '+pp.tdunit
                    tit_filename = stregex(buffer[bufnr].filename,'[0-9_]+\.mmclx',/extract)
                    ixmitte=(ixrhia[irhi]+ixrhib[irhi])/2
                    dati_title =  unixtime2bdate((*(buffer[bufnr].ptr_time))[ixrhi[nixrhi-1]], nice=2,utc=1)
                    if pp.tit eq '' then begin
                        tit = pp.ti + pp.b4tit + 'RHI Scan, Azi=' + strgg(azi_itr[ixmitte],'(i5)') + '  '+dati_title+ ' ' + buffer[bufnr].location
                        ;tit = pp.ti + pp.b4tit + tit_filename + ' '+ buffer[bufnr].long_name +"  "+dati_title+ ' ' + buffer[bufnr].location
                    endif else tit=pp.tit
                    if tit ne 'timebar' then begin
                        xyouts,0.49,0.93,tit, /noclip,alignment=0.5,/normal,chars=pp.charsize
                    endif else begin
                        if nmovie gt 1 then begin
                           ssx=!x & ssy=!y & ssp=!p
                           sxtf=!x.TICKFORMAT & !x.TICKFORMAT='LABEL_DATE'
                           sxtu=!X.TICKUNITS[0] & !X.TICKUNITS[0]=pp.xtu
                           dummy = LABEL_DATE(DATE_FORMAT='%h:%i')
                           plot,[0,1],[0,1],xrange=[jultime_left[ixrhia[0]],jultime_right[ixrhib[nrhi-1]]],yrange=[0,1], /nodata, /noerase, xstyle=9, ystyle=5,$
                               position=[[0.07,0.95],[0.87,0.97]],xticklen=0.3
                           JulTimeOfScan0 = jultime_left [ixrhia[irhi]]
                           JulTimeOfScan1 = jultime_right[ixrhib[irhi]]
                           dJulTime=JulTimeOfScan1-JulTimeOfScan0
                           polyfill,JulTimeOfScan0+dJulTime*[-0.5,+1.5,1.5,-0.5,-0.5],[0,0,1,1,0],color=2
                           !x.TICKFORMAT = sxtf & !X.TICKUNITS[0]=sxtu
                           !x=ssx & !y=ssy & !p=ssp
                        endif
                    endelse
                    case !d.name of
                        'X':if nmovie gt 1 then wait,0.2
                        'Z': begin
                           write_png, FnRhi +'.'+ string(irhi,format='(i3.3)')+'.png', TVRD() , ct_red, ct_green, ct_blue
                        end
                        else:
                    endcase
                endfor
                ; append some black pictures at the end of a movie
                if !D.name eq 'Z' then begin
                    erase
                    for irhi=i1rhi, i1rhi+ round(nmovie*0.04-1.5) do begin
                        write_png, FnRhi +'.'+ string(irhi,format='(i3.3)')+'.png', TVRD() , ct_red, ct_green, ct_blue
                    endfor
                endif
            endif
        end
        cmd eq 'trm': begin
            sel_region=select_region_by_mouse(dir=1,first_day_0000=first_day_0000)
            pp.tr = jul2unixtime(sel_region.x)/3600d0 - first_day_0000 
            print,pp.tr[0]," bis ", pp.tr[1]
        end
        cmd eq 'export': begin
            if narg ge 2 then exfilename = arg[1] else exfilename="ViewNcdfExport.sav"
            exfilename = arg[1]
            if bufnr ge 0 && bufnr lt nbuffer && buffer[bufnr].name ne 'unknown' then begin
                if n_elements(*(buffer[bufnr].ptr_range)) gt 1 then begin
                    nhei = ihr[1]-ihr[0]+1
                    nutime = itr[1] - itr[0] +1
                    heis = (*(buffer[bufnr].ptr_range))[ihr[0]:ihr[1]]
                    utimes = (*(buffer[bufnr].ptr_time))[itr[0]:itr[1]]
                    filename = buffer[bufnr].filename
                    dataname = buffer[bufnr].name
                    aveprof=fltarr(nhei) & ednutime=1.0/float(nutime) & for ihei=0,nhei-1 do aveprof[ihei]=total(prof[ihei,*])*ednutime
                    save,filename=exfilename,nhei,nutime,heis,utimes,filename,dataname,prof,aveprof
                endif else print,'No data for exporting in buffer number' + arg[2]
            endif else begin
               print,'The buffer must be plotted by thc before'
            endelse
        end
        cmd eq 'noisecom' or cmd eq 'nc': begin
            noisecom_ex=0
            if narg ge 2 then gvh_filename = arg[1] else gvh_filename=fn4ld[0]+".sav"
            print,"HSDco or HSDcx of NoiseCom measurment should be plotted by thc before"
            print,"Pls. select the time range by mouse in which the noise source was switched on!"
            ok_on = average_profiles(select_region_by_mouse(dir=1,firstday=first_day_0000),buffer[bufnr],jultime_right,HSD_on,heis)
            print,"Pls. select the time range by mouse in which the noise source was switched off!"
            ok_off = average_profiles(select_region_by_mouse(dir=1,firstday=first_day_0000),buffer[bufnr],jultime_right,HSD_off,heis,itr_off)
            ; die naechste Zeile war auskommentiert. Bin mir nicht sicher ob das vielleicht besser war.
            medHSD_on=median(HSD_on)
            medHSD_off=median(HSD_off)
            print,"Pls. select the time range by mouse in which the IF attn was set to 30 dB and noise source was off!"
            ok_attn = average_profiles(select_region_by_mouse(dir=1,firstday=first_day_0000),buffer[bufnr],jultime_right,HSD_adc,heis)
            if ok_attn then padc=median(HSD_adc) else padc=pp.padc
            nf=pp.pnoisecom/(median((HSD_on-padc)/(medHSD_off-padc)-1.))
            print, "Noise Figure = ",db(nf),' dB   AdcNoisePow = ',padc
            ; in der naechsten Zeile war ein minus statt /. Nach der Korrektur muss das calida-file neu generiert werden
            GainVsRange=(HSD_on-HSD_off)/medHSD_off
            GainVsRange = 0.001 > GainVsRange/median(GainVsRange)   ; soll nicht mehr als 30 dB korrigieren   
            plot,heis,GainVsRange,xra=[0,6000]
            PowCalCo=median((*(buffer[bufnr].ptr_cpw1))[itr_off[0]:itr_off[1]])
            PowCalCx=median((*(buffer[bufnr].ptr_cpw2))[itr_off[0]:itr_off[1]])
            PowNoiCo=median((*(buffer[bufnr].ptr_npw1))[itr_off[0]:itr_off[1]])
            PowNoiCx=median((*(buffer[bufnr].ptr_npw2))[itr_off[0]:itr_off[1]])
            nfft=buffer[bufnr].nfft
            save,filename=gvh_filename,heis,GainVsRange,nf,PowCalCo,PowCalCx,PowNoiCo,PowNoiCx,medHSD_on,medHSD_off,padc,nfft
        end
        cmd eq 'smooth': begin
            if narg ge 2 then bufnr = fix(arg[1])-1
            if bufnr ge 0 && bufnr lt nbuffer && narg eq 4 then begin
                nsmooth_t = fix(arg[2])
                nsmooth_r = fix(arg[3])
                *buffer[bufnr].ptr_var = smooth(*buffer[bufnr].ptr_var,[nsmooth_r,nsmooth_t],/nan)
            endif else begin
                print,'usage: smooth <BufferNumber> <nbox_time> <nbox_range>'
            endelse
        end
        (cmd eq 'scatter' or cmd eq 'sc') and narg eq 3: begin
            bufnr1 = fix(arg[1])-1
            bufnr2 = fix(arg[2])-1
            if bufnr1 ge 0 && bufnr1 lt nbuffer && n_elements(*buffer[bufnr1].ptr_prof) gt 1 $
            && bufnr2 ge 0 && bufnr2 lt nbuffer && n_elements(*buffer[bufnr2].ptr_prof) gt 1 then begin
                ; time (dwell matchnig)
                if pp.maxtdsyn gt 0 then maxtdsyn=pp.maxtdsyn else begin
                      maxtdsyn = ((*(buffer[bufnr1].ptr_hours))[1]-(*(buffer[bufnr1].ptr_hours))[0]) $
                    > ( (*(buffer[bufnr2].ptr_hours))[1]-(*(buffer[bufnr2].ptr_hours))[0])
                endelse
                hours1 = *(buffer[bufnr1].ptr_hours)
                hours2 = *(buffer[bufnr2].ptr_hours)
                ntr = n_elements(hours1)
                nmatcheddwells=0L
                ixmatched1=lonarr(ntr)
                ixmatched2=lonarr(ntr)
                for iti=0L, ntr-1L do begin
                    tdsyn = min(abs(hours1[iti]-hours2),ixsysn)
                    if tdsyn le maxtdsyn*36d2 then begin
                        ixmatched1[nmatcheddwells]=iti
                        ixmatched2[nmatcheddwells]=ixsysn
                        nmatcheddwells++
                    endif
                endfor
                if nmatcheddwells gt 0 then begin
                    ; range matching yet only a pp.scatter_rg_shift can be set by the user
                    matchzrg=zrg-abs(pp.scatter_rg_shift)
                    if matchzrg gt 0 then begin
                        ix0machedrg1 = - (pp.scatter_rg_shift < 0)
                        ix0machedrg2 = + (pp.scatter_rg_shift > 0)
                        scatterx=reform((*(buffer[bufnr1].ptr_prof))[ixmatched1[0:nmatcheddwells-1],ix0machedrg1:ix0machedrg1+matchzrg-1])
                        scattery=reform((*(buffer[bufnr2].ptr_prof))[ixmatched2[0:nmatcheddwells-1],ix0machedrg2:ix0machedrg2+matchzrg-1])
                        scatter_gd=where(scatterx ge (buffer[bufnr1].yrange)[0] and scatterx le (buffer[bufnr1].yrange)[1] and $
                                         scattery ge (buffer[bufnr2].yrange)[0] and scattery le (buffer[bufnr2].yrange)[1],scatter_ngd)
                        if scatter_ngd ge 1 then begin
                            scatterx=scatterx[scatter_gd]
                            scattery=scattery[scatter_gd]
                            plot,scatterx,$
                                scattery,$
                                xra=buffer[bufnr1].yrange,yra=buffer[bufnr2].yrange,psym=3,xst=1,yst=1,$
                                xtit=buffer[bufnr1].filename+' '+buffer[bufnr1].valuetitle,$
                                ytit=buffer[bufnr2].filename+' '+buffer[bufnr2].valuetitle
                            oplot,buffer[bufnr1].yrange,buffer[bufnr2].yrange,color=12
                            ; oplot,scatterx,scattery,psym=3
                            scatterdiff0 = median(scattery-scatterx)
                            if pp.scattertol lt 0 then scattertol=((buffer[bufnr1].yrange)[1]-(buffer[bufnr1].yrange)[0])/2. else scattertol=pp.scattertol
                            scatterclose = where(abs(scattery-scatterx-scatterdiff0) le scattertol,scatternclose)
                            if (scatternclose gt 1) then begin
                                scatterdif=total(scattery[scatterclose]-scatterx[scatterclose])/float(scatternclose)
                                scattervarvar= sqrt(total((scattery[scatterclose]-scatterx[scatterclose]-scatterdif)^2)/float(scatternclose))
                                oplot,buffer[bufnr1].yrange,buffer[bufnr2].yrange+scatterdif,color=2
                                print,'average difference: ',scatterdif,'  variance: ',scattervarvar
                            endif
                        endif
                    endif else print,'no matching rangegates'
                endif else print,'no dwells with matching time'
            endif else begin
               print,'usage: scatter <BufferNumber4xAxis>  <BufferNumber4yAxis>'
               print,'The buffers <BufferNumber4*> must be filled by load-cmd before.'
            endelse
        end
        cmd eq 'calc': begin
            bufnrSRC=-1
            bufnrTRG=-1
            right2eq = 0
            for i=1, narg-1 do begin
                if arg[i] eq '=' then right2eq=1
                bufnr = fix((stregex(arg[i],'^#([1-9])$',/subexp,/extract))[1])-1
                bufnr_str = string(bufnr,format='(I1.1)')
                if bufnr_str ne '*' && bufnr le 8 then begin
                    arg[i] = '*buffer['+bufnr_str+'].ptr_var'
                    if right2eq eq 0 then bufnrTRG=bufnr
                    if right2eq and bufnrSRC lt 0 then bufnrSRC=bufnr
                endif
                bufnr = fix((stregex(arg[i],'^H#([1-9])$',/subexp,/extract))[1])-1
                bufnr_str = string(bufnr,format='(I1.1)')
                if right2eq eq 1 && bufnr_str ne '*' && bufnr le 8 then begin
                    arg[i] = '(*buffer['+bufnr_str+'].ptr_range # replicate(1.0,n_elements(*buffer['+bufnr_str+'].ptr_time)))'
                endif
            endfor
            if bufnrSRC ge 0 and bufnrTRG ge 0 then buffer[bufnrTRG] = buffer[bufnrSRC]
            buffer[bufnrTRG].ptr_var = ptr_new(*buffer[bufnrSRC].ptr_var)
            ;buffer[bufnrTRG].ptr_prof = ptr_new(*buffer[bufnrSRC].ptr_prof)
            buffer[bufnrTRG].ptr_time = ptr_new(*buffer[bufnrSRC].ptr_time)
            ;buffer[bufnrTRG].ptr_hours = ptr_new(*buffer[bufnrSRC].ptr_hours)
            buffer[bufnrTRG].ptr_range = ptr_new(*buffer[bufnrSRC].ptr_range)
            ;buffer[bufnrTRG].ptr_hei = ptr_new(*buffer[bufnrSRC].ptr_hei)
            buffer[bufnrTRG].ptr_tpow = ptr_new(*buffer[bufnrSRC].ptr_tpow)
            returnvalue = execute(strjoin(arg[1:narg-1],' '))
        end
        cmd eq 'histo': begin
            if narg eq 1 then begin
                bufnrs = [bufnr] & nbufnrs=1
            endif else begin
                bufnrs = (fix(arg[1:narg-1])-1)
                nbufnrs=narg-1
            endelse
            for i=0,nbufnrs-1 do begin
                bufnr=bufnrs[i]
                if bufnr lt 0 || bufnr ge nbuffer || n_elements(*buffer[bufnr].ptr_prof) le 1 then begin
                    print,'No data is in buffer number'+string(bufnr+1)
                    goto,emptybuffer
                endif
            endfor
            bufnr=bufnrs[0]
            dzra = buffer[bufnr].yrange[1] - buffer[bufnr].yrange[0]
            x = findgen(pp.nbin)/float(pp.nbin)*dzra+buffer[bufnr].yrange[0]
            histo=histogram(*buffer[bufnr].ptr_prof,min=(buffer[bufnr].yrange)[0],max=(buffer[bufnr].yrange)[1],nbins=pp.nbin)
            plot,x,histo,xrange=buffer[bufnr].yrange,xst=1,ytit='Distribution Frequency / ' + string(dzra/pp.nbin,format='(g7.1)'),$
                    xtit=valuetitle,ylog=1,yrange=[1,max(histo)],/nodata
            for i=0,nbufnrs-1 do begin
                bufnr=bufnrs[i]
                if n_elements(*buffer[bufnr].ptr_prof) gt 1 then begin
                    histo=histogram(*buffer[bufnr].ptr_prof,min=x[0],max=x[pp.nbin-1],nbins=pp.nbin)
                    oplot,x,histo,color=i+1
                endif else print,'No data in the buffer.'
            endfor
            emptybuffer:
        end
        cmd eq 'sequence' or cmd eq 'zr': begin
            last_draw_arg = arg
            arg[0]='&'
            IxPlt = where(arg eq '&' or arg eq '|', zPltO) ; plots and oplots
            dummy = where(arg eq '&', zPlt) ; only plots that need a new frame
            multi_save=!p.multi
            erase
            iPlt=0
            for iPltO=0,zPltO-1 do begin
                ; Default Values for the plotting parameters:
                kpzr = { $
                    ra:[0.0,-1.0],$
                    ti:'',$
                    os:0.0,$
                    tr:[0d0,-1d0],$
                    co:!p.color $
                }
                ; Find out what to plot:
                plt_item = arg[IxPlt[iPltO] + 1]
                ; search for h<bufnr>.<rg> plot range gate <rg> of buffer <bufnr>
                greped = stregex(plt_item,'^h([1-9])\.([0-9]*)$',/subexp,/extract,/fold)
                if greped[0] ne '' then begin 
                   rg=long(greped[2])
                   bufnr=long(greped[1])-1L
                endif else rg=-1
                ; search for srvi.<tag> where <tag> is a tagname defined in read_srvinfo.pro
                greped = stregex(plt_item,'^srvi\.(.*)$',/subexp,/extract,/fold)
                if greped[0] ne '' then begin 
                   srvi=greped[1]
                endif else srvi=''
                LogItemNr = (stregex(plt_item,'^l([0-9]*)$',/subexp,/extract,/fold))[1]
                ; search for l<log_item_nr> where <log_item_nr> is the number of the log parameter
                ; loaded by grep_log before
                ZrIsThere = 0
                case 1 of
                    rg ge 0: begin
                        if n_elements(*buffer[bufnr].ptr_prof) gt 1 then begin
                            ZrData = (*buffer[bufnr].ptr_prof)[*,rg]
                            ZrTime = *buffer[bufnr].ptr_hours
                            zti = n_elements(ZrTime)
                            ZrTrange = [ZrTime[0],ZrTime[zti-1]]
                            ZrHight = (*buffer[bufnr].ptr_hei)[rg] 
                            kpzr.ti = buffer[bufnr].valuetitle + ' ' + string(ZrHight/pp.td,format='(f6.2)') + ' '+pp.tdunit
                            kpzr.ra = buffer[bufnr].yrange
                            ZrIsThere = 1
                        endif
                    end
                    srvi ne '': begin
                        ncdfidv = NCDF_VARID(ncdfid, srvi ) & NCDF_VARGET, ncdfid, ncdfidv, ZrData
                        ZrTime = *buffer[bufnr].ptr_hours
                        zti = n_elements(ZrTime)
                        ZrTrange = [ZrTime[0],ZrTime[zti-1]]
                        kpzr.ti = srvi
                        kpzr.ra = [min(ZrData,max=sdmax),sdmax]
                        ZrIsThere = 1
                    end
                    else: begin
                        if ( n_elements(log_params) gt 0 ) then begin
                            if (LogItemNr ne '') then begin
                                ZrDataIx = long(LogItemNr)
                            endif else begin
                                ZrDataIx = (where(log_params eq plt_item))[0]
                            endelse
                            if (ZrDataIx ge 0) then begin
                                ZrData = log_data.(ZrDataIx+1)
                                ZrTime = log_time
                                ZrTrange = [log_time[0],log_time[log_n-1]]
                                kpzr.ti = log_params[ZrDataIx]
                                kpzr.ra = [0.,-1.]
                                ZrIsThere = 1
                            endif
                        endif  
                    end
                endcase
                
                ; Overwrite the default plotting parameters kpzr by the settings supplied in the cmd line
                kpzr0 = IxPlt[iPltO] + 2          ; index at which the key word parameters in arg start
                if iPltO lt zPltO-1 then begin
                    kpzr1 = IxPlt[iPltO + 1]-1
                endif else begin
                    kpzr1 = narg -1
                endelse
                nkpzr = kpzr1 - kpzr0+1
                if nkpzr gt 0 then begin
                    argv2stru,arg[kpzr0:kpzr1],kpzr
                endif
                
                ; make the plot
                if ZrIsThere then begin
                    if iPlt eq 0 then begin
                        ; select the timerange in the first plot so that all plots will have the same time range
                        if kpzr.tr[0] lt kpzr.tr[1] then begin
                            ZrTrange4all = kpzr.tr
                        endif else begin
                            ZrTrange4all = ZrTrange
                        endelse
                    endif 
                    if kpzr.tr[0] lt kpzr.tr[1] then ZrTrangeX = kpzr.tr else ZrTrangeX = ZrTrange4all
                    ZrData += kpzr.os
                    if arg[IxPlt[iPltO]] eq '&' then begin
                        ; make a new frame
                        !p.multi=[++iPlt,1,zPlt]
                        if kpzr.ra[0] gt kpzr.ra[1] then begin
                            kpzr.ra = autorange(ZrData,99.9,8.) ; the range is chosen to cover 98 percent of the data 
                                                               ; and afterwards it is increased by 4 percent to each side.
                        endif
                        plot,ZrTime,ZrData,/nodata,yrange=kpzr.ra,ytit=kpzr.ti,xrange=ZrTrangeX,xst=1,yst=1,charsi=pp.charsize
                        last_ra = kpzr.ra
                    endif else kpzr.ra = last_ra
                    oplot,ZrTime,ZrData,color=kpzr.co,min=kpzr.ra[0],max=kpzr.ra[1]
                    oplot,ZrTime,ZrData,color=kpzr.co,psym=pp.zrsym
                endif
            endfor
            !p.multi=multi_save
        end
        cmd eq 'grep_log' or cmd eq 'gl': begin
            nparam = (where(arg[1:narg-1] eq ':'))[0]
            if nparam gt 0 then begin
                log_params = arg[1:nparam]
                spawn,'grep_log '+tota(arg[1:narg-1]+' ')+'> /tmp/grep_log.tmp'
                log_data = read_ascii('/tmp/grep_log.tmp', TEMPLATE={ VERSION:1.0, DATASTART:0, $
                    DELIMITER:32b, MISSINGVALUE:9e29, COMMENTSYMBOL:'#', FIELDCOUNT:nparam+1,$
                    FIELDTYPES:[3,replicate(4,nparam)], FIELDNAMES:['time',log_params],$
                    FIELDLOCATIONS:replicate(-1,nparam+1), FIELDGROUPS:lindgen(nparam+1)})
                spawn,'rm /tmp/grep_log.tmp'
                if n_elements(first_day_0000) ne 1 then first_day_0000 = long(log_data.time[0]/(3600d0*24d0))*24d0
                log_time = log_data.time/36d2 - first_day_0000
                log_n = n_elements(log_time)
                ; log_data.data[(where(log_params eq param]))[0],*)
            endif else print,'Pls. use a detached colon to separate parameters from log file names'
        end
        cmd eq 'postscript' or cmd eq 'ps' or cmd eq 'print': begin
            if narg eq 2 then psfile = arg[1] else psfile="viewncdf.ps"
            if (!d.name ne 'PS') then set_plot,'PS'
            device,/color,bits_per_pixel=8,filename=psfile ; ,yoffset= 27,xoffset=0.7
        end
        cmd eq 'png': begin
            if narg ge 2 then png_fn = arg[1] else begin
                png_fn =  'viewncdf' + tag_mom + '.png'
            endelse
            case !d.name of
                'X':write_png, png_fn, TVRD(true=1)
                'Z': begin
                    write_png, png_fn, TVRD() , ct_red, ct_green, ct_blue
                end
                else:
            endcase
        end
        cmd eq 'close':begin
            device,/close
            if startup_device ne 'PS' then set_plot,'X'
        end
        cmd eq 'set': begin
            if narg ge 2 then set_tag_name = arg[1] else set_tag_name='HELP'
            if strupcase(set_tag_name) eq 'HELP' then begin
                pp_tags = tag_names(pp)
                pph_tags = tag_names(pph)
                for i=0,n_elements(pp_tags)-1 do begin
                    helptext_tag_number = (where(pph_tags eq pp_tags[i]))[0]
                    helptext = helptext_tag_number ge 0 ? pph.(helptext_tag_number) : ''
                    print_table,pp_tags[i]+':',helptext,tabw=20,linew=79
                endfor
            endif else begin
                set_tag_number = (where(strupcase(set_tag_name) eq tag_names(pp)))[0]
                if set_tag_number ge 0 then begin
                    if narg eq 2 then print,set_tag_name+' :',pp.(set_tag_number) else begin
                        for i=0,(narg-3) < (n_elements(pp.(set_tag_number))-1) do begin
                            if strupcase(arg[i+2]) ne 'SKIP' then pp.(set_tag_number)[i] = arg[i+2]
                        endfor
                    endelse
                endif else print,'unknown variable name'
            endelse
        end
        cmd eq 'bset': begin
            bufnr = fix(arg[1])-1
            if bufnr ge 0 && bufnr lt nbuffer-1 && buffer[bufnr].name ne 'unknown' then begin
                if narg ge 3 then set_tag_name = arg[2] else set_tag_name='HELP'
                if strupcase(set_tag_name) eq 'HELP' then begin
                    pp_tags = tag_names(buffer[bufnr])
                    pph_tags = tag_names(buffer_help)
                    for i=0,n_elements(pp_tags)-1 do begin
                        helptext_tag_number = (where(pph_tags eq pp_tags[i]))[0]
                        helptext = helptext_tag_number ge 0 ? buffer_help.(helptext_tag_number) : ''
                        print_table,pp_tags[i]+':',helptext,tabw=20,linew=79
                    endfor
                endif else begin
                    set_tag_number = (where(strupcase(set_tag_name) eq tag_names(buffer[bufnr])))[0]
                    if set_tag_number ge 0 then begin
                        if narg eq 3 then print,set_tag_name+' :',buffer[bufnr].(set_tag_number) else begin
                            for i=0,(narg-4) < (n_elements(buffer[bufnr].(set_tag_number))-1) do begin
                                if strupcase(arg[i+3]) ne 'SKIP' then begin
                                    newval=arg[i+3]
                                    regex = stregex(newval,"^\'(.*)\'",/extr,/subex)
                                    if regex[0] ne '' then newval=regex[1]
                                    buffer[bufnr].(set_tag_number)[i] = newval 
                                endif
                            endfor
                        endelse
                    endif else print,'unknown variable name'
                endelse
            endif else begin
                print,'usage: bset <BufferNumber> <name> <value1> [<value2> ...]  '
                print,'The buffer <BufferNumber> must be filled by load-cmd before.'
           endelse
        end
        cmd eq 'info': begin
        end
        cmd eq 'stop': if lmgr(/vm) then goto,ende else stop
        cmd eq 'x' or cmd eq 'quit' or cmd eq 'exit': goto,ende
        else: begin
            setcmd = ''
            for i=0 , narg - 1 do setcmd = setcmd + arg[i] + ' '
            if lmgr(/vm) then print,'VM license can not execute '+setcmd $
            else dummi=execute( setcmd )
        end
    endcase
endwhile

ende:
if !d.name eq "X" then wdelete
exit
end

