function near_field_correction, lambda, antenna_diameter, range, decibel=decibel

; Calculates near field correction to be applied to radars, assuming Gaussian 
; antenna patterns corresponding to eq. 17 in Stephen M. Sekelsky: Near-field 
; reflectivity and antenna Boresight gain correction for millimeter-wave
; radars, JTECH, 19, 2002
;
; Syntax:
;
;   result = near_field_correction(lambda, antenna_diameter, range, decibel=decibel)
;
; Return value:
;
;   Returns the signal correction value either in linear units or in 
;   dBz 
; 
; Arguments:
;
;  lambda            - wavelength of the radar in m
;  antenna_diameter  - diameter of antenna in m
;  range             - height range vector
;
; Keyword:
;
;  decibel - control the returned value, if this keyword parameter is set then 
;            the correction is given dBz 
;
; UG 13.10.2010


; far-field distance
rf = 2 * antenna_diameter^2 / lambda 


linear_corr = 1. / (  (5.26e-5 + (range / rf)^2.5 )/(0.0117 + (range / rf)^2.5 ) )
db_corr = 10*alog10(linear_corr)

if keyword_set(decibel) then return, db_corr else return, linear_corr

end