pro spcs2mom,arguments
;
; obsolete
; 
; Reads the spectra from pds files and estimates 3 spectral moments of the nmulti most 
; dominant peaks. If nmulti=1 and global=1 then it estimates the 3 spectral moments of 
; the complete spectra as the DSP software does.
;
; Calling Sequence: 
;
; unix> idl spcs2mom infile=data/20040602_1000.pds outfile=20040602.9.pdf nmulti=9
;
; IDL > spcs2mom, 'infile=data/20040602_1000.pds outfile=xxx_off.pdf'
; 
; infile      File to read the pds data from. Multible files with contain successive 
;             (in time) data may be given as array of strings.
;
; outfile     File for writing the output data.
;
; nmulti      number of most dominant peaks from which moments should be saved.
; 
; See the structure pph below for a deskription of the other keyword parameters.
;
; Matthias Bauer-Pfundstein bauer@metek.de
;
common startup,startup_done
if n_elements(startup_done) ne 1 then begin
@startup
endif

; Arguments can be provided by two methods:
; - If spcs2mom is called by an interactive IDL session the first (and only) 
; string parameter arguments which contains a space separated list is split to
; the strarr argv.
; - Aditionally strings contained in the environment variables ARG1, ARG2, ..., 
; ARG9, ARG10, ... are prepended to argv. The script idl packs its calling arguments
; these environment variables. This is done as IDL can not extract the shell 
; variables like $1, $2,...
argv = env2argv(arguments)

kp = { $
    infile:'', $
    outfile:'' $
}
argv2stru,argv,kp

; Default values for the Ploting parameters. These Parameters can be changed by the command "set"
pp={ $
nave:10,$
nmulti:1,$
eformat:0,$
global:1,$
expected_vrange:2,$
of0:-1,$     
hilde_method:3,$ 
ofdiv:9.,$
ndiv:16,$
rgclutter:0,$
max_sholder_dbc:-0.20,$
fftwin:[0.75, 0.3, 0.16, 0.065, 0.023, 0.008, 0.0025],$
gsm_n:-1, $
gsm_alpha:8.6, $
peak_thr:9.0 ,$
peak_sig_thr:0.05,$
gap4dom:5, $
snyc_cx_peak_pos:1,$
modvuar:0$
}
argv2stru,argv,pp

; online descripion for the above parameters
pph={ $
nave: 'Number of spectral averages',$
nmulti: "Number of  peaks for which the spectra should be saved. If 0 the global moments are estimated." , $
eformat: 'use an efficient format for output data',$
global: "If 1, the global moments are estimated instead of using multipeak identyfication." , $
expected_vrange: 'In global mode it saves some CPU time to preselect the expected radial velocity. Large error will occure for peaks that are (n*1/2)*UAVR different from the expected value!', $
of0: "ofs0, parameter for detection threshold", $
hilde_method: "enumerate: 0 take noise from noise gate, 1 hildebrand-sekhon, 2 hildebrand-fischer-henemut, 3 HSdiv",$
ofdiv: "Used by HSdiv, similar to ofs0 for determination of detection threshold. divisions from minimum to this thresold are regarded as noise",$
ndiv: "Used by HSdiv, number of parts to which the spectrum is divided",$
rgclutter:"maximum height in which clutter hould be removed",$
max_sholder_dbc:"theroretically the sholders of the clutter in the spectra should be -6 dbc. Non clutter signal may cover the sholders. Then no clutter removal is performed",$
fftwin:"decaying factor for the neighbor spectral lines in clutter removal",$
gsm_n:'number of points of convol. Kernel for smoothing before peak search', $
gsm_alpha:'1/width of gausian kernel for smoothing, see gsm_n', $
peak_thr: 'Threshold for peak separation, canion method. Canions sepatate peaks if the relative depth of the canion inbtween is greater than peak_thr.', $ 
gap4dom: "Peaks are separated in any case if more than gap4dom values are below detection thresold between them",$
snyc_cx_peak_pos: "If 1, the peak ranges estimated in the co-spectrum are used for moment estimation in the co- and the cross-spectra",$
modvuar: "Restrict the calculated 1st moments to +/-0.5 velocity un-ambiguty range, by doing a modolo operation"$
}

; open the pds file and read the fileheader
case kp.infile of 
    '': lun=0
    'data': begin
        spawn,"tail_s2m 2 /data/\*.pds",unit=lun
    end
    else: openr,lun,kp.infile,/get_lun
endcase
openw,outlun,kp.outfile,/get_lun
; read obsolete header which is in the first 1024 bytes of each file
ppar = read_hdr(lun,hdr)
nmulti=pp.nmulti
write_hdr,outlun, ppar, pp.eformat ? 0 : nmulti

if pp.of0 gt 0 then of0 = pp.of0 else of0 = ppar.of0

zchan=2 ; number of channels
if nmulti ge 2 && pp.global then begin
    pp.global = 0
    print,'Setting global to 0 as nmulti != 1'
endif
if ~ pp.snyc_cx_peak_pos and pp.eformat then begin
    print,'setting snyc_cx_peak_pos=1 as efficient format only works with snyc_cx_peak_pos'
    pp.snyc_cx_peak_pos = 1
endif
maxchg = 512L
nmom=3
zrg = -1L
lig3=lindgen(nmom)
nfft=0  ; nfft of the spectra in momory. If hdr.nfft differs then a new buffer for noisspectra is created.
while not eof(lun) do begin
    ; read one dwell of spectra
resume:
    while (rmh=read_mainframe_header(lun, hdr)) eq 0 do begin
        if rmh eq -1L then goto,ende
        ppar = read_ppar_frame( lun, hdr)
        out_ppar=ppar
        out_ppar.raw = 0
        out_ppar.ihp = out_ppar.raw_gate1
        out_ppar.chg = out_ppar.raw_gate2-out_ppar.raw_gate1+1+2
        out_ppar.avc = ppar.avc * pp.nave
        write_ppar_frame,outlun, out_ppar
    endwhile
    if zrg ne hdr.raw_zrg then begin
        ; create some arrays
        zrg = hdr.raw_zrg
        moments = fltarr(zchan, maxchg, nmom+1, nmulti)
        npeaks = bytarr(maxchg) ; needed for eformat
        if pp.snyc_cx_peak_pos then peak_ifrqs_ptrs=ptrarr(maxchg)
    endif
    if hdr.nfft ne nfft then begin
        nfft=hdr.nfft
        nave=pp.nave & ednave = 1./float(nave)
        ix2ord = (lindgen(nfft) + nfft/2 ) mod nfft
        dx = hdr.VUAR/nfft
        x = cmod(dx * findgen(nfft), hdr.VUAR,0)
        ed_laprf = 1./(hdr.la * hdr.prf)
        xord=x[ix2ord]*ed_laprf
        if pp.expected_vrange gt -1000.0 then begin
            xord_expect_vrange = cmod(x[ix2ord],hdr.VUAR,-pp.expected_vrange) * ed_laprf 
        endif
        ; buffers for the spectra
        iave=0
        avespc  = fltarr(nfft,zrg+2,zchan)
        avespcq = fltarr(nfft,zrg+2,zchan)
        tpow = 0.0
    endif

    if iave eq 0 then begin
        ; clear the buffers to which the spectra will be averaged
        avespc[*,*,*]=0.0
        avespcq[*,*,*]=0.0
        tpow = 0.0
    endif
    
    ptr_free,peak_ifrqs_ptrs
    if ptr_valid(p_srvinfo) then ptr_free,p_srvinfo
    if ptr_valid(p_data) then heap_free,p_data    ; replaces ptr_free,(*p_data).sp1 & ptr_free,(*p_data).sp2 & ptr_free,p_data
    
    ; read the spectra here:
    read1pds_dwell, lun, hdr, ppar, p_srvinfo, p_data, extract=['SP1','SP2']
    if n_elements( (*(*p_data).sp1)[*,0] ) ne nfft then begin
        if eof(lun) then goto,ende
        print,'corrupted data'
        goto,resume
    endif

    if n_elements(sp_tagnr) ne 1 then begin
        sp_tagnr = [(where(tag_names(*p_data) eq 'SP1'))[0],(where(tag_names(*p_data) eq 'SP2'))[0]]
        cal_tagnr = [(where(tag_names(*p_srvinfo) eq 'CPW1'))[0],(where(tag_names(*p_srvinfo) eq 'CPW2'))[0]]
        noise_tagnr = [(where(tag_names(*p_srvinfo) eq 'NPW1'))[0],(where(tag_names(*p_srvinfo) eq 'NPW2'))[0]]
    endif

    ; Save the spectra in one array
    for ichan=0,zchan-1 do begin
        avespc[*,*,ichan] += (*(*p_data).(sp_tagnr[ichan]))[*,*]
        avespcq[*,*,ichan] += ((*(*p_data).(sp_tagnr[ichan]))[*,*])^2
        ; avespcq[*,*,ichan] += (avespc[*,*,ichan])^2   ; see whats faster
        tpow += (*p_srvinfo).tpow
    endfor
    iave++

    if iave eq pp.nave then begin
        for ichan=0,zchan-1 do begin
            ; Do the final adjustments of the spectra averaging
            avespc = ednave * avespc
            avespcq = ednave * avespcq - avespc^2
            (*p_srvinfo).tpow = ednave * tpow
            iave=0
            
            ; calculate the noise power and the calibration power from the averaged spec. (sliding average over the last 16 spectra is no more implemented.)
            NoiseLevel = median(avespc[*,zrg,ichan])
            CalNoiseLevel = median(avespc[*,zrg+1,ichan])
            (*p_srvinfo).(noise_tagnr[ichan]) = NoiseLevel * nfft
            (*p_srvinfo).(cal_tagnr[ichan]) = CalNoiseLevel * nfft
            ; the thermistor power should be averaged
            
            if pp.hilde_method eq 0 then begin
                det_thres_from_noisegate = NoiseLevel * ( 1 + of0/10. / sqrt(ppar.avc))
            endif else begin
                det_thres_fac = ( 1 + of0/10. / sqrt(ppar.avc))
            endelse
            noise_snr = NoiseLevel*float(nfft)
            
            ; do the moment estimation for each range gate and each channel
            for irg = 0, zrg-1 do begin
                spc = avespc[*,irg,ichan]
                var = avespcq[*,irg,ichan]
                if pp.hilde_method eq 0 then begin
                    det_thres = det_thres_from_noisegate > (max(spc) / linear(ppar.SWT))
                endif else begin
                    NoiseLevel = hilde(spc,method=pp.hilde_method, ofdiv=pp.ofdiv, ndiv=pp.ndiv, n_spcave=ppar.avc)
                    det_thres = NoiseLevel * det_thres_fac
                endelse

                ; clutter removal
                if irg le pp.rgclutter then begin
                    remove_clutter,spc,NoiseLevel,det_thres,pp
                endif
                
                spcord = [spc[nfft/2:nfft-1],spc[0:nfft/2-1]] ; Not neccessaryly neccessary
                varord = [var[nfft/2:nfft-1],var[0:nfft/2-1]]
                if pp.global eq 1 then begin
                    ; estimate moments of the complete spectrum as done by the dsp software (global)
                    ix_above_thres = where(spcord ge det_thres, nix_above)
                    if nix_above gt 0 then begin
                        ; noise removal and thresholding
                        spc_above_9noise = spcord[ix_above_thres] - NoiseLevel
                        if pp.expected_vrange lt -1000. then begin
                            dummy = max(spc_above_9noise,ixxmax) 
                            xx = cmod(xord[ix_above_thres],hdr.VUAR,xord[ixxmax])
                        endif else begin
                            xx = xord_expect_vrange[ix_above_thres]
                        endelse
                        ; moment estimation
                        moments[ichan, irg, lig3, 0] = calc_moms(spc_above_9noise,xx,noise_snr)
                        moments[ichan, irg, 3, 0] = total(varord[ix_above_thres]*spc_above_9noise) / moments[ichan, irg, 0, 0] 
                        if pp.modvuar then moments[ichan, irg, ipeak, 1L] =  cmod(moments[ichan, irg, ipeak, 1L], hdr.VUAR,0)
                    endif else begin
                        moments[ichan, irg, *, 0] = [0.0, -9999.0, -9999.0, 0.0]
                    endelse
                endif else begin
                    ; estimate the moments of all peaks (multi peak)
                    ; use canion algorithm for identifying dominant peaks    
                    if ~pp.snyc_cx_peak_pos ||  ichan eq 0 then begin
                        if pp.gsm_n gt 0 then spc4id_pk = gsmooth(spcord > det_thres,n=pp.gsm_n,alpha=pp.gsm_alpha) $
                                        else spc4id_pk = spcord > det_thres
                        peak_ifrqs = identify_peaks_by_canion(spc4id_pk,det_thres,peak_thr_fac=pp.peak_thr/sqrt(ppar.avc),$
                            gap4dom=pp.gap4dom,peak_sig_thr=pp.peak_sig_thr,maxis=maxis)
                        if peak_ifrqs[0,0] ne -1 then begin
                            ; sort the peaks by their maxi values saved in maxis
                            peak_ifrqs = peak_ifrqs[*,reverse(sort(maxis))]
                        endif
                        if pp.snyc_cx_peak_pos && ichan eq 0 then begin
                            ; save the result of identify_peaks for re-using for the cross-channel
                            peak_ifrqs_ptrs[irg] = ptr_new(peak_ifrqs)
                        endif
                    endif
                    if pp.snyc_cx_peak_pos && ichan eq 1 then begin 
                        peak_ifrqs = *peak_ifrqs_ptrs[irg]
                    endif
                    npeak = n_elements(peak_ifrqs)/2 ; this is 0 in case peak_ifrqs=-1
                    if ichan eq 0 then npeaks[irg] = npeak < nmulti
                    if peak_ifrqs[0,0] eq -1 then npeak=0
                    for ipeak=0,nmulti-1 do begin
                        if ipeak lt npeak then begin
                            ixa = peak_ifrqs[0,ipeak] & ixb = peak_ifrqs[1,ipeak]
                            if ixa lt ixb then begin
                                spc_of_peak = spcord[ixa:ixb]-NoiseLevel
                                xx_of_peak = xord[ixa:ixb]
                                var_of_peak = varord[ixa:ixb]
                            endif else begin
                                spc_of_peak = [spcord[ixa:nfft-1],spcord[0:ixb]]-NoiseLevel
                                xx_of_peak = [xord[ixa:nfft-1],xord[0:ixb]+hdr.VUAR] ; calculated 1st moments will be in -VUAR/2:VUAR
                                var_of_peak = [varord[ixa:nfft-1],varord[0:ixb]]
                            endelse
                            moments[ichan,irg,lig3,ipeak] = calc_moms(spc_of_peak, xx_of_peak,noise_snr)
                            moments[ichan,irg,3,ipeak] = total(var_of_peak*spc_of_peak) / moments[ichan, irg, 0, ipeak]
                            if pp.modvuar then begin
                                moments[ichan,irg,1L,ipeak]=cmod(moments[ichan,irg,1L,ipeak],hdr.VUAR,pp.expected_vrange)
                            endif
                        endif else begin
                            moments[ichan,irg,*,ipeak] = [0.0, -9999.0, -9999.0, 0.0]
                        endelse
                    endfor
                endelse  ; multi peak
            endfor  ; irg
        endfor  ; ichan
        ; write the moments to file
        if pp.eformat then begin
            size_moms=total(npeaks)*4L*4L
            len_mainframe = 8L + hdr.len_srvi + 8L + maxchg + 8L + size_moms 
            writeu,outlun,'HALO', len_mainframe
            writeu,outlun,'SRVI', hdr.len_srvi,*p_srvinfo
            writeu,outlun,'ENPK', maxchg, npeaks
            writeu,outlun,'EMOM', size_moms
            for irg=0,maxchg-1 do begin
                if npeaks[irg] gt 0 then writeu,outlun,moments[*, irg, *, 0:npeaks[irg]-1]
            endfor
        endif else begin
            ; len_mainframe = 8L + hdr.len_srvi + 8L + 0L + 4L * (8L + maxchg * 4 * 2) + 8L 
            len_mainframe = 8L + hdr.len_srvi + 8L + 0L + 3L * (8L + maxchg * 4 * 2) + 8L 
            writeu,outlun,'HALO', len_mainframe
            writeu,outlun,'SRVI', hdr.len_srvi, *p_srvinfo
            writeu,outlun,'FFTD', 0L
            writeu,outlun,'SNRD', maxchg * 4 * 2 * nmulti, moments[*, *, 0, *]
            writeu,outlun,'VELD', maxchg * 4 * 2 * nmulti, moments[*, *, 1, *]
            writeu,outlun,'RMSD', maxchg * 4 * 2 * nmulti, moments[*, *, 2, *]
            ; writeu,outlun,'SVRD', maxchg * 4 * 2 * nmulti, moments[*, *, 3, *]
            writeu,outlun,'EXPD',0L
        endelse
    endif ; if iave eq pp.nave
endwhile

ende:
close,lun
close,outlun
if ptr_valid(p_srvinfo) then heap_free,p_srvinfo
if ptr_valid(p_data) then heap_free,p_data
exit
end
