function petenko_filter, rf, ve, wd

; filter using a mask and median filtering for real 'pds' data 
; (s. Report of Igor Petenko: Determination of macro physical
;  cloud parameters from measurements of the Lindenberg cloud radar MIRA36)
;
; Arguments:
;
;    rf    - input matrix of reflectivity (missing values equal NaN)
;    ve    - input matrix of Doppler velocity
;    wd    - input matrix of spectral width
;    returnvalue yp    - output binary matrix (1: cloud pixel, 0, no cloud pixel)




; written by Igor Petenko, December 2004/January 2005
; modified by UG 04.01.2004


; settings for clutter removal
wd_thres = 4       ; removing points at lowest gates with too large rms
v_thres  = 8247    ; removing points with V=0.08247 which is observed in clutters (at lowest about 3 km)

s = size(rf)
r = s[2] ; rows (height)
c = s[1] ; columns (time)


; Grid parameters and thresholds:
;nm=[5,5]   ; size of box
n1_th=9    ; thresholds of number of "1s" in the ROUNDED box 5x5
;!!!!!( n1_th_o(1)=9 is found empirically)
; for SQUARE box n1_th=11, ( n1_th=13 is equivalent to p_thresh=5*10^-12 for a box 5x5. NB!!! n1_th=11 is better!!!)
;===================================================

; creation of BINARY MASK (0-1) after threshold filtering for REFLECTIVITY

n1 = where(finite(rf));n1 = find(rf>NaN);
r1 = replicate(0, c, r); ; binary REFLECTIVITY matrix after threshold filtering
r1[n1] = 1 ; '0' in a binary REFLECTIVITY matrix after threshold filtering
n1 = where((wd gt wd_thres),count); removing points at lowest gates with too large rms
r1_1 = replicate(1, c, r); ; binary REFLECTIVITY matrix after threshold filtering
if count gt 0 then r1_1[n1] = 0 ;
r1[*, 0:9]=r1[*, 0:9]*r1_1[*, 0:9] ;;applying only for lowest 9 gates
n1 = where(round(100000*ve) eq v_thres, count) ; removing points with V=0.08247 which is observed in clutters (at lowest 3 km)
r1_1 = replicate(1, c, r); ; binary REFLECTIVITY matrix after threshold filtering
if count gt 0 then r1_1[n1] = 0 ;
r1[*, 0:100]=r1[*, 0:100]*r1_1[*, 0:100] ;applying only for lowest 100 gates
rr = rf * r1; ; REFLECTIVITY values matrix after threshold filtering


;////////////////////////////////////////////////
; CLOUD MASK FILTERING using ROUNDED filter box

; Applying a filter ROUNDED grid (kernel) ;1

;          ooo       5x5
;         ooooo
;         ooooo
;         ooooo
;          ooo
dn0 = 5  ; % threshold of the difference of up and down parts of a filter grid
dn1 = 5  ; % (n1_th-dn1) - threshold for the number of '1' pixels to check for belonging to the boundary

; PADDING parameters
pads=2   ;
;padding  is 'symmetric';


 r1=r1
 rp=rr


; ????%PADDING ???????

r1= [[r1[*,2]], [r1[*,1]], [r1[*,*]], [r1[*,r-2]], [r1[*,r-3]]] ; rows
r1= [r1[2,*], r1[1,*], r1[*,*], r1[c-2,*], r1[c-3,*]] ; columns
;    r1 = padarray(r1, padsize, 'symmetric', 'both');
;    ;rp = padarray(rp, padsize, 'symmetric', 'both');

yp = replicate(0, c, r)

;% creation of the matrix of sums over 5 horizontal pixels
end1= r + 4 -1
end2= c + 4 -1
r1_5 = r1(0:end2-4, *)+ r1(1:end2-3, *)+r1(2:end2-2, *)+r1(3:end2-1, *)+r1(4:end2, *); % sum 5 pixels in horizontal direction (over columns)-->n1_h
r1_a = r1(0:end2-4, *)+ r1(4:end2, *); % sum of 2 angle pixels in horizontal direction
;% creation a matrix of sums over 5 horizontal and 5 vertical pixels
r1_55 = r1_5(*, 0:end1-4)+ r1_5(*, 1:end1-3)+r1_5(*, 2:end1-2)+r1_5(*, 3:end1-1)+r1_5(*, 4:end1);  % number of '1' in the square box 5X5
r1_aa = r1_a(*, 0:end1-4)+ r1_a(*, 4:end1);  % sum of 2 angle pixels in horizontal direction and 2 angle pixels in vertical direction
r1_ro = r1_55- r1_aa; % number of '1' in the 'Rounded' box 5x5 ---> n1_a
r1_52 = r1_5(*, 0:end1-1)+ r1_5(*, 1:end1);  % number of '1' in the square box 2row X 5column
r1_52d = r1_52(*, 0:end1-3)- r1_a(*, 0:end1-4); % number of '1' in Down 2 rows of the pixel box -->n1_d
r1_52u = r1_52(*, 3:end1-1)- r1_a(*, 4:end1); % number of '1' in Up 2 rows of the pixel box --> n1_u
r1_ud = abs(r1_52u - r1_52d); % the difference between Up and Down parts -->n1_ud

;% checking criteria
i = where( ((r1_ro ge n1_th) and (abs(r1_ud) lt dn0)) or ((r1_ro ge n1_th-dn1) and (abs(r1_ud) gt 2) and (r1_5(*, 2:end1-2) gt 3))  ); % NB!!! dn0=5 is optimal

if i[0] ge 0L then yp(i)=1

;%===================================================
s2 = size(yp);
r = s2[2]
c = s2[1]
; print, s2
;;;;;;;;;

;===================================================

; Median filtering
med_gr=5; ; size of grid for Median filtering of binary array
;med_gr_v=3; ; size of grid for Median filtering of value matrix
rep=1;
for k=1,rep do begin
yp = median(yp, [med_gr]) ; median filtering of binary matrix
endfor ; k

return,yp



END;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
