function RadarConstant, hdr, serverinfo, calida, do_corrbycal, doLowRangeAttenuation, raw=raw, SnrCorfa, RadarConstant5
; Calculate the factors SnrCorfa[*,Co] and SnrCorfa[*,Cx] which should be multiplied with the SNRs to correct for:
;     1) The power in noise gate serverinfo.npowx is smaller than it would be if the LNA would be terminated by a matched load at 300 K
;        As the SNRs had been devided by serverinfo.npowx they are too big and have to be multiplied by a factor > 1.
;     2) vanisching gain in the lowest range gates.
;     3) if do_corrbycal is selected, correct by npow/cpow*cpow0/npow0. 
;        cpow0/npow0 = so called calibration (~ 10 db) measured while noisecom measurement, 
;        cpow /npow  = so called calibration observed at current measuremnet.
;     4) STC-Mode
;     5) Near Field Correction which is calculated by an estimation from ewan depending on the antenna diameter
; and a factor RadarConstant which has to be multiplied to the corrected SNRs to get the Ze values. Also RadarConstant5 = the radar constant at 5 km 
; height is passed to output. The later is the  product: 
;               [ RadarConstant at 5km * SnrCorfa at 5 km and for co    channel, 
;                 RadarConstant at 5km * SnrCorfa at 5 km and for cross channel]
; Note SnrCorfa has nearly no range dependance above 1 km, so SnrCorfa at 5 km = SnrCorfa at last range gate. 
;
; RadarConstant also calculates a factor RadarConstant which has to be multiplied to the corrected SNRs to get the Ze values.

; These factors are range dependend. Therefore this routine provides two vektors (RadarConstant, SnrCorfa[*,Co], and SnrCorfa[*,Cx]) 
; with one element each per selected range gate.
; The range dependance is mainly due to 1/R^2 in the radar equation. There is an additional rage dependance
; because the pin modulators need some time
; to close after the transmitting pulse. This is is measured during the calibration by the Noisecon source. 
; The Data gained from the calibration with the NoiseCom source is provided by the structure calida. 
;
;
; Matthias 
;
common RadarConstant,zrg,SnrCorfa8,C0,refvals,Hq7H0q,PRef7PNoi50,StcHei,doLowRangeAtt
    if n_elements(raw) ne 1 then raw=0    ; normally 0 as calculating Ze is normally done with the moment and not the spectra
    if raw eq 0 then zrgh=hdr.mom_zrg else zrgh=hdr.raw_zrg
    if n_elements(zrg) eq 0 || zrg ne zrgh || doLowRangeAttenuation ne doLowRangeAtt then begin
        pp={$
            PTAV0:30.  ,$    ; [W] Reference value of the average transmitter power at the receiver output (The internal thermistor 
                        $    ; is calibrated so that it shows the Power at the receiver output. At the transmitter output the power is usually 1.4 dB greater). 
            tau0:200e-9 ,$   ; [s] Pulse width = 1/Bandwidth
            PRF0:5000. ,$    ; [Hz]   Reference Pulse repetition frequency
            H0:5000.   ,$    ; [m]    Reference Height
            lambda:0.00845 ,$; [m]    Wavelength 
            clight:299792458.,$; speed of light m/s
            AGain:linear(49.75),$; Antenna Gain
            ABeamWidth:0.5 * !pi/180.,$
            NfRec0:linear(6.7),$; Reference Value of the total noise figure at the receiver input (including losses in the TR-switch and Couplers). 
                               $; calida.nf[Co/Cx] = value of the receiver noise aktually measured by the noisecom source 
            WGlength:0.65,$     ; single path waveguide length including the inside the feed
            AntennaDiameter4NearFieldCorr:1.0,$ ; Antenna Diameter used for the near field correction, if < 0.1 no near field correction is applied
            KBT0 : 4e-21,$ ; [J] 
            kwq : 0.93,$
            $; x=(findgen(10000)-5000)/5000.
            $; sinc=sin(x*!pi)/x
            $; sinc[5000]=sinc[5001]
            $; sinc /= sinc[5000]
            $; TxRecbandwidthMatchingLoss = total((sinc/ssinc)*sinc) 
            $; TxRecbandwidthMatchingLoss= 1./0.765767 ; Die anderen Losses sind auch als positive dB definiert
            TxRecbandwidthMatchingLoss: 1. $  ; erstmal fehlt mir der Mut zu diesen 1.2 dB
        }
common magic4chs,magic4chs,FzkPdsFmt,syspar
        config2stru,pp,'radarconstant',[syspar.iam+'.config']
        LossWG = linear(0.6 * pp.WGlength*2)  ; Losses in the waveguides, both ways (receiving and transmitting, assuming 0.6 dB/m loss). 
        zrg=zrgh
        doLowRangeAtt = doLowRangeAttenuation

        C0= (1e18 * 1024.0 * alog(2.0) * pp.lambda^2 * pp.KBT0 * LossWG * pp.TxRecbandwidthMatchingLoss * pp.NfRec0 * pp.PRF0 * pp.H0^2) $
             /(!pi^3 * pp.clight * pp.kwq * pp.AGain^2 * pp.ABeamWidth^2 * pp.PTAV0 * pp.tau0)
        C0 = C0 

        if raw eq 0 then H = findgen(zrg) * hdr.delta_h + hdr.mom_h_min $
                    else H = findgen(zrg) * hdr.delta_h + hdr.raw_h_min
        Hq7H0q = (H/pp.H0)^2
        SnrCorfa8 = fltarr(zrg,2)
        SnrCorfa8[*,0] = calida.nf[0]/(pp.NfRec0*calida.CorrLNAmatch[0])  
        SnrCorfa8[*,1] = calida.nf[1]/(pp.NfRec0*calida.CorrLNAmatch[1])
        if doLowRangeAtt then begin
            zGainVsRange=n_elements( calida.GainVsRange[*,0])
            vertical_averaging=round(hdr.PULSE_WIDTH/2.00000e-07)
            if vertical_averaging gt 1 then begin
                GainVsRangeCo_calida=calida.GainVsRange[*,0]
                GainVsRangeCx_calida=calida.GainVsRange[*,1]
                hei=findgen(zrg)*hdr.DELTA_H+hdr.MOM_H_MIN
                heicalida=(5.+findgen(zGainVsRange))*hdr.DELTA_H/vertical_averaging
                GainVsRangeCo=fltarr(zrg)
                GainVsRangeCx=fltarr(zrg)
                for irg=0,zrg-1 do begin
                    ix=where(abs(heicalida-hei[irg]) ge hdr.DELTA_H/2,nix)
                    if nix ge 1 then begin
                        GainVsRangeCo[irg] = total(GainVsRangeCo_calida[ix])/nix
                        GainVsRangeCx[irg] = total(GainVsRangeCx_calida[ix])/nix
                    endif else begin
                        dummy=min(abs(heicalida-hei[irg]),ix)
                        GainVsRangeCo[irg] = total(GainVsRangeCo_calida[ix])
                        GainVsRangeCx[irg] = total(GainVsRangeCx_calida[ix])
                    endelse
                endfor
                SnrCorfa8[*,0] /= GainVsRangeCo
                SnrCorfa8[*,1] /= GainVsRangeCx
            endif else begin
                ihs0 = round(hdr.MOM_H_MIN/hdr.DELTA_H)  ; lowest range gate (integer). GainVsRange has been recorded with ihs0=5
                ihsa=ihs0-5L & ihsb = (ihsa+zrg < zGainVsRange) -1L
                SnrCorfa8[0:ihsb-ihsa,0] /= calida.GainVsRange[ihsa:ihsb,0]
                SnrCorfa8[0:ihsb-ihsa,1] /= calida.GainVsRange[ihsa:ihsb,1]
            endelse
        endif
        if pp.AntennaDiameter4NearFieldCorr gt 0.1 then begin
            nfc = near_field_correction( 299792458./35.5e9, pp.AntennaDiameter4NearFieldCorr, H)
            SnrCorfa8[*,0] *= nfc 
            SnrCorfa8[*,1] *= nfc
        endif
        refvals = pp.PTAV0*pp.tau0/pp.PRF0
        PRef7PNoi50 = calida.Calibr0 * calida.CorrLNAmatch  ; Power of internal reference noise at the time of the calibration measurment / receiver noise with matched load @ 300 K
        StcHei = (where(h*2/hdr.c gt 6e-6 ))[0]
    endif    

    if do_corrbycal then begin
        SnrCorfa = [[SnrCorfa8[*,0] * (serverinfo.npw1 * PRef7PNoi50[0] / serverinfo.cpw1)],$
                   [SnrCorfa8[*,1] * (serverinfo.npw2 * PRef7PNoi50[1] / serverinfo.cpw2)]]
    endif else begin
        ; ### adc noise correction #####
        SnrCorfa = [[SnrCorfa8[*,0] * serverinfo.npw1 / ( serverinfo.npw1 - calida.adcnoise[0] * hdr.nfft )],$
                   [SnrCorfa8[*,1] * serverinfo.npw2 / ( serverinfo.npw2 - calida.adcnoise[1] * hdr.nfft )]]
    endelse
    
    if hdr.att then begin
        StcCorfaCo=linear(15.0)
        SnrCorfa[0:StcHei-2,0] *= StcCorfaCo 
        StcCorfaCx=linear(15.0)
        SnrCorfa[0:StcHei-2,1] *= StcCorfaCx
    endif
    
    RadarConstant5 = ( C0 * hdr.prf * refvals / (serverinfo.tpow/hdr.fzk100 * hdr.pulse_width) ) 
    RadarConstant  =  RadarConstant5 * Hq7H0q
    RadarConstant5 =  RadarConstant5 * reform(SnrCorfa[zrg-1,*],2)
    return, RadarConstant
end

