pro remove_clutter,spc,noise,det_thres,pp
    ; remove clutter from a spectrum which is distrubuted symmetrically around zero velocity (index 0 in spc)
    ; spc = Spectrum of one range gate, zero velocity at index 0
    ; noise = Noise level
    ; det_thres = detection threshold
    ; pp = Structure containing the following Parameters:
    ; Parameters tested with Data from September + October 2005:
    ; pp.max_sholder_dbc=-1.10  ; theoretically the shoulders should be -6 dbc, -1.1 dB works good in practice
    ;                  Non clutter signal may cover the the clutter. The typically one of the shoulders is higer.
    ;                  In this case no clutter removal is done.
    ; pp.fftwin=[0.45,.055,0.014, 0.003,,0.002,,0.001, 0] ; decaying factor for the neighbors typically used for strong clutter
    ; pp.fftwin=[0.31, 0.0042, 0.001, 0, 0, 0, 0, 0] ; decaying factor (fourier transfomed fft window) for normal clutter
common remove_clutter,max_sholder_oc,n,nop
    if n_elements(max_sholder_oc) ne 1 then begin
        max_sholder_oc=linear(pp.max_sholder_dbc)
        n = n_elements(spc)
        nop = max(where( pp.fftwin gt 0.0 ))+1L
    endif
    pc = spc[0]   ; power of the spectral component at zero Doppler shift
    signif_neg = 2*noise - det_thres
    pc9n = pc - noise 
    ;ospc=spc
    if pc ge det_thres then begin   ; just for saving cpu time
        bigger_sholder = spc[1] > spc[n-1]  
        if bigger_sholder-noise lt pc9n*max_sholder_oc then begin
            spc[0] = noise 
            for i=0,nop-1 do begin
                pcs = pc9n * pp.fftwin[i]
                spc[i+1] = (spc[i+1] - pcs) > signif_neg
                spc[n-i-1] = (spc[n-i-1] - pcs) > signif_neg
            endfor
        endif
    endif
    ;plot,db(ospc),yra=[-7,30],psym=10
    ;oplot,db(spc),col=1,psym=10
    ;stop
end
