pro spc2ncspc,arguments
;   
; Reads the spectra from pds or zspc files and saves them in NetCdf format
;  
;
; Calling Sequence: 
;
; unix> idl spc2ncspc infile=data/20040602_1000.zspc outfile=20040602.9.ncspc
;
; IDL > spc2nc, 'infile=data/20040602_1000.pds outfile=20040602.ncspc'
; 
; infile      File(s) to read the pds data from. Multible files with contain successive 
;             (in time) data may be given as array of strings.
;
; outfile     File for writing the output data.
;
; Arguments are passes as keyword parameters in the form <keyword>=<value1>[:<value2>...]
; - If mmclx is called by an interactive IDL session all keyword parameters are 
;   passed in the first string argument as a space separated list. Each keword-value pair must
;   not contain blanks. 
; - if mmclx is called from the csh-script idl each argument ($1, $2, ...) is idl 
;   processed as one keword-value pair. Again, each keword-value pair may not contain blanks.
;
; See the structure pph below for a description of the other keyword parameters.
; 
; Matthias Bauer
;
common startup,startup_done
if n_elements(startup_done) ne 1 then begin
@startup
endif

argv = env2argv(arguments)

kp = { $
    infile:'', $
    config:'',$
    gziped:-1, $
    iam:'nobody', $   ; radar name, may be specified to evaluate data from another MIRA36
    outfile:'' $
}
argv2stru,argv,kp
kp.iam = whoami(kp.iam)
common magic4chs,magic4chs,FzkPdsFmt,syspar

; Default values for the Ploting parameters. These Parameters can be changed by the command "set"
pp={ $
of0:-1,$     ; ofs0, parameter for detection threshold
LDRof07of0:0.8181818,$ ; As the spectral range ixa,ixb in cross channel is synced to co channel ofs0 may be reduced by LDRof07of0 in the cross channel.
hilde_method:3,$ ; enumerate: 1 hildebrand-sekhon, 2 hildebrand-fischer-henemut, 3 HSdiv
dwellsperfile:1800L,$; 
ofdiv:9.,$ ; Used by HSdiv, similar to ofs0 for determination of detection threshold. divisions from minimum to this thresold are regarded as noise
ndiv:16 $ ; Used by HSdiv, number of parts to which the spectrum is divided
}

config2stru,pp,'spc2ncspc',[kp.iam+'.config',kp.config]

argv2stru,argv,pp

zrg = -1L
nfft = 0  ; nfft of the spectra in memory. If hdr.nfft differs then a new buffer for noisspectra is created.
idwell=0L

if kp.infile eq 'data' then begin
    zfiles=1
    infiles=[kp.infile]
endif else begin
    infiles = findfile(kp.infile)
    infiles_oldsyntax = strsplit(kp.infile,':',/extract)
    if n_elements(infiles_oldsyntax) gt n_elements(infiles) then infiles=infiles_oldsyntax
    if infiles[0] eq '' then begin
        print,'infile not found: ' + kp.infile
        goto,ende
    endif else zfiles = n_elements(infiles)
endelse
for ifiles=0,zfiles-1 do begin
    ; open the pds file and read the fileheader
    if infiles[ifiles] eq 'data' then begin
        spawn,"tail_s2m 2 /data/\*.pds",unit=lun
        online = 1
    endif else begin
        if kp.gziped ge 0 then gziped=kp.gziped else begin
            if strpos(infiles[ifiles],'.zspc') eq strlen(infiles[ifiles])-5 or $ 
               strpos(infiles[ifiles],'.zspca') eq strlen(infiles[ifiles])-6 or $
               strpos(infiles[ifiles],'.gz')  eq strlen(infiles[ifiles])-3       then gziped=1 else gziped=0
        endelse
        openr,lun,infiles[ifiles],/get_lun,compress=gziped
        online = 0
    endelse

    ppar = read_hdr(lun,hdr)


    while not eof(lun) do begin
        ; read one dwell of spectra
        resume:
        itime = idwell mod pp.dwellsperfile
        if itime eq 0L then begin
            iidwell=idwell/pp.dwellsperfile
            if iidwell ge 1L then ncdf_close,ncdfid
            ; write the file headers only for the first input file
            zchan = hdr.pol+1L ; number of channels
            extras=['SPC','HSDco']
            if zchan eq 2 then extras = [extras, 'HSDcx']
            ncdfid = header2ncdf(kp.outfile+'.'+string(iidwell,format='(i2.2)'),ppar,hdr,ptypes=ptypes,extras=extras)
            itime=0
        endif

        if FzkPdsFmt then begin
            while (rmh=read_mainframe_header(lun, hdr)) eq 0 do begin
                if rmh eq -1L then goto,ende
                ppar = read_ppar_frame( lun, hdr)
            endwhile
        endif
        if zrg ne (hdr.raw_zrg < hdr.mom_zrg) or hdr.nfft ne nfft then begin
            ; create some arrays
            zrg = long(hdr.raw_zrg < hdr.mom_zrg)
            if hdr.raw_zrg gt hdr.mom_zrg then print,'raw_zrg is greater than mom_zrg. Normall it indicates that something is wron with the ppar header. Assuming the smaller one is correct'
            heis = findgen(zrg) * hdr.delta_h + hdr.raw_h_min
            
            nfft=hdr.nfft & nffth = nfft/2L 
            
            ; the spectra are sorted so that 0 is in the middle (nffth) and the falling dropplets are at bins greater than nffth
            if syspar.dwd_me eq -1 then begin
                if syspar.pds_spectra_centered then begin
                    lind = (nfft-lindgen(nfft)) mod nfft
                endif else begin
                    lind = (nfft-lindgen(nfft)+nffth) mod nfft 
                    ; in case of DWD radar the spectrum has to be swapped. For consistency the velocities in pds-format have to 
                    ; be swapped again. So when using only pds output swapping twice compensates, but its necessary nevertheless for 
                    ; correct velocities in the dmp-data. Note calc_moms() turns the sign again to make the velocity of falling drops negative. oO
                endelse
            endif else begin
                lind = (lindgen(nfft)+nffth) mod nfft
            endelse
            hilde_nls = fltarr(zrg,zchan)
            spcs=fltarr(nfft,zrg,zchan)
        endif

        if ptr_valid(p_srvinfo) then ptr_free,p_srvinfo
        if ptr_valid(p_data) then heap_free,p_data    ; replaces ptr_free,(*p_data).sp1 & ptr_free,(*p_data).sp2 & ptr_free,p_data
        
        ; read the spectra here:
        if zchan eq 2 then extract=['SP1','SP2'] else extract=['SP1'] 
        read1pds_dwell, lun, hdr, ppar, p_srvinfo, p_data, extract=extract

        if n_elements( (*(*p_data).sp1)[*,0] ) ne nfft then begin
            if eof(lun) then goto,ende
            print,'corrupted data'
            goto,resume
        endif
        srvinfo2ncdf,ncdfid,itime,*p_srvinfo,hdr.fzk100

        if n_elements(sp_tagnr) ne 1 then begin
            sp_tagnr = [(where(tag_names(*p_data) eq 'SP1'))[0],(where(tag_names(*p_data) eq 'SP2'))[0]]
            cal_tagnr = [(where(tag_names(*p_srvinfo) eq 'CPW1'))[0],(where(tag_names(*p_srvinfo) eq 'CPW2'))[0]]
            noise_tagnr = [(where(tag_names(*p_srvinfo) eq 'NPW1'))[0],(where(tag_names(*p_srvinfo) eq 'NPW2'))[0]]
        endif

        for ichan=0L,zchan-1L do begin
            for irg=0L,zrg-1 do begin
                spc = (*(*p_data).(sp_tagnr[ichan]))[*,irg] 
                if ppar.osc eq 4 then begin
                    hilde_nls[irg,ichan] = (*(*p_data).hsd)[irg,ichan]
                endif else begin
                    hilde_nls[irg,ichan] = hilde(spc,method=pp.hilde_method, ofdiv=pp.ofdiv, ndiv=pp.ndiv, n_spcave=ppar.avc)
                endelse
                spcs[0,irg,ichan] = spc[lind]
            endfor
        endfor

        NCDF_VARPUT, ncdfid, 'SPCco', spcs[*,*,0L], count=[nfft,zrg,1], offset=[0,0,itime]
        if zchan eq 2 then NCDF_VARPUT, ncdfid, 'SPCcx', spcs[*,*,1L], count=[nfft,zrg,1], offset=[0,0,itime]
        NCDF_VARPUT, ncdfid, 'HSDco', hilde_nls[*,0], count=[zrg,1], offset=[0,itime]
        if zchan eq 2 then NCDF_VARPUT, ncdfid, 'HSDcx', hilde_nls[*,1], count=[zrg,1], offset=[0,itime]

        idwell++
            
    endwhile ; while not eof of one fiöe
    close,lun
endfor ; ifiles

ende:
NCDF_CLOSE, ncdfid
if ptr_valid(p_srvinfo) then heap_free,p_srvinfo
if ptr_valid(p_data) then heap_free,p_data
exit
end

