pro spcs2dmp,arguments
;   
; Reads the spectra from pds files and estimates 3 spectral moments of the 16 most 
; dominant peaks.  
;
; Calling Sequence: 
;
; unix> idl spcs2dmp infile=data/20040602_1000.pds outfile=20040602.9.pdf 
;
; IDL > spcs2dmp, 'infile=data/20040602_1000.pds outfile=xxx_off.pdf'
; 
; infile      File to read the pds data from. Multible files with contain successive 
;             (in time) data may be given as array of strings.
;
; outfile     File for writing the output data.
;
; Arguments are passes as keyword parameters in the form <keyword>=<value1>[:<value2>...]
; - If mmclx is called by an interactive IDL session all keyword parameters are 
;   passed in the first string argument as a space separated list. Each keword-value pair must
;   not contain blanks. 
; - if mmclx is called from the csh-script idl each argument ($1, $2, ...) is idl 
;   processed as one keword-value pair. Again, each keword-value pair may not contain blanks.
;
; See the structure pph below for a description of the other keyword parameters.
; 
; Matthias Bauer
;
common startup,startup_done
common magic4chs,magic4chs,FzkPdsFmt
if n_elements(startup_done) ne 1 then begin
@startup
endif

argv = env2argv(arguments)

kp = { $
    infile:'', $
    iam:'nobody', $   ; radar name, may be specified to evaluate data from another MIRA36
    config:'', $
    outpddsp:'', $  ; If set, extract the moments calculated by th DSP to this file
    outpdglobal:'', $  ; If set, calculate also the global moments and save them to this file
    outfile:'' $
}
argv2stru,argv,kp
kp.iam = whoami(kp.iam)

; Default values for the Ploting parameters. These Parameters can be changed by the command "set"
pp={ $
nave:1,$
of0:-1,$     
LDRof07of0:0.8181818,$
hilde_method:3,$ 
ofdiv:9.,$
ndiv:16,$
rgclutter:50,$
max_sholder_dbc:-0.20,$
; fftwin:[0.75, 0.3, 0.16, 0.065, 0.023, 0.008, 0.0025],$
fftwin:[0.31, 0.0042, 0.001, 0, 0, 0, 0, 0],$
gsm_n:-1, $
gsm_alpha:8.6, $
peak_thr:9.0 ,$
peak_sig_thr:0.05,$
gap4dom:5, $
expected_vrange:-2.,$
MeltHei:1500.,$
outdoortemp2metlhei:152., $ ; If outdoortemp is saved in the ps_err tag of the serverinfo structrure the MeltRgSearchCent is calculatete from it.
FixedValTpow:-1.0,$
modvuar:1$
}

config2stru,pp,'spcs2dmp',['local.config',kp.config]

argv2stru,argv,pp

; online descripion for the above parameters
pph={ $  
nave: 'Number of spectral averages',$
of0: "ofs0, parameter for detection threshold", $
LDRof07of0: "As the spectral range ixa,ixb in cross channel is synced to co channel ofs0 may be reduced by LDRof07of0 in the cross channel.", $
hilde_method: "enumerate: 1 hildebrand-sekhon, 2 hildebrand-fischer-henemut, 3 HSdiv",$
ofdiv: "Used by HSdiv, similar to ofs0 for determination of detection threshold. divisions from minimum to this thresold are regarded as noise",$
ndiv: "Used by HSdiv, number of parts to which the spectrum is divided",$
rgclutter:"maximum height in which clutter hould be removed",$
max_sholder_dbc:"theroretically the sholders of the clutter in the spectra should be -6 dbc. Non clutter signal may cover the sholders. Then no clutter removal is performed",$
fftwin:"decaying factor for the neighbor spectral lines in clutter removal",$
gsm_n:'number of points of convol. Kernel for smoothing before peak search', $
gsm_alpha:'1/width of gausian kernel for smoothing, see gsm_n', $
peak_thr: 'Threshold for peak separation, canion method. Canions sepatate peaks if the relative depth of the canion inbtween is greater than peak_thr.', $ 
gap4dom: "Peaks are separated in any case if more than gap4dom values are below detection thresold between them",$
expected_vrange: 'see modvuar', $
MeltHei: 'Height of melting layer, used for temperatur estimation for Mie correction.', $
FixedValTpow:'As thermistor is not working reliable it is better to introduce a fixed value of transmitter power here.',$
modvuar: "Restrict the calculated 1st moments to expected_vrange+/-0.5 velocity un-ambiguty range, by doing a modolo operation"$
}

openw,outlun,kp.outfile,/get_lun
if kp.outpddsp ne '' then openw,PdDspLun,kp.outpddsp,/get_lun
if kp.outpdglobal ne '' then openw,PdGlobalLun,kp.outpdglobal,/get_lun

; Konstants:
dwd_me = kp.Iam eq 'DWD' ? -1 : 1
maxchg = 512L ; fixed number of range gates in fzk-pds format  
nmulti = 16 ; must be ^2  (better dont change it!)

mietab,mietab 
; d=findgen(mietab.NDIA)*mietab.DDIA+mietab.DIA0
; plot,d*1000,db(mietab.mie7ray[*, 0]),yra=[-25,5],yst=1
; for i=1,mietab.NTEMP-1 do oplot,d*1000,db(mietab.mie7ray[*, i]),color=i mod 16
 
dmpstru = {SNR:-20000, SNM:-20000, LWC:-20000, RR:-20000, LDR:-20000, VEL:!values.f_nan, RMS: !values.f_nan, ixa:-20000, ixb:20000}
ndmpstru = N_Tags(dmpstru, /Length)  ; number of values passed in dmpformat 
; SNR = C * 10^(mom.SNR/1000.) = Int N(D)*D^6*Mie(D) dD, where C = H^2/H_0^2*P_0/P*C_0 = Radar Constant. Note, it's calculated without Mie correction.
; SNM = C * 10^(mom.SNM/1000.) = Int N(D)*D^6 dD. Note, it's calculated with Mie correction.
; LWC = rho * pi * C / 6 * 10^(mom.LWC/1000.) = Int N(D)*D^3 dD ; 10^(mom.LWC/1000.) = Sum spc(fi)/(D(fi)^3*Mie(D(fi))^3). Vertical Wind velocity ia assumed to be zero.
;  RR = rho * pi * C / 6 * 10^(mom.RR/1000.)  = Int N(D)*D^3*V_T(D) dD ; 10^(mom.LWC/1000.) = Sum VEL(fi)*spc(fi)/(D(fi)^3*Mie(D(fi))^3)
; LDR = 10^(mom.LDR/1000.) = SNR_cross/SNR_co
; mom.VEL = 1st moment in m/s
; mom.RMS = peak width in m/s
; mom.ixa, mom.ixb range of spectral lines containing the peak.

zrg = -1L
nfft = 0  ; nfft of the spectra in momory. If hdr.nfft differs then a new buffer for noisspectra is created.

if kp.infile eq 'data' then begin
    zfiles=1
    infiles=[kp.infile]
endif else begin
    infiles = findfile(kp.infile)
    infiles_oldsyntax = strsplit(kp.infile,':',/extract)
    if n_elements(infiles_oldsyntax) gt n_elements(infiles) then infiles=infiles_oldsyntax
    if infiles[0] eq '' then begin
        print,'infile not found: ' + kp.infile
        goto,ende
    endif else zfiles = n_elements(infiles)
endelse
for ifiles=0,zfiles-1 do begin
    ; open the pds file and read the fileheader
    if infiles[ifiles] eq 'data' then begin
        spawn,"tail_s2m 2 /data/\*.pds",unit=lun
        online = 1
    endif else begin
        openr,lun,infiles[ifiles],/get_lun
        online = 0
    endelse

    ppar = read_hdr(lun,hdr)

    if ifiles eq 0 then begin
        ; write the file headers only for the first input file
        out_ppar = ppar
        out_ppar.raw = 0
        out_ppar.ihp = out_ppar.raw_gate1
        out_ppar.chg = out_ppar.raw_gate2-out_ppar.raw_gate1+1+2
        out_ppar.avc = ppar.avc * pp.nave
        if pp.of0 gt 0 then begin
            of0 = pp.of0 
            out_ppar.of0 = of0
        endif else begin
            of0 = ppar.of0
        endelse
        write_hdr, outlun, out_ppar, 0 
        if kp.outpddsp ne '' then begin
            outpd_ppar = ppar
            outpd_ppar.raw = 0
            write_hdr, PdDspLun, outpd_ppar, 0 
            extract=['SP1','SP2','p1','p2','v1','v2','r1','r2']
        endif else extract=['SP1','SP2']
        if kp.outpdglobal ne '' then begin
            write_hdr, PdGlobalLun, out_ppar, 1
        endif
        
        SNR_min10 = of0 / ( 10.0 * hdr.nfft * sqrt(out_ppar.avc) )
        zchan = hdr.pol+1L ; number of channels
    endif

    while not eof(lun) do begin
        ; read one dwell of spectra
        resume:
        if FzkPdsFmt then begin
            while (rmh=read_mainframe_header(lun, hdr)) eq 0 do begin
                if rmh eq -1L then goto,ende
                ppar = read_ppar_frame( lun, hdr)
                out_ppar=ppar
                out_ppar.raw = 0
                out_ppar.ihp = out_ppar.raw_gate1
                out_ppar.chg = out_ppar.raw_gate2-out_ppar.raw_gate1+1+2
                out_ppar.avc = ppar.avc * pp.nave
                if pp.of0 gt 0 then begin
                    of0 = pp.of0 
                    out_ppar.of0 = of0
                endif else begin
                    of0 = ppar.of0
                endelse
                write_ppar_frame, outlun, out_ppar
                if kp.outpddsp ne '' then write_ppar_frame, PdDspLun, outpd_ppar
                if kp.outpdglobal ne '' then write_ppar_frame, PdGlobalLun, out_ppar
            endwhile
        endif
        if zrg ne hdr.raw_zrg then begin
            ; create some arrays
            zrg = long(hdr.raw_zrg)
            dmpstrus = replicate(dmpstru, maxchg, nmulti)
            npeaks = bytarr(zrg) ; needed for dmpformat
            heis = findgen(zrg) * hdr.delta_h + hdr.mom_h_min
            nues = 1+3.68e-5 * heis + 1.71e-9 * heis^2
            if kp.outpddsp ne '' then begin
                mom_zrg = hdr.mom_zrg
                moments = fltarr(zchan,maxchg)
                ed_laprf = 1./(hdr.la * hdr.prf)
            endif
            if kp.outpdglobal ne '' then begin
                nmom = 3           
                lig3=lindgen(nmom)
                gmoments = fltarr(zchan, maxchg, nmom)
                ed_laprf = 1./(hdr.la * hdr.prf)
            endif
        endif
        if hdr.nfft ne nfft then begin
            nfft=hdr.nfft & nffth = nfft/2L & nfftme=nfft-1L
            dx = hdr.VUAR/nfft
            x = cmod(dx * findgen(nfft), hdr.VUAR,0) ; vel=0 is at index=0, negative velocities are at the end
            ix2ord =   (lindgen(nfft)+nffth) mod nfft
            xord = x[ix2ord]  ; xord = -VUAR/2, ..., 0, ... +VUAR/2
            
            spcord=fltarr(nfft,zchan)
            ; the spectra are sorted so that 0 is in the middle (nffth) and the falling dropplets are at bins greater than nffth
            if dwd_me eq -1 then begin
                lind = (nfft-lindgen(nfft)+nffth) mod nfft 
                ; in case of DWD radar the spectrum has to be swapped. For consistency the velocities in pds-format have to 
                ; be swapped again. So when using only pds output swapping twice compensates, but its necessary nevertheless for 
                ; correct velocities in the dmp-data. Note calc_moms() turns the sign again to make the velocity of falling drops negative. oO
            endif else begin
                lind = (lindgen(nfft)+nffth) mod nfft
            endelse
            nave=pp.nave & ednave = 1./float(nave)
            ; buffers for the Spectra
            iave=0
            avespc = fltarr(nfft,zrg+2,zchan)
            tpow = 0.0
            hilde_nls = fltarr(zrg,zchan)
        endif

        if iave eq 0 then begin
            ; clear the buffer to which the spectra will be averaged
            avespc[*,*,*]=0.0
            tpow = 0.0
        endif
        
        if ptr_valid(p_srvinfo) then ptr_free,p_srvinfo
        if ptr_valid(p_data) then heap_free,p_data    ; replaces ptr_free,(*p_data).sp1 & ptr_free,(*p_data).sp2 & ptr_free,p_data
        
        ; read the spectra here:
        read1pds_dwell, lun, hdr, ppar, p_srvinfo, p_data, extract=extract
        if n_elements( (*(*p_data).sp1)[*,0] ) ne nfft then begin
            if eof(lun) then goto,ende
            print,'corrupted data'
            goto,resume
        endif

        (*p_srvinfo).ps_err = -1UL
        outdoortemp = outdoortemperature((*p_srvinfo).TIME_T)
        if finite(outdoortemp) then begin 
            pp.MeltHei = pp.outdoortemp2metlhei * outdoortemp
            (*p_srvinfo).ps_err = ulong(outdoortemp) + 100UL
        endif

        if pp.FixedValTpow gt 0 then (*p_srvinfo).tpow = pp.FixedValTpow
        
        if n_elements(temps) ne n_elements(heis) then begin
            temps = temperature(heis,(*p_srvinfo).TIME_T,MeltHei=pp.MeltHei,outdoortemp2metlhei=pp.outdoortemp2metlhei)
        endif
        if kp.outpddsp ne '' then begin
            ; Pass Moments calculated by the DSP in PDS format
            if FzkPdsFmt then begin
                len_mainframe = 8L + hdr.len_srvi + 8L + 0L + 3L * (8L + maxchg * 4 * 2) + 8L + 0L
                writeu,PdDspLun,magic4chs, len_mainframe
                writeu,PdDspLun,'SRVI', hdr.len_srvi, *p_srvinfo
                writeu,PdDspLun,'FFTD', 0L
                moments[0, 0:mom_zrg-1] = (*p_data).P1 & if zchan eq 2 then moments[1, 0:mom_zrg-1] = (*p_data).P2
                writeu,PdDspLun,'SNRD', maxchg * 4 * 2, moments
                moments[0, 0:mom_zrg-1] = (*p_data).V1 & if zchan eq 2 then moments[1, 0:mom_zrg-1] = (*p_data).V2
                writeu,PdDspLun,'VELD', maxchg * 4 * 2, moments*ed_laprf
                moments[0, 0:mom_zrg-1] = (*p_data).R1 & if zchan eq 2 then moments[1, 0:mom_zrg-1] = (*p_data).R2
                writeu,PdDspLun,'RMSD', maxchg * 4 * 2, moments*ed_laprf
                writeu,PdDspLun,'EXPD',0L
            endif else begin
                writeu,PdDspLun,magic4chs,-1L,*p_srvinfo
                writeu,PdDspLun,'FCOP',-1L
                writeu,PdDspLun,'FFFT',0L
                writeu,PdDspLun,'FINT',4L*(mom_zrg+2L),(*p_data).P1
                writeu,PdDspLun,'FVEL',4L*(mom_zrg+2L),dwd_me*ed_laprf*(*p_data).V1
                writeu,PdDspLun,'FRMS',4L*(mom_zrg+2L),ed_laprf*(*p_data).R1

                if zchan ge 2 then begin
                    writeu,PdDspLun,'FCRP',-1L
                    writeu,PdDspLun,'FFFT',0L
                    writeu,PdDspLun,'FINT',4L*(mom_zrg+2L),(*p_data).P2 
                    writeu,PdDspLun,'FVEL',4L*(mom_zrg+2L),dwd_me*ed_laprf*(*p_data).V2
                    writeu,PdDspLun,'FRMS',4L*(mom_zrg+2L),ed_laprf*(*p_data).R2
                endif
            endelse
            ; flush,PdDspLun
        endif
        
        if n_elements(sp_tagnr) ne 1 then begin
            sp_tagnr = [(where(tag_names(*p_data) eq 'SP1'))[0],(where(tag_names(*p_data) eq 'SP2'))[0]]
            cal_tagnr = [(where(tag_names(*p_srvinfo) eq 'CPW1'))[0],(where(tag_names(*p_srvinfo) eq 'CPW2'))[0]]
            noise_tagnr = [(where(tag_names(*p_srvinfo) eq 'NPW1'))[0],(where(tag_names(*p_srvinfo) eq 'NPW2'))[0]]
        endif

        ; Add the spectra to one averaging buffer
        for ichan=0L,zchan-1L do begin
            for irg=0L,zrg+1L do begin
                avespc[0L,irg,ichan] = avespc[*,irg,ichan] + (*(*p_data).(sp_tagnr[ichan]))[*,irg] ; avespc[nfft,zrg+2,zchan]
            endfor
        endfor
        tpow += (*p_srvinfo).tpow
        iave++

        if iave eq pp.nave then begin
            ; Do the final adjustments of the spectra averaging
            avespc[0,0,0] = ednave * avespc[*,*,*]
            (*p_srvinfo).tpow = ednave * tpow
            iave=0
            
            ; Do the calculations that are needed only once for all range gates
            det_thres_fac = ( 1 + of0/(10. * sqrt(out_ppar.avc)) ) 
            SNRminLDR = of0 * pp.LDRof07of0 / (10. * sqrt(out_ppar.avc))   ; nfft im Nenner?
            for ichan=0,zchan-1 do begin
                ; calculate the noise power and the calibration power from the averaged spec. 
                NoiseLevel = median(avespc[*,zrg,ichan])
                CalNoiseLevel = median(avespc[*,zrg+1,ichan])
                (*p_srvinfo).(noise_tagnr[ichan]) = NoiseLevel * nfft
                (*p_srvinfo).(cal_tagnr[ichan]) = CalNoiseLevel * nfft
                ; the thermistor power should be averaged??
            endfor ; ichan
                
            ; do the moment estimation for each range gate and each channel
            for irg = 0, zrg-1 do begin
                ; height and temp is needed for the Mie-correction
                temp = temps[irg] ; temp = temps[irg]  !!!
                nue = nues[irg]

                ; processing which is the same for all peaks
                for ichan=0,zchan-1 do begin
                    spc = avespc[*,irg,ichan]
                    hilde_nls[irg,ichan] = hilde(spc,method=pp.hilde_method, ofdiv=pp.ofdiv, ndiv=pp.ndiv, n_spcave=out_ppar.avc)
                    if irg le pp.rgclutter then begin
                        remove_clutter,spc,hilde_nls[irg,ichan],hilde_nls[irg,ichan] * det_thres_fac,pp
                    endif
                    spcord[0,ichan] = spc[lind] 
                endfor
                
                ; Global Moment Estimation
                if kp.outpdglobal ne '' then begin 
                    for ichan=0L,zchan-1L do begin 
                        det_thres = hilde_nls[irg,ichan] * det_thres_fac
                        ix_above_thres = where(spcord[*,ichan] ge det_thres, nix_above)
                        if nix_above gt 0 then begin
                            ; noise removal and thresholding
                            spc_above_9noise = spcord[ix_above_thres,ichan] - hilde_nls[irg,ichan]
                            dummy = max(spc_above_9noise,ixxmax) 
                            xx = cmod(xord[ix_above_thres],hdr.VUAR,xord[ix_above_thres[ixxmax]])
                            ; moment estimation
                            noise4snr = (*p_srvinfo).(noise_tagnr[ichan])
                            gmoments[ichan, irg, lig3] = calc_moms(spc_above_9noise,xx*ed_laprf,noise4snr)
                            if pp.modvuar then gmoments[ichan, irg, 1L] =  cmod(gmoments[ichan, irg, 1L], hdr.VUAR*ed_laprf,0)
                        endif else begin
                            gmoments[ichan, irg, lig3] = [0.0, -9999.0, -9999.0]
                        endelse
                    endfor ; ichan
                endif
                
                ; Peak Identification. It is done only in the co-channel
                det_thres = hilde_nls[irg,0] * det_thres_fac
                if pp.gsm_n gt 0 then spc_co4id_pk = gsmooth(spcord[*,0L] > det_thres, n=pp.gsm_n,alpha=pp.gsm_alpha) $
                                else spc_co4id_pk = spcord[*,0L] > det_thres
                peak_ifrqs = identify_peaks_by_canion(spc_co4id_pk,det_thres,peak_thr_fac=pp.peak_thr/sqrt(out_ppar.avc),$
                    gap4dom=pp.gap4dom, peak_sig_thr=pp.peak_sig_thr, maxis=maxis)
                npeak = n_elements(peak_ifrqs)/2 ; this is 0 in case peak_ifrqs=-1
                npeaks[irg] = npeak < nmulti
                if npeak gt 1  then begin
                    ; sort the peaks by their maxi values saved in maxis
                    peak_ifrqs = peak_ifrqs[*,reverse(sort(maxis))]
                endif

                ; Multi Peak Moment estimation
                for ipeak=0,nmulti-1 do begin
                    if ipeak lt npeak then begin
                        ixa = peak_ifrqs[0,ipeak] & ixb = peak_ifrqs[1,ipeak]
                        nlines = (ixb-ixa+nfft) mod nfft
                        
                        ; per peak co-channel processing
                        if ixa lt ixb then begin
                            spc_of_peak = spcord[ixa:ixb,0L]-hilde_nls[irg,0L]
                            xx_of_peak = xord[ixa:ixb]
                        endif else begin
                            spc_of_peak = [spcord[ixa:nfftme,0L],spcord[0:ixb,0L]]-hilde_nls[irg,0L]
                            xx_of_peak = [xord[ixa:nfftme],xord[0L:ixb] + hdr.VUAR] ; calculated 1st moments will be in -VUAR/2:VUAR
                        endelse
                        noise4snr=(*p_srvinfo).NPW1
                        dmpstrus[irg,ipeak] = calc_moms_mie(spc_of_peak, xx_of_peak, noise4snr, mietab, nue, temp, dmpstru, SNRco)
                        if pp.modvuar then begin
                            dmpstrus[irg,ipeak].VEL = cmod( dmpstrus[irg,ipeak].VEL, hdr.VUAR, pp.expected_vrange)
                        endif
                        
                        ; per peak cross-channel processing
                        if ixa lt ixb then begin
                            SNcx = total(spcord[ixa:ixb,1L]-hilde_nls[irg,1L])
                        endif else begin
                            SNcx = total([spcord[ixa:nfftme,1L],spcord[0L:ixb,1L]]-hilde_nls[irg,1])
                        endelse
                        ; Berechnung fur LDRdetectable wird doch wieder besser in mmclx gemacht
                        ; SNxc_min = sqrt(float(nlines)) * SNRminLDR * hilde_nls[irg,1] ; minimum Signal in cposs channel which is significantly above noise
                        ; dmpstrus[irg,ipeak].LDR =  SNcx ge SNxc_min ? fix(1000.*alog10(SNcx / (SNRco*(*p_srvinfo).NPW2))) : -19999
                        dmpstrus[irg,ipeak].LDR = SNcx ge 0.0 ?  fix(1000.*alog10(SNcx / (SNRco*(*p_srvinfo).NPW2))) : -5000

                        ; add index range information:
                        dmpstrus[irg,ipeak].ixa = ixa
                        dmpstrus[irg,ipeak].ixb = ixb
                    endif else begin
                        dmpstrus[irg,ipeak] = dmpstru
                    endelse
                endfor ; ipeak
            endfor  ; irg

            ; Write the calculated moments to file

            ; 1.) global moments calculated above in PDS format
            if kp.outpdglobal ne '' then begin
                if FzkPdsFmt then begin
                    len_mainframe = 8L + hdr.len_srvi + 8L + 0L + 3L * (8L + maxchg * 4 * 2) + 8L + 0L
                    writeu,PdGlobalLun,magic4chs, len_mainframe
                    writeu,PdGlobalLun,'SRVI', hdr.len_srvi, *p_srvinfo
                    writeu,PdGlobalLun,'FFTD', 0L
                    writeu,PdGlobalLun,'SNRD', maxchg * 4 * 2, gmoments[*,*,0L]
                    writeu,PdGlobalLun,'VELD', maxchg * 4 * 2, gmoments[*,*,1L]
                    writeu,PdGlobalLun,'RMSD', maxchg * 4 * 2, gmoments[*,*,2L]
                    writeu,PdGlobalLun,'EXPD',0L
                endif else begin
                    writeu,PdGlobalLun,magic4chs,-1L,*p_srvinfo
                    writeu,PdGlobalLun,'FCOP',-1L
                    writeu,PdGlobalLun,'FFFT',0L
                    NoisePows=total((*(*p_data).sp1)[*,zrg])/float(nfft)
                    CalPows=total((*(*p_data).sp1)[*,zrg+1])/float(nfft)
                    writeu,PdGlobalLun,'FINT',4L*(zrg+2L),gmoments[0L,0:zrg-1L,0L],NoisePows,CalPows
                    writeu,PdGlobalLun,'FVEL',4L*(zrg+2L),dwd_me*gmoments[0L,0:zrg-1L,1L],0.0,0.0
                    writeu,PdGlobalLun,'FRMS',4L*(zrg+2L),gmoments[0L,0:zrg-1L,2L],0.0,0.0

                    if zchan ge 2 then begin
                        writeu,PdGlobalLun,'FCRP',-1L
                        writeu,PdGlobalLun,'FFFT',0L
                        NoisePows = total((*(*p_data).sp2)[*,zrg])/float(nfft)
                        CalPows = total((*(*p_data).sp2)[*,zrg+1])/float(nfft)
                        writeu,PdGlobalLun,'FINT',4L*(zrg+2L),gmoments[1L,0:zrg-1L,0L],NoisePows,CalPows
                        writeu,PdGlobalLun,'FVEL',4L*(zrg+2L),dwd_me*gmoments[1L,0:zrg-1L,1L],0.0,0.0
                        writeu,PdGlobalLun,'FRMS',4L*(zrg+2L),gmoments[1L,0:zrg-1L,2L],0.0,0.0
                    endif
                endelse
                ; flush,PdGlobalLun
            endif

            ; 2.) Multi Peak Moments
            len_srvi = N_Tags(*p_srvinfo,/Length)
            tnpeaks = long(total(npeaks)) 
            len_hsd = zrg * zchan * 4L
            len_mom = tnpeaks * ndmpstru 
            len_mainframe = 8L+len_srvi + 8L+len_hsd  + 8L+zrg + 8L+len_mom
            writeu,outlun,magic4chs, len_mainframe
            if FzkPdsFmt then srvi_sgnt='SRVI' else srvi_sgnt=magic4chs
            writeu,outlun,srvi_sgnt, len_srvi, *p_srvinfo
            writeu,outlun,'HSDD', len_hsd,hilde_nls[0L:zrg-1L,0:zchan-1L]*nfft
            writeu,outlun,'NPKS', zrg, npeaks
            writeu,outlun,'DMPD', len_mom
            for irg=0,zrg-1 do begin                          ; das geht vielleicht auch ohne schleife
                for ipeak=0,npeaks[irg]-1 do begin
                    writeu,outlun,dmpstrus[irg, ipeak]
                endfor
            endfor
            ; flush,outlun
        endif ; if iave eq pp.nave
    endwhile ; while not eof of one file
    close,lun
endfor ; ifiles

ende:
close,outlun
if kp.outpddsp ne '' then close,PdDspLun
if kp.outpdglobal ne '' then close,PdGlobalLun
if ptr_valid(p_srvinfo) then heap_free,p_srvinfo
if ptr_valid(p_data) then heap_free,p_data
exit
end
