pro ttview,arguments
; 
; Tool for viewing which peaks in the spectrum have been assigned to which target-type
;
; 
; 
; unix> idl ttview pdsfile=data/20040602_1000.pds ttfile=20040602_1000.tt
;
; The .tt file has to be generated by
; 
; unix > idl mmclx_mie infile=x.dmp outfile=ddd.mmclx ttoutfile=x.tt
;
; Where the .dmp file is the multi peak moment output from spcs2dmp
;
;
; The following commands can be used:
;-----------------------------------------------------------------------------------------------------------------
;     ttview > set rg 137     # sets the range gate at which moment estimation should be performed
;     ttview > ttview          # do moment estimation
;
;     ttview > + [increment]  # read the next dwell from the pds file
;                               # noise estimation works only correctly after reaching dwell # 16.
;
;-----------------------------------------------------------------------------------------------------------------
;    ttview > set <parameter> <valu1> [<value2> ...]
;
; Change the default value of a plotting parameter. If <value1> is omited the
; current value is plotted. All the plotting parameters which 
; are available can be shown by
;
;    ttview > set 
;
; Their curren Value can be queried by
;
;    ttview > set rg          # prints the range gate on which moment estimation is performed.
;
;    ttview > postscript <filename>
;
; Directs the output of the following plot commands to the given postscript filename.
; for flushing and closing the postscript file the command
;
;    ttview > close
;
; must be given. After that the plot out put is directed to the screen again.
;----------------------------------------------------------------------------------------------------------------
;    ttview > png <filename>
;
; exports the screen to png-format
;-----------------------------------------------------------------------------------------------------------------
;    ttview > info
;
; gives some information from the file header
;-----------------------------------------------------------------------------------------------------------------
;    ttview > x
;
; exits ttview
;----------------------------------------------------------------------------------------------------------------
; If no command is recognized then the command is executed by the IDL interpreter.
;
; Matthias Bauer-Pfundstein bauer@metek.de
;
common startup,startup_done
if n_elements(startup_done) ne 1 then begin
@startup
endif

; Arguments can be provided by two methods:
; - If ttview is called by an interactive IDL session the first (and only) 
; string parameter arguments which contains a space separated list is split to
; the strarr argv.
; - Aditionally strings contained in the environment variables ARG1, ARG2, ..., 
; ARG9, ARG10, ... are prepended to argv. The script idl packs its calling arguments
; these environment variables. This is done as IDL can not extract the shell 
; variables like $1, $2,...
argv = env2argv(arguments)

kp = { $
    infile:'', $
    ttfile:'',$
  iam:'nobody', $
    maxtt:18000L,$
    max_dwells:6000 $
}
argv2stru,argv,kp
kp.iam = whoami(kp.iam)


; Default values for the Ploting parameters. These Parameters can be changed by the command "set"
pp={ $
rg:50,$          ; see comments below!
badcol :15, $
srange:[-3,67.], $
vrange:[!values.f_nan,!values.f_nan], $
charsize:1.5, $
thick:2.4, $
cxlinestyle:0,$
cxcolor:15,$
ttbl:2.0,$
ttbvo:0.016,$
ti:'!3',$
colt:'smooth' $
}
argv2stru,argv,pp

; online descripion for the above parameters
pph={ $
rg:      'gate for calculating moments', $
badcol : 'Grey, color for plotting values which are not plausible.', $
srange:  'Zrange for spectra', $
vrange:  'velocity range for plotting spectra', $
charsize: 'See graphic keywords in IDL documentation', $
thick:   'See graphic keywords in IDL documentation', $
cxlinestyle: 'Line style for oplotting the cross spectrum',$
cxcolor: 'Color for oplotting the cross spectrum',$
ttbl:    'Length of the ticks indicating the peak bounderies',$
ttbvo:   'Vel. offset of the ticks indicating the peak bounderies',$
ti:     'Can be uses to pass a Hershly-style font directive used for all annotations', $
colt:   'Color table, the predefined colortables reside in the directory idl/colors' $
}

startcolor=30
ocolt='none'
td=1000.

!p.multi=[0,1,1]
if !d.name eq 'X' then begin
    while !d.window ge 0 do wdelete
    screen_size=get_screen_size()
    scale=screen_size/[1280.,1024]
    maxcolor=512
    window,/free,retain=2,colors=maxcolor,title='ttview',xpos=600*scale[0],ypos=600*scale[1], $
        xsize=680*scale[0],ysize=400*scale[1]
        spc_win=!d.window
    wset,spc_win
endif

; open the pds file and read the fileheader
get_lun,lun & openr,lun,kp.infile
file_pos = lonarr(kp.max_dwells)
ipos=0L
ppar = read_hdr(lun,hdr)
fzk=1
;if fzk then begin
;    ppar = read_ppar_frame( lun, hdr)
;endif

ntt=0
ixtt=-1
if kp.ttfile ne '' then begin
    ; cash all the time stamps and their file positions of the tt-file
    openr,luntt,kp.ttfile,/get_lun
    while not eof(luntt) do begin
        fstatx = fstat(luntt)
        timettx=0L
        read_tt,luntt,datattx,timettx
        if timettx eq 0L then begin
            print,'problems reading tt file'
            exit
        endif
        if ntt eq 0 then begin
            zrgtt = n_elements(datattx[*,0])
            zpktt = n_elements(datattx[*,0])
            postt=lonarr(kp.maxtt)
            timett=lonarr(kp.maxtt)
        endif
        postt[ntt] = fstatx.cur_ptr
        timett[ntt]=timettx
        if ntt lt (kp.maxtt-1) then ntt++ else begin
            print,'maxtt too small'
            goto,closett
        endelse
    endwhile
closett:
    close,luntt
    free_lun,luntt
    postt = postt[0:ntt-1]
    timett = timett[0:ntt-1]
endif

zchan=2 ; number of channels
nfft=0  ; nfft of the noise spectra in momory. If hdr.nfft differs then a new buffer for noisspectra is created.
read1dwell=1
while 1 do begin
    if read1dwell then begin
        ; remeber the positions in file where the dwells begin. Needed for the '-' command!
        point_lun, -lun, posx & file_pos[ipos] = posx 
        if ++ipos ge kp.max_dwells then begin
            shiftpos=kp.max_dwells/2
            file_pos[0]=file_pos[shiftpos:kp.max_dwells-1L]
            ipos -= shiftpos
        endif
        ; read parameter headers if necessary
        if fzk then begin
            while (rmh=read_mainframe_header(lun, hdr)) eq 0 do begin
                if rmh eq -1L then goto,ende
                ppar = read_ppar_frame( lun, hdr)
            endwhile
        endif
        ; gnerate some arrays that depend on the parameters in pparm
        if hdr.nfft ne nfft then begin
            nfft=hdr.nfft
            ix2ord = (lindgen(nfft) + nfft/2 ) mod nfft
            dx = hdr.VUAR/nfft
            x = cmod(dx * findgen(nfft), hdr.VUAR,0)
            xord=x[ix2ord]
        endif
        if n_elements(p_srvinfo) then heap_free,p_srvinfo
        if n_elements(p_data) then heap_free,p_data
        ; read the p_srvinfo structure and the spectra
        read1pds_dwell, lun, hdr, ppar, p_srvinfo, p_data
        read1dwell=0
    endif
    
    ; color table
    if ocolt ne pp.colt then begin
        loadcolor,startcolor,colorfile=pp.colt,zcolor=zcolor 
        ocolt=pp.colt
    endif
 
    ; restrict gr to existing range gates
    rgx = (pp.rg > ppar.ihp) < (ppar.ihp+ppar.chg-3) 
    rgx = ( rgx < (ppar.RAW_GATE2+2)) > (ppar.RAW_GATE1)
    if pp.rg ne rgx then begin
        print,'setting rg to the range where spectra and mom data are available',ppar.RAW_GATE1,ppar.RAW_GATE2
        pp.rg = rgx
    endif
    rg4sp = pp.rg - ppar.RAW_GATE1  ; richtig
    ; extract the data fromt the range gate that should be investigated
    rg4mom = pp.rg - ppar.IHP  ; nur für DSP-Momente
    dnsp_co = (*p_srvinfo).npw1/float(nfft)
    dnsp_cx = (*p_srvinfo).npw2/float(nfft)
    nspcord_co = ((*(*p_data).sp1)[*,hdr.raw_zrg])[ix2ord]
    nspcord_cx = ((*(*p_data).sp2)[*,hdr.raw_zrg])[ix2ord]
    tnsp_co=total(nspcord_co)/float(nfft)
    tnsp_cx=total(nspcord_cx)/float(nfft)
    spcco =  (1./tnsp_co) * (*(*p_data).SP1)[*,rg4sp]
    spccoord = spcco[ix2ord]
    spccx = (1./tnsp_cx) * (*(*p_data).SP2)[*,rg4sp]
    spccxord = spccx[ix2ord]

    
    ; plot
    if finite(pp.vrange[0]) then xra=pp.vrange else xra=[-0.5,.5]*hdr.vuar
    tit = ''
    time = (*p_srvinfo).TIME_T
    hei = pp.rg*hdr.delta_h + hdr.raw_h_min
    dati = unixtime2bdate(time, nice=2,utc=1) +' '+string(time)
    plot,xord,db(spccoord), $
        xtit=pp.ti+'Velocity [m/s]',ytit=pp.ti+'SNR [dB]',$
        tit=pp.ti+'Spectrum  RG=' + string(pp.rg,format='(I3)') + ' ('+string(hei) + ') ' $
        +tit+'  '+dati, $
        yrange=pp.srange,xst=1,yst=1,psym=10, xrange=xra, thick=pp.thick, charsize=pp.charsize
    oplot,xord,db(spccxord),linest=pp.cxlinestyle,thick=pp.thick,color=pp.cxcolor
    
    ; read the coresponding dwell from the ttfile
    ; and taint the co-spectrum
    if ntt gt 0 then begin
        ixttx = where(time eq timett,nixtt)
        if nixtt eq 1 then begin
            if ixttx ne ixtt then begin
                ixtt = ixttx
                openr,luntt,kp.ttfile,/get_lun
                point_lun, luntt, postt[ixttx]
                    read_tt,luntt,datatt,timettx,npk
                close,luntt
                free_lun,luntt
            endif
            for itt=0,npk[rg4sp]-1 do begin
                case datatt[rg4sp,itt].ttype of
                   -1:ttco=3
                    0:ttco=2
                    1:ttco=5
                    3:ttco=6
                    else:ttco=5
                endcase
                ; ixax = ((nfft-datatt[rg4sp,itt].ixb)+nfft) mod nfft ; fuer DWD falsch herum ? 
                ; ixbx = ((nfft-datatt[rg4sp,itt].ixa)+nfft) mod nfft
                ixax=datatt[rg4sp,itt].ixa   ; andere WoRas?
                ixbx=datatt[rg4sp,itt].ixb
                if ixbx gt ixax then begin
                    oplot,xord[ixax:ixbx],db(spccoord[ixax:ixbx]),color=ttco,thick=pp.thick,psym=10
                endif else begin
                    oplot,xord[ixax:nfft-1],db(spccoord[ixax:nfft-1]),color=ttco,thick=pp.thick
                    oplot,xord[0:ixbx],db(spccoord[0:ixbx]),color=ttco,thick=pp.thick
                endelse
                oplot,xord[[ixax,ixax]]+pp.ttbvo,db(spccoord[[ixax,ixax]])+[-1,1]*pp.ttbl,color=ttco,thick=pp.thick,linesty=1
                oplot,xord[[ixbx,ixbx]]-pp.ttbvo,db(spccoord[[ixbx,ixbx]])+[-1,1]*pp.ttbl,color=ttco,thick=pp.thick,linesty=1
            endfor
        endif else begin
            ixtt=-1
            print,'tt file does not contain data for the current spectrum'
        endelse
    endif

    arg = myprompt('ttview >')
artarg:
    cmd = arg[0] &  narg=n_elements(arg)
    case 1 of
        cmd eq '+': begin 
            if narg eq 2 then stride = long(arg[1]) else stride = 1L 
            if stride gt 1 then begin
                for i=0,stride-2 do begin
                    if fzk then begin
                        while read_mainframe_header(lun, hdr) eq 0 do begin
                            ppar = read_ppar_frame( lun, hdr)
                        endwhile
                    endif
                    read1pds_dwell, lun, hdr, ppar, p_srvinfo, p_data
                endfor
            endif
            read1dwell=1
        end
        cmd eq '-': begin 
            if narg eq 2 then stride = long(arg[1]) else stride = 1L 
            point_lun,lun,file_pos[(ipos-stride-1) > 0L]
            ipos = (ipos - stride - 1L) > 0L
            read1dwell=1
        end
        cmd eq 'u': begin 
            if narg eq 2 then stride = long(arg[1]) else stride = 1L 
            pp.rg = pp.rg + stride 
        end
        cmd eq 'd': begin 
            if narg eq 2 then stride = long(arg[1]) else stride = 1L 
            pp.rg = pp.rg - stride 
        end
        cmd eq 'postscript' or cmd eq 'ps': begin
            if narg eq 2 then psfile = arg[1] else psfile='ttview.ps'
            set_plot,'PS'
            device,/color,bits_per_pixel=8,filename=psfile ; ,yoffset= 27,xoffset=0.7
        end
        cmd eq 'close':begin
            device,/close
            set_plot,'X'
        end
        cmd eq 'png': begin
            if narg ge 2 then png_fn = arg[1] else begin
                png_fn = infile + '_' + tag_mom + '.png'
            endelse
            screen_dump = TVRD(true=1)
            WRITE_PNG, png_fn, screen_dump
        end
        cmd eq 'set': begin
            if narg ge 2 then set_tag_name = arg[1] else set_tag_name='HELP'
            if strupcase(set_tag_name) eq 'HELP' then begin
                pp_tags = tag_names(pp)
                pph_tags = tag_names(pph)
                for i=0,n_elements(pp_tags)-1 do begin
                    helptext_tag_number = (where(pph_tags eq pp_tags[i]))[0]
                    helptext = helptext_tag_number ge 0 ? pph.(helptext_tag_number) : ''
                    print_table,pp_tags[i]+':',helptext,tabw=20,linew=79
                endfor
            endif else begin
                set_tag_number = (where(strupcase(set_tag_name) eq tag_names(pp)))[0]
                if set_tag_number ge 0 then begin
                    if narg eq 2 then print,set_tag_name+' :',pp.(set_tag_number) else begin
                        for i=0,(narg-3) < (n_elements(pp.(set_tag_number))-1) do begin
                            if strupcase(arg[i+2]) ne 'SKIP' then pp.(set_tag_number)[i] = arg[i+2]
                        endfor
                    endelse
                endif else print,'unknown variable name'
            endelse
        end
        cmd eq 'info': begin
            help,hdr,/struct
            help,ppar,/struct
        end
        cmd eq 'stop': if lmgr(/vm) then goto,ende else stop
        cmd eq 'x': goto,ende
        else: begin
            setcmd = ''
            for i=0 , narg - 1 do setcmd = setcmd + arg[i] + ' '
            if lmgr(/vm) then print,'VM license can not execute '+setcmd $
            else dummi=execute( setcmd )
        end
    endcase
endwhile

ende:
close,lun
if !d.name eq 'X' then begin
    while !d.window ge 0 do wdelete
endif
if n_elements(p_srvinfo) then heap_free,p_srvinfo
if n_elements(p_data) then heap_free,p_data

end
