pro polarimage,image,radsl,radsr,fisl,fisr,xrange=xrange,yrange=yrange, zrange=zrange, $
         zcol, startcol, badcol,charsize=charsize,thick=thick,ztit=ztit,xytit=xytit,xstyle=xstyle,ystyle=ystyle,$
         saturate=saturate,nullcol=nullcol,nulltol=nulltol
;
; Plot RHI or PPI scans.
;    image[rg,ti] rg=0..zrg-1 number of range gates, ti=0..zti-1 number of data chunks in the selected time range
;    radsl[rg],radsr[rg] lower and upper bound of the range covered by each range gate
;    fisl[ti],fisr[ti]   left and right bound of the angle (elevation or arumuth) which where covered dureing the measuring time of each chunk of data
;    xrange,yrange range in which the polar diagrams should be drawn
;    startcol the number at which the color table starts (typpicaly 30)
;    zcol number of elements in the color table
;    badcol This color should be used for NaN-values
;    nullcol,nulltol  if abs values in image are smaller than nultol then the color is set to nulcol
;    saturate: if bit 0 is set then values smaller than zrange[0] are set to zrange[0]
;          and if bit 1 is set then values biger than zrange[1] are set to zrange[1]
;    ztit=ztit,xytit=xytit,rtit=rtit
;    charsize=charsize,thick=thick,xstyle=xstyle,ystyle=ystyle    some plotting keywords.
;
; A method for gridding data related to sperical coordinates which is not used below:
;ix=where((elv-ym) le 90 and  abs(azic) le 90 )
;xx=sin(!pi/180.*azic[ix])*cos(!pi/180.*(elv[ix]-ym))
;yy=sin(!pi/180.*(elv[ix]-ym)) & pco=pcoa[ix]
;TRIANGULATE, xx, yy, trp
;pcog=griddata(xx,yy,pco,dimen=[400,200],delta=[1,1.04]/200.,start=[-1,-0.04]-[1,1.04]/400.,method="NearestNeighbor",TRIANGLES=trp)
;mimage,pcog,[-1,1],[-0.04,1],[-20,60],zcolor, startcolor,badcol,ztit='dB',heitit='Elevation [deg]',charsi=1.5
;WRITE_PNG, fn+'.Co_vol.png', TVRD(true=1)
;
; Kerstin Ebel, modified Matthias Bauer-Pfundstein
;
    default_value,xstyle,1
    default_value,ystyle,1
    default_value,saturate,2b
    default_value,nulltol,0.1
    ticklen=-0.01

    ; Dimensions of the image to be rendered
    image_size = size(image)
    if image_size[0] lt 2 then message, 'image not 2D'
    ztime = long(image_size[1])
    zrg = long(image_size[2])
    if n_elements(radsl) ne n_elements(radsr) or n_elements(radsr) ne zrg then begin
        print,"radal, radsr, and image[*,0] must have the same dimensions"
        return
    endif
    if n_elements(fisl) ne n_elements(fisr) or n_elements(fisr) ne ztime then begin
        print,"fisl, fisr, and image[0,*] must have the same dimensions"
        return
    endif
    maxrad=max(radsl)
    default_value,xrange,[-maxrad,maxrad]
    default_value,yrange,[-maxrad,maxrad]
    default_value,zrange,[min(image,max=maxz),maxz]
    dzra = zrange[1] - zrange[0]
    if  dzra lt 0 then message, 'zrange must be positive'

    ;Positions of the image window in normalized coordinates
    ;x0im_nrm  = 0.33
    x0im_nrm  = 0.07
    x1im_nrm  = 0.87
    y0im_nrm  = 0.1
    y1im_nrm  = 0.9

    ; define the coordinate system for the image plot
    plot,[0,0],/nodata,xrange=xrange,yrange=yrange, $
            ticklen=ticklen,xstyle=xstyle,ystyle=ystyle, $
            position=[[x0im_nrm,y0im_nrm],[x1im_nrm,y1im_nrm]],$
            ytitle=xytit, xtitle=xytit,charthick=thick, charsize=charsize
    
    ;image skalieren, dabei die "schlechten" Werte rausschmeissen -> in scl_image speichern.
    ixnan = where( ~ finite(image),nixnan )
    if nixnan gt 0 then image[ixnan] = zrange[0] - 10000.0
    definiert = image ge zrange[0] and image le zrange[1]
    scl_image = (( image - zrange[0] ) * (zcol-1) / ( zrange[1]-zrange[0] ) + startcol) * definiert + badcol * (definiert ne 1) 
    ;============================PATRICS MODIFIKATION=======================
    if (saturate and 1b) ne 0 then begin
        idx=where(image lt zrange[0] and image gt zrange[0]-9999., count)
        if count gt 0 then scl_image[idx]=startcol+0
    endif
    if (saturate and 2b) ne 0 then begin
        idx=where(image gt zrange[1], count)
        if count gt 0 then scl_image[idx]=startcol+zcol-1
    endif
    ;===========================ENDE PATRICS MODIFIKATION===================
    ;Falls nullcol gesetzt ist, werden Werte, die genau 0 sind mit nullcol markiert
    if n_elements(nullcol) ne 0 then begin
        ixnull=where(abs (image) lt nulltol,nixtol)
        if nixnull then scl_image[ixnull] = nullcol
    endif

    for iti=0L,ztime-1 do begin
        for irg=0L,zrg-1 do begin
            zwiebel_rad = [ radsr[irg], radsr[irg], radsl[irg], radsl[irg], radsr[irg]]
            zwiebel_fi  = [  fisr[iti],  fisl[iti],  fisl[iti],  fisr[iti],  fisr[iti]]
            zwiebelx = zwiebel_rad * sin(zwiebel_fi*!PI/180.)
            zwiebely = zwiebel_rad * cos(zwiebel_fi*!PI/180.)
            if total(zwiebelx lt xrange[0]) + total(zwiebelx gt xrange[1]) + total(zwiebely lt yrange[0]) + total(zwiebely gt yrange[1]) eq 0 then $
                polyfill, zwiebelx , zwiebely , color=scl_image[iti,irg]
        endfor
    endfor

    ; replot the frame arround the image as parts of it may be covered
    plot,[0,0],/nodata,xrange=xrange,yrange=yrange, $
            ticklen=ticklen,xstyle=xstyle,ystyle=ystyle, $
            position=[[x0im_nrm,y0im_nrm],[x1im_nrm,y1im_nrm]],$
            ytitle=xytit, xtitle=xytit,charthick=thick, charsize=charsize, /noerase

    
    ; save coordinate system. It will we restored after drawing the scale to provide the
    ; possibility to oplot something after exiting this procedure.
    ssx=!x & ssy=!y & ssp=!p

    ; ===================
    ; draw the colorbar |   Das sollte ich auch mal durch die COLORBAR-Routine von D.Fanning ersetzen
    ; ===================
    scale = findgen(1,zcol) + startcol ; two dimensional array containing the image for the scale. 
                                    ; It will be inflated by poly_2d in case of pixle devices.
    sz_scale = size(scale)
    zscale_x = float(sz_scale[1])
    zscale_y = float(sz_scale[2])


    ;Position (0=left, 1=right) of the SCale window in NoRMalized (to 1) coordinates
    x0sc_nrm = 0.89
    x1sc_nrm = 0.91
    y0sc_nrm = y0im_nrm
    y1sc_nrm = y1im_nrm

    ; define plot area
    plot,[0,0], xrange=[0,1],yrange=[0,1], /nodata, /noerase, xstyle=5, ystyle=5, $
        position=[[x0sc_nrm,y0sc_nrm],[x1sc_nrm,y1sc_nrm]]

    ; !x.window are the normalized coordinates of the plot region, !d.x_vsize is the number of pixles of the plot device
    ; !y.window !d.y_vsize dito for y direction
    xsc_px = !x.window * !d.x_vsize   ; x range of the scale in device coordinates (PiXels) 
    xd_sc_px = xsc_px[1]-xsc_px[0]    ; pixel size of the scale in x direction

    ysc_px = !y.window * !d.y_vsize   ; y range of the scale in device coordinates (PiXels)
    yd_sc_px = ysc_px[1]-ysc_px[0]    ; pixel size of the scale in y direction 

    if (!d.flags and 1) ne 0 then begin     ; Postscript (scalable device)
        tv, scale, xsc_px[0], ysc_px[0],$
            xsize=xd_sc_px, ysize=yd_sc_px
    endif else begin                        ; screen
        tv,  $
            congrid( $                             ; poly_2d inflates the the scale to a two-dim pixle array
                scale, $                           ; array to be inflated
                xd_sc_px, yd_sc_px,  $             ; x- and y-size of the pixle array
                keyword_set(interp)  $             ; type of fit. 0 = neares neighbor
            ), $
            xsc_px[0]+1, ysc_px[0]+1               ; position where the lower left corner should be placed in device coordinates
    endelse

    ; replot the frame arround the scale as parts of it may be covered
    oplot,[0,1],[0,0],/noclip
    oplot,[0,0],[0,1],/noclip
    oplot,[0,1],[1,1],/noclip
    if zcol le 20 then yticks = zcol else yticks=0
    axis, ticklen=0, ystyle=1, yaxis=1, yrange=[zrange[0],zrange[1]], yticks=yticks,$
        ytitle=ztit, /ynozero,charthick=thick, charsize=charsize

    ; restore the image coordinate system for later oplots
    !x=ssx & !y=ssy & !p=ssp

end
