#!/usr/bin/perl
# talk to control client = ttcclient
# Provide some function which are useful to handle the communucation with the MIRA36 radarserver.
#
# The following function can be accessed by the last function exec1crcmd which interpretes 
# and executes one so called CR command. 
#
# Maybe some perl packages need to be installed by:
#
#    unix > perl -MCPAN -e shell
#    cpan > install Bundle::CPAN
#
# Matthias
#
use Time::HiRes qw( usleep gettimeofday );  # install Time::HiRescd from www.cpan.org!
use Expect;
use strict;

my $exp;    # static!
sub cclogin ($) {
# Log in to the radarserver by the account crctl. If the requested connection type view_control
# is not available retry relogin_tries times. The function returns the type of the connection 
# which was actually esteblished. 
    # my $view_control=1; $view_control = $_[0] if $_[0]; # 1 = control/view, 2 = view only
    my $view_control = $_[0] ; # 1 = control/view, 0 = view only, 2= urgent

    use constant timeout => 10;
    use constant timeout4cmds => 27;

    my $relogin_tries=10;
    my $relogin_delay=30;
    my $view_control_r = -1;
    while ( $view_control != $view_control_r ){
	$exp = new Expect('/opt/mira36/bin/cclient', 28386);

	# make it a bit more silent
	$exp->log_stdout(0);

	# login
	unless ( $exp->expect(timeout, 'login: ') ){die "$0: did not get a login prompt from the radarserver\n" ;} 
	$exp->send("crctl\n");
	unless ( $exp->expect(timeout, 'Password: ') ){die "$0: radarserver did not ask for a passwd\n" ;}
	$exp->send("crctl\n");         # dwd
	# $exp->send("radarserver\n"); # nasa
	unless ( $exp->expect(timeout, 'administrator): ') ){die "$0: radarserver did not ask for the session level\n" ;}
	$exp->send("1\n");
	unless ( $exp->expect(timeout, 'Urgent): ') ){die "$0: radarserver did not ask if you like a view or a control session\n" ;}
	# my $vc_code = $view_control == 2 ? 0 : 1 ;
	my $vc_code = $view_control;
	$exp->send("$vc_code\n");
	$view_control_r = $exp->expect(timeout, 'Access: View/Control', 'Access: View only', 'Access denied') ;
	if ($view_control_r) {
	    if ($view_control_r == 1 ) { print STDERR "Control session accepted\n";}
	    elsif ($view_control_r == 2 ) { print STDERR "View-only control session accepted\n";}
	    elsif ($view_control_r == 3 ) { print "Access denied\n" ; return -1;}
	} else {print "timeout while waiting for Access type\n"; return -1;} 
	unless ( $exp->expect(timeout, 'Ready>> ' ) ){print "timeout while waiting for \"Ready>>\" after login\n"; return -1;}
	if ( $view_control_r == 2 ){$view_control_r=0;}
	if ( $view_control == 2 && $view_control_r == 1 ){$view_control_r=2;}
	if ( $view_control_r != $view_control){
	    print STDERR "Wrong view-control type. req:$view_control ret:$view_control_r Making a new attempt after $relogin_delay s.\n";
	    cclogout(); 
	    die "Trying to get the requested view-control type too many times\n" if ( $relogin_tries--  <= 0 ); 
	    sleep $relogin_delay;
	}
    }
    $view_control_r;
}

sub cclogout {
# Close the connection to the cclient
    if ($exp){
	$exp->send("quit\n");
	unless( $exp->expect(timeout, 'finished!') ) { print STDERR "server did not say finish! quiting\n";} 
	close($exp);
	print STDERR "control session finished\n";
	return 1;
    }
    return 0;
}

# $SIG{'INT'} = 'CLEANUP';
# sub CLEANUP {
#     cclogout();
#     print "\n\nCaught Interrupt (^C), Aborting\n";
#     exit(1);
# }


END{
# Close the connection on exit under most conditions
    cclogout();
}

sub srvcmd {
# send one server command to the radarserver and return the result(s) in the array @val 
# if the command was a get command.
    my $cmd = $_[0];
    my $dry=0 ; $dry = $_[1] if $_[1];

    my @val;
    chomp $cmd;
    if (! $dry ){
	$exp->send("$cmd\n");
	usleep 100000;
	if ( $cmd eq 'stop' ){ 
	    print "radarserver stopped\n"; close($exp); undef $exp ; exit 0;
	}
	my ($ixmatch,$returncode,$matched,$text_before,$text_after) = $exp->expect(timeout4cmds, 'Ready>> ');
	if ( $ixmatch && "$ixmatch" != 1 ) { 
	    die "timeout while waiting for \"Ready>>\" after $_ \n";
	}
	if ( $text_after ) { print STDERR "radarserver sent \"$text_after\" after Ready>>";}
	my $i=0;
	if ( $text_before =~ /ERROR/ ) { print STDERR "$text_before"; }
	if ( $text_before =~ /val= *([a-fA-F0-9.]+)/ ) {
	    print STDERR "$cmd ==> $1\n";
	    $val[0] = $1;
	} elsif ($cmd =~ /^get prc/){
	    my @srv_answer = split '\n',$text_before;
	    my $success = 0;
	    foreach my $srv_answer_line (@srv_answer) { 
		if ( $success && $srv_answer_line =~ /^([\-0-9.]+) - / ) {
		    print STDERR "$cmd [$i] ==> $srv_answer_line\n" ;
		    $val[$i++]=$1;
		} elsif ( $success && $srv_answer_line =~ /^([0-9.]+) +\(([0-9]+)\)/ ) {
		    print STDERR "$cmd [$i] ==> $srv_answer_line\n" ;
		    $val[$i++]=$1;
	        }
		$success=1 if $srv_answer_line =~ /SUCCESS/;
	    }
	} elsif ( $text_before =~ /Values: *([ 0-9.]+)/ ) {
	    print STDERR "$cmd ==> $1\n";
            @val = split ' +',$1;
	}
    } else { print STDERR ":$cmd:\n";}
    @val;
}

sub trbcmd {
# prepend set set/cmn/msc/0tr to the trb cmd and pass it to srvcmd
    my $trbcmd = $_[0];
    my $dry=0 ; $dry = $_[1] if $_[1];

    chomp $trbcmd; $trbcmd =~ s/^\s*// ; $trbcmd =~ s/\s*$//;
    my $srvcmd = "set set/cmn/csl/0tr \'$trbcmd\'";
    my @val;
    if (! $dry){
        @val = srvcmd($srvcmd);
    } else { print STDERR ":$srvcmd:\n"; }
    @val;
}

sub srvcrank {
# Increase or decrease the value of a numeric parameter.
    my ($path, $inc, $min, $max) = @_;
    my $dry=0 ; $dry = $_[4] if $_[4];
    my @val_org = srvcmd("get $path");
    if ( $#val_org != 0){print STDERR "Parameter has more than one value\n";return 0}
    my $new = $val_org[0] + $inc;
    if ($new >= $min && $new <= $max) {
        my $srvcmd = "set $path $new";
	if (! $dry){
            my @val_set = srvcmd($srvcmd);
	    print STDERR "val returned from set:@val_set\n";
	} else {print STDERR ":$srvcmd:\n";}
    } else {print STDERR "$val_org[0] + $inc is not in the expected range\n";}
    my @val_new = srvcmd("get $path");
    @val_new;
}

sub trbcrank {
# Increase or decrease the value of a numeric parameter on trb level.
    my ($trbcmd_, $inc, $min, $max) = @_;  # $trbcmd_ dose not include the value or get nore the single quotes
    my $dry=0 ; $dry = $_[4] if $_[4];
    my @val_org = srvcmd("set set/cmn/msc/0tr \'$trbcmd_ get\'");
    if ( $#val_org != 0){print STDERR "Parameter has more than one value\n";return 0}
    my $new = $val_org[0] + $inc;
    if ($new >= $min && $new <= $max) {
        my $srvcmd = "set set/cmn/msc/0tr \'$trbcmd_ $new\'";
        if (! $dry){
            my @val_set = srvcmd($srvcmd);
	    print STDERR "val returned from set:@val_set\n";
	} else {print STDERR ":$srvcmd:\n";}
    } else {print STDERR "$val_org[0] + $inc is not in the expected range\n";}
    my @val_new = srvcmd("set set/cmn/msc/0tr \'$trbcmd_ get\'");
    @val_new;
}

{
my ($StartTime,$StartTimeMicro) = gettimeofday();
my $GeisterSekunde = $StartTime - ($StartTime % (3600*24)); 
# The first second $GeisterSekunde of the current Day (at program startup) is used 
# as zero of the time scale for improoving the precision.
sub sectil{
# Calculate the time in seconds untill the next action should be performed.
# The time when an action should be performed is characerized by the first two 
# aruments which are given in units of seconds: 
#     $tsched     = offset in this cycle at which a concrete action is scheduled
#     $cycletime  = lengthe of the period
# E.g. if an action should be taken 5 munutes after each full hour then before
# doing it you should wait for
#     usleep ( sectil(300, 3600) );
# or simply:
#     crcmd > waittil 300 3600
#
# Matthias
#
    my $tsched = $_[0];
    my $cycletime = $_[1];
    my ($NowSec,$NowMicro) = gettimeofday ();
    my $SecondOfDay = $NowSec-$GeisterSekunde;
    my $ts = ( $tsched - ($NowSec-$GeisterSekunde) ) % $cycletime  - $NowMicro * 1e-6;
    $ts += $cycletime if $ts < 0 ;
    return $ts;
}}


#############################################################################################
{
my @val=(0); 
my %my;

sub interpolate {
# Emulate what Lary Wall calls interpolate. (used by the following routine)
    no strict 'vars';
    $my{dddddd}=1;
    my $s = shift;
    $s =~ s/``/\222\223\224/g;    # \222 is a character which is hopefully never needed
    while ($s =~ /^(.*)`([^`]*)`(.*)$/){
        $s = $1 . eval(sprintf "\"$2\"") . $3;
    }
    $s =~ s/\222\223\224/`/g;
    $s;
}


sub exec1crcmd {
# Interprete and execute a single crcmd which is passed as string
# in the first argument. Depending on the type of the command one of the routines above 
# is called for performing the dirty detailes. 
#
# The second argument is used to pass some keyword parameters, e.g. the connection type. 
#
# The Array @val is returned by each command. In case of a get-type
# server or trb command it contains the output from the radarserver.
# In case of a shell command it contains its stdout lines. In case
# of the pl command it passes the eval-ed value. 
#
# Note that @val is static to exec1crcmd. Therefore the @val which was
# the result of one crcmd can be used for further 
# calculations in the following pl command. e.g.:
#    srv get set/cmn/msc/0ah
#    pl $my{a_heating_onoff} = $val[0]
#
# Additionaly perl style interpolation can be used for inserting the 
# elements of @val, or user variables saved in the hash %my as to
# a cr command line. If these variables are braced by back-quotes 
# they are interpolated to their values.
#    pl $my{hi} = 'hello'
#    sh echo `$my{hi}`
# prints hello
# 
#
# The following types of commands are implemented:
#
# Server Commands:
# ================
#     srv <set|get> <path> [<index>] <value>
#
# Lines preceded by the string "srv" are sent though a control
# connection to the radarserver (excluding the "srv"). A control
# connection is established at the startup of crcmd unless the key
# word parameter control_view=2 is set on the calling command line.
#
#
# TRB Commands:
# =============
#     trb '<cmd>'
#
# Lines begining with a "trb" are passed to the TRB. E.g.:
#     trb 'calibr on'
# is converted to the server command: 
#     set set/cmn/msc/0tr 'calibr on'
# Note, the single quotes must be given!
#
#
# Crank Parameter, Server-level:
# ==============================
#     srvcrank <path> <in/de-crement> <new_min> <new_max>
#
# Change the value of a parameter by <in/de-vrement>. This is only
# done if the target value will lie between min and max.
#
#
# Crank Parameter, TRB-level:
# ===========================
#     trbcrank '<cmd_base>' <in/de-crement> <new_min> <new_max>
#
# Dito for TRB commands where <cmd_base> is the TRB command excluding
# the get or the value. This works only with the TRB commands with
# standard behaivour. Note that there are some exceptions. 
#
#
# Shell Commands:
# ===============
#     sh <bourn shell command>
#
# Pass a command to the bourn shell (sh). Each line of the stdout of the shell command is 
# returned in one element in @val.
#
#
# Perl Commands:
# ==============
#     pl <percommand>
# 
# Pass a command to the perl interpreter. The command will be eval'ed in the context
# of exec1crcmd. 
#
# Wait untill a periodically scheduled time:
# ==========================================
#     waittil <offset> <period length>
# 
# See the description at the definition of sub seconds_to_wait for more description.
#
#
# Matthias 
#
    my $crcmd = $_[0];
    my $Rkp = $_[1]; 

# Some preprocessing
    chomp $crcmd ;                               # remove eol char
    $crcmd =~ s/\s*#.*$//;                       # remove comments
    return ('empty line') if $crcmd =~ /^\s*$/;  # return if nothing is left after removin the comments

    $crcmd = interpolate($crcmd);

    if ($crcmd =~ /^\s*(srv|get|set)\s+(.+)$/ ) {							# srv command
        my $srvcmd;
	if ( $1 eq 'set' or $1 eq 'get') {
	    $srvcmd = $1 . ' ' . $2 ;
	} else {
	    $srvcmd = $2 ;
	}
	if ( !( $srvcmd =~ /^\s*set\s/) || $Rkp->{view_control} >= 1 ){
	    @val = srvcmd($srvcmd,$Rkp->{dry});
	    print STDERR "val=@val\n" if $srvcmd =~ /^\s*get\s/ ;
	} else { print STDERR "set commands are only allowed in view-control mode\n"; }
    }

    elsif ($crcmd =~ /^\s*trb +\'(.*)\'\s*$/){						# trb command
        if ( $Rkp->{view_control} >= 1 ){
	    @val = trbcmd($1,$Rkp->{dry});
	    print STDERR "val=@val\n" if $crcmd =~ /get/ ;
	} else {print STDERR "TRB commands are only allowed in view-control mode\n";}
    }

    elsif ($crcmd =~ /^\s*srvcrank\s+(\S\S\S\/\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s*$/){	# srvcrank
        if ( $Rkp->{view_control} >= 1 ){
            @val = srvcrank($1,$2,$3,$4,$Rkp->{dry}); 
	} else {print STDERR "srvcrank commands are only allowed in view-control mode\n";}
    }

    elsif ($crcmd =~ /^\s*trbcrank\s+\'(.*)\'\s+(\S+)\s+(\S+)\s+(\S+)\s*$/){		# trbcrank
        if ( $Rkp->{view_control} >= 1 ){
            @val = trbcrank($1,$2,$3,$4,$Rkp->{dry});
	} else {print STDERR "srvcrank commands are only allowed in view-control mode (view_control=1)\n";}
    }

    elsif ( $crcmd =~ /^\s*sh +(.*)$/ ){						# shell command
        # @val = system($1) if $1;
	if ($1){ 
	    open( ShHand , $1 . ' |' ) || die "Can't execute: $1\n";
	    @val = <ShHand>;
	}
    }

    elsif ( $crcmd =~ /^\s*pl +(.*)$/ ){						# perl command
        no strict 'vars';
	$my{dddddd}=1;
        @val = eval($1) if $1; 
    }

    elsif ( $crcmd =~ /^\s*(wait|sec)til\s+(\d+)\s+(\d+)\s*$/ ){			# perl command
        my $wait = $1;
        my $sec = sectil ($2,$3);
	usleep($sec*1e+6) if ($wait eq 'wait');
        @val = ($sec) ; 
    }

    else { print STDERR "unknown command or wrong syntax: $crcmd\n" ; @val=(); }

    return @val ;
}}

1;   # don't delete this line!
